const char luarocks_admin_cache_lua[] =
"\n"
"--- Module handling the LuaRocks local cache.\n"
"-- Adds a rock or rockspec to a rocks server.\n"
"local cache = {}\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local util = require(\"luarocks.util\")\n"
"\n"
"function cache.get_upload_server(server)\n"
"   if not server then server = cfg.upload_server end\n"
"   if not server then\n"
"      return nil, \"No server specified and no default configured with upload_server.\"\n"
"   end\n"
"   return server, cfg.upload_servers and cfg.upload_servers[server]\n"
"end\n"
"\n"
"function cache.get_server_urls(server, upload_server)\n"
"   local download_url = server\n"
"   local login_url = nil\n"
"   if upload_server then\n"
"      if upload_server.rsync then download_url = \"rsync://\"..upload_server.rsync\n"
"      elseif upload_server.http then download_url = \"http://\"..upload_server.http\n"
"      elseif upload_server.ftp then download_url = \"ftp://\"..upload_server.ftp\n"
"      end\n"
"      \n"
"      if upload_server.ftp then login_url = \"ftp://\"..upload_server.ftp\n"
"      elseif upload_server.sftp then login_url = \"sftp://\"..upload_server.sftp\n"
"      end\n"
"   end\n"
"   return download_url, login_url\n"
"end\n"
"\n"
"function cache.split_server_url(url, user, password)\n"
"   local protocol, server_path = dir.split_url(url)\n"
"   if protocol == \"file\" then\n"
"      server_path = fs.absolute_name(server_path)\n"
"   elseif server_path:match(\"@\") then\n"
"      local credentials\n"
"      credentials, server_path = server_path:match(\"([^@]*)@(.*)\")\n"
"      if credentials:match(\":\") then\n"
"         user, password = credentials:match(\"([^:]*):(.*)\")\n"
"      else\n"
"         user = credentials\n"
"      end\n"
"   end\n"
"   local local_cache = cfg.local_cache .. \"/\" .. server_path:gsub(\"[\\\\/]\", \"_\")\n"
"   return local_cache, protocol, server_path, user, password\n"
"end\n"
"\n"
"local function download_cache(protocol, server_path, user, password)\n"
"   os.remove(\"index.html\")\n"
"   -- TODO abstract away explicit 'wget' call\n"
"   if protocol == \"rsync\" then\n"
"      local srv, path = server_path:match(\"([^/]+)(/.+)\")\n"
"      return fs.execute(cfg.variables.RSYNC..\" \"..cfg.variables.RSYNCFLAGS..\" -e ssh \"..user..\"@\"..srv..\":\"..path..\"/ ./\")\n"
"   elseif protocol == \"file\" then\n"
"      return fs.copy_contents(server_path, \".\")\n"
"   else \n"
"      local login_info = \"\"\n"
"      if user then login_info = \" --user=\"..user end\n"
"      if password then login_info = login_info .. \" --password=\"..password end\n"
"      return fs.execute(cfg.variables.WGET..\" --no-cache -q -m -np -nd \"..protocol..\"://\"..server_path..login_info)\n"
"   end\n"
"end\n"
"\n"
"function cache.refresh_local_cache(url, given_user, given_password)\n"
"   local local_cache, protocol, server_path, user, password = cache.split_server_url(url, given_user, given_password)\n"
"\n"
"   local ok, err = fs.make_dir(local_cache)\n"
"   if not ok then\n"
"      return nil, \"Failed creating local cache dir: \"..err\n"
"   end\n"
"\n"
"   fs.change_dir(local_cache)\n"
"   \n"
"   util.printout(\"Refreshing cache \"..local_cache..\"...\")\n"
"\n"
"   ok = download_cache(protocol, server_path, user, password)\n"
"   if not ok then\n"
"      return nil, \"Failed downloading cache.\"\n"
"   end\n"
"\n"
"   return local_cache, protocol, server_path, user, password\n"
"end\n"
"\n"
"return cache\n"
""
;
