const char luarocks_admin_cmd_add_lua[] =
"\n"
"--- Module implementing the luarocks-admin \"add\" command.\n"
"-- Adds a rock or rockspec to a rocks server.\n"
"local add = {}\n"
"\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local util = require(\"luarocks.util\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local writer = require(\"luarocks.manif.writer\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local cache = require(\"luarocks.admin.cache\")\n"
"local index = require(\"luarocks.admin.index\")\n"
"\n"
"add.help_summary = \"Add a rock or rockspec to a rocks server.\"\n"
"add.help_arguments = \"[--server=<server>] [--no-refresh] {<rockspec>|<rock>...}\"\n"
"add.help = [[\n"
"Arguments are local files, which may be rockspecs or rocks.\n"
"The flag --server indicates which server to use.\n"
"If not given, the default server set in the upload_server variable\n"
"from the configuration file is used instead.\n"
"\n"
"--no-refresh  The local cache should not be refreshed\n"
"              prior to generation of the updated manifest.\n"
"--index       Produce an index.html file for the manifest.\n"
"              This flag is automatically set if an index.html\n"
"              file already exists.\n"
"]]\n"
"\n"
"local function zip_manifests()\n"
"   for ver in util.lua_versions() do\n"
"      local file = \"manifest-\"..ver\n"
"      local zip = file..\".zip\"\n"
"      fs.delete(dir.path(fs.current_dir(), zip))\n"
"      fs.zip(zip, file)\n"
"   end\n"
"end\n"
"\n"
"local function add_files_to_server(refresh, rockfiles, server, upload_server, do_index)\n"
"   assert(type(refresh) == \"boolean\" or not refresh)\n"
"   assert(type(rockfiles) == \"table\")\n"
"   assert(type(server) == \"string\")\n"
"   assert(type(upload_server) == \"table\" or not upload_server)\n"
"   \n"
"   local download_url, login_url = cache.get_server_urls(server, upload_server)\n"
"   local at = fs.current_dir()\n"
"   local refresh_fn = refresh and cache.refresh_local_cache or cache.split_server_url\n"
"   \n"
"   local local_cache, protocol, server_path, user, password = refresh_fn(download_url, cfg.upload_user, cfg.upload_password)\n"
"   if not local_cache then\n"
"      return nil, protocol\n"
"   end\n"
"   \n"
"   if not login_url then\n"
"      login_url = protocol..\"://\"..server_path\n"
"   end\n"
"   \n"
"   local ok, err = fs.change_dir(at)\n"
"   if not ok then return nil, err end\n"
"   \n"
"   local files = {}\n"
"   for _, rockfile in ipairs(rockfiles) do\n"
"      if fs.exists(rockfile) then\n"
"         util.printout(\"Copying file \"..rockfile..\" to \"..local_cache..\"...\")\n"
"         local absolute = fs.absolute_name(rockfile)\n"
"         fs.copy(absolute, local_cache, \"read\")\n"
"         table.insert(files, dir.base_name(absolute))\n"
"      else\n"
"         util.printerr(\"File \"..rockfile..\" not found\")\n"
"      end\n"
"   end\n"
"   if #files == 0 then\n"
"      return nil, \"No files found\"\n"
"   end\n"
"\n"
"   local ok, err = fs.change_dir(local_cache)\n"
"   if not ok then return nil, err end\n"
"\n"
"   util.printout(\"Updating manifest...\")\n"
"   writer.make_manifest(local_cache, \"one\", true)\n"
"   \n"
"   zip_manifests()\n"
"   \n"
"   if fs.exists(\"index.html\") then\n"
"      do_index = true\n"
"   end\n"
"\n"
"   if do_index then\n"
"      util.printout(\"Updating index.html...\")\n"
"      index.make_index(local_cache)\n"
"   end\n"
"\n"
"   local login_info = \"\"\n"
"   if user then login_info = \" -u \"..user end\n"
"   if password then login_info = login_info..\":\"..password end\n"
"   if not login_url:match(\"/$\") then\n"
"      login_url = login_url .. \"/\"\n"
"   end\n"
"\n"
"   if do_index then\n"
"      table.insert(files, \"index.html\")\n"
"   end\n"
"   table.insert(files, \"manifest\")\n"
"   for ver in util.lua_versions() do\n"
"      table.insert(files, \"manifest-\"..ver)\n"
"      table.insert(files, \"manifest-\"..ver..\".zip\")\n"
"   end\n"
"\n"
"   -- TODO abstract away explicit 'curl' call\n"
"\n"
"   local cmd\n"
"   if protocol == \"rsync\" then\n"
"      local srv, path = server_path:match(\"([^/]+)(/.+)\")\n"
"      cmd = cfg.variables.RSYNC..\" \"..cfg.variables.RSYNCFLAGS..\" -e ssh \"..local_cache..\"/ \"..user..\"@\"..srv..\":\"..path..\"/\"\n"
"   elseif protocol == \"file\" then\n"
"      return fs.copy_contents(local_cache, server_path)\n"
"   elseif upload_server and upload_server.sftp then\n"
"      local part1, part2 = upload_server.sftp:match(\"^([^/]*)/(.*)$\")\n"
"      cmd = cfg.variables.SCP..\" \"..table.concat(files, \" \")..\" \"..user..\"@\"..part1..\":/\"..part2\n"
"   else\n"
"      cmd = cfg.variables.CURL..\" \"..login_info..\" -T '{\"..table.concat(files, \",\")..\"}' \"..login_url\n"
"   end\n"
"\n"
"   util.printout(cmd)\n"
"   return fs.execute(cmd)\n"
"end\n"
"\n"
"function add.command(flags, ...)\n"
"   local files = {...}\n"
"   if #files < 1 then\n"
"      return nil, \"Argument missing. \"..util.see_help(\"add\", \"luarocks-admin\")\n"
"   end\n"
"   local server, server_table = cache.get_upload_server(flags[\"server\"])\n"
"   if not server then return nil, server_table end\n"
"   return add_files_to_server(not flags[\"no-refresh\"], files, server, server_table, flags[\"index\"])\n"
"end\n"
"\n"
"\n"
"return add\n"
""
;
