const char luarocks_admin_cmd_make_manifest_lua[] =
"\n"
"--- Module implementing the luarocks-admin \"make_manifest\" command.\n"
"-- Compile a manifest file for a repository.\n"
"local make_manifest = {}\n"
"\n"
"local writer = require(\"luarocks.manif.writer\")\n"
"local index = require(\"luarocks.admin.index\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local util = require(\"luarocks.util\")\n"
"local deps = require(\"luarocks.deps\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local dir = require(\"luarocks.dir\")\n"
"\n"
"make_manifest.help_summary = \"Compile a manifest file for a repository.\"\n"
"\n"
"make_manifest.help = [[\n"
"<argument>, if given, is a local repository pathname.\n"
"\n"
"--local-tree  If given, do not write versioned versions of the manifest file.\n"
"              Use this when rebuilding the manifest of a local rocks tree.\n"
"]]\n"
"\n"
"--- Driver function for \"make_manifest\" command.\n"
"-- @param repo string or nil: Pathname of a local repository. If not given,\n"
"-- the default local repository configured as cfg.rocks_dir is used.\n"
"-- @return boolean or (nil, string): True if manifest was generated,\n"
"-- or nil and an error message.\n"
"function make_manifest.command(flags, repo)\n"
"   assert(type(repo) == \"string\" or not repo)\n"
"   repo = repo or cfg.rocks_dir\n"
"  \n"
"   util.printout(\"Making manifest for \"..repo)\n"
"   \n"
"   if repo:match(\"/lib/luarocks\") and not flags[\"local-tree\"] then\n"
"      util.warning(\"This looks like a local rocks tree, but you did not pass --local-tree.\")\n"
"   end\n"
"   \n"
"   local ok, err = writer.make_manifest(repo, deps.get_deps_mode(flags), not flags[\"local-tree\"])\n"
"   if ok and not flags[\"local-tree\"] then\n"
"      util.printout(\"Generating index.html for \"..repo)\n"
"      index.make_index(repo)\n"
"   end\n"
"   if flags[\"local-tree\"] then\n"
"      for luaver in util.lua_versions() do\n"
"         fs.delete(dir.path(repo, \"manifest-\"..luaver))\n"
"      end\n"
"   end\n"
"   return ok, err\n"
"end\n"
"\n"
"return make_manifest\n"
""
;
