const char luarocks_admin_cmd_remove_lua[] =
"\n"
"--- Module implementing the luarocks-admin \"remove\" command.\n"
"-- Removes a rock or rockspec from a rocks server.\n"
"local admin_remove = {}\n"
"\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local util = require(\"luarocks.util\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local writer = require(\"luarocks.manif.writer\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local cache = require(\"luarocks.admin.cache\")\n"
"local index = require(\"luarocks.admin.index\")\n"
"\n"
"admin_remove.help_summary = \"Remove a rock or rockspec from a rocks server.\"\n"
"admin_remove.help_arguments = \"[--server=<server>] [--no-refresh] {<rockspec>|<rock>...}\"\n"
"admin_remove.help = [[\n"
"Arguments are local files, which may be rockspecs or rocks.\n"
"The flag --server indicates which server to use.\n"
"If not given, the default server set in the upload_server variable\n"
"from the configuration file is used instead.\n"
"The flag --no-refresh indicates the local cache should not be refreshed\n"
"prior to generation of the updated manifest.\n"
"]]\n"
"\n"
"local function remove_files_from_server(refresh, rockfiles, server, upload_server)\n"
"   assert(type(refresh) == \"boolean\" or not refresh)\n"
"   assert(type(rockfiles) == \"table\")\n"
"   assert(type(server) == \"string\")\n"
"   assert(type(upload_server) == \"table\" or not upload_server)\n"
"\n"
"   local download_url, login_url = cache.get_server_urls(server, upload_server)\n"
"   local at = fs.current_dir()\n"
"   local refresh_fn = refresh and cache.refresh_local_cache or cache.split_server_url\n"
"   \n"
"   local local_cache, protocol, server_path, user, password = refresh_fn(download_url, cfg.upload_user, cfg.upload_password)\n"
"   if not local_cache then\n"
"      return nil, protocol\n"
"   end\n"
"   if protocol ~= \"rsync\" then\n"
"      return nil, \"This command requires 'rsync', check your configuration.\"\n"
"   end\n"
"   \n"
"   local ok, err = fs.change_dir(at)\n"
"   if not ok then return nil, err end\n"
"   \n"
"   local nr_files = 0\n"
"   for _, rockfile in ipairs(rockfiles) do\n"
"      local basename = dir.base_name(rockfile)\n"
"      local file = dir.path(local_cache, basename)\n"
"      util.printout(\"Removing file \"..file..\"...\")\n"
"      fs.delete(file)\n"
"      if not fs.exists(file) then\n"
"         nr_files = nr_files + 1\n"
"      else\n"
"         util.printerr(\"Failed removing \"..file)\n"
"      end\n"
"   end\n"
"   if nr_files == 0 then\n"
"      return nil, \"No files removed.\"\n"
"   end\n"
"\n"
"   local ok, err = fs.change_dir(local_cache)\n"
"   if not ok then return nil, err end\n"
"\n"
"   util.printout(\"Updating manifest...\")\n"
"   writer.make_manifest(local_cache, \"one\", true)\n"
"   util.printout(\"Updating index.html...\")\n"
"   index.make_index(local_cache)\n"
"\n"
"   local srv, path = server_path:match(\"([^/]+)(/.+)\")\n"
"   local cmd = cfg.variables.RSYNC..\" \"..cfg.variables.RSYNCFLAGS..\" --delete -e ssh \"..local_cache..\"/ \"..user..\"@\"..srv..\":\"..path..\"/\"\n"
"\n"
"   util.printout(cmd)\n"
"   fs.execute(cmd)\n"
"\n"
"   return true\n"
"end\n"
"\n"
"function admin_remove.command(flags, ...)\n"
"   local files = {...}\n"
"   if #files < 1 then\n"
"      return nil, \"Argument missing. \"..util.see_help(\"remove\", \"luarocks-admin\")\n"
"   end\n"
"   local server, server_table = cache.get_upload_server(flags[\"server\"])\n"
"   if not server then return nil, server_table end\n"
"   return remove_files_from_server(not flags[\"no-refresh\"], files, server, server_table)\n"
"end\n"
"\n"
"\n"
"return admin_remove\n"
""
;
