const char luarocks_admin_index_lua[] =
"\n"
"--- Module which builds the index.html page to be used in rocks servers.\n"
"local index = {}\n"
"\n"
"local util = require(\"luarocks.util\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local vers = require(\"luarocks.core.vers\")\n"
"local persist = require(\"luarocks.persist\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local manif = require(\"luarocks.manif\")\n"
"\n"
"local ext_url_target = ' target=\"_blank\"'\n"
"\n"
"local index_header = [[\n"
"<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"
"<html>\n"
"<head>\n"
"<title>Available rocks</title>\n"
"<meta http-equiv=\"content-type\" content=\"text/html; charset=iso-8859-1\">\n"
"<style>\n"
"body {\n"
"   background-color: white;\n"
"   font-family: \"bitstream vera sans\", \"verdana\", \"sans\";\n"
"   font-size: 14px;\n"
"}\n"
"a {\n"
"   color: #0000c0;\n"
"   text-decoration: none;\n"
"}\n"
"a.pkg {\n"
"   color: black;\n"
"}\n"
"a:hover {\n"
"   text-decoration: underline;\n"
"}\n"
"td.main {\n"
"   border-style: none;\n"
"}\n"
"blockquote {\n"
"   font-size: 12px;\n"
"}\n"
"td.package {\n"
"   background-color: #f0f0f0;\n"
"   vertical-align: top;\n"
"}\n"
"td.spacer {\n"
"   height: 5px;\n"
"}\n"
"td.version {\n"
"   background-color: #d0d0d0;\n"
"   vertical-align: top;\n"
"   text-align: left;\n"
"   padding: 5px;\n"
"   width: 100px;\n"
"}\n"
"p.manifest {\n"
"   font-size: 8px;\n"
"}\n"
"</style>\n"
"</head>\n"
"<body>\n"
"<h1>Available rocks</h1>\n"
"<p>\n"
"Lua modules available from this location for use with <a href=\"http://www.luarocks.org\">LuaRocks</a>:\n"
"</p>\n"
"<table class=\"main\">\n"
"]]\n"
"\n"
"local index_package_begin = [[\n"
"<td class=\"package\">\n"
"<p><a name=\"$anchor\"></a><a href=\"#$anchor\" class=\"pkg\"><b>$package</b></a> - $summary<br/>\n"
"</p><blockquote><p>$detailed<br/>\n"
"$externaldependencies\n"
"<font size=\"-1\"><a href=\"$original\">latest sources</a> $homepage | License: $license</font></p>\n"
"</blockquote></a></td>\n"
"<td class=\"version\">\n"
"]]\n"
"\n"
"local index_package_end = [[\n"
"</td></tr>\n"
"<tr><td colspan=\"2\" class=\"spacer\"></td></tr>\n"
"]]\n"
"\n"
"local index_footer_begin = [[\n"
"</table>\n"
"<p class=\"manifest\">\n"
"<a href=\"manifest\">manifest file</a>\n"
"]]\n"
"local index_manifest_ver = [[\n"
"&bull; <a href=\"manifest-$VER\">Lua $VER manifest file</a> (<a href=\"manifest-$VER.zip\">zip</a>) \n"
"]]\n"
"local index_footer_end = [[\n"
"</p>\n"
"</body>\n"
"</html>\n"
"]]\n"
"\n"
"function index.format_external_dependencies(rockspec)\n"
"   if rockspec.external_dependencies then\n"
"      local deplist = {}\n"
"      local listed_set = {}\n"
"      local plats = nil\n"
"      for name, desc in util.sortedpairs(rockspec.external_dependencies) do\n"
"         if name ~= \"platforms\" then\n"
"            table.insert(deplist, name:lower())\n"
"            listed_set[name] = true\n"
"         else\n"
"            plats = desc\n"
"         end\n"
"      end\n"
"      if plats then\n"
"         for plat, entries in util.sortedpairs(plats) do\n"
"            for name, desc in util.sortedpairs(entries) do\n"
"               if not listed_set[name] then\n"
"                  table.insert(deplist, name:lower() .. \" (on \"..plat..\")\")\n"
"               end\n"
"            end\n"
"         end\n"
"      end\n"
"      return '<p><b>External dependencies:</b> ' .. table.concat(deplist, ',&nbsp;').. '</p>'\n"
"   else\n"
"      return \"\"\n"
"   end\n"
"end\n"
"\n"
"function index.make_index(repo)\n"
"   if not fs.is_dir(repo) then\n"
"      return nil, \"Cannot access repository at \"..repo\n"
"   end\n"
"   local manifest = manif.load_manifest(repo)\n"
"   local out = io.open(dir.path(repo, \"index.html\"), \"w\")\n"
"   \n"
"   out:write(index_header)\n"
"   for package, version_list in util.sortedpairs(manifest.repository) do\n"
"      local latest_rockspec = nil\n"
"      local output = index_package_begin\n"
"      for version, data in util.sortedpairs(version_list, vers.compare_versions) do\n"
"         local versions = {}\n"
"         output = output..version..':&nbsp;'\n"
"         table.sort(data, function(a,b) return a.arch < b.arch end)\n"
"         for _, item in ipairs(data) do\n"
"            local file\n"
"            if item.arch == 'rockspec' then\n"
"               file = (\"%s-%s.rockspec\"):format(package, version)\n"
"               if not latest_rockspec then latest_rockspec = file end\n"
"            else\n"
"               file = (\"%s-%s.%s.rock\"):format(package, version, item.arch)\n"
"            end\n"
"            table.insert(versions, '<a href=\"'..file..'\">'..item.arch..'</a>')\n"
"         end\n"
"         output = output .. table.concat(versions, ',&nbsp;') .. '<br/>'\n"
"      end\n"
"      output = output .. index_package_end\n"
"      if latest_rockspec then\n"
"         local rockspec = persist.load_into_table(dir.path(repo, latest_rockspec))\n"
"         local descript = rockspec.description or {}\n"
"         local vars = {\n"
"            anchor = package,\n"
"            package = rockspec.package,\n"
"            original = rockspec.source.url,\n"
"            summary = descript.summary or \"\",\n"
"            detailed = descript.detailed or \"\",\n"
"            license = descript.license or \"N/A\",\n"
"            homepage = descript.homepage and ('| <a href=\"'..descript.homepage..'\"'..ext_url_target..'>project homepage</a>') or \"\",\n"
"            externaldependencies = index.format_external_dependencies(rockspec)\n"
"         }\n"
"         vars.detailed = vars.detailed:gsub(\"\\n\\n\", \"</p><p>\"):gsub(\"%s+\", \" \")\n"
"         vars.detailed = vars.detailed:gsub(\"(https\?://[a-zA-Z0-9%.%%-_%+%[%]=%\?&/$@;:]+)\", '<a href=\"%1\"'..ext_url_target..'>%1</a>')\n"
"         output = output:gsub(\"$(%w+)\", vars)\n"
"      else\n"
"         output = output:gsub(\"$anchor\", package)\n"
"         output = output:gsub(\"$package\", package)\n"
"         output = output:gsub(\"$(%w+)\", \"\")\n"
"      end\n"
"      out:write(output)\n"
"   end\n"
"   out:write(index_footer_begin)\n"
"   for ver in util.lua_versions() do\n"
"      out:write((index_manifest_ver:gsub(\"$VER\", ver)))\n"
"   end\n"
"   out:write(index_footer_end)\n"
"   out:close()\n"
"end\n"
"\n"
"return index\n"
""
;
