const char luarocks_build_lua[] =
"\n"
"local build = {}\n"
"\n"
"local path = require(\"luarocks.path\")\n"
"local util = require(\"luarocks.util\")\n"
"local fun = require(\"luarocks.fun\")\n"
"local fetch = require(\"luarocks.fetch\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local deps = require(\"luarocks.deps\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local repos = require(\"luarocks.repos\")\n"
"local writer = require(\"luarocks.manif.writer\")\n"
"\n"
"build.opts = util.opts_table(\"build.opts\", {\n"
"   need_to_fetch = \"boolean\",\n"
"   minimal_mode = \"boolean\",\n"
"   deps_mode = \"string\",\n"
"   build_only_deps = \"boolean\",\n"
"   namespace = \"string\?\",\n"
"   branch = \"boolean\",\n"
"   verify = \"boolean\",\n"
"})\n"
"\n"
"do\n"
"   --- Write to the current directory the contents of a table,\n"
"   -- where each key is a file name and its value is the file content.\n"
"   -- @param files table: The table of files to be written.\n"
"   local function extract_from_rockspec(files)\n"
"      for name, content in pairs(files) do\n"
"         local fd = io.open(dir.path(fs.current_dir(), name), \"w+\")\n"
"         fd:write(content)\n"
"         fd:close()\n"
"      end\n"
"   end\n"
"\n"
"   --- Applies patches inlined in the build.patches section\n"
"   -- and extracts files inlined in the build.extra_files section\n"
"   -- of a rockspec.\n"
"   -- @param rockspec table: A rockspec table.\n"
"   -- @return boolean or (nil, string): True if succeeded or\n"
"   -- nil and an error message.\n"
"   function build.apply_patches(rockspec)\n"
"      assert(rockspec:type() == \"rockspec\")\n"
"\n"
"      if not (rockspec.build.extra_files or rockspec.build.patches) then\n"
"         return true\n"
"      end\n"
"\n"
"      local fd = io.open(fs.absolute_name(\".luarocks.patches.applied\"), \"r\")\n"
"      if fd then\n"
"         fd:close()\n"
"         return true\n"
"      end\n"
"\n"
"      if rockspec.build.extra_files then\n"
"         extract_from_rockspec(rockspec.build.extra_files)\n"
"      end\n"
"      if rockspec.build.patches then\n"
"         extract_from_rockspec(rockspec.build.patches)\n"
"         for patch, patchdata in util.sortedpairs(rockspec.build.patches) do\n"
"            util.printout(\"Applying patch \"..patch..\"...\")\n"
"            local create_delete = rockspec:format_is_at_least(\"3.0\")\n"
"            local ok, err = fs.apply_patch(tostring(patch), patchdata, create_delete)\n"
"            if not ok then\n"
"               return nil, \"Failed applying patch \"..patch\n"
"            end\n"
"         end\n"
"      end\n"
"\n"
"      fd = io.open(fs.absolute_name(\".luarocks.patches.applied\"), \"w\")\n"
"      if fd then\n"
"         fd:close()\n"
"      end\n"
"      return true\n"
"   end\n"
"end\n"
"\n"
"local function check_macosx_deployment_target(rockspec)\n"
"   local target = rockspec.build.macosx_deployment_target\n"
"   local function minor(version) \n"
"      return tonumber(version and version:match(\"^[^.]+%.([^.]+)\"))\n"
"   end\n"
"   local function patch_variable(var)\n"
"      if rockspec.variables[var]:match(\"MACOSX_DEPLOYMENT_TARGET\") then\n"
"         rockspec.variables[var] = (rockspec.variables[var]):gsub(\"MACOSX_DEPLOYMENT_TARGET=[^ ]*\", \"MACOSX_DEPLOYMENT_TARGET=\"..target)\n"
"      else\n"
"         rockspec.variables[var] = \"env MACOSX_DEPLOYMENT_TARGET=\"..target..\" \"..rockspec.variables[var]\n"
"      end\n"
"   end\n"
"   if cfg.is_platform(\"macosx\") and rockspec:format_is_at_least(\"3.0\") and target then\n"
"      local version = util.popen_read(\"sw_vers -productVersion\")\n"
"      local versionminor = minor(version)\n"
"      local targetminor = minor(target)\n"
"      if targetminor > versionminor then\n"
"         return nil, (\"This rock requires Mac OSX 10.%d, and you are running 10.%d.\"):format(targetminor, versionminor)\n"
"      end\n"
"      patch_variable(\"CC\")\n"
"      patch_variable(\"LD\")\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"local function process_dependencies(rockspec, opts)\n"
"   if not opts.build_only_deps then\n"
"      local ok, err, errcode = deps.check_external_deps(rockspec, \"build\")\n"
"      if err then\n"
"         return nil, err, errcode\n"
"      end\n"
"   end\n"
"\n"
"   local ok, err, errcode = deps.check_lua(rockspec.variables)\n"
"   if not ok then\n"
"      return nil, err, errcode\n"
"   end\n"
"\n"
"   if opts.deps_mode == \"none\" then\n"
"      util.warning(\"skipping dependency checks.\")\n"
"      return true\n"
"   end\n"
"   if not opts.build_only_deps then\n"
"      if next(rockspec.build_dependencies) then\n"
"         local ok, err, errcode = deps.fulfill_dependencies(rockspec, \"build_dependencies\", opts.deps_mode, opts.verify)\n"
"         if err then\n"
"            return nil, err, errcode\n"
"         end\n"
"      end\n"
"   end\n"
"   local ok, err, errcode = deps.fulfill_dependencies(rockspec, \"dependencies\", opts.deps_mode, opts.verify)\n"
"   if err then\n"
"      return nil, err, errcode\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"local function fetch_and_change_to_source_dir(rockspec, opts)\n"
"   if opts.minimal_mode then\n"
"      return true\n"
"   end\n"
"   if opts.need_to_fetch then\n"
"      if opts.branch then\n"
"         rockspec.source.branch = opts.branch\n"
"      end\n"
"      local ok, source_dir, errcode = fetch.fetch_sources(rockspec, true)\n"
"      if not ok then\n"
"         return nil, source_dir, errcode\n"
"      end\n"
"      local err\n"
"      ok, err = fs.change_dir(source_dir)\n"
"      if not ok then\n"
"         return nil, err\n"
"      end\n"
"   elseif rockspec.source.file then\n"
"      local ok, err = fs.unpack_archive(rockspec.source.file)\n"
"      if not ok then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"   fs.change_dir(rockspec.source.dir)\n"
"   return true\n"
"end\n"
"\n"
"local function prepare_install_dirs(name, version)\n"
"   local dirs = {\n"
"      lua = { name = path.lua_dir(name, version), is_module_path = true, perms = \"read\" },\n"
"      lib = { name = path.lib_dir(name, version), is_module_path = true, perms = \"exec\" },\n"
"      bin = { name = path.bin_dir(name, version), is_module_path = false, perms = \"exec\" },\n"
"      conf = { name = path.conf_dir(name, version), is_module_path = false, perms = \"read\" },\n"
"   }\n"
"\n"
"   for _, d in pairs(dirs) do\n"
"      local ok, err = fs.make_dir(d.name)\n"
"      if not ok then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"\n"
"   return dirs\n"
"end\n"
"\n"
"local function run_build_driver(rockspec)\n"
"   local btype = rockspec.build.type\n"
"   if btype == \"none\" then\n"
"      return true\n"
"   end\n"
"   -- Temporary compatibility\n"
"   if btype == \"module\" then\n"
"      util.printout(\"Do not use 'module' as a build type. Use 'builtin' instead.\")\n"
"      btype = \"builtin\"\n"
"      rockspec.build.type = btype\n"
"   end\n"
"   if cfg.accepted_build_types and not fun.contains(cfg.accepted_build_types, btype) then\n"
"      return nil, \"This rockspec uses the '\"..btype..\"' build type, which is blocked by the 'accepted_build_types' setting in your LuaRocks configuration.\"\n"
"   end\n"
"   local pok, driver = pcall(require, \"luarocks.build.\" .. btype)\n"
"   if not pok or type(driver) ~= \"table\" then\n"
"      return nil, \"Failed initializing build back-end for build type '\"..btype..\"': \"..driver\n"
"   end\n"
"   local ok, err = driver.run(rockspec)\n"
"   if not ok then\n"
"      return nil, \"Build error: \" .. err\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"local install_files\n"
"do\n"
"   --- Install files to a given location.\n"
"   -- Takes a table where the array part is a list of filenames to be copied.\n"
"   -- In the hash part, other keys, if is_module_path is set, are identifiers\n"
"   -- in Lua module format, to indicate which subdirectory the file should be\n"
"   -- copied to. For example, install_files({[\"foo.bar\"] = \"src/bar.lua\"}, \"boo\")\n"
"   -- will copy src/bar.lua to boo/foo.\n"
"   -- @param files table or nil: A table containing a list of files to copy in\n"
"   -- the format described above. If nil is passed, this function is a no-op.\n"
"   -- Directories should be delimited by forward slashes as in internet URLs.\n"
"   -- @param location string: The base directory files should be copied to.\n"
"   -- @param is_module_path boolean: True if string keys in files should be\n"
"   -- interpreted as dotted module paths.\n"
"   -- @param perms string (\"read\" or \"exec\"): Permissions of the newly created\n"
"   -- files installed.\n"
"   -- Directories are always created with the default permissions.\n"
"   -- @return boolean or (nil, string): True if succeeded or\n"
"   -- nil and an error message.\n"
"   local function install_to(files, location, is_module_path, perms)\n"
"      assert(type(files) == \"table\" or not files)\n"
"      assert(type(location) == \"string\")\n"
"      if not files then\n"
"         return true\n"
"      end\n"
"      for k, file in pairs(files) do\n"
"         local dest = location\n"
"         local filename = dir.base_name(file)\n"
"         if type(k) == \"string\" then\n"
"            local modname = k\n"
"            if is_module_path then\n"
"               dest = dir.path(location, path.module_to_path(modname))\n"
"               local ok, err = fs.make_dir(dest)\n"
"               if not ok then return nil, err end\n"
"               if filename:match(\"%.lua$\") then\n"
"                  local basename = modname:match(\"([^.]+)$\")\n"
"                  filename = basename..\".lua\"\n"
"               end\n"
"            else\n"
"               dest = dir.path(location, dir.dir_name(modname))\n"
"               local ok, err = fs.make_dir(dest)\n"
"               if not ok then return nil, err end\n"
"               filename = dir.base_name(modname)\n"
"            end\n"
"         else\n"
"            local ok, err = fs.make_dir(dest)\n"
"            if not ok then return nil, err end\n"
"         end\n"
"         local ok = fs.copy(dir.path(file), dir.path(dest, filename), perms)\n"
"         if not ok then\n"
"            return nil, \"Failed copying \"..file\n"
"         end\n"
"      end\n"
"      return true\n"
"   end\n"
"\n"
"   local function install_default_docs(name, version)\n"
"      local patterns = { \"readme\", \"license\", \"copying\", \".*%.md\" }\n"
"      local dest = dir.path(path.install_dir(name, version), \"doc\")\n"
"      local has_dir = false\n"
"      for file in fs.dir() do\n"
"         for _, pattern in ipairs(patterns) do\n"
"            if file:lower():match(\"^\"..pattern) then\n"
"               if not has_dir then\n"
"                  fs.make_dir(dest)\n"
"                  has_dir = true\n"
"               end\n"
"               fs.copy(file, dest, \"read\")\n"
"               break\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   install_files = function(rockspec, dirs)\n"
"      local name, version = rockspec.name, rockspec.version\n"
"\n"
"      if rockspec.build.install then\n"
"         for k, d in pairs(dirs) do\n"
"            local ok, err = install_to(rockspec.build.install[k], d.name, d.is_module_path, d.perms)\n"
"            if not ok then return nil, err end\n"
"         end\n"
"      end\n"
"\n"
"      local copy_directories = rockspec.build.copy_directories\n"
"      local copying_default = false\n"
"      if not copy_directories then\n"
"         copy_directories = {\"doc\"}\n"
"         copying_default = true\n"
"      end\n"
"\n"
"      local any_docs = false\n"
"      for _, copy_dir in pairs(copy_directories) do\n"
"         if fs.is_dir(copy_dir) then\n"
"            local dest = dir.path(path.install_dir(name, version), copy_dir)\n"
"            fs.make_dir(dest)\n"
"            fs.copy_contents(copy_dir, dest)\n"
"            any_docs = true\n"
"         else\n"
"            if not copying_default then\n"
"               return nil, \"Directory '\"..copy_dir..\"' not found\"\n"
"            end\n"
"         end\n"
"      end\n"
"      if not any_docs then\n"
"         install_default_docs(name, version)\n"
"      end\n"
"\n"
"      return true\n"
"   end\n"
"end\n"
"\n"
"local function write_rock_dir_files(rockspec, opts)\n"
"   local name, version = rockspec.name, rockspec.version\n"
"\n"
"   fs.copy(rockspec.local_abs_filename, path.rockspec_file(name, version), \"read\")\n"
"\n"
"   local ok, err = writer.make_rock_manifest(name, version)\n"
"   if not ok then return nil, err end\n"
"\n"
"   ok, err = writer.make_namespace_file(name, version, opts.namespace)\n"
"   if not ok then return nil, err end\n"
"\n"
"   return true\n"
"end\n"
"\n"
"--- Build and install a rock given a rockspec.\n"
"-- @param rockspec_file string: local or remote filename of a rockspec.\n"
"-- @param need_to_fetch boolean: true if sources need to be fetched,\n"
"-- false if the rockspec was obtained from inside a source rock.\n"
"-- @param minimal_mode boolean: true if there's no need to fetch,\n"
"-- unpack or change dir (this is used by \"luarocks make\"). Implies\n"
"-- need_to_fetch = false.\n"
"-- @param deps_mode string: Dependency mode: \"one\" for the current default tree,\n"
"-- \"all\" for all trees, \"order\" for all trees with priority >= the current default,\n"
"-- \"none\" for no trees.\n"
"-- @param build_only_deps boolean: true to build the listed dependencies only.\n"
"-- @param namespace string\?: a namespace for the rockspec\n"
"-- @param branch string\?: a forced branch to use\n"
"-- @return (string, string) or (nil, string, [string]): Name and version of\n"
"-- installed rock if succeeded or nil and an error message followed by an error code.\n"
"function build.build_rockspec(rockspec, opts)\n"
"   assert(rockspec:type() == \"rockspec\")\n"
"   assert(opts:type() == \"build.opts\")\n"
"\n"
"   if not rockspec.build then\n"
"      if rockspec:format_is_at_least(\"3.0\") then\n"
"         rockspec.build = {\n"
"            type = \"builtin\"\n"
"         }\n"
"      else\n"
"         return nil, \"Rockspec error: build table not specified\"\n"
"      end\n"
"   end\n"
"\n"
"   if not rockspec.build.type then\n"
"      if rockspec:format_is_at_least(\"3.0\") then\n"
"         rockspec.build.type = \"builtin\"\n"
"      else\n"
"         return nil, \"Rockspec error: build type not specified\"\n"
"      end\n"
"   end\n"
"\n"
"   local ok, err = process_dependencies(rockspec, opts)\n"
"   if not ok then return nil, err end\n"
"\n"
"   local name, version = rockspec.name, rockspec.version\n"
"   if opts.build_only_deps then\n"
"      return name, version\n"
"   end   \n"
"\n"
"   if repos.is_installed(name, version) then\n"
"      repos.delete_version(name, version, opts.deps_mode)\n"
"   end\n"
"\n"
"   ok, err = fetch_and_change_to_source_dir(rockspec, opts)\n"
"   if not ok then return nil, err end\n"
"   \n"
"   local dirs, err = prepare_install_dirs(name, version)\n"
"   if not dirs then return nil, err end\n"
"   \n"
"   local rollback = util.schedule_function(function()\n"
"      fs.delete(path.install_dir(name, version))\n"
"      fs.remove_dir_if_empty(path.versions_dir(name))\n"
"   end)\n"
"\n"
"   ok, err = build.apply_patches(rockspec)\n"
"   if not ok then return nil, err end\n"
"   \n"
"   ok, err = check_macosx_deployment_target(rockspec)\n"
"   if not ok then return nil, err end\n"
"   \n"
"   ok, err = run_build_driver(rockspec)\n"
"   if not ok then return nil, err end\n"
"   \n"
"   ok, err = install_files(rockspec, dirs)\n"
"   if not ok then return nil, err end\n"
"   \n"
"   for _, d in pairs(dirs) do\n"
"      fs.remove_dir_if_empty(d.name)\n"
"   end\n"
"\n"
"   fs.pop_dir()\n"
"   if opts.need_to_fetch then\n"
"      fs.pop_dir()\n"
"   end\n"
"\n"
"   ok, err = write_rock_dir_files(rockspec, opts)\n"
"   if not ok then return nil, err end\n"
"\n"
"   ok, err = repos.deploy_files(name, version, repos.should_wrap_bin_scripts(rockspec), opts.deps_mode)\n"
"   if not ok then return nil, err end\n"
"   \n"
"   util.remove_scheduled_function(rollback)\n"
"   rollback = util.schedule_function(function()\n"
"      repos.delete_version(name, version, opts.deps_mode)\n"
"   end)\n"
"\n"
"   ok, err = repos.run_hook(rockspec, \"post_install\")\n"
"   if not ok then return nil, err end\n"
"\n"
"   util.announce_install(rockspec)\n"
"   util.remove_scheduled_function(rollback)\n"
"   return name, version\n"
"end\n"
"\n"
"return build\n"
""
;
