const char luarocks_build_builtin_lua[] =
"\n"
"--- A builtin build system: back-end to provide a portable way of building C-based Lua modules.\n"
"local builtin = {}\n"
"\n"
"local unpack = unpack or table.unpack\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"local path = require(\"luarocks.path\")\n"
"local util = require(\"luarocks.util\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local dir = require(\"luarocks.dir\")\n"
"\n"
"function builtin.autodetect_external_dependencies(build)\n"
"   if not build or not build.modules then\n"
"      return nil\n"
"   end\n"
"   local extdeps = {}\n"
"   local any = false\n"
"   for _, data in pairs(build.modules) do\n"
"      if type(data) == \"table\" and data.libraries then\n"
"         local libraries = data.libraries\n"
"         if type(libraries) == \"string\" then\n"
"            libraries = { libraries }\n"
"         end\n"
"         local incdirs = {}\n"
"         local libdirs = {}\n"
"         for _, lib in ipairs(libraries) do\n"
"            local upper = lib:upper():gsub(\"%+\", \"P\"):gsub(\"[^%w]\", \"_\")\n"
"            any = true\n"
"            extdeps[upper] = { library = lib }\n"
"            table.insert(incdirs, \"$(\" .. upper .. \"_INCDIR)\")\n"
"            table.insert(libdirs, \"$(\" .. upper .. \"_LIBDIR)\")\n"
"         end\n"
"         if not data.incdirs then\n"
"            data.incdirs = incdirs\n"
"         end\n"
"         if not data.libdirs then\n"
"            data.libdirs = libdirs\n"
"         end\n"
"      end\n"
"   end\n"
"   return any and extdeps or nil\n"
"end\n"
"\n"
"local function autoextract_libs(external_dependencies, variables)\n"
"   if not external_dependencies then\n"
"      return nil, nil, nil\n"
"   end\n"
"   local libs = {}\n"
"   local incdirs = {}\n"
"   local libdirs = {}\n"
"   for name, data in pairs(external_dependencies) do\n"
"      if data.library then\n"
"         table.insert(libs, data.library)\n"
"         table.insert(incdirs, variables[name .. \"_INCDIR\"])\n"
"         table.insert(libdirs, variables[name .. \"_LIBDIR\"])\n"
"      end\n"
"   end\n"
"   return libs, incdirs, libdirs\n"
"end\n"
"\n"
"do\n"
"   local function get_cmod_name(file)\n"
"      local fd = io.open(dir.path(fs.current_dir(), file), \"r\")\n"
"      if not fd then return nil end\n"
"      local data = fd:read(\"*a\")\n"
"      fd:close()\n"
"      return (data:match(\"int%s+luaopen_([a-zA-Z0-9_]+)\"))\n"
"   end\n"
"\n"
"   local skiplist = {\n"
"      [\"spec\"] = true,\n"
"      [\".luarocks\"] = true,\n"
"      [\"lua_modules\"] = true,\n"
"      [\"test.lua\"] = true,\n"
"      [\"tests.lua\"] = true,\n"
"   }\n"
"\n"
"   function builtin.autodetect_modules(libs, incdirs, libdirs)\n"
"      local modules = {}\n"
"      local install\n"
"      local copy_directories\n"
"\n"
"      local prefix = \"\"\n"
"      for _, parent in ipairs({\"src\", \"lua\", \"lib\"}) do\n"
"         if fs.is_dir(parent) then\n"
"            fs.change_dir(parent)\n"
"            prefix = parent..\"/\"\n"
"            break\n"
"         end\n"
"      end\n"
"\n"
"      for _, file in ipairs(fs.find()) do\n"
"         local base = file:match(\"^([^\\\\/]*)\")\n"
"         if not skiplist[base] then\n"
"            local luamod = file:match(\"(.*)%.lua$\")\n"
"            if luamod then\n"
"               modules[path.path_to_module(file)] = prefix..file\n"
"            else\n"
"               local cmod = file:match(\"(.*)%.c$\")\n"
"               if cmod then\n"
"                  local modname = get_cmod_name(file) or path.path_to_module(file:gsub(\"%.c$\", \".lua\"))\n"
"                  modules[modname] = {\n"
"                     sources = prefix..file,\n"
"                     libraries = libs,\n"
"                     incdirs = incdirs,\n"
"                     libdirs = libdirs,\n"
"                  }\n"
"               end\n"
"            end\n"
"         end\n"
"      end\n"
"\n"
"      if prefix ~= \"\" then\n"
"         fs.pop_dir()\n"
"      end\n"
"\n"
"      local bindir = (fs.is_dir(\"src/bin\") and \"src/bin\")\n"
"                  or (fs.is_dir(\"bin\") and \"bin\")\n"
"      if bindir then\n"
"         install = { bin = {} }\n"
"         for _, file in ipairs(fs.list_dir(bindir)) do\n"
"            table.insert(install.bin, dir.path(bindir, file))\n"
"         end\n"
"      end\n"
"\n"
"      for _, directory in ipairs({ \"doc\", \"docs\", \"samples\", \"tests\" }) do\n"
"         if fs.is_dir(directory) then\n"
"            if not copy_directories then\n"
"               copy_directories = {}\n"
"            end\n"
"            table.insert(copy_directories, directory)\n"
"         end\n"
"      end\n"
"\n"
"      return modules, install, copy_directories\n"
"   end\n"
"end\n"
"\n"
"--- Run a command displaying its execution on standard output.\n"
"-- @return boolean: true if command succeeds (status code 0), false\n"
"-- otherwise.\n"
"local function execute(...)\n"
"   io.stdout:write(table.concat({...}, \" \")..\"\\n\")\n"
"   return fs.execute(...)\n"
"end\n"
"\n"
"--- Driver function for the builtin build back-end.\n"
"-- @param rockspec table: the loaded rockspec.\n"
"-- @return boolean or (nil, string): true if no errors occurred,\n"
"-- nil and an error message otherwise.\n"
"function builtin.run(rockspec)\n"
"   assert(rockspec:type() == \"rockspec\")\n"
"   local compile_object, compile_library, compile_static_library\n"
"\n"
"   local build = rockspec.build\n"
"   local variables = rockspec.variables\n"
"   local checked_lua_h = false\n"
"\n"
"   local function add_flags(extras, flag, flags)\n"
"      if flags then\n"
"         if type(flags) ~= \"table\" then\n"
"            flags = { tostring(flags) }\n"
"         end\n"
"         util.variable_substitutions(flags, variables)\n"
"         for _, v in ipairs(flags) do\n"
"            table.insert(extras, flag:format(v))\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   if cfg.is_platform(\"mingw32\") then\n"
"      compile_object = function(object, source, defines, incdirs)\n"
"         local extras = {}\n"
"         add_flags(extras, \"-D%s\", defines)\n"
"         add_flags(extras, \"-I%s\", incdirs)\n"
"         return execute(variables.CC..\" \"..variables.CFLAGS, \"-c\", \"-o\", object, \"-I\"..variables.LUA_INCDIR, source, unpack(extras))\n"
"      end\n"
"      compile_library = function(library, objects, libraries, libdirs)\n"
"         local extras = { unpack(objects) }\n"
"         add_flags(extras, \"-L%s\", libdirs)\n"
"         add_flags(extras, \"-l%s\", libraries)\n"
"         extras[#extras+1] = dir.path(variables.LUA_LIBDIR, variables.LUALIB)\n"
"         extras[#extras+1] = \"-l\" .. (variables.MSVCRT or \"m\")\n"
"         local ok = execute(variables.LD..\" \"..variables.LIBFLAG, \"-o\", library, unpack(extras))\n"
"         return ok\n"
"      end\n"
"      --[[ TODO disable static libs until we fix the conflict in the manifest, which will take extending the manifest format.\n"
"      compile_static_library = function(library, objects, libraries, libdirs, name)\n"
"         local ok = execute(variables.AR, \"rc\", library, unpack(objects))\n"
"         if ok then\n"
"            ok = execute(variables.RANLIB, library)\n"
"         end\n"
"         return ok\n"
"      end\n"
"      ]]\n"
"   elseif cfg.is_platform(\"win32\") then\n"
"      compile_object = function(object, source, defines, incdirs)\n"
"         local extras = {}\n"
"         add_flags(extras, \"-D%s\", defines)\n"
"         add_flags(extras, \"-I%s\", incdirs)\n"
"         return execute(variables.CC..\" \"..variables.CFLAGS, \"-c\", \"-Fo\"..object, \"-I\"..variables.LUA_INCDIR, source, unpack(extras))\n"
"      end\n"
"      compile_library = function(library, objects, libraries, libdirs, name)\n"
"         local extras = { unpack(objects) }\n"
"         add_flags(extras, \"-libpath:%s\", libdirs)\n"
"         add_flags(extras, \"%s.lib\", libraries)\n"
"         local basename = dir.base_name(library):gsub(\".[^.]*$\", \"\")\n"
"         local deffile = basename .. \".def\"\n"
"         local def = io.open(dir.path(fs.current_dir(), deffile), \"w+\")\n"
"         local exported_name = name:gsub(\"%.\", \"_\")\n"
"         exported_name = exported_name:match('^[^%-]+%-(.+)$') or exported_name\n"
"         def:write(\"EXPORTS\\n\")\n"
"         def:write(\"luaopen_\"..exported_name..\"\\n\")\n"
"         def:close()\n"
"         local ok = execute(variables.LD, \"-dll\", \"-def:\"..deffile, \"-out:\"..library, dir.path(variables.LUA_LIBDIR, variables.LUALIB), unpack(extras))\n"
"         local basedir = \"\"\n"
"         if name:find(\"%.\") ~= nil then\n"
"            basedir = name:gsub(\"%.%w+$\", \"\\\\\")\n"
"            basedir = basedir:gsub(\"%.\", \"\\\\\")\n"
"         end\n"
"         local manifestfile = basedir .. basename..\".dll.manifest\"\n"
"\n"
"         if ok and fs.exists(manifestfile) then\n"
"            ok = execute(variables.MT, \"-manifest\", manifestfile, \"-outputresource:\"..basedir..basename..\".dll;2\")\n"
"         end\n"
"         return ok\n"
"      end\n"
"      --[[ TODO disable static libs until we fix the conflict in the manifest, which will take extending the manifest format.\n"
"      compile_static_library = function(library, objects, libraries, libdirs, name)\n"
"         local ok = execute(variables.AR, \"-out:\"..library, unpack(objects))\n"
"         return ok\n"
"      end\n"
"      ]]\n"
"   else\n"
"      compile_object = function(object, source, defines, incdirs)\n"
"         local extras = {}\n"
"         add_flags(extras, \"-D%s\", defines)\n"
"         add_flags(extras, \"-I%s\", incdirs)\n"
"         return execute(variables.CC..\" \"..variables.CFLAGS, \"-I\"..variables.LUA_INCDIR, \"-c\", source, \"-o\", object, unpack(extras))\n"
"      end\n"
"      compile_library = function (library, objects, libraries, libdirs)\n"
"         local extras = { unpack(objects) }\n"
"         add_flags(extras, \"-L%s\", libdirs)\n"
"         if cfg.gcc_rpath then\n"
"            add_flags(extras, \"-Wl,-rpath,%s:\", libdirs)\n"
"         end\n"
"         add_flags(extras, \"-l%s\", libraries)\n"
"         if cfg.link_lua_explicitly then\n"
"            extras[#extras+1] = \"-L\"..variables.LUA_LIBDIR\n"
"            extras[#extras+1] = \"-llua\"\n"
"         end\n"
"         return execute(variables.LD..\" \"..variables.LIBFLAG, \"-o\", library, unpack(extras))\n"
"      end\n"
"      compile_static_library = function(library, objects, libraries, libdirs, name)\n"
"         local ok = execute(variables.AR, \"rc\", library, unpack(objects))\n"
"         if ok then\n"
"            ok = execute(variables.RANLIB, library)\n"
"         end\n"
"         return ok\n"
"      end\n"
"   end\n"
"\n"
"   local ok, err\n"
"   local lua_modules = {}\n"
"   local lib_modules = {}\n"
"   local luadir = path.lua_dir(rockspec.name, rockspec.version)\n"
"   local libdir = path.lib_dir(rockspec.name, rockspec.version)\n"
"   \n"
"   if not build.modules then\n"
"      if rockspec:format_is_at_least(\"3.0\") then\n"
"         local libs, incdirs, libdirs = autoextract_libs(rockspec.external_dependencies, rockspec.variables)\n"
"         local install, copy_directories\n"
"         build.modules, install, copy_directories = builtin.autodetect_modules(libs, incdirs, libdirs)\n"
"         build.install = build.install or install\n"
"         build.copy_directories = build.copy_directories or copy_directories\n"
"      else\n"
"         return nil, \"Missing build.modules table\"\n"
"      end\n"
"   end\n"
"   for name, info in pairs(build.modules) do\n"
"      local moddir = path.module_to_path(name)\n"
"      if type(info) == \"string\" then\n"
"         local ext = info:match(\"%.([^.]+)$\")\n"
"         if ext == \"lua\" then\n"
"            local filename = dir.base_name(info)\n"
"            if filename == \"init.lua\" and not name:match(\"%.init$\") then\n"
"               moddir = path.module_to_path(name..\".init\")\n"
"            else\n"
"               local basename = name:match(\"([^.]+)$\")\n"
"               filename = basename..\".lua\"\n"
"            end\n"
"            local dest = dir.path(luadir, moddir, filename)\n"
"            lua_modules[info] = dest\n"
"         else\n"
"            info = {info}\n"
"         end\n"
"      end\n"
"      if type(info) == \"table\" then\n"
"         if not checked_lua_h then\n"
"            local lua_incdir, lua_h = variables.LUA_INCDIR, \"lua.h\"\n"
"            if not fs.exists(dir.path(lua_incdir, lua_h)) then\n"
"               return nil, \"Lua header file \"..lua_h..\" not found (looked in \"..lua_incdir..\"). \\n\" ..\n"
"                           \"You need to install the Lua development package for your system.\"\n"
"            end\n"
"            checked_lua_h = true\n"
"         end\n"
"         local objects = {}\n"
"         local sources = info.sources\n"
"         if info[1] then sources = info end\n"
"         if type(sources) == \"string\" then sources = {sources} end\n"
"         for _, source in ipairs(sources) do\n"
"            local object = source:gsub(\"%.[^.]*$\", \".\"..cfg.obj_extension)\n"
"            if not object then\n"
"               object = source..\".\"..cfg.obj_extension\n"
"            end\n"
"            ok = compile_object(object, source, info.defines, info.incdirs)\n"
"            if not ok then\n"
"               return nil, \"Failed compiling object \"..object\n"
"            end\n"
"            table.insert(objects, object)\n"
"         end\n"
"         local module_name = name:match(\"([^.]*)$\")..\".\"..util.matchquote(cfg.lib_extension)\n"
"         if moddir ~= \"\" then\n"
"            module_name = dir.path(moddir, module_name)\n"
"            ok, err = fs.make_dir(moddir)\n"
"            if not ok then return nil, err end\n"
"         end\n"
"         lib_modules[module_name] = dir.path(libdir, module_name)\n"
"         ok = compile_library(module_name, objects, info.libraries, info.libdirs, name)\n"
"         if not ok then\n"
"            return nil, \"Failed compiling module \"..module_name\n"
"         end\n"
"         --[[ TODO disable static libs until we fix the conflict in the manifest, which will take extending the manifest format.\n"
"         module_name = name:match(\"([^.]*)$\")..\".\"..util.matchquote(cfg.static_lib_extension)\n"
"         if moddir ~= \"\" then\n"
"            module_name = dir.path(moddir, module_name)\n"
"         end\n"
"         lib_modules[module_name] = dir.path(libdir, module_name)\n"
"         ok = compile_static_library(module_name, objects, info.libraries, info.libdirs, name)\n"
"         if not ok then\n"
"            return nil, \"Failed compiling static library \"..module_name\n"
"         end\n"
"         ]]\n"
"      end\n"
"   end\n"
"   for _, mods in ipairs({{ tbl = lua_modules, perms = \"read\" }, { tbl = lib_modules, perms = \"exec\" }}) do\n"
"      for name, dest in pairs(mods.tbl) do\n"
"         fs.make_dir(dir.dir_name(dest))\n"
"         ok, err = fs.copy(name, dest, mods.perms)\n"
"         if not ok then\n"
"            return nil, \"Failed installing \"..name..\" in \"..dest..\": \"..err\n"
"         end\n"
"      end\n"
"   end\n"
"   if fs.is_dir(\"lua\") then\n"
"      ok, err = fs.copy_contents(\"lua\", luadir)\n"
"      if not ok then\n"
"         return nil, \"Failed copying contents of 'lua' directory: \"..err\n"
"      end\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"return builtin\n"
""
;
