const char luarocks_build_cmake_lua[] =
"\n"
"--- Build back-end for CMake-based modules.\n"
"local cmake = {}\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"local util = require(\"luarocks.util\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"\n"
"--- Driver function for the \"cmake\" build back-end.\n"
"-- @param rockspec table: the loaded rockspec.\n"
"-- @return boolean or (nil, string): true if no errors occurred,\n"
"-- nil and an error message otherwise.\n"
"function cmake.run(rockspec)\n"
"   assert(rockspec:type() == \"rockspec\")\n"
"   local build = rockspec.build\n"
"   local variables = build.variables or {}\n"
"\n"
"   -- Pass Env variables\n"
"   variables.CMAKE_MODULE_PATH=os.getenv(\"CMAKE_MODULE_PATH\")\n"
"   variables.CMAKE_LIBRARY_PATH=os.getenv(\"CMAKE_LIBRARY_PATH\")\n"
"   variables.CMAKE_INCLUDE_PATH=os.getenv(\"CMAKE_INCLUDE_PATH\")\n"
"\n"
"   util.variable_substitutions(variables, rockspec.variables)\n"
"\n"
"   local ok, err_msg = fs.is_tool_available(rockspec.variables.CMAKE, \"CMake\")\n"
"   if not ok then\n"
"      return nil, err_msg\n"
"   end\n"
"\n"
"   -- If inline cmake is present create CMakeLists.txt from it.\n"
"   if type(build.cmake) == \"string\" then\n"
"      local cmake_handler = assert(io.open(fs.current_dir()..\"/CMakeLists.txt\", \"w\"))\n"
"      cmake_handler:write(build.cmake)\n"
"      cmake_handler:close()\n"
"   end\n"
"\n"
"   -- Execute cmake with variables.\n"
"   local args = \"\"\n"
"   \n"
"   -- Try to pick the best generator. With msvc and x64, CMake does not select it by default so we need to be explicit.\n"
"   if cfg.cmake_generator then\n"
"      args = args .. ' -G\"'..cfg.cmake_generator.. '\"'\n"
"   elseif cfg.is_platform(\"windows\") and cfg.target_cpu:match(\"x86_64$\") then\n"
"      args = args .. \" -DCMAKE_GENERATOR_PLATFORM=x64\"\n"
"   end\n"
"\n"
"   for k,v in pairs(variables) do\n"
"      args = args .. ' -D' ..k.. '=\"' ..tostring(v).. '\"'\n"
"   end\n"
"\n"
"   if not fs.execute_string(rockspec.variables.CMAKE..\" -H. -Bbuild.luarocks \"..args) then\n"
"      return nil, \"Failed cmake.\"\n"
"   end\n"
"\n"
"   local do_build, do_install\n"
"   if rockspec:format_is_at_least(\"3.0\") then\n"
"      do_build   = (build.build_pass   == nil) and true or build.build_pass\n"
"      do_install = (build.install_pass == nil) and true or build.install_pass\n"
"   else\n"
"      do_build = true\n"
"      do_install = true\n"
"   end\n"
"\n"
"   if do_build then\n"
"      if not fs.execute_string(rockspec.variables.CMAKE..\" --build build.luarocks --config Release\") then\n"
"         return nil, \"Failed building.\"\n"
"      end\n"
"   end\n"
"   if do_install then\n"
"      if not fs.execute_string(rockspec.variables.CMAKE..\" --build build.luarocks --target install --config Release\") then\n"
"         return nil, \"Failed installing.\"\n"
"      end\n"
"   end\n"
"   \n"
"   return true\n"
"end\n"
"\n"
"return cmake\n"
""
;
