const char luarocks_build_make_lua[] =
"\n"
"--- Build back-end for using Makefile-based packages.\n"
"local make = {}\n"
"\n"
"local unpack = unpack or table.unpack\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"local util = require(\"luarocks.util\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"\n"
"--- Call \"make\" with given target and variables\n"
"-- @param make_cmd string: the make command to be used (typically\n"
"-- configured through variables.MAKE in the config files, or\n"
"-- the appropriate platform-specific default).\n"
"-- @param pass boolean: If true, run make; if false, do nothing.\n"
"-- @param target string: The make target; an empty string indicates \n"
"-- the default target.\n"
"-- @param variables table: A table containing string-string key-value\n"
"-- pairs representing variable assignments to be passed to make.\n"
"-- @return boolean: false if any errors occurred, true otherwise.\n"
"local function make_pass(make_cmd, pass, target, variables)\n"
"   assert(type(pass) == \"boolean\")\n"
"   assert(type(target) == \"string\")\n"
"   assert(type(variables) == \"table\")\n"
"\n"
"   local assignments = {}\n"
"   for k,v in pairs(variables) do\n"
"      table.insert(assignments, k..\"=\"..v)\n"
"   end\n"
"   if pass then\n"
"      return fs.execute(make_cmd..\" \"..target, unpack(assignments))\n"
"   else\n"
"      return true\n"
"   end\n"
"end\n"
"\n"
"--- Driver function for the \"make\" build back-end.\n"
"-- @param rockspec table: the loaded rockspec.\n"
"-- @return boolean or (nil, string): true if no errors occurred,\n"
"-- nil and an error message otherwise.\n"
"function make.run(rockspec)\n"
"   assert(rockspec:type() == \"rockspec\")\n"
"\n"
"   local build = rockspec.build\n"
"   \n"
"   if build.build_pass == nil then build.build_pass = true end\n"
"   if build.install_pass == nil then build.install_pass = true end\n"
"   build.build_variables = build.build_variables or {}\n"
"   build.install_variables = build.install_variables or {}\n"
"   build.build_target = build.build_target or \"\"\n"
"   build.install_target = build.install_target or \"install\"\n"
"   local makefile = build.makefile or cfg.makefile\n"
"   if makefile then\n"
"      -- Assumes all make's accept -f. True for POSIX make, GNU make and Microsoft nmake.\n"
"      build.build_target = \"-f \"..makefile..\" \"..build.build_target\n"
"      build.install_target = \"-f \"..makefile..\" \"..build.install_target\n"
"   end\n"
"\n"
"   if build.variables then   \n"
"      for var, val in pairs(build.variables) do\n"
"         build.build_variables[var] = val \n"
"         build.install_variables[var] = val\n"
"      end\n"
"   end\n"
"\n"
"   util.warn_if_not_used(build.build_variables, { CFLAGS=true }, \"variable %s was not passed in build_variables\")\n"
"\n"
"   util.variable_substitutions(build.build_variables, rockspec.variables)\n"
"   util.variable_substitutions(build.install_variables, rockspec.variables)\n"
"   \n"
"   local auto_variables = { \"CC\" }\n"
"   \n"
"   for _, variable in pairs(auto_variables) do\n"
"      if not build.build_variables[variable] then\n"
"         build.build_variables[variable] = rockspec.variables[variable]\n"
"      end\n"
"      if not build.install_variables[variable] then\n"
"         build.install_variables[variable] = rockspec.variables[variable]\n"
"      end\n"
"   end\n"
"\n"
"   -- backwards compatibility \n"
"   local make_cmd = cfg.make or rockspec.variables.MAKE\n"
"\n"
"   local ok = make_pass(make_cmd, build.build_pass, build.build_target, build.build_variables)\n"
"   if not ok then\n"
"      return nil, \"Failed building.\"\n"
"   end\n"
"   ok = make_pass(make_cmd, build.install_pass, build.install_target, build.install_variables)\n"
"   if not ok then\n"
"      return nil, \"Failed installing.\"\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"return make\n"
""
;
