const char luarocks_cmd_lua[] =
"\n"
"--- Functions for command-line scripts.\n"
"local cmd = {}\n"
"\n"
"local unpack = unpack or table.unpack\n"
"\n"
"local loader = require(\"luarocks.loader\")\n"
"local util = require(\"luarocks.util\")\n"
"local path = require(\"luarocks.path\")\n"
"local deps = require(\"luarocks.deps\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local fun = require(\"luarocks.fun\")\n"
"local fs = require(\"luarocks.fs\")\n"
"\n"
"local program = util.this_program(\"luarocks\")\n"
"\n"
"cmd.errorcodes = {\n"
"   OK = 0,\n"
"   UNSPECIFIED = 1,\n"
"   PERMISSIONDENIED = 2,\n"
"   CONFIGFILE = 3,\n"
"   CRASH = 99\n"
"}\n"
"\n"
"local function check_popen()\n"
"   local popen_ok, popen_result = pcall(io.popen, \"\")\n"
"   if popen_ok then\n"
"      if popen_result then\n"
"         popen_result:close()\n"
"      end\n"
"   else\n"
"      io.stderr:write(\"Your version of Lua does not support io.popen,\\n\")\n"
"      io.stderr:write(\"which is required by LuaRocks. Please check your Lua installation.\\n\")\n"
"      os.exit(cmd.errorcodes.UNSPECIFIED)\n"
"   end\n"
"end\n"
"\n"
"local function check_if_config_is_present(detected, try)\n"
"   local versions = detected.lua_version \n"
"                    and { detected.lua_version }\n"
"                    or  util.lua_versions(\"descending\")\n"
"   return fun.find(versions, function(v)\n"
"      if util.exists(dir.path(try, \".luarocks\", \"config-\"..v..\".lua\")) then\n"
"         detected.project_dir = try\n"
"         detected.lua_version = v\n"
"         return detected\n"
"      end\n"
"   end)\n"
"end\n"
"\n"
"local process_tree_flags\n"
"do\n"
"   local function replace_tree(flags, root, tree)\n"
"      root = dir.normalize(root)\n"
"      flags[\"tree\"] = root\n"
"      path.use_tree(tree or root)\n"
"   end\n"
"\n"
"   local function strip_trailing_slashes()\n"
"      if type(cfg.root_dir) == \"string\" then\n"
"        cfg.root_dir = cfg.root_dir:gsub(\"/+$\", \"\")\n"
"      else\n"
"        cfg.root_dir.root = cfg.root_dir.root:gsub(\"/+$\", \"\")\n"
"      end\n"
"      cfg.rocks_dir = cfg.rocks_dir:gsub(\"/+$\", \"\")\n"
"      cfg.deploy_bin_dir = cfg.deploy_bin_dir:gsub(\"/+$\", \"\")\n"
"      cfg.deploy_lua_dir = cfg.deploy_lua_dir:gsub(\"/+$\", \"\")\n"
"      cfg.deploy_lib_dir = cfg.deploy_lib_dir:gsub(\"/+$\", \"\")\n"
"   end\n"
"\n"
"   process_tree_flags = function(flags, project_dir)\n"
"   \n"
"      if flags[\"global\"] then\n"
"         cfg.local_by_default = false\n"
"      end\n"
"\n"
"      if flags[\"tree\"] then\n"
"         local named = false\n"
"         for _, tree in ipairs(cfg.rocks_trees) do\n"
"            if type(tree) == \"table\" and flags[\"tree\"] == tree.name then\n"
"               if not tree.root then\n"
"                  return nil, \"Configuration error: tree '\"..tree.name..\"' has no 'root' field.\"\n"
"               end\n"
"               replace_tree(flags, tree.root, tree)\n"
"               named = true\n"
"               break\n"
"            end\n"
"         end\n"
"         if not named then\n"
"            local root_dir = fs.absolute_name(flags[\"tree\"])\n"
"            replace_tree(flags, root_dir)\n"
"         end\n"
"      elseif flags[\"local\"] then\n"
"         if not cfg.home_tree then\n"
"            return nil, \"The --local flag is meant for operating in a user's home directory.\\n\"..\n"
"               \"You are running as a superuser, which is intended for system-wide operation.\\n\"..\n"
"               \"To force using the superuser's home, use --tree explicitly.\"\n"
"         else\n"
"            replace_tree(flags, cfg.home_tree)\n"
"         end\n"
"      elseif flags[\"project-tree\"] then\n"
"         local tree = flags[\"project-tree\"]\n"
"         table.insert(cfg.rocks_trees, 1, { name = \"project\", root = tree } )\n"
"         loader.load_rocks_trees()\n"
"         path.use_tree(tree)\n"
"      elseif cfg.local_by_default then\n"
"         if cfg.home_tree then\n"
"            replace_tree(flags, cfg.home_tree)\n"
"         end\n"
"      elseif project_dir then\n"
"         local project_tree = project_dir .. \"/lua_modules\"\n"
"         table.insert(cfg.rocks_trees, 1, { name = \"project\", root = project_tree } )\n"
"         loader.load_rocks_trees()\n"
"         path.use_tree(project_tree)\n"
"      else\n"
"         local trees = cfg.rocks_trees\n"
"         path.use_tree(trees[#trees])\n"
"      end\n"
"\n"
"      strip_trailing_slashes()\n"
"\n"
"      cfg.variables.ROCKS_TREE = cfg.rocks_dir\n"
"      cfg.variables.SCRIPTS_DIR = cfg.deploy_bin_dir\n"
"\n"
"      return true\n"
"   end\n"
"end\n"
"\n"
"local function process_server_flags(flags)\n"
"   if flags[\"server\"] then\n"
"      local protocol, pathname = dir.split_url(flags[\"server\"])\n"
"      table.insert(cfg.rocks_servers, 1, protocol..\"://\"..pathname)\n"
"   end\n"
"\n"
"   if flags[\"dev\"] then\n"
"      local append_dev = function(s) return dir.path(s, \"dev\") end\n"
"      local dev_servers = fun.traverse(cfg.rocks_servers, append_dev)\n"
"      cfg.rocks_servers = fun.concat(dev_servers, cfg.rocks_servers)\n"
"   end\n"
"\n"
"   if flags[\"only-server\"] then\n"
"      if flags[\"dev\"] then\n"
"         return nil, \"--only-server cannot be used with --dev\"\n"
"      end\n"
"      if flags[\"server\"] then\n"
"         return nil, \"--only-server cannot be used with --server\"\n"
"      end\n"
"      cfg.rocks_servers = { flags[\"only-server\"] }\n"
"   end\n"
"\n"
"   return true\n"
"end\n"
"\n"
"local function get_lua_version(flags)\n"
"   if flags[\"lua-version\"] then\n"
"      return flags[\"lua-version\"] \n"
"   end\n"
"   local dirs = {}\n"
"   local next_i = 1\n"
"   if flags[\"project-tree\"] then\n"
"      dirs[next_i] = dir.path(flags[\"project-tree\"], \"..\", \".luarocks\")\n"
"      next_i = next_i + 1\n"
"   end\n"
"   if cfg.home_tree then\n"
"      dirs[next_i] = dir.path(cfg.home_tree, \".luarocks\")\n"
"      next_i = next_i + 1\n"
"   end\n"
"   dirs[next_i] = cfg.sysconfdir\n"
"   for i=1, next_i do\n"
"      local f = dir.path(dirs[i], \"default-lua-version.lua\")\n"
"      local mod, err = loadfile(f, \"t\")\n"
"      if mod then\n"
"         local pok, ver = pcall(mod)\n"
"         if pok and type(ver) == \"string\" and ver:match(\"%d+.%d+\") then\n"
"            if flags[\"verbose\"] then\n"
"               util.printout(\"Defaulting to Lua \" .. ver .. \" based on \" .. f .. \" ...\")\n"
"            end\n"
"            return ver\n"
"         end\n"
"      end\n"
"   end\n"
"   return nil\n"
"end\n"
"\n"
"--- Main command-line processor.\n"
"-- Parses input arguments and calls the appropriate driver function\n"
"-- to execute the action requested on the command-line, forwarding\n"
"-- to it any additional arguments passed by the user.\n"
"-- @param description string: Short summary description of the program.\n"
"-- @param commands table: contains the loaded modules representing commands.\n"
"-- @param external_namespace string: where to look for external commands.\n"
"-- @param ... string: Arguments given on the command-line.\n"
"function cmd.run_command(description, commands, external_namespace, ...)\n"
"\n"
"   check_popen()\n"
"\n"
"   local function error_handler(err)\n"
"      local mode = \"Arch.: \" .. (cfg and cfg.arch or \"unknown\")\n"
"      if package.config:sub(1, 1) == \"\\\\\" then\n"
"         if cfg and cfg.fs_use_modules then\n"
"            mode = mode .. \" (fs_use_modules = true)\"\n"
"         end\n"
"      end\n"
"      return debug.traceback(\"LuaRocks \"..cfg.program_version..\n"
"         \" bug (please report at https://github.com/luarocks/luarocks/issues).\\n\"..\n"
"         mode..\"\\n\"..err, 2)\n"
"   end\n"
"\n"
"   --- Display an error message and exit.\n"
"   -- @param message string: The error message.\n"
"   -- @param exitcode number: the exitcode to use\n"
"   local function die(message, exitcode)\n"
"      assert(type(message) == \"string\", \"bad error, expected string, got: \" .. type(message))\n"
"      util.printerr(\"\\nError: \"..message)\n"
"\n"
"      local ok, err = xpcall(util.run_scheduled_functions, error_handler)\n"
"      if not ok then\n"
"         util.printerr(\"\\nError: \"..err)\n"
"         exitcode = cmd.errorcodes.CRASH\n"
"      end\n"
"\n"
"      os.exit(exitcode or cmd.errorcodes.UNSPECIFIED)\n"
"   end\n"
"\n"
"   local function process_arguments(...)\n"
"      local args = {...}\n"
"      local cmdline_vars = {}\n"
"      local last = #args\n"
"      for i = 1, #args do\n"
"         if args[i] == \"--\" then\n"
"            last = i - 1\n"
"            break\n"
"         end\n"
"      end\n"
"      for i = last, 1, -1 do\n"
"         local arg = args[i]\n"
"         if arg:match(\"^[^-][^=]*=\") then\n"
"            local var, val = arg:match(\"^([A-Z_][A-Z0-9_]*)=(.*)\")\n"
"            if val then\n"
"               cmdline_vars[var] = val\n"
"               table.remove(args, i)\n"
"            else\n"
"               die(\"Invalid assignment: \"..arg)\n"
"            end\n"
"         end\n"
"      end\n"
"      local nonflags = { util.parse_flags(unpack(args)) }\n"
"      local flags = table.remove(nonflags, 1)\n"
"      if flags.ERROR then\n"
"         die(flags.ERROR..\" See --help.\")\n"
"      end\n"
"\n"
"      -- Compatibility for old names of some flags\n"
"      if flags[\"to\"] then flags[\"tree\"] = flags[\"to\"] end\n"
"      if flags[\"from\"] then flags[\"server\"] = flags[\"from\"] end\n"
"      if flags[\"nodeps\"] then flags[\"deps-mode\"] = \"none\" end\n"
"      if flags[\"only-from\"] then flags[\"only-server\"] = flags[\"only-from\"] end\n"
"      if flags[\"only-sources-from\"] then flags[\"only-sources\"] = flags[\"only-sources-from\"] end\n"
"\n"
"      return flags, nonflags, cmdline_vars\n"
"   end\n"
"\n"
"   local flags, nonflags, cmdline_vars = process_arguments(...)\n"
"\n"
"   if flags[\"timeout\"] then   -- setting it in the config file will kick-in earlier in the process\n"
"      local timeout = tonumber(flags[\"timeout\"])\n"
"      if timeout then\n"
"         cfg.connection_timeout = timeout\n"
"      else\n"
"         die \"Argument error: --timeout expects a numeric argument.\"\n"
"      end\n"
"   end\n"
"\n"
"   local command\n"
"   if flags[\"help\"] or #nonflags == 0 then\n"
"      command = \"help\"\n"
"   else\n"
"      command = table.remove(nonflags, 1)\n"
"   end\n"
"   command = command:gsub(\"-\", \"_\")\n"
"\n"
"   if command == \"config\" then\n"
"      if nonflags[1] == \"lua_version\" and nonflags[2] then\n"
"         flags[\"lua-version\"] = nonflags[2]\n"
"      elseif nonflags[1] == \"lua_dir\" and nonflags[2] then\n"
"         flags[\"lua-dir\"] = nonflags[2]\n"
"      end\n"
"   end\n"
"\n"
"   local lua_version = get_lua_version(flags)\n"
"\n"
"   if flags[\"deps-mode\"] and not deps.check_deps_mode_flag(flags[\"deps-mode\"]) then\n"
"      die(\"Invalid entry for --deps-mode.\")\n"
"   end\n"
"   \n"
"   local lua_found = false\n"
"\n"
"   local detected\n"
"   if flags[\"lua-dir\"] then\n"
"      local err\n"
"      detected, err = util.find_lua(flags[\"lua-dir\"], lua_version)\n"
"      if not detected then\n"
"         die(err)\n"
"      end\n"
"      lua_found = true\n"
"      assert(detected.lua_version)\n"
"      assert(detected.lua_dir)\n"
"   elseif lua_version then\n"
"      local path_sep = (package.config:sub(1, 1) == \"\\\\\" and \";\" or \":\")\n"
"      for bindir in os.getenv(\"PATH\"):gmatch(\"[^\"..path_sep..\"]+\") do\n"
"         local parentdir = bindir:gsub(\"[\\\\/][^\\\\/]+[\\\\/]\?$\", \"\")\n"
"         detected = util.find_lua(dir.path(parentdir), lua_version)\n"
"         if detected then\n"
"            break\n"
"         end\n"
"         detected = util.find_lua(bindir, lua_version)\n"
"         if detected then\n"
"            break\n"
"         end\n"
"      end\n"
"      if detected then\n"
"         lua_found = true\n"
"      else\n"
"         detected = {\n"
"            lua_version = lua_version,\n"
"         }\n"
"      end\n"
"   end\n"
"\n"
"   if flags[\"project-tree\"] then\n"
"      local project_tree = flags[\"project-tree\"]:gsub(\"[/\\\\][^/\\\\]+$\", \"\")\n"
"      detected = detected or {}\n"
"      detected.project_dir = project_tree\n"
"      local d = check_if_config_is_present(detected, project_tree)\n"
"      if d then\n"
"         detected = d\n"
"      end\n"
"   else\n"
"      detected = detected or {}\n"
"      local try = \".\"\n"
"      for _ = 1, 10 do -- FIXME detect when root dir was hit instead\n"
"         if util.exists(try .. \"/.luarocks\") and util.exists(try .. \"/lua_modules\") then\n"
"            local d = check_if_config_is_present(detected, try)\n"
"            if d then\n"
"               detected = d\n"
"               break\n"
"            end\n"
"         elseif util.exists(try .. \"/.luarocks-no-project\") then\n"
"            break\n"
"         end\n"
"         try = try .. \"/..\"\n"
"      end\n"
"   end\n"
"\n"
"   -- FIXME A quick hack for the experimental Windows build\n"
"   if os.getenv(\"LUAROCKS_CROSS_COMPILING\") then\n"
"      cfg.each_platform = function()\n"
"         local i = 0\n"
"         local plats = { \"unix\", \"linux\" }\n"
"         return function()\n"
"            i = i + 1\n"
"            return plats[i]\n"
"         end\n"
"      end\n"
"      fs.init()\n"
"   end\n"
"\n"
"   -----------------------------------------------------------------------------\n"
"   local ok, err = cfg.init(detected, util.warning)\n"
"   if not ok then\n"
"      die(err)\n"
"   end\n"
"   -----------------------------------------------------------------------------\n"
"\n"
"   fs.init()\n"
"\n"
"   if not lua_found then\n"
"      if cfg.variables.LUA_DIR then\n"
"         local found = util.find_lua(cfg.variables.LUA_DIR, cfg.lua_version)\n"
"         if found then\n"
"            lua_found = true\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   if not lua_found then\n"
"      local lua_version = lua_version or cfg.lua_version\n"
"      util.warning(\"Could not find a Lua interpreter for version \" ..\n"
"                   lua_version .. \" in your PATH. \" ..\n"
"                   \"Modules may not install with the correct configurations. \" ..\n"
"                   \"You may want to specify to the path prefix to your build \" ..\n"
"                   \"of Lua \" .. lua_version .. \" using --lua-dir\")\n"
"   end\n"
"   cfg.lua_found = lua_found\n"
"\n"
"   if detected.project_dir then\n"
"      detected.project_dir = fs.absolute_name(detected.project_dir)\n"
"   end\n"
"\n"
"   if flags[\"version\"] then\n"
"      util.printout(program..\" \"..cfg.program_version)\n"
"      util.printout(description)\n"
"      util.printout()\n"
"      os.exit(cmd.errorcodes.OK)\n"
"   end\n"
"\n"
"   if external_namespace then\n"
"      for _, module_name in ipairs(fs.modules(external_namespace)) do\n"
"         if not commands[module_name] then\n"
"            commands[module_name] = external_namespace..\".\"..module_name\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   if flags[\"verbose\"] then\n"
"      cfg.verbose = true\n"
"      fs.verbose()\n"
"   end\n"
"\n"
"   if (not fs.current_dir()) or fs.current_dir() == \"\" then\n"
"      die(\"Current directory does not exist. Please run LuaRocks from an existing directory.\")\n"
"   end\n"
"\n"
"   ok, err = process_tree_flags(flags, detected.project_dir)\n"
"   if not ok then\n"
"      die(err)\n"
"   end\n"
"\n"
"   ok, err = process_server_flags(flags)\n"
"   if not ok then\n"
"      die(err)\n"
"   end\n"
"\n"
"   if flags[\"only-sources\"] then\n"
"      cfg.only_sources_from = flags[\"only-sources\"]\n"
"   end\n"
"\n"
"   if command ~= \"help\" then\n"
"      for k, v in pairs(cmdline_vars) do\n"
"         cfg.variables[k] = v\n"
"      end\n"
"   end\n"
"\n"
"   -- if running as superuser, use system cache dir\n"
"   if not cfg.home_tree then\n"
"      cfg.local_cache = dir.path(fs.system_cache_dir(), \"luarocks\")\n"
"   end\n"
"\n"
"   if commands[command] then\n"
"      local cmd_mod = require(commands[command])\n"
"      local call_ok, ok, err, exitcode = xpcall(function()\n"
"         if command == \"help\" then\n"
"            return cmd_mod.command(description, commands, unpack(nonflags))\n"
"         else\n"
"            return cmd_mod.command(flags, unpack(nonflags))\n"
"         end\n"
"      end, error_handler)\n"
"      if not call_ok then\n"
"         die(ok, cmd.errorcodes.CRASH)\n"
"      elseif not ok then\n"
"         die(err, exitcode)\n"
"      end\n"
"   else\n"
"      die(\"Unknown command: \"..command)\n"
"   end\n"
"   util.run_scheduled_functions()\n"
"end\n"
"\n"
"return cmd\n"
""
;
