const char luarocks_cmd_doc_lua[] =
"\n"
"--- Module implementing the LuaRocks \"doc\" command.\n"
"-- Shows documentation for an installed rock.\n"
"local doc = {}\n"
"\n"
"local util = require(\"luarocks.util\")\n"
"local queries = require(\"luarocks.queries\")\n"
"local search = require(\"luarocks.search\")\n"
"local path = require(\"luarocks.path\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local fetch = require(\"luarocks.fetch\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local download = require(\"luarocks.download\")\n"
"\n"
"doc.help_summary = \"Show documentation for an installed rock.\"\n"
"\n"
"doc.help = [[\n"
"<argument> is an existing package name.\n"
"Without any flags, tries to load the documentation\n"
"using a series of heuristics.\n"
"With these flags, return only the desired information:\n"
"\n"
"--home      Open the home page of project.\n"
"--list      List documentation files only.\n"
"\n"
"For more information about a rock, see the 'show' command.\n"
"]]\n"
"\n"
"local function show_homepage(homepage, name, version)\n"
"   if not homepage then\n"
"      return nil, \"No 'homepage' field in rockspec for \"..name..\" \"..version\n"
"   end\n"
"   util.printout(\"Opening \"..homepage..\" ...\")\n"
"   fs.browser(homepage)\n"
"   return true\n"
"end\n"
"\n"
"local function try_to_open_homepage(name, version)\n"
"   local temp_dir, err = fs.make_temp_dir(\"doc-\"..name..\"-\"..(version or \"\"))\n"
"   if not temp_dir then\n"
"      return nil, \"Failed creating temporary directory: \"..err\n"
"   end\n"
"   util.schedule_function(fs.delete, temp_dir)\n"
"   local ok, err = fs.change_dir(temp_dir)\n"
"   if not ok then return nil, err end\n"
"   local filename, err = download.download(\"rockspec\", name, version)\n"
"   if not filename then return nil, err end\n"
"   local rockspec, err = fetch.load_local_rockspec(filename)\n"
"   if not rockspec then return nil, err end\n"
"   fs.pop_dir()\n"
"   local descript = rockspec.description or {}\n"
"   if not descript.homepage then return nil, \"No homepage defined for \"..name end\n"
"   return show_homepage(descript.homepage, name, version)\n"
"end\n"
"\n"
"--- Driver function for \"doc\" command.\n"
"-- @param name or nil: an existing package name.\n"
"-- @param version string or nil: a version may also be passed.\n"
"-- @return boolean: True if succeeded, nil on errors.\n"
"function doc.command(flags, name, version)\n"
"   if not name then\n"
"      return nil, \"Argument missing. \"..util.see_help(\"doc\")\n"
"   end\n"
"\n"
"   name = util.adjust_name_and_namespace(name, flags)\n"
"   local query = queries.new(name, version)\n"
"   local iname, iversion, repo = search.pick_installed_rock(query, flags[\"tree\"])\n"
"   if not iname then\n"
"      util.printout(name..(version and \" \"..version or \"\")..\" is not installed. Looking for it in the rocks servers...\")\n"
"      return try_to_open_homepage(name, version)\n"
"   end\n"
"   name, version = iname, iversion\n"
"   \n"
"   local rockspec, err = fetch.load_local_rockspec(path.rockspec_file(name, version, repo))\n"
"   if not rockspec then return nil,err end\n"
"   local descript = rockspec.description or {}\n"
"\n"
"   if flags[\"home\"] then\n"
"      return show_homepage(descript.homepage, name, version)\n"
"   end\n"
"\n"
"   local directory = path.install_dir(name, version, repo)\n"
"   \n"
"   local docdir\n"
"   local directories = { \"doc\", \"docs\" }\n"
"   for _, d in ipairs(directories) do\n"
"      local dirname = dir.path(directory, d)\n"
"      if fs.is_dir(dirname) then\n"
"         docdir = dirname\n"
"         break\n"
"      end\n"
"   end\n"
"   if not docdir then\n"
"      if descript.homepage and not flags[\"list\"] then\n"
"         util.printout(\"Local documentation directory not found -- opening \"..descript.homepage..\" ...\")\n"
"         fs.browser(descript.homepage)\n"
"         return true\n"
"      end\n"
"      return nil, \"Documentation directory not found for \"..name..\" \"..version\n"
"   end\n"
"\n"
"   docdir = dir.normalize(docdir):gsub(\"/+\", \"/\")\n"
"   local files = fs.find(docdir)\n"
"   local htmlpatt = \"%.html\?$\"\n"
"   local extensions = { htmlpatt, \"%.md$\", \"%.txt$\",  \"%.textile$\", \"\" }\n"
"   local basenames = { \"index\", \"readme\", \"manual\" }\n"
"   \n"
"   local porcelain = flags[\"porcelain\"]\n"
"   if #files > 0 then\n"
"      util.title(\"Documentation files for \"..name..\" \"..version, porcelain)\n"
"      if porcelain then\n"
"         for _, file in ipairs(files) do\n"
"            util.printout(docdir..\"/\"..file)\n"
"         end\n"
"      else\n"
"         util.printout(docdir..\"/\")\n"
"         for _, file in ipairs(files) do\n"
"            util.printout(\"\\t\"..file)\n"
"         end\n"
"      end\n"
"   end\n"
"   \n"
"   if flags[\"list\"] then\n"
"      return true\n"
"   end\n"
"   \n"
"   for _, extension in ipairs(extensions) do\n"
"      for _, basename in ipairs(basenames) do\n"
"         local filename = basename..extension\n"
"         local found\n"
"         for _, file in ipairs(files) do\n"
"            if file:lower():match(filename) and ((not found) or #file < #found) then\n"
"               found = file\n"
"            end\n"
"         end\n"
"         if found then\n"
"            local pathname = dir.path(docdir, found)\n"
"            util.printout()\n"
"            util.printout(\"Opening \"..pathname..\" ...\")\n"
"            util.printout()\n"
"            local ok = fs.browser(pathname)\n"
"            if not ok and not pathname:match(htmlpatt) then\n"
"               local fd = io.open(pathname, \"r\")\n"
"               util.printout(fd:read(\"*a\"))\n"
"               fd:close()\n"
"            end\n"
"            return true\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   return true\n"
"end\n"
"\n"
"\n"
"return doc\n"
""
;
