const char luarocks_cmd_download_lua[] =
"\n"
"--- Module implementing the luarocks \"download\" command.\n"
"-- Download a rock from the repository.\n"
"local cmd_download = {}\n"
"\n"
"local util = require(\"luarocks.util\")\n"
"local download = require(\"luarocks.download\")\n"
"\n"
"cmd_download.help_summary = \"Download a specific rock file from a rocks server.\"\n"
"cmd_download.help_arguments = \"[--all] [--arch=<arch> | --source | --rockspec] [<name> [<version>]]\"\n"
"cmd_download.help = [[\n"
"--all          Download all files if there are multiple matches.\n"
"--source       Download .src.rock if available.\n"
"--rockspec     Download .rockspec if available.\n"
"--arch=<arch>  Download rock for a specific architecture.\n"
"]]\n"
"\n"
"--- Driver function for the \"download\" command.\n"
"-- @param name string: a rock name.\n"
"-- @param version string or nil: if the name of a package is given, a\n"
"-- version may also be passed.\n"
"-- @return boolean or (nil, string): true if successful or nil followed\n"
"-- by an error message.\n"
"function cmd_download.command(flags, name, version)\n"
"   assert(type(version) == \"string\" or not version)\n"
"   if type(name) ~= \"string\" and not flags[\"all\"] then\n"
"      return nil, \"Argument missing. \"..util.see_help(\"download\")\n"
"   end\n"
"\n"
"   name = util.adjust_name_and_namespace(name, flags)\n"
"\n"
"   if not name then name, version = \"\", \"\" end\n"
"\n"
"   local arch\n"
"\n"
"   if flags[\"source\"] then\n"
"      arch = \"src\"\n"
"   elseif flags[\"rockspec\"] then\n"
"      arch = \"rockspec\"\n"
"   elseif flags[\"arch\"] then\n"
"      arch = flags[\"arch\"]\n"
"   end\n"
"   \n"
"   local dl, err = download.download(arch, name:lower(), version, flags[\"all\"])\n"
"   return dl and true, err\n"
"end\n"
"\n"
"return cmd_download\n"
""
;
