const char luarocks_cmd_help_lua[] =
"\n"
"--- Module implementing the LuaRocks \"help\" command.\n"
"-- This is a generic help display module, which\n"
"-- uses a global table called \"commands\" to find commands\n"
"-- to show help for; each command should be represented by a\n"
"-- table containing \"help\" and \"help_summary\" fields.\n"
"local help = {}\n"
"\n"
"local util = require(\"luarocks.util\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local dir = require(\"luarocks.dir\")\n"
"\n"
"local program = util.this_program(\"luarocks\")\n"
"\n"
"help.help_summary = \"Help on commands. Type '\"..program..\" help <command>' for more.\"\n"
"\n"
"help.help_arguments = \"[<command>]\"\n"
"help.help = [[\n"
"<command> is the command to show help for.\n"
"]]\n"
"\n"
"local function print_banner()\n"
"   util.printout(\"\\nLuaRocks \"..cfg.program_version..\", the Lua package manager\")\n"
"end\n"
"\n"
"local function print_section(section)\n"
"   util.printout(\"\\n\"..section)\n"
"end\n"
"\n"
"local function get_status(status)\n"
"   if status then\n"
"      return \"ok\"\n"
"   else\n"
"      return \"not found\"\n"
"   end\n"
"end\n"
"\n"
"--- Driver function for the \"help\" command.\n"
"-- @param command string or nil: command to show help for; if not\n"
"-- given, help summaries for all commands are shown.\n"
"-- @return boolean or (nil, string): true if there were no errors\n"
"-- or nil and an error message if an invalid command was requested.\n"
"function help.command(description, commands, command)\n"
"   assert(type(description) == \"string\")\n"
"   assert(type(commands) == \"table\")\n"
"\n"
"   if not command then\n"
"      print_banner()\n"
"      print_section(\"NAME\")\n"
"      util.printout(\"\\t\"..program..[[ - ]]..description)\n"
"      print_section(\"SYNOPSIS\")\n"
"      util.printout(\"\\t\"..program..[[ [<flags...>] [VAR=VALUE]... <command> [<argument>] ]])\n"
"      print_section(\"GENERAL OPTIONS\")\n"
"      util.printout([[\n"
"\tThese apply to all commands, as appropriate:\n"
"\n"
"\t--dev                  Enable the sub-repositories in rocks servers\n"
"\t                       for rockspecs of in-development versions\n"
"\t--server=<server>      Fetch rocks/rockspecs from this server\n"
"\t                       (takes priority over config file)\n"
"\t--only-server=<server> Fetch rocks/rockspecs from this server only\n"
"\t                       (overrides any entries in the config file)\n"
"\t--only-sources=<url>   Restrict downloads to paths matching the\n"
"\t                       given URL.\n"
"        --lua-dir=<prefix>     Which Lua installation to use.\n"
"\t--lua-version=<ver>    Which Lua version to use.\n"
"\t--tree=<tree>          Which tree to operate on.\n"
"\t--local                Use the tree in the user's home directory.\n"
"\t                       To enable it, see ']]..program..[[ help path'.\n"
"\t--global               Use the system tree when `local_by_default` is `true`.\n"
"\t--verbose              Display verbose output of commands executed.\n"
"\t--timeout=<seconds>    Timeout on network operations, in seconds.\n"
"\t                       0 means no timeout (wait forever).\n"
"\t                       Default is ]]..tostring(cfg.connection_timeout)..[[.]])\n"
"      print_section(\"VARIABLES\")\n"
"      util.printout([[\n"
"\tVariables from the \"variables\" table of the configuration file\n"
"\tcan be overridden with VAR=VALUE assignments.]])\n"
"      print_section(\"COMMANDS\")\n"
"      for name, modname in util.sortedpairs(commands) do\n"
"         local cmd = require(modname)\n"
"         util.printout(\"\", name)\n"
"         util.printout(\"\\t\", cmd.help_summary)\n"
"      end\n"
"      print_section(\"CONFIGURATION\")\n"
"      util.printout(\"\\tLua version: \" .. cfg.lua_version)\n"
"      if cfg.luajit_version then\n"
"         util.printout(\"\\tLuaJIT version: \" .. cfg.luajit_version)\n"
"      end\n"
"      util.printout()\n"
"      util.printout(\"\\tConfiguration files:\")\n"
"      local conf = cfg.config_files\n"
"      util.printout(\"\\t\\tSystem  : \".. dir.normalize(conf.system.file) .. \" (\" .. get_status(conf.system.found) ..\")\")\n"
"      if conf.user.file then\n"
"         util.printout(\"\\t\\tUser    : \".. dir.normalize(conf.user.file) .. \" (\" .. get_status(conf.user.found) ..\")\")\n"
"      else\n"
"         util.printout(\"\\t\\tUser    : disabled in this LuaRocks installation.\")\n"
"      end\n"
"      if conf.project then\n"
"         util.printout(\"\\t\\tProject : \".. dir.normalize(conf.project.file) .. \" (\" .. get_status(conf.project.found) ..\")\")\n"
"      end\n"
"      util.printout()\n"
"      util.printout(\"\\tRocks trees in use: \")\n"
"      for _, tree in ipairs(cfg.rocks_trees) do\n"
"         if type(tree) == \"string\" then\n"
"            util.printout(\"\\t\\t\"..dir.normalize(tree))\n"
"         else\n"
"            local name = tree.name and \" (\\\"\"..tree.name..\"\\\")\" or \"\"\n"
"            util.printout(\"\\t\\t\"..dir.normalize(tree.root)..name)\n"
"         end\n"
"      end\n"
"      util.printout()\n"
"   else\n"
"      command = command:gsub(\"-\", \"_\")\n"
"      local cmd = commands[command] and require(commands[command])\n"
"      if cmd then\n"
"         local arguments = cmd.help_arguments or \"<argument>\"\n"
"         print_banner()\n"
"         print_section(\"NAME\")\n"
"         util.printout(\"\\t\"..program..\" \"..command..\" - \"..cmd.help_summary)\n"
"         print_section(\"SYNOPSIS\")\n"
"         util.printout(\"\\t\"..program..\" \"..command..\" \"..arguments)\n"
"         print_section(\"DESCRIPTION\")\n"
"         util.printout(\"\",(cmd.help:gsub(\"\\n\",\"\\n\\t\"):gsub(\"\\n\\t$\",\"\")))\n"
"         print_section(\"SEE ALSO\")\n"
"         if cmd.help_see_also then\n"
"            util.printout(cmd.help_see_also)\n"
"         end\n"
"         util.printout(\"\",\"'\"..program..\" help' for general options and configuration.\\n\")\n"
"      else\n"
"         return nil, \"Unknown command: \"..command\n"
"      end\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"return help\n"
""
;
