const char luarocks_cmd_init_lua[] =
"\n"
"local init = {}\n"
"\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local path = require(\"luarocks.path\")\n"
"local deps = require(\"luarocks.deps\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local util = require(\"luarocks.util\")\n"
"local persist = require(\"luarocks.persist\")\n"
"local write_rockspec = require(\"luarocks.cmd.write_rockspec\")\n"
"\n"
"init.help_summary = \"Initialize a directory for a Lua project using LuaRocks.\"\n"
"init.help_arguments = \"[<name> [<version>]]\"\n"
"init.help = [[\n"
"<name> is the project name.\n"
"<version> is an optional project version.\n"
"\n"
"--reset                  Delete .luarocks/config-5.x.lua and ./lua\n"
"                         and generate new ones.\n"
"\n"
"Options for specifying rockspec data:\n"
"\n"
"--license=\"<string>\"     A license string, such as \"MIT/X11\" or \"GNU GPL v3\".\n"
"--summary=\"<txt>\"        A short one-line description summary.\n"
"--detailed=\"<txt>\"       A longer description string.\n"
"--homepage=<url>         Project homepage.\n"
"--lua-versions=<ver>     Supported Lua versions. Accepted values are \"5.1\", \"5.2\",\n"
"                         \"5.3\", \"5.1,5.2\", \"5.2,5.3\", or \"5.1,5.2,5.3\".\n"
"--rockspec-format=<ver>  Rockspec format version, such as \"1.0\" or \"1.1\".\n"
"--lib=<lib>[,<lib>]      A comma-separated list of libraries that C files need to\n"
"                         link to.\n"
"]]\n"
"\n"
"local function write_gitignore(entries)\n"
"   local gitignore = \"\"\n"
"   local fd = io.open(\".gitignore\", \"r\")\n"
"   if fd then\n"
"      gitignore = fd:read(\"*a\")\n"
"      fd:close()\n"
"      gitignore = \"\\n\" .. gitignore .. \"\\n\"\n"
"   end\n"
"   \n"
"   fd = io.open(\".gitignore\", gitignore and \"a\" or \"w\")\n"
"   for _, entry in ipairs(entries) do\n"
"      entry = \"/\" .. entry\n"
"      if not gitignore:find(\"\\n\"..entry..\"\\n\", 1, true) then\n"
"         fd:write(entry..\"\\n\")\n"
"      end\n"
"   end\n"
"   fd:close()\n"
"end\n"
"\n"
"--- Driver function for \"init\" command.\n"
"-- @return boolean: True if succeeded, nil on errors.\n"
"function init.command(flags, name, version)\n"
"\n"
"   local pwd = fs.current_dir()\n"
"\n"
"   if not name then\n"
"      name = dir.base_name(pwd)\n"
"      if name == \"/\" then\n"
"         return nil, \"When running from the root directory, please specify the <name> argument\"\n"
"      end\n"
"   end\n"
"\n"
"   util.title(\"Initializing project '\" .. name .. \"' for Lua \" .. cfg.lua_version .. \" ...\")\n"
"\n"
"   util.printout(\"Checking your Lua installation ...\")\n"
"   if not cfg.lua_found then\n"
"      return nil, \"Lua installation is not found.\"\n"
"   end\n"
"   local ok, err = deps.check_lua(cfg.variables)\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"   \n"
"   local has_rockspec = false\n"
"   for file in fs.dir() do\n"
"      if file:match(\"%.rockspec$\") then\n"
"         has_rockspec = true\n"
"         break\n"
"      end\n"
"   end\n"
"\n"
"   if not has_rockspec then\n"
"      local ok, err = write_rockspec.command(flags, name, version or \"dev\", pwd)\n"
"      if not ok then\n"
"         util.printerr(err)\n"
"      end\n"
"   end\n"
"\n"
"   local ext = cfg.wrapper_suffix\n"
"   local luarocks_wrapper = \"luarocks\" .. ext\n"
"   local lua_wrapper = \"lua\" .. ext\n"
"\n"
"   util.printout(\"Adding entries to .gitignore ...\")\n"
"   write_gitignore({ luarocks_wrapper, lua_wrapper, \"lua_modules\" })\n"
"\n"
"   util.printout(\"Preparing ./.luarocks/ ...\")\n"
"   fs.make_dir(\".luarocks\")\n"
"   local config_file = \".luarocks/config-\" .. cfg.lua_version .. \".lua\"\n"
"\n"
"   if flags[\"reset\"] then\n"
"      fs.delete(lua_wrapper)\n"
"      fs.delete(config_file)\n"
"   end\n"
"   \n"
"   local config_tbl, err = persist.load_config_file_if_basic(config_file, cfg)\n"
"   if config_tbl then\n"
"      local globals = {\n"
"         \"lua_interpreter\",\n"
"         \"luajit_version\",\n"
"      }\n"
"      for _, v in ipairs(globals) do\n"
"         if cfg[v] then\n"
"            config_tbl[v] = cfg[v]\n"
"         end\n"
"      end\n"
"\n"
"      local varnames = {\n"
"         \"LUA_DIR\",\n"
"         \"LUA_INCDIR\",\n"
"         \"LUA_LIBDIR\",\n"
"         \"LUA_BINDIR\",\n"
"         \"LUA_INTERPRETER\",\n"
"      }\n"
"      for _, varname in ipairs(varnames) do\n"
"         if cfg.variables[varname] then\n"
"            config_tbl.variables = config_tbl.variables or {}\n"
"            config_tbl.variables[varname] = cfg.variables[varname]\n"
"         end\n"
"      end\n"
"      local ok, err = persist.save_from_table(config_file, config_tbl)\n"
"      if ok then\n"
"         util.printout(\"Wrote \" .. config_file)\n"
"      else\n"
"         util.printout(\"Failed writing \" .. config_file .. \": \" .. err)\n"
"      end\n"
"   else\n"
"      util.printout(\"Will not attempt to overwrite \" .. config_file)\n"
"   end\n"
"   \n"
"   ok, err = persist.save_default_lua_version(\".luarocks\", cfg.lua_version)\n"
"   if not ok then\n"
"      util.printout(\"Failed setting default Lua version: \" .. err)\n"
"   end\n"
"\n"
"   util.printout(\"Preparing ./lua_modules/ ...\")\n"
"\n"
"   fs.make_dir(\"lua_modules/lib/luarocks/rocks-\" .. cfg.lua_version)\n"
"   local tree = dir.path(pwd, \"lua_modules\")\n"
"\n"
"   luarocks_wrapper = dir.path(\".\", luarocks_wrapper)\n"
"   if not fs.exists(luarocks_wrapper) then\n"
"      util.printout(\"Preparing \" .. luarocks_wrapper .. \" ...\")\n"
"      fs.wrap_script(arg[0], \"luarocks\", \"none\", nil, nil, \"--project-tree\", tree)\n"
"   else\n"
"      util.printout(luarocks_wrapper .. \" already exists. Not overwriting it!\")\n"
"   end\n"
"\n"
"   lua_wrapper = dir.path(\".\", lua_wrapper)\n"
"   local write_lua_wrapper = true\n"
"   if fs.exists(lua_wrapper) then\n"
"      if not util.lua_is_wrapper(lua_wrapper) then\n"
"         util.printout(lua_wrapper .. \" already exists and does not look like a wrapper script. Not overwriting.\")\n"
"         write_lua_wrapper = false\n"
"      end\n"
"   end\n"
"   \n"
"   if write_lua_wrapper then\n"
"      local interp = dir.path(cfg.variables[\"LUA_BINDIR\"], cfg.lua_interpreter)\n"
"      if util.check_lua_version(interp, cfg.lua_version) then\n"
"         util.printout(\"Preparing \" .. lua_wrapper .. \" for version \" .. cfg.lua_version .. \"...\")\n"
"         path.use_tree(tree)\n"
"         fs.wrap_script(nil, \"lua\", \"all\")\n"
"      else\n"
"         util.warning(\"No Lua interpreter detected for version \" .. cfg.lua_version .. \". Not creating \" .. lua_wrapper)\n"
"      end\n"
"   end\n"
"\n"
"   return true\n"
"end\n"
"\n"
"return init\n"
""
;
