const char luarocks_cmd_lint_lua[] =
"\n"
"--- Module implementing the LuaRocks \"lint\" command.\n"
"-- Utility function that checks syntax of the rockspec.\n"
"local lint = {}\n"
"\n"
"local util = require(\"luarocks.util\")\n"
"local download = require(\"luarocks.download\")\n"
"local fetch = require(\"luarocks.fetch\")\n"
"\n"
"lint.help_summary = \"Check syntax of a rockspec.\"\n"
"lint.help_arguments = \"<rockspec>\"\n"
"lint.help = [[\n"
"This is a utility function that checks the syntax of a rockspec.\n"
"\n"
"It returns success or failure if the text of a rockspec is\n"
"syntactically correct.\n"
"]]\n"
"\n"
"function lint.command(flags, input)\n"
"   if not input then\n"
"      return nil, \"Argument missing. \"..util.see_help(\"lint\")\n"
"   end\n"
"   \n"
"   local filename = input\n"
"   if not input:match(\".rockspec$\") then\n"
"      local err\n"
"      filename, err = download.download(\"rockspec\", input:lower())\n"
"      if not filename then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"\n"
"   local rs, err = fetch.load_local_rockspec(filename)\n"
"   if not rs then\n"
"      return nil, \"Failed loading rockspec: \"..err\n"
"   end\n"
"\n"
"   local ok = true\n"
"   \n"
"   -- This should have been done in the type checker, \n"
"   -- but it would break compatibility of other commands.\n"
"   -- Making 'lint' alone be stricter shouldn't be a problem,\n"
"   -- because extra-strict checks is what lint-type commands\n"
"   -- are all about.\n"
"   if not rs.description.license then\n"
"      util.printerr(\"Rockspec has no license field.\")\n"
"      ok = false\n"
"   end\n"
"\n"
"   return ok, ok or filename..\" failed consistency checks.\"\n"
"end\n"
"\n"
"return lint\n"
""
;
