const char luarocks_cmd_list_lua[] =
"\n"
"--- Module implementing the LuaRocks \"list\" command.\n"
"-- Lists currently installed rocks.\n"
"local list = {}\n"
"\n"
"local search = require(\"luarocks.search\")\n"
"local queries = require(\"luarocks.queries\")\n"
"local vers = require(\"luarocks.core.vers\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local util = require(\"luarocks.util\")\n"
"local path = require(\"luarocks.path\")\n"
"\n"
"list.help_summary = \"List currently installed rocks.\"\n"
"list.help_arguments = \"[--porcelain] <filter>\"\n"
"list.help = [[\n"
"<filter> is a substring of a rock name to filter by.\n"
"\n"
"--outdated    List only rocks for which there is a\n"
"              higher version available in the rocks server.\n"
"\n"
"--porcelain   Produce machine-friendly output.\n"
"]]\n"
"\n"
"local function check_outdated(trees, query)\n"
"   local results_installed = {}\n"
"   for _, tree in ipairs(trees) do\n"
"      search.local_manifest_search(results_installed, path.rocks_dir(tree), query)\n"
"   end\n"
"   local outdated = {}\n"
"   for name, versions in util.sortedpairs(results_installed) do\n"
"      versions = util.keys(versions)\n"
"      table.sort(versions, vers.compare_versions)\n"
"      local latest_installed = versions[1]\n"
"\n"
"      local query_available = queries.new(name:lower())\n"
"      local results_available, err = search.search_repos(query_available)\n"
"      \n"
"      if results_available[name] then\n"
"         local available_versions = util.keys(results_available[name])\n"
"         table.sort(available_versions, vers.compare_versions)\n"
"         local latest_available = available_versions[1]\n"
"         local latest_available_repo = results_available[name][latest_available][1].repo\n"
"         \n"
"         if vers.compare_versions(latest_available, latest_installed) then\n"
"            table.insert(outdated, { name = name, installed = latest_installed, available = latest_available, repo = latest_available_repo })\n"
"         end\n"
"      end\n"
"   end\n"
"   return outdated\n"
"end\n"
"\n"
"local function list_outdated(trees, query, porcelain)\n"
"   util.title(\"Outdated rocks:\", porcelain)\n"
"   local outdated = check_outdated(trees, query)\n"
"   for _, item in ipairs(outdated) do\n"
"      if porcelain then\n"
"         util.printout(item.name, item.installed, item.available, item.repo)\n"
"      else\n"
"         util.printout(item.name)\n"
"         util.printout(\"   \"..item.installed..\" < \"..item.available..\" at \"..item.repo)\n"
"         util.printout()\n"
"      end\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"--- Driver function for \"list\" command.\n"
"-- @param filter string or nil: A substring of a rock name to filter by.\n"
"-- @param version string or nil: a version may also be passed.\n"
"-- @return boolean: True if succeeded, nil on errors.\n"
"function list.command(flags, filter, version)\n"
"   local query = queries.new(filter and filter:lower() or \"\", version, true)\n"
"   local trees = cfg.rocks_trees\n"
"   local title = \"Rocks installed for Lua \"..cfg.lua_version\n"
"   if flags[\"tree\"] then\n"
"      trees = { flags[\"tree\"] }\n"
"      title = title .. \" in \" .. flags[\"tree\"]\n"
"   end\n"
"   \n"
"   if flags[\"outdated\"] then\n"
"      return list_outdated(trees, query, flags[\"porcelain\"])\n"
"   end\n"
"   \n"
"   local results = {}\n"
"   for _, tree in ipairs(trees) do\n"
"      local ok, err, errcode = search.local_manifest_search(results, path.rocks_dir(tree), query)\n"
"      if not ok and errcode ~= \"open\" then\n"
"         util.warning(err)\n"
"      end\n"
"   end\n"
"   util.title(title, flags[\"porcelain\"])\n"
"   search.print_result_tree(results, flags[\"porcelain\"])\n"
"   return true\n"
"end\n"
"\n"
"return list\n"
""
;
