const char luarocks_cmd_new_version_lua[] =
"\n"
"--- Module implementing the LuaRocks \"new_version\" command.\n"
"-- Utility function that writes a new rockspec, updating data from a previous one.\n"
"local new_version = {}\n"
"\n"
"local util = require(\"luarocks.util\")\n"
"local download = require(\"luarocks.download\")\n"
"local fetch = require(\"luarocks.fetch\")\n"
"local persist = require(\"luarocks.persist\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local type_rockspec = require(\"luarocks.type.rockspec\")\n"
"\n"
"new_version.help_summary = \"Auto-write a rockspec for a new version of a rock.\"\n"
"new_version.help_arguments = \"[--tag=<tag>] [<package>|<rockspec>] [<new_version>] [<new_url>]\"\n"
"new_version.help = [[\n"
"This is a utility function that writes a new rockspec, updating data\n"
"from a previous one.\n"
"\n"
"If a package name is given, it downloads the latest rockspec from the\n"
"default server. If a rockspec is given, it uses it instead. If no argument\n"
"is given, it looks for a rockspec same way 'luarocks make' does.\n"
"\n"
"If the version number is not given and tag is passed using --tag,\n"
"it is used as the version, with 'v' removed from beginning.\n"
"Otherwise, it only increments the revision number of the given\n"
"(or downloaded) rockspec.\n"
"\n"
"If a URL is given, it replaces the one from the old rockspec with the\n"
"given URL. If a URL is not given and a new version is given, it tries\n"
"to guess the new URL by replacing occurrences of the version number\n"
"in the URL or tag. It also tries to download the new URL to determine\n"
"the new MD5 checksum.\n"
"\n"
"If a tag is given, it replaces the one from the old rockspec. If there is\n"
"an old tag but no new one passed, it is guessed in the same way URL is.\n"
"\n"
"WARNING: it writes the new rockspec to the current directory,\n"
"overwriting the file if it already exists.\n"
"]]\n"
"\n"
"local function try_replace(tbl, field, old, new)\n"
"   if not tbl[field] then\n"
"      return false\n"
"   end\n"
"   local old_field = tbl[field]\n"
"   local new_field = tbl[field]:gsub(old, new)\n"
"   if new_field ~= old_field then\n"
"      util.printout(\"Guessing new '\"..field..\"' field as \"..new_field)\n"
"      tbl[field] = new_field\n"
"      return true      \n"
"   end\n"
"   return false\n"
"end\n"
"\n"
"-- Try to download source file using URL from a rockspec.\n"
"-- If it specified MD5, update it.\n"
"-- @return (true, false) if MD5 was not specified or it stayed same,\n"
"-- (true, true) if MD5 changed, (nil, string) on error.\n"
"local function check_url_and_update_md5(out_rs)\n"
"   local file, temp_dir = fetch.fetch_url_at_temp_dir(out_rs.source.url, \"luarocks-new-version-\"..out_rs.package)\n"
"   if not file then\n"
"      util.warning(\"invalid URL - \"..temp_dir)\n"
"      return true, false\n"
"   end\n"
"\n"
"   local inferred_dir, found_dir = fetch.find_base_dir(file, temp_dir, out_rs.source.url, out_rs.source.dir)\n"
"   if not inferred_dir then\n"
"      return nil, found_dir\n"
"   end\n"
"\n"
"   if found_dir and found_dir ~= inferred_dir then\n"
"      out_rs.source.dir = found_dir\n"
"   end\n"
"\n"
"   if file then\n"
"      if out_rs.source.md5 then\n"
"         util.printout(\"File successfully downloaded. Updating MD5 checksum...\")\n"
"         local new_md5, err = fs.get_md5(file)\n"
"         if not new_md5 then\n"
"            return nil, err\n"
"         end\n"
"         local old_md5 = out_rs.source.md5\n"
"         out_rs.source.md5 = new_md5\n"
"         return true, new_md5 ~= old_md5\n"
"      else\n"
"         util.printout(\"File successfully downloaded.\")\n"
"         return true, false\n"
"      end\n"
"   end\n"
"end\n"
" \n"
"local function update_source_section(out_rs, url, tag, old_ver, new_ver)\n"
"   if tag then\n"
"      out_rs.source.tag = tag\n"
"   end\n"
"   if url then\n"
"      out_rs.source.url = url\n"
"      return check_url_and_update_md5(out_rs)\n"
"   end\n"
"   if new_ver == old_ver then\n"
"      return true\n"
"   end\n"
"   if out_rs.source.dir then\n"
"      try_replace(out_rs.source, \"dir\", old_ver, new_ver)\n"
"   end\n"
"   if out_rs.source.file then\n"
"      try_replace(out_rs.source, \"file\", old_ver, new_ver)\n"
"   end\n"
"   if try_replace(out_rs.source, \"url\", old_ver, new_ver) then\n"
"      return check_url_and_update_md5(out_rs)\n"
"   end\n"
"   if tag or try_replace(out_rs.source, \"tag\", old_ver, new_ver) then\n"
"      return true\n"
"   end\n"
"   -- Couldn't replace anything significant, use the old URL.\n"
"   local ok, md5_changed = check_url_and_update_md5(out_rs)\n"
"   if not ok then\n"
"      return nil, md5_changed\n"
"   end\n"
"   if md5_changed then\n"
"      util.warning(\"URL is the same, but MD5 has changed. Old rockspec is broken.\")\n"
"   end\n"
"   return true\n"
"end\n"
" \n"
"function new_version.command(flags, input, version, url)\n"
"   if not input then\n"
"      local err\n"
"      input, err = util.get_default_rockspec()\n"
"      if not input then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"   assert(type(input) == \"string\")\n"
"   \n"
"   local filename, err\n"
"   if input:match(\"rockspec$\") then\n"
"      filename, err = fetch.fetch_url(input)\n"
"      if not filename then\n"
"         return nil, err\n"
"      end\n"
"   else\n"
"      filename, err = download.download(\"rockspec\", input:lower())\n"
"      if not filename then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"\n"
"   local valid_rs, err = fetch.load_rockspec(filename)\n"
"   if not valid_rs then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   local old_ver, old_rev = valid_rs.version:match(\"(.*)%-(%d+)$\")\n"
"   local new_ver, new_rev\n"
"\n"
"   if flags.tag and not version then\n"
"      version = flags.tag:gsub(\"^v\", \"\")\n"
"   end\n"
"   \n"
"   if version then\n"
"      new_ver, new_rev = version:match(\"(.*)%-(%d+)$\")\n"
"      new_rev = tonumber(new_rev)\n"
"      if not new_rev then\n"
"         new_ver = version\n"
"         new_rev = 1\n"
"      end\n"
"   else\n"
"      new_ver = old_ver\n"
"      new_rev = tonumber(old_rev) + 1\n"
"   end\n"
"   local new_rockver = new_ver:gsub(\"-\", \"\")\n"
"   \n"
"   local out_rs, err = persist.load_into_table(filename)\n"
"   local out_name = out_rs.package:lower()\n"
"   out_rs.version = new_rockver..\"-\"..new_rev\n"
"\n"
"   local ok, err = update_source_section(out_rs, url, flags.tag, old_ver, new_ver)\n"
"   if not ok then return nil, err end\n"
"\n"
"   if out_rs.build and out_rs.build.type == \"module\" then\n"
"      out_rs.build.type = \"builtin\"\n"
"   end\n"
"   \n"
"   local out_filename = out_name..\"-\"..new_rockver..\"-\"..new_rev..\".rockspec\"\n"
"   \n"
"   persist.save_from_table(out_filename, out_rs, type_rockspec.order)\n"
"   \n"
"   util.printout(\"Wrote \"..out_filename)\n"
"\n"
"   local valid_out_rs, err = fetch.load_local_rockspec(out_filename)\n"
"   if not valid_out_rs then\n"
"      return nil, \"Failed loading generated rockspec: \"..err\n"
"   end\n"
"   \n"
"   return true\n"
"end\n"
"\n"
"return new_version\n"
""
;
