const char luarocks_cmd_pack_lua[] =
"\n"
"--- Module implementing the LuaRocks \"pack\" command.\n"
"-- Creates a rock, packing sources or binaries.\n"
"local cmd_pack = {}\n"
"\n"
"local util = require(\"luarocks.util\")\n"
"local pack = require(\"luarocks.pack\")\n"
"local signing = require(\"luarocks.signing\")\n"
"local queries = require(\"luarocks.queries\")\n"
"\n"
"cmd_pack.help_summary = \"Create a rock, packing sources or binaries.\"\n"
"cmd_pack.help_arguments = \"{<rockspec>|<name> [<version>]}\"\n"
"cmd_pack.help = [[\n"
"--sign     Produce a signature file as well.\n"
"\n"
"Argument may be a rockspec file, for creating a source rock,\n"
"or the name of an installed package, for creating a binary rock.\n"
"In the latter case, the app version may be given as a second\n"
"argument.\n"
"]]\n"
"\n"
"--- Driver function for the \"pack\" command.\n"
"-- @param arg string:  may be a rockspec file, for creating a source rock,\n"
"-- or the name of an installed package, for creating a binary rock.\n"
"-- @param version string or nil: if the name of a package is given, a\n"
"-- version may also be passed.\n"
"-- @return boolean or (nil, string): true if successful or nil followed\n"
"-- by an error message.\n"
"function cmd_pack.command(flags, arg, version)\n"
"   assert(type(version) == \"string\" or not version)\n"
"   if type(arg) ~= \"string\" then\n"
"      return nil, \"Argument missing. \"..util.see_help(\"pack\")\n"
"   end\n"
"\n"
"   local file, err\n"
"   if arg:match(\".*%.rockspec\") then\n"
"      file, err = pack.pack_source_rock(arg)\n"
"   else\n"
"      local name = util.adjust_name_and_namespace(arg, flags)\n"
"      local query = queries.new(name, version)\n"
"      file, err = pack.pack_installed_rock(query, flags[\"tree\"])\n"
"   end\n"
"   return pack.report_and_sign_local_file(file, err, flags[\"sign\"])\n"
"end\n"
"\n"
"return cmd_pack\n"
""
;
