const char luarocks_cmd_path_lua[] =
"\n"
"--- @module luarocks.path_cmd\n"
"-- Driver for the `luarocks path` command.\n"
"local path_cmd = {}\n"
"\n"
"local util = require(\"luarocks.util\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local fs = require(\"luarocks.fs\")\n"
"\n"
"path_cmd.help_summary = \"Return the currently configured package path.\"\n"
"path_cmd.help_arguments = \"\"\n"
"path_cmd.help = [[\n"
"Returns the package path currently configured for this installation\n"
"of LuaRocks, formatted as shell commands to update LUA_PATH and LUA_CPATH. \n"
"\n"
"--no-bin       Do not export the PATH variable\n"
"\n"
"--append       Appends the paths to the existing paths. Default is to prefix\n"
"               the LR paths to the existing paths.\n"
"\n"
"--lr-path      Exports the Lua path (not formatted as shell command)\n"
"\n"
"--lr-cpath     Exports the Lua cpath (not formatted as shell command)\n"
"\n"
"--lr-bin       Exports the system path (not formatted as shell command)\n"
"\n"
"\n"
"On Unix systems, you may run: \n"
"  eval `luarocks path`\n"
"And on Windows: \n"
"  luarocks path > \"%temp%\\_lrp.bat\" && call \"%temp%\\_lrp.bat\" && del \"%temp%\\_lrp.bat\"\n"
"]]\n"
"\n"
"--- Driver function for \"path\" command.\n"
"-- @return boolean This function always succeeds.\n"
"function path_cmd.command(flags)\n"
"   local lr_path, lr_cpath, lr_bin = cfg.package_paths(flags[\"tree\"])\n"
"   local path_sep = cfg.export_path_separator\n"
"\n"
"   if flags[\"lr-path\"] then\n"
"      util.printout(util.cleanup_path(lr_path, ';', cfg.lua_version))\n"
"      return true\n"
"   elseif flags[\"lr-cpath\"] then\n"
"      util.printout(util.cleanup_path(lr_cpath, ';', cfg.lua_version))\n"
"      return true\n"
"   elseif flags[\"lr-bin\"] then\n"
"      util.printout(util.cleanup_path(lr_bin, path_sep))\n"
"      return true\n"
"   end\n"
"   \n"
"   if flags[\"append\"] then\n"
"      lr_path = package.path .. \";\" .. lr_path\n"
"      lr_cpath = package.cpath .. \";\" .. lr_cpath\n"
"      lr_bin = os.getenv(\"PATH\") .. path_sep .. lr_bin\n"
"   else\n"
"      lr_path =  lr_path.. \";\" .. package.path\n"
"      lr_cpath = lr_cpath .. \";\" .. package.cpath\n"
"      lr_bin = lr_bin .. path_sep .. os.getenv(\"PATH\")\n"
"   end\n"
"   \n"
"   local lpath_var, lcpath_var = util.lua_path_variables()\n"
"\n"
"   util.printout(fs.export_cmd(lpath_var, util.cleanup_path(lr_path, ';', cfg.lua_version)))\n"
"   util.printout(fs.export_cmd(lcpath_var, util.cleanup_path(lr_cpath, ';', cfg.lua_version)))\n"
"   if not flags[\"no-bin\"] then\n"
"      util.printout(fs.export_cmd(\"PATH\", util.cleanup_path(lr_bin, path_sep)))\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"return path_cmd\n"
""
;
