const char luarocks_cmd_purge_lua[] =
"\n"
"--- Module implementing the LuaRocks \"purge\" command.\n"
"-- Remove all rocks from a given tree.\n"
"local purge = {}\n"
"\n"
"local util = require(\"luarocks.util\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local path = require(\"luarocks.path\")\n"
"local search = require(\"luarocks.search\")\n"
"local vers = require(\"luarocks.core.vers\")\n"
"local repos = require(\"luarocks.repos\")\n"
"local writer = require(\"luarocks.manif.writer\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local remove = require(\"luarocks.remove\")\n"
"local queries = require(\"luarocks.queries\")\n"
"local cmd = require(\"luarocks.cmd\")\n"
"\n"
"purge.help_summary = \"Remove all installed rocks from a tree.\"\n"
"purge.help_arguments = \"--tree=<tree> [--old-versions]\"\n"
"purge.help = [[\n"
"This command removes rocks en masse from a given tree.\n"
"By default, it removes all rocks from a tree.\n"
"\n"
"The --tree argument is mandatory: luarocks purge does not\n"
"assume a default tree.\n"
"\n"
"--old-versions  Keep the highest-numbered version of each\n"
"                rock and remove the other ones. By default\n"
"                it only removes old versions if they are\n"
"                not needed as dependencies. This can be\n"
"                overridden with the flag --force.\n"
"]]\n"
"\n"
"function purge.command(flags)\n"
"   local tree = flags[\"tree\"]\n"
"\n"
"   if type(tree) ~= \"string\" then\n"
"      return nil, \"The --tree argument is mandatory. \"..util.see_help(\"purge\")\n"
"   end\n"
"   \n"
"   local results = {}\n"
"   if not fs.is_dir(tree) then\n"
"      return nil, \"Directory not found: \"..tree\n"
"   end\n"
"\n"
"   local ok, err = fs.check_command_permissions(flags)\n"
"   if not ok then return nil, err, cmd.errorcodes.PERMISSIONDENIED end\n"
"\n"
"   search.local_manifest_search(results, path.rocks_dir(tree), queries.all())\n"
"\n"
"   local sort = function(a,b) return vers.compare_versions(b,a) end\n"
"   if flags[\"old-versions\"] then\n"
"      sort = vers.compare_versions\n"
"   end\n"
"\n"
"   for package, versions in util.sortedpairs(results) do\n"
"      for version, _ in util.sortedpairs(versions, sort) do\n"
"         if flags[\"old-versions\"] then\n"
"            util.printout(\"Keeping \"..package..\" \"..version..\"...\")\n"
"            local ok, err = remove.remove_other_versions(package, version, flags[\"force\"], flags[\"force-fast\"])\n"
"            if not ok then\n"
"               util.printerr(err)\n"
"            end\n"
"            break\n"
"         else\n"
"            util.printout(\"Removing \"..package..\" \"..version..\"...\")\n"
"            local ok, err = repos.delete_version(package, version, \"none\", true)\n"
"            if not ok then\n"
"               util.printerr(err)\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"   return writer.make_manifest(cfg.rocks_dir, \"one\")\n"
"end\n"
"\n"
"return purge\n"
""
;
