const char luarocks_cmd_remove_lua[] =
"\n"
"--- Module implementing the LuaRocks \"remove\" command.\n"
"-- Uninstalls rocks.\n"
"local cmd_remove = {}\n"
"\n"
"local remove = require(\"luarocks.remove\")\n"
"local util = require(\"luarocks.util\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local search = require(\"luarocks.search\")\n"
"local path = require(\"luarocks.path\")\n"
"local deps = require(\"luarocks.deps\")\n"
"local writer = require(\"luarocks.manif.writer\")\n"
"local queries = require(\"luarocks.queries\")\n"
"local cmd = require(\"luarocks.cmd\")\n"
"\n"
"cmd_remove.help_summary = \"Uninstall a rock.\"\n"
"cmd_remove.help_arguments = \"[--force|--force-fast] <name> [<version>]\"\n"
"cmd_remove.help = [[\n"
"Argument is the name of a rock to be uninstalled.\n"
"If a version is not given, try to remove all versions at once.\n"
"Will only perform the removal if it does not break dependencies.\n"
"To override this check and force the removal, use --force.\n"
"To perform a forced removal without reporting dependency issues,\n"
"use --force-fast.\n"
"\n"
"]]..util.deps_mode_help()\n"
"\n"
"--- Driver function for the \"remove\" command.\n"
"-- @param name string: name of a rock. If a version is given, refer to\n"
"-- a specific version; otherwise, try to remove all versions.\n"
"-- @param version string: When passing a package name, a version number\n"
"-- may also be given.\n"
"-- @return boolean or (nil, string, exitcode): True if removal was\n"
"-- successful, nil and an error message otherwise. exitcode is optionally returned.\n"
"function cmd_remove.command(flags, name, version)\n"
"   if type(name) ~= \"string\" then\n"
"      return nil, \"Argument missing. \"..util.see_help(\"remove\")\n"
"   end\n"
"\n"
"   name = util.adjust_name_and_namespace(name, flags)\n"
"   \n"
"   local deps_mode = flags[\"deps-mode\"] or cfg.deps_mode\n"
"   \n"
"   local ok, err = fs.check_command_permissions(flags)\n"
"   if not ok then return nil, err, cmd.errorcodes.PERMISSIONDENIED end\n"
"   \n"
"   local rock_type = name:match(\"%.(rock)$\") or name:match(\"%.(rockspec)$\")\n"
"   local filename = name\n"
"   if rock_type then\n"
"      name, version = path.parse_name(filename)\n"
"      if not name then return nil, \"Invalid \"..rock_type..\" filename: \"..filename end\n"
"   end\n"
"\n"
"   local results = {}\n"
"   name = name:lower()\n"
"   search.local_manifest_search(results, cfg.rocks_dir, queries.new(name, version))\n"
"   if not results[name] then\n"
"      return nil, \"Could not find rock '\"..name..(version and \" \"..version or \"\")..\"' in \"..path.rocks_tree_to_string(cfg.root_dir)\n"
"   end\n"
"\n"
"   local ok, err = remove.remove_search_results(results, name, deps_mode, flags[\"force\"], flags[\"force-fast\"])\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   writer.check_dependencies(nil, deps.get_deps_mode(flags))\n"
"   return true\n"
"end\n"
"\n"
"return cmd_remove\n"
""
;
