const char luarocks_cmd_search_lua[] =
"\n"
"--- Module implementing the LuaRocks \"search\" command.\n"
"-- Queries LuaRocks servers.\n"
"local cmd_search = {}\n"
"\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local util = require(\"luarocks.util\")\n"
"local search = require(\"luarocks.search\")\n"
"local queries = require(\"luarocks.queries\")\n"
"local results = require(\"luarocks.results\")\n"
"\n"
"cmd_search.help_summary = \"Query the LuaRocks servers.\"\n"
"cmd_search.help_arguments = \"[--source] [--binary] { <name> [<version>] | --all }\"\n"
"cmd_search.help = [[\n"
"--source     Return only rockspecs and source rocks,\n"
"             to be used with the \"build\" command.\n"
"--binary     Return only pure Lua and binary rocks (rocks that can be used\n"
"             with the \"install\" command without requiring a C toolchain).\n"
"--all        List all contents of the server that are suitable to\n"
"             this platform, do not filter by name.\n"
"--porcelain  Return a machine readable format.\n"
"]]\n"
"\n"
"--- Splits a list of search results into two lists, one for \"source\" results\n"
"-- to be used with the \"build\" command, and one for \"binary\" results to be\n"
"-- used with the \"install\" command.\n"
"-- @param result_tree table: A search results table.\n"
"-- @return (table, table): Two tables, one for source and one for binary\n"
"-- results.\n"
"local function split_source_and_binary_results(result_tree)\n"
"   local sources, binaries = {}, {}\n"
"   for name, versions in pairs(result_tree) do\n"
"      for version, repositories in pairs(versions) do\n"
"         for _, repo in ipairs(repositories) do\n"
"            local where = sources\n"
"            if repo.arch == \"all\" or repo.arch == cfg.arch then\n"
"               where = binaries\n"
"            end\n"
"            local entry = results.new(name, version, repo.repo, repo.arch)\n"
"            search.store_result(where, entry)\n"
"         end\n"
"      end\n"
"   end\n"
"   return sources, binaries\n"
"end\n"
"\n"
"--- Driver function for \"search\" command.\n"
"-- @param name string: A substring of a rock name to search.\n"
"-- @param version string or nil: a version may also be passed.\n"
"-- @return boolean or (nil, string): True if build was successful; nil and an\n"
"-- error message otherwise.\n"
"function cmd_search.command(flags, name, version)\n"
"\n"
"   name = util.adjust_name_and_namespace(name, flags)\n"
"\n"
"   if flags[\"all\"] then\n"
"      name, version = \"\", nil\n"
"   end\n"
"\n"
"   if type(name) ~= \"string\" and not flags[\"all\"] then\n"
"      return nil, \"Enter name and version or use --all. \"..util.see_help(\"search\")\n"
"   end\n"
"   \n"
"   local query = queries.new(name:lower(), version, true)\n"
"   local result_tree, err = search.search_repos(query)\n"
"   local porcelain = flags[\"porcelain\"]\n"
"   local full_name = name .. (version and \" \" .. version or \"\")\n"
"   util.title(full_name .. \" - Search results for Lua \"..cfg.lua_version..\":\", porcelain, \"=\")\n"
"   local sources, binaries = split_source_and_binary_results(result_tree)\n"
"   if next(sources) and not flags[\"binary\"] then\n"
"      util.title(\"Rockspecs and source rocks:\", porcelain)\n"
"      search.print_result_tree(sources, porcelain)\n"
"   end\n"
"   if next(binaries) and not flags[\"source\"] then    \n"
"      util.title(\"Binary and pure-Lua rocks:\", porcelain)\n"
"      search.print_result_tree(binaries, porcelain)\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"return cmd_search\n"
""
;
