const char luarocks_cmd_show_lua[] =
"--- Module implementing the LuaRocks \"show\" command.\n"
"-- Shows information about an installed rock.\n"
"local show = {}\n"
"\n"
"local queries = require(\"luarocks.queries\")\n"
"local search = require(\"luarocks.search\")\n"
"local dir = require(\"luarocks.core.dir\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local util = require(\"luarocks.util\")\n"
"local path = require(\"luarocks.path\")\n"
"local fetch = require(\"luarocks.fetch\")\n"
"local manif = require(\"luarocks.manif\")\n"
"local repos = require(\"luarocks.repos\")\n"
"\n"
"show.help_summary = \"Show information about an installed rock.\"\n"
"\n"
"show.help = [[\n"
"<argument> is an existing package name.\n"
"Without any flags, show all module information.\n"
"With these flags, return only the desired information:\n"
"\n"
"--home       home page of project\n"
"--modules    all modules provided by this package as used by require()\n"
"--deps       packages this package depends on\n"
"--build-deps build-only dependencies for this package\n"
"--test-deps  dependencies for testing this package\n"
"--rockspec   the full path of the rockspec file\n"
"--mversion   the package version\n"
"--rock-tree  local tree where rock is installed\n"
"--rock-dir   data directory of the installed rock\n"
"]]\n"
"\n"
"\n"
"local friendly_template = [[\n"
"          :\n"
"\?namespace:${namespace}/${package} ${version} - ${summary}\n"
"!namespace:${package} ${version} - ${summary}\n"
"          :\n"
"*detailed :${detailed}\n"
"\?detailed :\n"
"\?license  :License:      \\t${license}\n"
"\?homepage :Homepage:     \\t${homepage}\n"
"\?issues   :Issues:       \\t${issues}\n"
"\?labels   :Labels:       \\t${labels}\n"
"\?location :Installed in: \\t${location}\n"
"\?commands :\n"
"\?commands :Commands:\n"
"*commands :\\t${name} (${file})\n"
"\?modules  :\n"
"\?modules  :Modules:\n"
"*modules  :\\t${name} (${file})\n"
"\?bdeps    :\n"
"\?bdeps    :Has build dependency on:\n"
"*bdeps    :\\t${name} (${label})\n"
"\?tdeps    :\n"
"\?tdeps    :Tests depend on:\n"
"*tdeps    :\\t${name} (${label})\n"
"\?deps     :\n"
"\?deps     :Depends on:\n"
"*deps     :\\t${name} (${label})\n"
"\?ideps    :\n"
"\?ideps    :Indirectly pulling:\n"
"*ideps    :\\t${name} (${label})\n"
"          :\n"
"]]\n"
"\n"
"local porcelain_template = [[\n"
"\?namespace:namespace\\t${namespace}\n"
"\?package  :package\\t${package}\n"
"\?version  :version\\t${version}\n"
"\?summary  :summary\\t${summary}\n"
"*detailed :detailed\\t${detailed}\n"
"\?license  :license\\t${license}\n"
"\?homepage :homepage\\t${homepage}\n"
"\?issues   :issues\\t${issues}\n"
"\?labels   :labels\\t${labels}\n"
"\?location :location\\t${location}\n"
"*commands :command\\t${name}\\t${file}\n"
"*modules  :module\\t${name}\\t${file}\n"
"*bdeps    :build_dependency\\t${name}\\t${label}\n"
"*tdeps    :test_dependency\\t${name}\\t${label}\n"
"*deps     :dependency\\t${name}\\t${label}\n"
"*ideps    :indirect_dependency\\t${name}\\t${label}\n"
"]]\n"
"\n"
"local function keys_as_string(t, sep)\n"
"   local keys = util.keys(t)\n"
"   table.sort(keys)\n"
"   return table.concat(keys, sep or \" \")\n"
"end\n"
"\n"
"local function word_wrap(line) \n"
"   local width = tonumber(os.getenv(\"COLUMNS\")) or 80\n"
"   if width > 80 then width = 80 end\n"
"   if #line > width then\n"
"      local brk = width\n"
"      while brk > 0 and line:sub(brk, brk) ~= \" \" do\n"
"         brk = brk - 1\n"
"      end\n"
"      if brk > 0 then\n"
"         return line:sub(1, brk-1) .. \"\\n\" .. word_wrap(line:sub(brk+1))\n"
"      end\n"
"   end\n"
"   return line\n"
"end\n"
"\n"
"local function format_text(text)\n"
"   text = text:gsub(\"^%s*\",\"\"):gsub(\"%s$\", \"\"):gsub(\"\\n[ \\t]+\",\"\\n\"):gsub(\"([^\\n])\\n([^\\n])\",\"%1 %2\")\n"
"   local paragraphs = util.split_string(text, \"\\n\\n\")\n"
"   for n, line in ipairs(paragraphs) do\n"
"      paragraphs[n] = word_wrap(line)\n"
"   end\n"
"   return (table.concat(paragraphs, \"\\n\\n\"):gsub(\"%s$\", \"\"))\n"
"end\n"
"\n"
"local function installed_rock_label(dep, tree)\n"
"   local installed, version\n"
"   if cfg.rocks_provided[dep.name] then\n"
"      installed, version = true, cfg.rocks_provided[dep.name]\n"
"   else\n"
"      installed, version = search.pick_installed_rock(dep, tree)\n"
"   end\n"
"   return installed and \"using \"..version or \"missing\"\n"
"end\n"
"\n"
"local function render(template, data)\n"
"   local out = {}\n"
"   for cmd, var, line in template:gmatch(\"(.)([a-z]*)%s*:([^\\n]*)\\n\") do\n"
"      line = line:gsub(\"\\\\t\", \"\\t\")\n"
"      local d = data[var]\n"
"      if cmd == \" \" then\n"
"         table.insert(out, line)\n"
"      elseif cmd == \"\?\" or cmd == \"*\" or cmd == \"!\" then\n"
"         if (cmd == \"!\" and d == nil)\n"
"             or (cmd ~= \"!\" and (type(d) == \"string\"\n"
"                                 or (type(d) == \"table\" and next(d)))) then\n"
"            local n = cmd == \"*\" and #d or 1\n"
"            for i = 1, n do\n"
"               local tbl = cmd == \"*\" and d[i] or data\n"
"               table.insert(out, (line:gsub(\"${([a-z]+)}\", tbl)))\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"   return table.concat(out, \"\\n\")\n"
"end\n"
"\n"
"local function adjust_path(name, version, basedir, pathname, suffix)\n"
"   pathname = dir.path(basedir, pathname)\n"
"   local vpathname = path.versioned_name(pathname, basedir, name, version)\n"
"   return (fs.exists(vpathname)\n"
"          and vpathname\n"
"          or pathname) .. (suffix or \"\")\n"
"end\n"
"\n"
"local function modules_to_list(name, version, repo)\n"
"   local ret = {}\n"
"   local rock_manifest = manif.load_rock_manifest(name, version, repo)\n"
"\n"
"   local lua_dir = path.deploy_lua_dir(repo)\n"
"   local lib_dir = path.deploy_lib_dir(repo)\n"
"   repos.recurse_rock_manifest_entry(rock_manifest.lua, function(pathname)\n"
"      table.insert(ret, {\n"
"         name = path.path_to_module(pathname),\n"
"         file = adjust_path(name, version, lua_dir, pathname),\n"
"      })\n"
"   end)\n"
"   repos.recurse_rock_manifest_entry(rock_manifest.lib, function(pathname)\n"
"      table.insert(ret, {\n"
"         name = path.path_to_module(pathname),\n"
"         file = adjust_path(name, version, lib_dir, pathname),\n"
"      })\n"
"   end)\n"
"   table.sort(ret, function(a, b)\n"
"      if a.name == b.name then\n"
"         return a.file < b.file\n"
"      end\n"
"      return a.name < b.name\n"
"   end)\n"
"   return ret\n"
"end\n"
"\n"
"local function commands_to_list(name, version, repo)\n"
"   local ret = {}\n"
"   local rock_manifest = manif.load_rock_manifest(name, version, repo)\n"
"\n"
"   local bin_dir = path.deploy_bin_dir(repo)\n"
"   repos.recurse_rock_manifest_entry(rock_manifest.bin, function(pathname)\n"
"      pathname = adjust_path(name, version, bin_dir, pathname)\n"
"      table.insert(ret, {\n"
"         name = pathname,\n"
"         file = adjust_path(name, version, bin_dir, pathname, cfg.wrapper_suffix),\n"
"      })\n"
"   end)\n"
"   table.sort(ret, function(a, b)\n"
"      if a.name == b.name then\n"
"         return a.file < b.file\n"
"      end\n"
"      return a.name < b.name\n"
"   end)\n"
"   return ret\n"
"end\n"
"\n"
"local function deps_to_list(dependencies, tree)\n"
"   local ret = {}\n"
"   for _, dep in ipairs(dependencies or {}) do\n"
"      table.insert(ret, { name = tostring(dep), label = installed_rock_label(dep, tree) })\n"
"   end\n"
"   return ret\n"
"end\n"
"\n"
"local function indirect_deps(mdeps, rdeps, tree)\n"
"   local ret = {}\n"
"   local direct_deps = {}\n"
"   for _, dep in ipairs(rdeps) do\n"
"      direct_deps[dep] = true\n"
"   end\n"
"   for dep_name in util.sortedpairs(mdeps or {}) do\n"
"      if not direct_deps[dep_name] then\n"
"         table.insert(ret, { name = tostring(dep_name), label = installed_rock_label(queries.new(dep_name), tree) })\n"
"      end\n"
"   end\n"
"   return ret\n"
"end\n"
"\n"
"local function show_rock(template, namespace, name, version, rockspec, repo, minfo, tree)\n"
"   local desc = rockspec.description or {}\n"
"   local data = {\n"
"      namespace = namespace,\n"
"      package = rockspec.package,\n"
"      version = rockspec.version,\n"
"      summary = desc.summary or \"\",\n"
"      detailed = desc.detailed and util.split_string(format_text(desc.detailed), \"\\n\"),\n"
"      license = desc.license,\n"
"      homepage = desc.homepage,\n"
"      issues = desc.issues_url,\n"
"      labels = desc.labels and table.concat(desc.labels, \", \"),\n"
"      location = path.rocks_tree_to_string(repo),\n"
"      commands = commands_to_list(name, version, repo),\n"
"      modules = modules_to_list(name, version, repo),\n"
"      bdeps = deps_to_list(rockspec.build_dependencies, tree),\n"
"      tdeps = deps_to_list(rockspec.test_dependencies, tree),\n"
"      deps = deps_to_list(rockspec.dependencies, tree),\n"
"      ideps = indirect_deps(minfo.dependencies, rockspec.dependencies, tree),\n"
"   }\n"
"   util.printout(render(template, data))\n"
"end\n"
"\n"
"--- Driver function for \"show\" command.\n"
"-- @param name or nil: an existing package name.\n"
"-- @param version string or nil: a version may also be passed.\n"
"-- @return boolean: True if succeeded, nil on errors.\n"
"function show.command(flags, name, version)\n"
"   if not name then\n"
"      return nil, \"Argument missing. \"..util.see_help(\"show\")\n"
"   end\n"
"\n"
"   name = util.adjust_name_and_namespace(name, flags)\n"
"   local query = queries.new(name, version)\n"
"   \n"
"   local repo, repo_url\n"
"   name, version, repo, repo_url = search.pick_installed_rock(query, flags[\"tree\"])\n"
"   if not name then\n"
"      return nil, version\n"
"   end\n"
"   local tree = path.rocks_tree_to_string(repo)\n"
"   local directory = path.install_dir(name, version, repo)\n"
"   local namespace = path.read_namespace(name, version, tree)\n"
"   local rockspec_file = path.rockspec_file(name, version, repo)\n"
"   local rockspec, err = fetch.load_local_rockspec(rockspec_file)\n"
"   if not rockspec then return nil,err end\n"
"\n"
"   local descript = rockspec.description or {}\n"
"   local manifest, err = manif.load_manifest(repo_url)\n"
"   if not manifest then return nil,err end\n"
"   local minfo = manifest.repository[name][version][1]\n"
"\n"
"   if flags[\"rock-tree\"] then util.printout(tree)\n"
"   elseif flags[\"rock-namespace\"] then util.printout(namespace)\n"
"   elseif flags[\"rock-dir\"] then util.printout(directory)\n"
"   elseif flags[\"home\"] then util.printout(descript.homepage)\n"
"   elseif flags[\"rock-license\"] then util.printout(descript.license)\n"
"   elseif flags[\"issues\"] then util.printout(descript.issues_url)\n"
"   elseif flags[\"labels\"] then util.printout(descript.labels and table.concat(descript.labels, \"\\n\"))\n"
"   elseif flags[\"modules\"] then util.printout(keys_as_string(minfo.modules, \"\\n\"))\n"
"   elseif flags[\"deps\"] then\n"
"      for _, dep in ipairs(rockspec.dependencies) do\n"
"         util.printout(tostring(dep))\n"
"      end\n"
"   elseif flags[\"build-deps\"] then\n"
"      for _, dep in ipairs(rockspec.build_dependencies) do\n"
"         util.printout(tostring(dep))\n"
"      end\n"
"   elseif flags[\"test-deps\"] then\n"
"      for _, dep in ipairs(rockspec.test_dependencies) do\n"
"         util.printout(tostring(dep))\n"
"      end\n"
"   elseif flags[\"rockspec\"] then util.printout(rockspec_file)\n"
"   elseif flags[\"mversion\"] then util.printout(version)\n"
"   elseif flags[\"porcelain\"] then\n"
"      show_rock(porcelain_template, namespace, name, version, rockspec, repo, minfo, flags[\"tree\"])\n"
"   else\n"
"      show_rock(friendly_template, namespace, name, version, rockspec, repo, minfo, flags[\"tree\"])\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"return show\n"
""
;
