const char luarocks_cmd_upload_lua[] =
"\n"
"local upload = {}\n"
"\n"
"local signing = require(\"luarocks.signing\")\n"
"local util = require(\"luarocks.util\")\n"
"local fetch = require(\"luarocks.fetch\")\n"
"local pack = require(\"luarocks.pack\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local Api = require(\"luarocks.upload.api\")\n"
"\n"
"upload.help_summary = \"Upload a rockspec to the public rocks repository.\"\n"
"upload.help_arguments = \"[--skip-pack] [--api-key=<key>] [--force] <rockspec>\"\n"
"upload.help = [[\n"
"<rockspec>       Pack a source rock file (.src.rock extension),\n"
"                 upload rockspec and source rock to server.\n"
"\n"
"--skip-pack      Do not pack and send source rock.\n"
"\n"
"--api-key=<key>  Give it an API key. It will be stored for subsequent uses.\n"
"\n"
"--temp-key=<key> Use the given a temporary API key in this invocation only.\n"
"                 It will not be stored.\n"
"\n"
"--force          Replace existing rockspec if the same revision of\n"
"                 a module already exists. This should be used only \n"
"                 in case of upload mistakes: when updating a rockspec,\n"
"                 increment the revision number instead.\n"
"\n"
"--sign           Upload a signature file alongside each file as well.\n"
"]]\n"
"\n"
"local function is_dev_version(version)\n"
"   return version:match(\"^dev\") or version:match(\"^scm\")\n"
"end\n"
"\n"
"function upload.command(flags, fname)\n"
"   if not fname then\n"
"      return nil, \"Missing rockspec. \"..util.see_help(\"upload\")\n"
"   end\n"
"\n"
"   local api, err = Api.new(flags)\n"
"   if not api then\n"
"      return nil, err\n"
"   end\n"
"   if cfg.verbose then\n"
"      api.debug = true\n"
"   end\n"
"\n"
"   local rockspec, err, errcode = fetch.load_rockspec(fname)\n"
"   if err then\n"
"      return nil, err, errcode\n"
"   end\n"
"\n"
"   util.printout(\"Sending \" .. tostring(fname) .. \" ...\")\n"
"   local res, err = api:method(\"check_rockspec\", {\n"
"      package = rockspec.package,\n"
"      version = rockspec.version\n"
"   })\n"
"   if not res then return nil, err end\n"
"   \n"
"   if not res.module then\n"
"      util.printout(\"Will create new module (\" .. tostring(rockspec.package) .. \")\")\n"
"   end\n"
"   if res.version and not flags[\"force\"] then\n"
"      return nil, \"Revision \"..rockspec.version..\" already exists on the server. \"..util.see_help(\"upload\")\n"
"   end\n"
"\n"
"   local sigfname\n"
"   local rock_sigfname\n"
"\n"
"   if flags[\"sign\"] then\n"
"      sigfname, err = signing.sign_file(fname)\n"
"      if err then\n"
"         return nil, \"Failed signing rockspec: \" .. err\n"
"      end\n"
"      util.printout(\"Signed rockspec: \"..sigfname)\n"
"   end\n"
"\n"
"   local rock_fname\n"
"   if not flags[\"skip-pack\"] and not is_dev_version(rockspec.version) then\n"
"      util.printout(\"Packing \" .. tostring(rockspec.package))\n"
"      rock_fname, err = pack.pack_source_rock(fname)\n"
"      if not rock_fname then\n"
"         return nil, err\n"
"      end\n"
"      if flags[\"sign\"] then\n"
"         rock_sigfname, err = signing.sign_file(rock_fname)\n"
"         if err then\n"
"            return nil, \"Failed signing rock: \" .. err\n"
"         end\n"
"         util.printout(\"Signed packed rock: \"..rock_sigfname)\n"
"      end\n"
"   end\n"
"   \n"
"   local multipart = require(\"luarocks.upload.multipart\")\n"
"\n"
"   res, err = api:method(\"upload\", nil, {\n"
"     rockspec_file = multipart.new_file(fname),\n"
"     rockspec_sig = sigfname and multipart.new_file(sigfname),\n"
"   })\n"
"   if not res then return nil, err end\n"
"   \n"
"   if res.is_new and #res.manifests == 0 then\n"
"      util.printerr(\"Warning: module not added to root manifest due to name taken.\")\n"
"   end\n"
"   \n"
"   local module_url = res.module_url\n"
"   \n"
"   if rock_fname then\n"
"      if (not res.version) or (not res.version.id) then\n"
"         return nil, \"Invalid response from server.\"\n"
"      end\n"
"      util.printout((\"Sending \" .. tostring(rock_fname) .. \" ...\"))\n"
"      res, err = api:method(\"upload_rock/\" .. (\"%d\"):format(res.version.id), nil, {\n"
"         rock_file = multipart.new_file(rock_fname),\n"
"         rock_sig = rock_sigfname and multipart.new_file(rock_sigfname),\n"
"      })\n"
"      if not res then return nil, err end\n"
"   end\n"
"   \n"
"   util.printout()\n"
"   util.printout(\"Done: \" .. tostring(module_url))\n"
"   util.printout()\n"
"   return true\n"
"end\n"
"\n"
"return upload\n"
""
;
