const char luarocks_cmd_write_rockspec_lua[] =
"\n"
"local write_rockspec = {}\n"
"\n"
"local builtin = require(\"luarocks.build.builtin\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local fetch = require(\"luarocks.fetch\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local persist = require(\"luarocks.persist\")\n"
"local rockspecs = require(\"luarocks.rockspecs\")\n"
"local type_rockspec = require(\"luarocks.type.rockspec\")\n"
"local util = require(\"luarocks.util\")\n"
"\n"
"write_rockspec.help_summary = \"Write a template for a rockspec file.\"\n"
"write_rockspec.help_arguments = \"[--output=<file> ...] [<name>] [<version>] [<url>|<path>]\"\n"
"write_rockspec.help = [[\n"
"This command writes an initial version of a rockspec file,\n"
"based on a name, a version, and a location (an URL or a local path).\n"
"If only two arguments are given, the first one is considered the name and the\n"
"second one is the location.\n"
"If only one argument is given, it must be the location.\n"
"If no arguments are given, current directory is used as location.\n"
"LuaRocks will attempt to infer name and version if not given,\n"
"using 'dev' as a fallback default version.\n"
"\n"
"Note that the generated file is a _starting point_ for writing a\n"
"rockspec, and is not guaranteed to be complete or correct.\n"
"\n"
"--output=<file>          Write the rockspec with the given filename.\n"
"                         If not given, a file is written in the current\n"
"                         directory with a filename based on given name and version.\n"
"--license=\"<string>\"     A license string, such as \"MIT/X11\" or \"GNU GPL v3\".\n"
"--summary=\"<txt>\"        A short one-line description summary.\n"
"--detailed=\"<txt>\"       A longer description string.\n"
"--homepage=<url>         Project homepage.\n"
"--lua-versions=<ver>     Supported Lua versions.  Accepted values are: \"5.1\", \"5.2\",\n"
"                         \"5.3\", \"5.4\", \"5.1,5.2\", \"5.2,5.3\", \"5.3,5.4\", \"5.1,5.2,5.3\",\n"
"                         \"5.2,5.3,5.4\", or \"5.1,5.2,5.3,5.4\"\n"
"--rockspec-format=<ver>  Rockspec format version, such as \"1.0\" or \"1.1\".\n"
"--tag=<tag>              Tag to use. Will attempt to extract version number from it.\n"
"--lib=<lib>[,<lib>]      A comma-separated list of libraries that C files need to\n"
"                         link to.\n"
"]]\n"
"\n"
"local function open_file(name)\n"
"   return io.open(dir.path(fs.current_dir(), name), \"r\")\n"
"end\n"
"\n"
"local function fetch_url(rockspec)\n"
"   local file, temp_dir, err_code, err_file, err_temp_dir = fetch.fetch_sources(rockspec, false)\n"
"   if err_code == \"source.dir\" then\n"
"      file, temp_dir = err_file, err_temp_dir\n"
"   elseif not file then\n"
"      util.warning(\"Could not fetch sources - \"..temp_dir)\n"
"      return false\n"
"   end\n"
"   util.printout(\"File successfully downloaded. Making checksum and checking base dir...\")\n"
"   if dir.is_basic_protocol(rockspec.source.protocol) then\n"
"      rockspec.source.md5 = fs.get_md5(file)\n"
"   end\n"
"   local inferred_dir, found_dir = fetch.find_base_dir(file, temp_dir, rockspec.source.url)\n"
"   return true, found_dir or inferred_dir, temp_dir\n"
"end\n"
"\n"
"local lua_version_dep = {\n"
"   [\"5.1\"] = \"lua ~> 5.1\",\n"
"   [\"5.2\"] = \"lua ~> 5.2\",\n"
"   [\"5.3\"] = \"lua ~> 5.3\",\n"
"   [\"5.4\"] = \"lua ~> 5.4\",\n"
"   [\"5.1,5.2\"] = \"lua >= 5.1, < 5.3\",\n"
"   [\"5.2,5.3\"] = \"lua >= 5.2, < 5.4\",\n"
"   [\"5.3,5.4\"] = \"lua >= 5.3, < 5.5\",\n"
"   [\"5.1,5.2,5.3\"] = \"lua >= 5.1, < 5.4\",\n"
"   [\"5.2,5.3,5.4\"] = \"lua >= 5.2, < 5.5\",\n"
"   [\"5.1,5.2,5.3,5.4\"] = \"lua >= 5.1, < 5.5\",\n"
"}\n"
"\n"
"local simple_scm_protocols = {\n"
"   git = true,\n"
"   [\"git+http\"] = true,\n"
"   [\"git+https\"] = true,\n"
"   [\"git+ssh\"] = true,\n"
"   hg = true,\n"
"   [\"hg+http\"] = true,\n"
"   [\"hg+https\"] = true,\n"
"   [\"hg+ssh\"] = true,\n"
"}\n"
"\n"
"local detect_url\n"
"do\n"
"   local function detect_url_from_command(program, args, directory)\n"
"      local command = fs.Q(cfg.variables[program:upper()]).. \" \"..args\n"
"      local pipe = io.popen(fs.command_at(directory, fs.quiet_stderr(command)))\n"
"      if not pipe then return nil end\n"
"      local url = pipe:read(\"*a\"):match(\"^([^\\r\\n]+)\")\n"
"      pipe:close()\n"
"      if not url then return nil end\n"
"      if url:match(\"^[^@:/]+@[^@:/]+:.*$\") then\n"
"         local u, h, p = url:match(\"^([^@]+)@([^:]+):(.*)$\")\n"
"         url = program..\"+ssh://\"..u..\"@\"..h..\"/\"..p\n"
"      elseif not util.starts_with(url, program..\"://\") then\n"
"         url = program..\"+\"..url\n"
"      end\n"
"\n"
"      if simple_scm_protocols[dir.split_url(url)] then\n"
"         return url\n"
"      end\n"
"   end\n"
"   \n"
"   local function detect_scm_url(directory)\n"
"      return detect_url_from_command(\"git\", \"config --get remote.origin.url\", directory) or\n"
"         detect_url_from_command(\"hg\", \"paths default\", directory)\n"
"   end\n"
"\n"
"   detect_url = function(url_or_dir)\n"
"      if url_or_dir:match(\"://\") then\n"
"         return url_or_dir\n"
"      else\n"
"         return detect_scm_url(url_or_dir) or \"*** please add URL for source tarball, zip or repository here ***\"\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"local function detect_homepage(url, homepage)\n"
"   if homepage then\n"
"      return homepage\n"
"   end\n"
"   local url_protocol, url_path = dir.split_url(url)\n"
"\n"
"   if simple_scm_protocols[url_protocol] then\n"
"      for _, domain in ipairs({\"github.com\", \"bitbucket.org\", \"gitlab.com\"}) do\n"
"         if util.starts_with(url_path, domain) then\n"
"            return \"https://\"..url_path:gsub(\"%.git$\", \"\")\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   return \"*** please enter a project homepage ***\"\n"
"end\n"
"\n"
"local function detect_description()\n"
"   local fd = open_file(\"README.md\") or open_file(\"README\")\n"
"   if not fd then return end\n"
"   local data = fd:read(\"*a\")\n"
"   fd:close()\n"
"   local paragraph = data:match(\"\\n\\n([^%[].-)\\n\\n\")\n"
"   if not paragraph then paragraph = data:match(\"\\n\\n(.*)\") end\n"
"   local summary, detailed\n"
"   if paragraph then\n"
"      detailed = paragraph\n"
"\n"
"      if #paragraph < 80 then\n"
"         summary = paragraph:gsub(\"\\n\", \"\")\n"
"      else\n"
"         summary = paragraph:gsub(\"\\n\", \" \"):match(\"([^.]*%.) \")\n"
"      end\n"
"   end\n"
"   return summary, detailed\n"
"end\n"
"\n"
"local licenses = {\n"
"   [78656] = \"MIT\",\n"
"   [49311] = \"ISC\",\n"
"}\n"
"\n"
"local function detect_license(data)\n"
"   local strip_copyright = (data:gsub(\"^Copyright [^\\n]*\\n\", \"\"))\n"
"   local sum = 0\n"
"   for i = 1, #strip_copyright do\n"
"      local num = string.byte(strip_copyright:sub(i,i))\n"
"      if num > 32 and num <= 128 then\n"
"         sum = sum + num\n"
"      end\n"
"   end\n"
"   return licenses[sum]\n"
"end\n"
"\n"
"local function check_license()\n"
"   local fd = open_file(\"COPYING\") or open_file(\"LICENSE\") or open_file(\"MIT-LICENSE.txt\")\n"
"   if not fd then return nil end\n"
"   local data = fd:read(\"*a\")\n"
"   fd:close()\n"
"   local license = detect_license(data)\n"
"   if license then\n"
"      return license, data\n"
"   end\n"
"   return nil, data\n"
"end\n"
"\n"
"local function fill_as_builtin(rockspec, libs)\n"
"   rockspec.build.type = \"builtin\"\n"
"\n"
"   local incdirs, libdirs\n"
"   if libs then\n"
"      incdirs, libdirs = {}, {}\n"
"      for _, lib in ipairs(libs) do\n"
"         local upper = lib:upper()\n"
"         incdirs[#incdirs+1] = \"$(\"..upper..\"_INCDIR)\"\n"
"         libdirs[#libdirs+1] = \"$(\"..upper..\"_LIBDIR)\"\n"
"      end\n"
"   end\n"
"   \n"
"   rockspec.build.modules, rockspec.build.install, rockspec.build.copy_directories = builtin.autodetect_modules(libs, incdirs, libdirs)\n"
"end\n"
"\n"
"local function rockspec_cleanup(rockspec)\n"
"   rockspec.source.file = nil\n"
"   rockspec.source.protocol = nil\n"
"   rockspec.source.identifier = nil\n"
"   rockspec.source.dir = nil\n"
"   rockspec.source.dir_set = nil\n"
"   rockspec.source.pathname = nil\n"
"   rockspec.variables = nil\n"
"   rockspec.name = nil\n"
"   rockspec.format_is_at_least = nil\n"
"   rockspec.local_abs_filename = nil\n"
"   rockspec.rocks_provided = nil\n"
"   for _, list in ipairs({\"dependencies\", \"build_dependencies\", \"test_dependencies\"}) do\n"
"      if rockspec[list] and not next(rockspec[list]) then\n"
"         rockspec[list] = nil\n"
"      end\n"
"   end\n"
"   for _, list in ipairs({\"dependencies\", \"build_dependencies\", \"test_dependencies\"}) do\n"
"      if rockspec[list] then\n"
"         for i, entry in ipairs(rockspec[list]) do\n"
"            rockspec[list][i] = tostring(entry)\n"
"         end\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"function write_rockspec.command(flags, name, version, url_or_dir)\n"
"\n"
"   name = util.adjust_name_and_namespace(name, flags)\n"
"\n"
"   if not name then\n"
"      url_or_dir = \".\"\n"
"   elseif not version then\n"
"      url_or_dir = name\n"
"      name = nil\n"
"   elseif not url_or_dir then\n"
"      url_or_dir = version\n"
"      version = nil\n"
"   end\n"
"\n"
"   if flags[\"tag\"] then\n"
"      if not version then\n"
"         version = flags[\"tag\"]:gsub(\"^v\", \"\")\n"
"      end\n"
"   end\n"
"\n"
"   local protocol, pathname = dir.split_url(url_or_dir)\n"
"   if protocol == \"file\" then\n"
"      if pathname == \".\" then\n"
"         name = name or dir.base_name(fs.current_dir())\n"
"      end\n"
"   elseif dir.is_basic_protocol(protocol) then\n"
"      local filename = dir.base_name(url_or_dir)\n"
"      local newname, newversion = filename:match(\"(.*)-([^-]+)\")\n"
"      if newname then\n"
"         name = name or newname\n"
"         version = version or newversion:gsub(\"%.[a-z]+$\", \"\"):gsub(\"%.tar$\", \"\")\n"
"      end\n"
"   else\n"
"      name = name or dir.base_name(url_or_dir):gsub(\"%.[^.]+$\", \"\")\n"
"   end\n"
"\n"
"   if not name then\n"
"      return nil, \"Could not infer rock name. \"..util.see_help(\"write_rockspec\")\n"
"   end\n"
"   version = version or \"dev\"\n"
"\n"
"   local filename = flags[\"output\"] or dir.path(fs.current_dir(), name:lower()..\"-\"..version..\"-1.rockspec\")\n"
"   \n"
"   local url = detect_url(url_or_dir)\n"
"   local homepage = detect_homepage(url, flags[\"homepage\"])\n"
"   \n"
"   local rockspec, err = rockspecs.from_persisted_table(filename, {\n"
"      rockspec_format = flags[\"rockspec-format\"],\n"
"      package = name,\n"
"      version = version..\"-1\",\n"
"      source = {\n"
"         url = url,\n"
"         tag = flags[\"tag\"],\n"
"      },\n"
"      description = {\n"
"         summary = flags[\"summary\"] or \"*** please specify description summary ***\",\n"
"         detailed = flags[\"detailed\"] or \"*** please enter a detailed description ***\",\n"
"         homepage = homepage,\n"
"         license = flags[\"license\"] or \"*** please specify a license ***\",\n"
"      },\n"
"      dependencies = {\n"
"         lua_version_dep[flags[\"lua-versions\"]],\n"
"      },\n"
"      build = {},\n"
"   })\n"
"   assert(not err, err)\n"
"   rockspec.source.protocol = protocol\n"
"   \n"
"   if not next(rockspec.dependencies) then\n"
"      util.warning(\"Please specify supported Lua versions with --lua-versions=<ver>. \"..util.see_help(\"write_rockspec\"))\n"
"   end\n"
"   \n"
"   local local_dir = url_or_dir\n"
"\n"
"   if url_or_dir:match(\"://\") then\n"
"      rockspec.source.file = dir.base_name(url_or_dir)\n"
"      if not dir.is_basic_protocol(rockspec.source.protocol) then\n"
"         if version ~= \"dev\" then\n"
"            rockspec.source.tag = flags[\"tag\"] or \"v\" .. version\n"
"         end\n"
"      end\n"
"      rockspec.source.dir = nil\n"
"      local ok, base_dir, temp_dir = fetch_url(rockspec)\n"
"      if ok then\n"
"         if base_dir ~= dir.base_name(url_or_dir) then\n"
"            rockspec.source.dir = base_dir\n"
"         end\n"
"      end\n"
"      if base_dir then\n"
"         local_dir = dir.path(temp_dir, base_dir)\n"
"      else\n"
"         local_dir = nil\n"
"      end\n"
"   end\n"
"   \n"
"   if not local_dir then\n"
"      local_dir = \".\"\n"
"   end\n"
"   \n"
"   local libs = nil\n"
"   if flags[\"lib\"] then\n"
"      libs = {}\n"
"      rockspec.external_dependencies = {}\n"
"      for lib in flags[\"lib\"]:gmatch(\"([^,]+)\") do\n"
"         table.insert(libs, lib)\n"
"         rockspec.external_dependencies[lib:upper()] = {\n"
"            library = lib\n"
"         }\n"
"      end\n"
"   end\n"
"\n"
"   local ok, err = fs.change_dir(local_dir)\n"
"   if not ok then return nil, \"Failed reaching files from project - error entering directory \"..local_dir end\n"
"\n"
"   if not (flags[\"summary\"] and flags[\"detailed\"]) then\n"
"      local summary, detailed = detect_description()\n"
"      rockspec.description.summary = flags[\"summary\"] or summary\n"
"      rockspec.description.detailed = flags[\"detailed\"] or detailed\n"
"   end\n"
"\n"
"   if not flags[\"license\"] then\n"
"      local license, fulltext = check_license()\n"
"      if license then\n"
"         rockspec.description.license = license\n"
"      elseif license then\n"
"         util.title(\"Could not auto-detect type for project license:\")\n"
"         util.printout(fulltext)\n"
"         util.printout()\n"
"         util.title(\"Please fill in the source.license field manually or use --license.\")\n"
"      end\n"
"   end\n"
"   \n"
"   fill_as_builtin(rockspec, libs)\n"
"      \n"
"   rockspec_cleanup(rockspec)\n"
"   \n"
"   persist.save_from_table(filename, rockspec, type_rockspec.order)\n"
"\n"
"   util.printout()   \n"
"   util.printout(\"Wrote template at \"..filename..\" -- you should now edit and finish it.\")\n"
"   util.printout()   \n"
"\n"
"   return true\n"
"end\n"
"\n"
"return write_rockspec\n"
""
;
