const char luarocks_core_cfg_lua[] =
"\n"
"--- Configuration for LuaRocks.\n"
"-- Tries to load the user's configuration file and\n"
"-- defines defaults for unset values. See the\n"
"-- <a href=\"http://luarocks.org/en/Config_file_format\">config\n"
"-- file format documentation</a> for details.\n"
"--\n"
"-- End-users shouldn't edit this file. They can override any defaults\n"
"-- set in this file using their system-wide or user-specific configuration\n"
"-- files. Run `luarocks` with no arguments to see the locations of\n"
"-- these files in your platform.\n"
"\n"
"local next, table, pairs, require, os, pcall, ipairs, package, tonumber, type, assert =\n"
"      next, table, pairs, require, os, pcall, ipairs, package, tonumber, type, assert\n"
"\n"
"local util = require(\"luarocks.core.util\")\n"
"local persist = require(\"luarocks.core.persist\")\n"
"local sysdetect = require(\"luarocks.core.sysdetect\")\n"
"\n"
"--------------------------------------------------------------------------------\n"
"\n"
"local program_version = \"3.1.1\"\n"
"local program_series = \"3.1\"\n"
"local major_version = (program_version:match(\"([^.]%.[^.])\")) or program_series\n"
"\n"
"local is_windows = package.config:sub(1,1) == \"\\\\\"\n"
"\n"
"-- Set order for platform overrides.\n"
"-- More general platform identifiers should be listed first,\n"
"-- more specific ones later.\n"
"local platform_order = {\n"
"   -- Unixes\n"
"   \"unix\",\n"
"   \"bsd\",\n"
"   \"solaris\",\n"
"   \"netbsd\",\n"
"   \"openbsd\",\n"
"   \"freebsd\",\n"
"   \"linux\",\n"
"   \"macosx\",\n"
"   \"cygwin\",\n"
"   \"msys\",\n"
"   \"haiku\",\n"
"   -- Windows\n"
"   \"windows\",\n"
"   \"win32\",\n"
"   \"mingw32\",\n"
"}\n"
"\n"
"local function detect_sysconfdir()\n"
"   local src = debug.getinfo(1, \"S\").source:gsub(\"\\\\\", \"/\"):gsub(\"/+\", \"/\")\n"
"   if src:sub(1, 1) == \"@\" then\n"
"      src = src:sub(2)\n"
"   end\n"
"   local basedir = src:match(\"^(.*)/luarocks/core/cfg.lua$\")\n"
"   if not basedir then\n"
"      return\n"
"   end\n"
"   -- If installed in a Unix-like tree, use a Unix-like sysconfdir\n"
"   local installdir = basedir:match(\"^(.*)/share/lua/[^/]*$\")\n"
"   if installdir then\n"
"      if installdir == \"/usr\" then\n"
"         return \"/etc/luarocks\"\n"
"      end\n"
"      return installdir .. \"/etc/luarocks\"\n"
"   end\n"
"   -- Otherwise, use base directory of sources\n"
"   return basedir\n"
"end\n"
"\n"
"local function set_confdirs(cfg, platforms, hardcoded)\n"
"   local sysconfdir = os.getenv(\"LUAROCKS_SYSCONFDIR\") or hardcoded.SYSCONFDIR\n"
"   if platforms.windows then\n"
"      cfg.home = os.getenv(\"APPDATA\") or \"c:\"\n"
"      cfg.home_tree = cfg.home..\"/luarocks\"\n"
"      cfg.homeconfdir = cfg.home_tree\n"
"      cfg.sysconfdir = sysconfdir or ((os.getenv(\"PROGRAMFILES\") or \"c:\") .. \"/luarocks\")\n"
"   else\n"
"      if not sysconfdir then\n"
"         sysconfdir = detect_sysconfdir()\n"
"      end\n"
"      cfg.home = hardcoded.HOMEDIR or os.getenv(\"HOME\") or \"\"\n"
"      cfg.localdir = hardcoded.LOCALDIR or cfg.home\n"
"      local home_tree_subdir = hardcoded.HOME_TREE_SUBDIR or \"/.luarocks\"\n"
"      cfg.homeconfdir = cfg.localdir .. home_tree_subdir\n"
"      cfg.sysconfdir = sysconfdir or \"/etc/luarocks\"\n"
"      cfg.home_tree = cfg.localdir .. home_tree_subdir\n"
"   end\n"
"end\n"
"\n"
"local load_config_file\n"
"do\n"
"   -- Create global environment for the config files;\n"
"   local function env_for_config_file(cfg, platforms)\n"
"      local e\n"
"      e = {\n"
"         home = cfg.home,\n"
"         lua_version = cfg.lua_version,\n"
"         platforms = util.make_shallow_copy(platforms),\n"
"         processor = cfg.target_cpu,   -- remains for compat reasons\n"
"         target_cpu = cfg.target_cpu,  -- replaces `processor`\n"
"         os_getenv = os.getenv,\n"
"         variables = cfg.variables or {},\n"
"         dump_env = function()\n"
"            -- debug function, calling it from a config file will show all\n"
"            -- available globals to that config file\n"
"            print(util.show_table(e, \"global environment\"))\n"
"         end,\n"
"      }\n"
"      return e\n"
"   end\n"
"\n"
"   -- Merge values from config files read into the `cfg` table\n"
"   local function merge_overrides(cfg, overrides)\n"
"      -- remove some stuff we do not want to integrate\n"
"      overrides.os_getenv = nil\n"
"      overrides.dump_env = nil\n"
"      -- remove tables to be copied verbatim instead of deeply merged\n"
"      if overrides.rocks_trees   then cfg.rocks_trees   = nil end\n"
"      if overrides.rocks_servers then cfg.rocks_servers = nil end\n"
"      -- perform actual merge\n"
"      util.deep_merge(cfg, overrides)\n"
"   end\n"
"\n"
"   local function update_platforms(platforms, overrides)\n"
"      if overrides[1] then\n"
"         for k, _ in pairs(platforms) do\n"
"            platforms[k] = nil\n"
"         end\n"
"         for _, v in ipairs(overrides) do\n"
"            platforms[v] = true\n"
"         end\n"
"         -- set some fallback default in case the user provides an incomplete configuration.\n"
"         -- LuaRocks expects a set of defaults to be available.\n"
"         if not (platforms.unix or platforms.windows) then\n"
"            platforms[is_windows and \"windows\" or \"unix\"] = true\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   -- Load config file and merge its contents into the `cfg` module table.\n"
"   -- @return filepath of successfully loaded file or nil if it failed\n"
"   load_config_file = function(cfg, platforms, filepath)\n"
"      local result, err, errcode = persist.load_into_table(filepath, env_for_config_file(cfg, platforms))\n"
"      if (not result) and errcode ~= \"open\" then\n"
"         -- errcode is either \"load\" or \"run\"; bad config file, so error out\n"
"         return nil, err, \"config\"\n"
"      end\n"
"      if result then\n"
"         -- success in loading and running, merge contents and exit\n"
"         update_platforms(platforms, result.platforms)\n"
"         result.platforms = nil\n"
"         merge_overrides(cfg, result)\n"
"         return filepath\n"
"      end\n"
"      return nil -- nothing was loaded\n"
"   end\n"
"end\n"
"\n"
"local platform_sets = {\n"
"   freebsd = { unix = true, bsd = true, freebsd = true },\n"
"   openbsd = { unix = true, bsd = true, openbsd = true },\n"
"   solaris = { unix = true, solaris = true },\n"
"   windows = { windows = true, win32 = true },\n"
"   cygwin = { unix = true, cygwin = true },\n"
"   macosx = { unix = true, bsd = true, macosx = true, macos = true },\n"
"   netbsd = { unix = true, bsd = true, netbsd = true },\n"
"   haiku = { unix = true, haiku = true },\n"
"   linux = { unix = true, linux = true },\n"
"   mingw = { windows = true, win32 = true, mingw32 = true, mingw = true },\n"
"   msys = { unix = true, cygwin = true, msys = true },\n"
"}\n"
"\n"
"local function make_platforms(system)\n"
"   -- fallback to Unix in unknown systems\n"
"   return platform_sets[system] or { unix = true }\n"
"end\n"
"\n"
"--------------------------------------------------------------------------------\n"
"\n"
"local function make_defaults(lua_version, target_cpu, platforms, home, hardcoded)\n"
"\n"
"   -- Configure defaults:\n"
"   local defaults = {\n"
"\n"
"      lua_interpreter = hardcoded.LUA_INTERPRETER or \"lua\",\n"
"      local_by_default = hardcoded.LOCAL_BY_DEFAULT or false,\n"
"      accept_unknown_fields = false,\n"
"      fs_use_modules = true,\n"
"      hooks_enabled = true,\n"
"      deps_mode = \"one\",\n"
"      check_certificates = false,\n"
"\n"
"      cache_timeout = 60,\n"
"      cache_fail_timeout = 86400,\n"
"      version_check_on_fail = true,\n"
"\n"
"      lua_modules_path = hardcoded.LUA_MODULES_LUA_SUBDIR or \"/share/lua/\"..lua_version,\n"
"      lib_modules_path = hardcoded.LUA_MODULES_LIB_SUBDIR or \"/lib/lua/\"..lua_version,\n"
"      rocks_subdir = hardcoded.ROCKS_SUBDIR or \"/lib/luarocks/rocks-\"..lua_version,\n"
"\n"
"      arch = \"unknown\",\n"
"      lib_extension = \"unknown\",\n"
"      obj_extension = \"unknown\",\n"
"      link_lua_explicitly = false,\n"
"\n"
"      rocks_servers = hardcoded.ROCKS_SERVERS or {\n"
"         {\n"
"           \"https://luarocks.org\",\n"
"           \"https://raw.githubusercontent.com/rocks-moonscript-org/moonrocks-mirror/master/\",\n"
"           \"http://luafr.org/moonrocks/\",\n"
"           \"http://luarocks.logiceditor.com/rocks\",\n"
"         }\n"
"      },\n"
"      disabled_servers = {},\n"
"\n"
"      upload = {\n"
"         server = \"https://luarocks.org\",\n"
"         tool_version = \"1.0.0\",\n"
"         api_version = \"1\",\n"
"      },\n"
"\n"
"      lua_extension = \"lua\",\n"
"      connection_timeout = 30,  -- 0 = no timeout\n"
"\n"
"      variables = {\n"
"         MAKE = \"make\",\n"
"         CC = \"cc\",\n"
"         LD = \"ld\",\n"
"         AR = \"ar\",\n"
"         RANLIB = \"ranlib\",\n"
"\n"
"         CVS = \"cvs\",\n"
"         GIT = \"git\",\n"
"         SSCM = \"sscm\",\n"
"         SVN = \"svn\",\n"
"         HG = \"hg\",\n"
"\n"
"         GPG = \"gpg\",\n"
"\n"
"         RSYNC = \"rsync\",\n"
"         WGET = \"wget\",\n"
"         SCP = \"scp\",\n"
"         CURL = \"curl\",\n"
"\n"
"         PWD = \"pwd\",\n"
"         MKDIR = \"mkdir\",\n"
"         RMDIR = \"rmdir\",\n"
"         CP = \"cp\",\n"
"         LS = \"ls\",\n"
"         RM = \"rm\",\n"
"         FIND = \"find\",\n"
"         CHMOD = \"chmod\",\n"
"         ICACLS = \"icacls\",\n"
"         MKTEMP = \"mktemp\",\n"
"\n"
"         ZIP = \"zip\",\n"
"         UNZIP = \"unzip -n\",\n"
"         GUNZIP = \"gunzip\",\n"
"         BUNZIP2 = \"bunzip2\",\n"
"         TAR = \"tar\",\n"
"\n"
"         MD5SUM = \"md5sum\",\n"
"         OPENSSL = \"openssl\",\n"
"         MD5 = \"md5\",\n"
"         TOUCH = \"touch\",\n"
"\n"
"         CMAKE = \"cmake\",\n"
"         SEVENZ = \"7z\",\n"
"\n"
"         RSYNCFLAGS = \"--exclude=.git -Oavz\",\n"
"         CURLNOCERTFLAG = \"\",\n"
"         WGETNOCERTFLAG = \"\",\n"
"      },\n"
"\n"
"      external_deps_subdirs = hardcoded.EXTERNAL_DEPS_SUBDIRS or {\n"
"         bin = \"bin\",\n"
"         lib = \"lib\",\n"
"         include = \"include\"\n"
"      },\n"
"      runtime_external_deps_subdirs = hardcoded.RUNTIME_EXTERNAL_DEPS_SUBDIRS or {\n"
"         bin = \"bin\",\n"
"         lib = \"lib\",\n"
"         include = \"include\"\n"
"      },\n"
"   }\n"
"\n"
"   if platforms.windows then\n"
"\n"
"      defaults.arch = \"win32-\"..target_cpu\n"
"      defaults.lib_extension = \"dll\"\n"
"      defaults.external_lib_extension = \"dll\"\n"
"      defaults.static_lib_extension = \"lib\"\n"
"      defaults.obj_extension = \"obj\"\n"
"      defaults.external_deps_dirs = { \"c:/external/\", \"c:/windows/system32\" }\n"
"\n"
"      defaults.makefile = \"Makefile.win\"\n"
"      defaults.variables.MAKE = \"nmake\"\n"
"      defaults.variables.CC = \"cl\"\n"
"      defaults.variables.RC = \"rc\"\n"
"      defaults.variables.LD = \"link\"\n"
"      defaults.variables.MT = \"mt\"\n"
"      defaults.variables.AR = \"lib\"\n"
"      defaults.variables.LUALIB = \"lua\"..lua_version..\".lib\"\n"
"      defaults.variables.CFLAGS = \"/nologo /MD /O2\"\n"
"      defaults.variables.LIBFLAG = \"/nologo /dll\"\n"
"\n"
"      defaults.external_deps_patterns = {\n"
"         bin = { \"\?.exe\", \"\?.bat\" },\n"
"         lib = { \"\?.lib\", \"\?.dll\", \"lib\?.dll\" },\n"
"         include = { \"\?.h\" }\n"
"      }\n"
"      defaults.runtime_external_deps_patterns = {\n"
"         bin = { \"\?.exe\", \"\?.bat\" },\n"
"         lib = { \"\?.dll\", \"lib\?.dll\" },\n"
"         include = { \"\?.h\" }\n"
"      }\n"
"      defaults.export_path_separator = \";\"\n"
"      defaults.wrapper_suffix = \".bat\"\n"
"\n"
"      local localappdata = os.getenv(\"LOCALAPPDATA\")\n"
"      if not localappdata then\n"
"         -- for Windows versions below Vista\n"
"         localappdata = (os.getenv(\"USERPROFILE\") or \"c:/Users/All Users\")..\"/Local Settings/Application Data\"\n"
"      end\n"
"      defaults.local_cache = localappdata..\"/LuaRocks/Cache\"\n"
"      defaults.web_browser = \"start\"\n"
"\n"
"      defaults.external_deps_subdirs.lib = { \"\", \"lib\", \"bin\" }\n"
"      defaults.runtime_external_deps_subdirs.lib = { \"\", \"lib\", \"bin\" }\n"
"      defaults.link_lua_explicitly = true\n"
"      defaults.fs_use_modules = false\n"
"   end\n"
"\n"
"   if platforms.mingw32 then\n"
"      defaults.obj_extension = \"o\"\n"
"      defaults.static_lib_extension = \"a\"\n"
"      defaults.external_deps_dirs = { \"c:/external/\", \"c:/mingw\", \"c:/windows/system32\" }\n"
"      defaults.cmake_generator = \"MinGW Makefiles\"\n"
"      defaults.variables.MAKE = \"mingw32-make\"\n"
"      defaults.variables.CC = \"mingw32-gcc\"\n"
"      defaults.variables.RC = \"windres\"\n"
"      defaults.variables.LD = \"mingw32-gcc\"\n"
"      defaults.variables.AR = \"ar\"\n"
"      defaults.variables.RANLIB = \"ranlib\"\n"
"      defaults.variables.CFLAGS = \"-O2\"\n"
"      defaults.variables.LIBFLAG = \"-shared\"\n"
"      defaults.makefile = \"Makefile\"\n"
"      defaults.external_deps_patterns = {\n"
"         bin = { \"\?.exe\", \"\?.bat\" },\n"
"         -- mingw lookup list from http://stackoverflow.com/a/15853231/1793220\n"
"         -- ...should we keep \?.lib at the end\? It's not in the above list.\n"
"         lib = { \"lib\?.dll.a\", \"\?.dll.a\", \"lib\?.a\", \"cyg\?.dll\", \"lib\?.dll\", \"\?.dll\", \"\?.lib\" },\n"
"         include = { \"\?.h\" }\n"
"      }\n"
"      defaults.runtime_external_deps_patterns = {\n"
"         bin = { \"\?.exe\", \"\?.bat\" },\n"
"         lib = { \"cyg\?.dll\", \"\?.dll\", \"lib\?.dll\" },\n"
"         include = { \"\?.h\" }\n"
"      }\n"
"   end\n"
"\n"
"   if platforms.unix then\n"
"      defaults.lib_extension = \"so\"\n"
"      defaults.static_lib_extension = \"a\"\n"
"      defaults.external_lib_extension = \"so\"\n"
"      defaults.obj_extension = \"o\"\n"
"      defaults.external_deps_dirs = { \"/usr/local\", \"/usr\", \"/\", hardcoded.PREFIX }\n"
"      defaults.variables.CFLAGS = \"-O2\"\n"
"      defaults.cmake_generator = \"Unix Makefiles\"\n"
"      defaults.variables.CC = \"gcc\"\n"
"      defaults.variables.LD = \"gcc\"\n"
"      defaults.gcc_rpath = true\n"
"      defaults.variables.LIBFLAG = \"-shared\"\n"
"\n"
"      defaults.external_deps_patterns = {\n"
"         bin = { \"\?\" },\n"
"         lib = { \"lib\?.a\", \"lib\?.so\", \"lib\?.so.*\" },\n"
"         include = { \"\?.h\" }\n"
"      }\n"
"      defaults.runtime_external_deps_patterns = {\n"
"         bin = { \"\?\" },\n"
"         lib = { \"lib\?.so\", \"lib\?.so.*\" },\n"
"         include = { \"\?.h\" }\n"
"      }\n"
"      defaults.export_path_separator = \":\"\n"
"      defaults.wrapper_suffix = \"\"\n"
"      defaults.local_cache = home..\"/.cache/luarocks\"\n"
"      if not defaults.variables.CFLAGS:match(\"-fPIC\") then\n"
"         defaults.variables.CFLAGS = defaults.variables.CFLAGS..\" -fPIC\"\n"
"      end\n"
"      defaults.web_browser = \"xdg-open\"\n"
"   end\n"
"\n"
"   if platforms.cygwin then\n"
"      defaults.lib_extension = \"so\" -- can be overridden in the config file for mingw builds\n"
"      defaults.arch = \"cygwin-\"..target_cpu\n"
"      defaults.cmake_generator = \"Unix Makefiles\"\n"
"      defaults.variables.CC = \"echo -llua | xargs gcc\"\n"
"      defaults.variables.LD = \"echo -llua | xargs gcc\"\n"
"      defaults.variables.LIBFLAG = \"-shared\"\n"
"      defaults.link_lua_explicitly = true\n"
"   end\n"
"\n"
"   if platforms.msys then\n"
"      -- msys is basically cygwin made out of mingw, meaning the subsytem is unixish\n"
"      -- enough, yet we can freely mix with native win32\n"
"      defaults.external_deps_patterns = {\n"
"         bin = { \"\?.exe\", \"\?.bat\", \"\?\" },\n"
"         lib = { \"lib\?.so\", \"lib\?.so.*\", \"lib\?.dll.a\", \"\?.dll.a\",\n"
"                 \"lib\?.a\", \"lib\?.dll\", \"\?.dll\", \"\?.lib\" },\n"
"         include = { \"\?.h\" }\n"
"      }\n"
"      defaults.runtime_external_deps_patterns = {\n"
"         bin = { \"\?.exe\", \"\?.bat\" },\n"
"         lib = { \"lib\?.so\", \"\?.dll\", \"lib\?.dll\" },\n"
"         include = { \"\?.h\" }\n"
"      }\n"
"   end\n"
"\n"
"   if platforms.bsd then\n"
"      defaults.variables.MAKE = \"gmake\"\n"
"   end\n"
"\n"
"   if platforms.macosx then\n"
"      defaults.variables.MAKE = \"make\"\n"
"      defaults.external_lib_extension = \"dylib\"\n"
"      defaults.arch = \"macosx-\"..target_cpu\n"
"      defaults.variables.LIBFLAG = \"-bundle -undefined dynamic_lookup -all_load\"\n"
"      local version = util.popen_read(\"sw_vers -productVersion\")\n"
"      version = tonumber(version and version:match(\"^[^.]+%.([^.]+)\")) or 3\n"
"      if version >= 10 then\n"
"         version = 8\n"
"      elseif version >= 5 then\n"
"         version = 5\n"
"      else\n"
"         defaults.gcc_rpath = false\n"
"      end\n"
"      defaults.variables.CC = \"env MACOSX_DEPLOYMENT_TARGET=10.\"..version..\" gcc\"\n"
"      defaults.variables.LD = \"env MACOSX_DEPLOYMENT_TARGET=10.\"..version..\" gcc\"\n"
"      defaults.web_browser = \"open\"\n"
"   end\n"
"\n"
"   if platforms.linux then\n"
"      defaults.arch = \"linux-\"..target_cpu\n"
"\n"
"      local gcc_arch = util.popen_read(\"gcc -print-multiarch 2>/dev/null\")\n"
"      if gcc_arch and gcc_arch ~= \"\" then\n"
"         defaults.external_deps_subdirs.lib = { \"lib\", \"lib/\" .. gcc_arch, \"lib64\" }\n"
"         defaults.runtime_external_deps_subdirs.lib = { \"lib\", \"lib/\" .. gcc_arch, \"lib64\" }\n"
"      else\n"
"         defaults.external_deps_subdirs.lib = { \"lib\", \"lib64\" }\n"
"         defaults.runtime_external_deps_subdirs.lib = { \"lib\", \"lib64\" }\n"
"      end\n"
"   end\n"
"\n"
"   if platforms.freebsd then\n"
"      defaults.arch = \"freebsd-\"..target_cpu\n"
"      defaults.gcc_rpath = false\n"
"      defaults.variables.CC = \"cc\"\n"
"      defaults.variables.LD = \"cc\"\n"
"   end\n"
"\n"
"   if platforms.openbsd then\n"
"      defaults.arch = \"openbsd-\"..target_cpu\n"
"   end\n"
"\n"
"   if platforms.netbsd then\n"
"      defaults.arch = \"netbsd-\"..target_cpu\n"
"   end\n"
"\n"
"   if platforms.solaris then\n"
"      defaults.arch = \"solaris-\"..target_cpu\n"
"      --defaults.platforms = {\"unix\", \"solaris\"}\n"
"      defaults.variables.MAKE = \"gmake\"\n"
"   end\n"
"\n"
"   -- Expose some more values detected by LuaRocks for use by rockspec authors.\n"
"   defaults.variables.LIB_EXTENSION = defaults.lib_extension\n"
"   defaults.variables.OBJ_EXTENSION = defaults.obj_extension\n"
"\n"
"   return defaults\n"
"end\n"
"\n"
"local function make_rocks_provided(lua_version, luajit_version)\n"
"   local rocks_provided = {}\n"
"   local rocks_provided_3_0 = {}\n"
"\n"
"   rocks_provided[\"lua\"] = lua_version..\"-1\"\n"
"\n"
"   if lua_version == \"5.2\" or lua_version == \"5.3\" then\n"
"      rocks_provided[\"bit32\"] = lua_version..\"-1\"\n"
"   end\n"
"\n"
"   if lua_version == \"5.3\" or lua_version == \"5.4\" then\n"
"      rocks_provided[\"utf8\"] = lua_version..\"-1\"\n"
"   end\n"
"\n"
"   if luajit_version then\n"
"      rocks_provided[\"luabitop\"] = luajit_version..\"-1\"\n"
"      rocks_provided_3_0[\"luajit\"] = luajit_version..\"-1\"\n"
"   end\n"
"\n"
"   if rawget(_G, '_TARANTOOL') then\n"
"      -- Tarantool\n"
"      local tarantool_version = _TARANTOOL:match(\"([^-]+)-\")\n"
"      rocks_provided[\"tarantool\"] = tarantool_version..\"-1\"\n"
"      rocks_provided_3_0[\"tarantool\"] = tarantool_version..\"-1\"\n"
"   end\n"
"\n"
"   return rocks_provided, rocks_provided_3_0\n"
"end\n"
"\n"
"local function use_defaults(cfg, defaults)\n"
"\n"
"   -- Populate some arrays with values from their 'defaults' counterparts\n"
"   -- if they were not already set by user.\n"
"   for _, entry in ipairs({\"variables\", \"rocks_provided\"}) do\n"
"      if not cfg[entry] then\n"
"         cfg[entry] = {}\n"
"      end\n"
"      for k,v in pairs(defaults[entry]) do\n"
"         if not cfg[entry][k] then\n"
"            cfg[entry][k] = v\n"
"         end\n"
"      end\n"
"   end\n"
"   util.deep_merge_under(defaults.rocks_provided_3_0, cfg.rocks_provided)\n"
"\n"
"   util.deep_merge_under(cfg, defaults)\n"
"\n"
"   -- FIXME get rid of this\n"
"   if not cfg.check_certificates then\n"
"      cfg.variables.CURLNOCERTFLAG = \"-k\"\n"
"      cfg.variables.WGETNOCERTFLAG = \"--no-check-certificate\"\n"
"   end\n"
"end\n"
"\n"
"--------------------------------------------------------------------------------\n"
"\n"
"local cfg = {}\n"
"\n"
"--- Initializes the LuaRocks configuration for variables, paths\n"
"-- and OS detection.\n"
"-- @param detected table containing information detected about the \n"
"-- environment. All fields below are optional:\n"
"-- * lua_version (in x.y format, e.g. \"5.3\")\n"
"-- * luajit_version (complete, e.g. \"2.1.0-beta3\")\n"
"-- * lua_bindir (e.g. \"/usr/local/bin\")\n"
"-- * lua_incdir (e.g. \"/usr/local/include/lua5.3/\")\n"
"-- * lua_libdir(e.g. \"/usr/local/lib\")\n"
"-- * lua_dir (e.g. \"/usr/local\")\n"
"-- * lua_interpreter (e.g. \"lua-5.3\")\n"
"-- * project_dir (a string with the path of the project directory\n"
"--   when using per-project environments, as created with `luarocks init`)\n"
"-- @param warning a logging function for warnings that takes a string\n"
"-- @return true on success; nil and an error message on failure.\n"
"function cfg.init(detected, warning)\n"
"   if cfg.initialized == true then\n"
"      return true\n"
"   end\n"
"\n"
"   detected = detected or {}\n"
"\n"
"   local hc_ok, hardcoded = pcall(require, \"luarocks.core.hardcoded\")\n"
"   if not hc_ok then\n"
"      hardcoded = {}\n"
"   end\n"
"\n"
"   local arg = rawget(_G, 'arg')\n"
"   local lua_version = detected.lua_version or hardcoded.LUA_VERSION or _VERSION:sub(5)\n"
"   local luajit_version = detected.luajit_version or hardcoded.LUAJIT_VERSION or (jit and jit.version:sub(8))\n"
"   local lua_interpreter = detected.lua_interpreter or hardcoded.LUA_INTERPRETER or (arg and arg[-1] and arg[-1]:gsub(\".*[\\\\/]\", \"\")) or (is_windows and \"lua.exe\" or \"lua\")\n"
"   local lua_bindir = detected.lua_bindir or hardcoded.LUA_BINDIR or (arg and arg[-1] and arg[-1]:gsub(\"[\\\\/][^\\\\/]+$\", \"\"))\n"
"   local lua_incdir = detected.lua_incdir or hardcoded.LUA_INCDIR\n"
"   local lua_libdir = detected.lua_libdir or hardcoded.LUA_LIBDIR\n"
"   local lua_dir = detected.lua_dir or hardcoded.LUA_DIR or (lua_bindir and lua_bindir:gsub(\"[\\\\/]bin$\", \"\"))\n"
"   \n"
"   local init = cfg.init\n"
"\n"
"   ----------------------------------------\n"
"   -- Reset the cfg table.\n"
"   ----------------------------------------\n"
"\n"
"   for k, _ in pairs(cfg) do\n"
"      cfg[k] = nil\n"
"   end\n"
"\n"
"   cfg.program_version = program_version\n"
"   cfg.program_series = program_series\n"
"   cfg.major_version = major_version\n"
"\n"
"   cfg.lua_version = lua_version\n"
"   cfg.luajit_version = luajit_version\n"
"   cfg.lua_interpreter = lua_interpreter\n"
"\n"
"   cfg.variables = {\n"
"      LUA_DIR = lua_dir,\n"
"      LUA_BINDIR = lua_bindir,\n"
"      LUA_INCDIR = lua_incdir,\n"
"      LUA_LIBDIR = lua_libdir,\n"
"   }\n"
"\n"
"   cfg.init = init\n"
"\n"
"   ----------------------------------------\n"
"   -- System detection.\n"
"   ----------------------------------------\n"
"\n"
"   -- A proper build of LuaRocks will hardcode the system\n"
"   -- and proc values with hardcoded.SYSTEM and hardcoded.PROCESSOR.\n"
"   -- If that is not available, we try to identify the system.\n"
"   local system, processor = sysdetect.detect()\n"
"   if hardcoded.SYSTEM then\n"
"      system = hardcoded.SYSTEM\n"
"   end\n"
"   if hardcoded.PROCESSOR then\n"
"      processor = hardcoded.PROCESSOR\n"
"   end\n"
"\n"
"   if system == \"windows\" then\n"
"      if os.getenv(\"VCINSTALLDIR\") then\n"
"         -- running from the Development Command prompt for VS 2017\n"
"         system = \"windows\"\n"
"      else\n"
"         local fd = io.open(\"/bin/sh\", \"r\")\n"
"         if fd then\n"
"            fd:close()\n"
"            system = \"msys\"\n"
"         else\n"
"            system = \"mingw\"\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   cfg.target_cpu = processor\n"
"\n"
"   local platforms = make_platforms(system)\n"
"\n"
"   ----------------------------------------\n"
"   -- Platform is determined.\n"
"   -- Let's load the config files.\n"
"   ----------------------------------------\n"
"\n"
"   local sys_config_file\n"
"   local home_config_file\n"
"   local project_config_file\n"
"   do\n"
"      set_confdirs(cfg, platforms, hardcoded)\n"
"      local name = \"config-\"..cfg.lua_version..\".lua\"\n"
"      sys_config_file = (cfg.sysconfdir .. \"/\" .. name):gsub(\"\\\\\", \"/\")\n"
"      home_config_file = (cfg.homeconfdir .. \"/\" .. name):gsub(\"\\\\\", \"/\")\n"
"      if detected.project_dir then\n"
"         project_config_file = detected.project_dir .. \"/.luarocks/\" .. name\n"
"      end\n"
"   end\n"
"\n"
"   -- Load system configuration file\n"
"   local sys_config_ok, err = load_config_file(cfg, platforms, sys_config_file)\n"
"   if err then\n"
"      return nil, err, \"config\"\n"
"   end\n"
"\n"
"   -- Load user configuration file (if allowed)\n"
"   local home_config_ok\n"
"   local project_config_ok\n"
"   if not hardcoded.FORCE_CONFIG then\n"
"      local env_var   = \"LUAROCKS_CONFIG_\" .. lua_version:gsub(\"%.\", \"_\")\n"
"      local env_value = os.getenv(env_var)\n"
"      if not env_value then\n"
"         env_var   = \"LUAROCKS_CONFIG\"\n"
"         env_value = os.getenv(env_var)\n"
"      end\n"
"      -- first try environment provided file, so we can explicitly warn when it is missing\n"
"      if env_value then\n"
"         local env_ok, err = load_config_file(cfg, platforms, env_value)\n"
"         if err then\n"
"            return nil, err, \"config\"\n"
"         elseif warning and not env_ok then\n"
"            warning(\"Warning: could not load configuration file `\"..env_value..\"` given in environment variable \"..env_var..\"\\n\")\n"
"         end\n"
"         if env_ok then\n"
"            home_config_ok = true\n"
"            home_config_file = env_value\n"
"         end\n"
"      end\n"
"\n"
"      -- try the alternative defaults if there was no environment specified file or it didn't work\n"
"      if not home_config_ok then\n"
"         home_config_ok, err = load_config_file(cfg, platforms, home_config_file)\n"
"         if err then\n"
"            return nil, err, \"config\"\n"
"         end\n"
"      end\n"
"\n"
"      -- finally, use the project-specific config file if any\n"
"      if detected.project_dir then\n"
"         project_config_ok, err = load_config_file(cfg, platforms, project_config_file)\n"
"         if err then\n"
"            return nil, err, \"config\"\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   ----------------------------------------\n"
"   -- Config files are loaded.\n"
"   -- Let's finish up the cfg table.\n"
"   ----------------------------------------\n"
"\n"
"   -- Settings detected or given via the CLI (i.e. --lua-dir) take precedence over config files:\n"
"   cfg.lua_version = detected.lua_version or cfg.lua_version\n"
"   cfg.luajit_version = detected.luajit_version or cfg.luajit_version\n"
"   cfg.lua_interpreter = detected.lua_interpreter or cfg.lua_interpreter\n"
"   cfg.variables.LUA_BINDIR = detected.lua_bindir or cfg.variables.LUA_BINDIR or lua_bindir\n"
"   cfg.variables.LUA_INCDIR = detected.lua_incdir or cfg.variables.LUA_INCDIR or lua_incdir\n"
"   cfg.variables.LUA_LIBDIR = detected.lua_libdir or cfg.variables.LUA_LIBDIR or lua_libdir\n"
"   cfg.variables.LUA_DIR = detected.lua_dir or cfg.variables.LUA_DIR or lua_dir\n"
"\n"
"   -- Build a default list of rocks trees if not given\n"
"   if cfg.rocks_trees == nil then\n"
"      cfg.rocks_trees = {}\n"
"      if cfg.home_tree then\n"
"         table.insert(cfg.rocks_trees, { name = \"user\", root = cfg.home_tree } )\n"
"      end\n"
"      if hardcoded.PREFIX and hardcoded.PREFIX ~= cfg.home_tree then\n"
"         table.insert(cfg.rocks_trees, { name = \"system\", root = hardcoded.PREFIX } )\n"
"      end\n"
"   end\n"
"\n"
"   local defaults = make_defaults(lua_version, processor, platforms, cfg.home, hardcoded)\n"
"\n"
"   if platforms.windows and hardcoded.WIN_TOOLS then\n"
"      local tools = { \"SEVENZ\", \"CP\", \"FIND\", \"LS\", \"MD5SUM\", \"PWD\", \"RMDIR\", \"WGET\", \"MKDIR\" }\n"
"      for _, tool in ipairs(tools) do\n"
"         defaults.variables[tool] = '\"' .. hardcoded.WIN_TOOLS .. \"/\" .. defaults.variables[tool] .. '.exe\"'\n"
"      end\n"
"   else\n"
"      defaults.fs_use_modules = true\n"
"   end\n"
"\n"
"   defaults.rocks_provided, defaults.rocks_provided_3_0 = make_rocks_provided(lua_version, luajit_version)\n"
"   use_defaults(cfg, defaults)\n"
"\n"
"   cfg.variables.LUA = cfg.variables.LUA or (cfg.variables.LUA_BINDIR and (cfg.variables.LUA_BINDIR .. \"/\" .. cfg.lua_interpreter):gsub(\"//\", \"/\"))\n"
"   cfg.user_agent = \"LuaRocks/\"..cfg.program_version..\" \"..cfg.arch\n"
"\n"
"   cfg.config_files = {\n"
"      project = detected.project_dir and {\n"
"         file = project_config_file,\n"
"         found = not not project_config_ok,\n"
"      },\n"
"      system = {\n"
"         file = sys_config_file,\n"
"         found = not not sys_config_ok,\n"
"      },\n"
"      user = {\n"
"         file = home_config_file,\n"
"         found = not not home_config_ok,\n"
"      },\n"
"      nearest = project_config_ok\n"
"                and project_config_file\n"
"                or (home_config_ok\n"
"                    and home_config_file\n"
"                    or sys_config_file),\n"
"   }\n"
"\n"
"   ----------------------------------------\n"
"   -- Attributes of cfg are set.\n"
"   -- Let's add some methods.\n"
"   ----------------------------------------\n"
"\n"
"   do\n"
"      local function make_paths_from_tree(tree)\n"
"         local lua_path, lib_path, bin_path\n"
"         if type(tree) == \"string\" then\n"
"            lua_path = tree..cfg.lua_modules_path\n"
"            lib_path = tree..cfg.lib_modules_path\n"
"            bin_path = tree..\"/bin\"\n"
"         else\n"
"            lua_path = tree.lua_dir or tree.root..cfg.lua_modules_path\n"
"            lib_path = tree.lib_dir or tree.root..cfg.lib_modules_path\n"
"            bin_path = tree.bin_dir or tree.root..\"/bin\"\n"
"         end\n"
"         return lua_path, lib_path, bin_path\n"
"      end\n"
"\n"
"      function cfg.package_paths(current)\n"
"         local new_path, new_cpath, new_bin = {}, {}, {}\n"
"         local function add_tree_to_paths(tree)\n"
"            local lua_path, lib_path, bin_path = make_paths_from_tree(tree)\n"
"            table.insert(new_path, lua_path..\"/\?.lua\")\n"
"            table.insert(new_path, lua_path..\"/\?/init.lua\")\n"
"            table.insert(new_cpath, lib_path..\"/\?.\"..cfg.lib_extension)\n"
"            table.insert(new_bin, bin_path)\n"
"         end\n"
"         if current then\n"
"            add_tree_to_paths(current)\n"
"         end\n"
"         for _,tree in ipairs(cfg.rocks_trees) do\n"
"            add_tree_to_paths(tree)\n"
"         end\n"
"         return table.concat(new_path, \";\"), table.concat(new_cpath, \";\"), table.concat(new_bin, cfg.export_path_separator)\n"
"      end\n"
"   end\n"
"\n"
"   function cfg.init_package_paths()\n"
"      local lr_path, lr_cpath, lr_bin = cfg.package_paths()\n"
"      package.path = util.cleanup_path(package.path .. \";\" .. lr_path, \";\", lua_version)\n"
"      package.cpath = util.cleanup_path(package.cpath .. \";\" .. lr_cpath, \";\", lua_version)\n"
"   end\n"
"\n"
"   --- Check if platform was detected\n"
"   -- @param name string: The platform name to check.\n"
"   -- @return boolean: true if LuaRocks is currently running on queried platform.\n"
"   function cfg.is_platform(name)\n"
"      assert(type(name) == \"string\")\n"
"      return platforms[name]\n"
"   end\n"
"\n"
"   function cfg.each_platform()\n"
"      local i = 0\n"
"      return function()\n"
"         local p\n"
"         repeat\n"
"            i = i + 1\n"
"            p = platform_order[i]\n"
"         until (not p) or platforms[p]\n"
"         return p\n"
"      end\n"
"   end\n"
"\n"
"   function cfg.print_platforms()\n"
"      local platform_keys = {}\n"
"      for k,_ in pairs(platforms) do\n"
"         table.insert(platform_keys, k)\n"
"      end\n"
"      table.sort(platform_keys)\n"
"      return table.concat(platform_keys, \", \")\n"
"   end\n"
"\n"
"   cfg.initialized = true\n"
"   return true\n"
"end\n"
"\n"
"return cfg\n"
""
;
