const char luarocks_core_manif_lua[] =
"\n"
"--- Core functions for querying manifest files.\n"
"local manif = {}\n"
"\n"
"local persist = require(\"luarocks.core.persist\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local dir = require(\"luarocks.core.dir\")\n"
"local require = nil\n"
"--------------------------------------------------------------------------------\n"
"\n"
"-- Table with repository identifiers as keys and tables mapping\n"
"-- Lua versions to cached loaded manifests as values.\n"
"local manifest_cache = {}\n"
"\n"
"--- Cache a loaded manifest.\n"
"-- @param repo_url string: The repository identifier.\n"
"-- @param lua_version string: Lua version in \"5.x\" format, defaults to installed version.\n"
"-- @param manifest table: the manifest to be cached.\n"
"function manif.cache_manifest(repo_url, lua_version, manifest)\n"
"   lua_version = lua_version or cfg.lua_version\n"
"   manifest_cache[repo_url] = manifest_cache[repo_url] or {}\n"
"   manifest_cache[repo_url][lua_version] = manifest\n"
"end\n"
"\n"
"--- Attempt to get cached loaded manifest.\n"
"-- @param repo_url string: The repository identifier.\n"
"-- @param lua_version string: Lua version in \"5.x\" format, defaults to installed version.\n"
"-- @return table or nil: loaded manifest or nil if cache is empty.\n"
"function manif.get_cached_manifest(repo_url, lua_version)\n"
"   lua_version = lua_version or cfg.lua_version\n"
"   return manifest_cache[repo_url] and manifest_cache[repo_url][lua_version]\n"
"end\n"
"\n"
"--- Back-end function that actually loads the manifest\n"
"-- and stores it in the manifest cache.\n"
"-- @param file string: The local filename of the manifest file.\n"
"-- @param repo_url string: The repository identifier.\n"
"-- @param lua_version string: Lua version in \"5.x\" format, defaults to installed version.\n"
"-- @return table or (nil, string, string): the manifest or nil,\n"
"-- error message and error code (\"open\", \"load\", \"run\").\n"
"function manif.manifest_loader(file, repo_url, lua_version)\n"
"   local manifest, err, errcode = persist.load_into_table(file)\n"
"   if not manifest then\n"
"      return nil, \"Failed loading manifest for \"..repo_url..\": \"..err, errcode\n"
"   end\n"
"   manif.cache_manifest(repo_url, lua_version, manifest)\n"
"   return manifest, err, errcode\n"
"end\n"
"\n"
"--- Load a local manifest describing a repository.\n"
"-- This is used by the luarocks.loader only.\n"
"-- @param repo_url string: URL or pathname for the repository.\n"
"-- @return table or (nil, string, string): A table representing the manifest,\n"
"-- or nil followed by an error message and an error code, see manifest_loader.\n"
"function manif.fast_load_local_manifest(repo_url)\n"
"   assert(type(repo_url) == \"string\")\n"
"\n"
"   local cached_manifest = manif.get_cached_manifest(repo_url)\n"
"   if cached_manifest then\n"
"      return cached_manifest\n"
"   end\n"
"\n"
"   local pathname = dir.path(repo_url, \"manifest\")\n"
"   return manif.manifest_loader(pathname, repo_url, nil, true)\n"
"end\n"
"\n"
"return manif\n"
""
;
