const char luarocks_core_path_lua[] =
"\n"
"--- Core LuaRocks-specific path handling functions.\n"
"local path = {}\n"
"\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local dir = require(\"luarocks.core.dir\")\n"
"local require = nil\n"
"--------------------------------------------------------------------------------\n"
"\n"
"function path.rocks_dir(tree)\n"
"   if tree == nil then\n"
"      tree = cfg.root_dir\n"
"   end\n"
"   if type(tree) == \"string\" then\n"
"      return dir.path(tree, cfg.rocks_subdir)\n"
"   end\n"
"   assert(type(tree) == \"table\")\n"
"   return tree.rocks_dir or dir.path(tree.root, cfg.rocks_subdir)\n"
"end\n"
"\n"
"--- Produce a versioned version of a filename.\n"
"-- @param file string: filename (must start with prefix)\n"
"-- @param prefix string: Path prefix for file\n"
"-- @param name string: Rock name\n"
"-- @param version string: Rock version\n"
"-- @return string: a pathname with the same directory parts and a versioned basename.\n"
"function path.versioned_name(file, prefix, name, version)\n"
"   assert(type(file) == \"string\")\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"\n"
"   local rest = file:sub(#prefix+1):gsub(\"^/*\", \"\")\n"
"   local name_version = (name..\"_\"..version):gsub(\"%-\", \"_\"):gsub(\"%.\", \"_\")\n"
"   return dir.path(prefix, name_version..\"-\"..rest)\n"
"end\n"
"\n"
"--- Convert a pathname to a module identifier.\n"
"-- In Unix, for example, a path \"foo/bar/baz.lua\" is converted to\n"
"-- \"foo.bar.baz\"; \"bla/init.lua\" returns \"bla.init\"; \"foo.so\" returns \"foo\".\n"
"-- @param file string: Pathname of module\n"
"-- @return string: The module identifier, or nil if given path is\n"
"-- not a conformant module path (the function does not check if the\n"
"-- path actually exists).\n"
"function path.path_to_module(file)\n"
"   assert(type(file) == \"string\")\n"
"\n"
"   local name = file:match(\"(.*)%.\"..cfg.lua_extension..\"$\")\n"
"   if name then\n"
"      name = name:gsub(\"/\", \".\")\n"
"   else\n"
"      name = file:match(\"(.*)%.\"..cfg.lib_extension..\"$\")\n"
"      if name then\n"
"         name = name:gsub(\"/\", \".\")\n"
"      --[[ TODO disable static libs until we fix the conflict in the manifest, which will take extending the manifest format.\n"
"      else\n"
"         name = file:match(\"(.*)%.\"..cfg.static_lib_extension..\"$\")\n"
"         if name then\n"
"            name = name:gsub(\"/\", \".\")\n"
"         end\n"
"      ]]\n"
"      end\n"
"   end\n"
"   if not name then name = file end\n"
"   name = name:gsub(\"^%.+\", \"\"):gsub(\"%.+$\", \"\")\n"
"   return name\n"
"end\n"
"\n"
"function path.deploy_lua_dir(tree)\n"
"   if type(tree) == \"string\" then\n"
"      return dir.path(tree, cfg.lua_modules_path)\n"
"   else\n"
"      assert(type(tree) == \"table\")\n"
"      return tree.lua_dir or dir.path(tree.root, cfg.lua_modules_path)\n"
"   end\n"
"end\n"
"\n"
"function path.deploy_lib_dir(tree)\n"
"   if type(tree) == \"string\" then\n"
"      return dir.path(tree, cfg.lib_modules_path)\n"
"   else\n"
"      assert(type(tree) == \"table\")\n"
"      return tree.lib_dir or dir.path(tree.root, cfg.lib_modules_path)\n"
"   end\n"
"end\n"
"\n"
"local is_src_extension = { [\".lua\"] = true, [\".tl\"] = true, [\".tld\"] = true, [\".moon\"] = true }\n"
"\n"
"--- Return the pathname of the file that would be loaded for a module, indexed.\n"
"-- @param file_name string: module file name as in manifest (eg. \"socket/core.so\")\n"
"-- @param name string: name of the package (eg. \"luasocket\")\n"
"-- @param version string: version number (eg. \"2.0.2-1\")\n"
"-- @param tree string: repository path (eg. \"/usr/local\")\n"
"-- @param i number: the index, 1 if version is the current default, > 1 otherwise.\n"
"-- This is done this way for use by select_module in luarocks.loader.\n"
"-- @return string: filename of the module (eg. \"/usr/local/lib/lua/5.1/socket/core.so\")\n"
"function path.which_i(file_name, name, version, tree, i)\n"
"   local deploy_dir\n"
"   local extension = file_name:match(\"%.[a-z]+$\")\n"
"   if is_src_extension[extension] then\n"
"      deploy_dir = path.deploy_lua_dir(tree)\n"
"      file_name = dir.path(deploy_dir, file_name)\n"
"   else\n"
"      deploy_dir = path.deploy_lib_dir(tree)\n"
"      file_name = dir.path(deploy_dir, file_name)\n"
"   end\n"
"   if i > 1 then\n"
"      file_name = path.versioned_name(file_name, deploy_dir, name, version)\n"
"   end\n"
"   return file_name\n"
"end\n"
"\n"
"return path\n"
""
;
