const char luarocks_core_persist_lua[] =
"\n"
"local persist = {}\n"
"\n"
"local require = nil\n"
"--------------------------------------------------------------------------------\n"
"\n"
"--- Load and run a Lua file in an environment.\n"
"-- @param filename string: the name of the file.\n"
"-- @param env table: the environment table.\n"
"-- @return (true, any) or (nil, string, string): true and the return value\n"
"-- of the file, or nil, an error message and an error code (\"open\", \"load\"\n"
"-- or \"run\") in case of errors.\n"
"local function run_file(filename, env)\n"
"   local fd, err = io.open(filename)\n"
"   if not fd then\n"
"      return nil, err, \"open\"\n"
"   end\n"
"   local str, err = fd:read(\"*a\")\n"
"   fd:close()\n"
"   if not str then\n"
"      return nil, err, \"open\"\n"
"   end\n"
"   str = str:gsub(\"^#![^\\n]*\\n\", \"\")\n"
"   local chunk, ran\n"
"   if _VERSION == \"Lua 5.1\" then -- Lua 5.1\n"
"      chunk, err = loadstring(str, filename)\n"
"      if chunk then\n"
"         setfenv(chunk, env)\n"
"         ran, err = pcall(chunk)\n"
"      end\n"
"   else -- Lua 5.2\n"
"      chunk, err = load(str, filename, \"t\", env)\n"
"      if chunk then\n"
"         ran, err = pcall(chunk)\n"
"      end\n"
"   end\n"
"   if not chunk then\n"
"      return nil, \"Error loading file: \"..err, \"load\"\n"
"   end\n"
"   if not ran then\n"
"      return nil, \"Error running file: \"..err, \"run\"\n"
"   end\n"
"   return true, err\n"
"end\n"
"\n"
"--- Load a Lua file containing assignments, storing them in a table.\n"
"-- The global environment is not propagated to the loaded file.\n"
"-- @param filename string: the name of the file.\n"
"-- @param tbl table or nil: if given, this table is used to store\n"
"-- loaded values.\n"
"-- @return (table, table) or (nil, string, string): a table with the file's assignments\n"
"-- as fields and set of undefined globals accessed in file,\n"
"-- or nil, an error message and an error code (\"open\"; couldn't open the file,\n"
"-- \"load\"; compile-time error, or \"run\"; run-time error)\n"
"-- in case of errors.\n"
"function persist.load_into_table(filename, tbl)\n"
"   assert(type(filename) == \"string\")\n"
"   assert(type(tbl) == \"table\" or not tbl)\n"
"\n"
"   local result = tbl or {}\n"
"   local globals = {}\n"
"   local globals_mt = {\n"
"      __index = function(t, k)\n"
"         globals[k] = true\n"
"      end\n"
"   }\n"
"   local save_mt = getmetatable(result)\n"
"   setmetatable(result, globals_mt)\n"
"   \n"
"   local ok, err, errcode = run_file(filename, result)\n"
"   \n"
"   setmetatable(result, save_mt)\n"
"\n"
"   if not ok then\n"
"      return nil, err, errcode\n"
"   end\n"
"   return result, globals\n"
"end\n"
"\n"
"return persist\n"
"\n"
""
;
