const char luarocks_core_sysdetect_lua[] =
"-- Detect the operating system and architecture without forking a subprocess.\n"
"--\n"
"-- We are not going for exhaustive list of every historical system here,\n"
"-- but aiming to cover every platform where LuaRocks is known to run.\n"
"-- If your system is not detected, patches are welcome!\n"
"\n"
"local sysdetect = {}\n"
"\n"
"local function hex(s)\n"
"   return s:gsub(\"$(..)\", function(x)\n"
"      return string.char(tonumber(x, 16))\n"
"   end)\n"
"end\n"
"\n"
"local function read_int8(fd)\n"
"   if io.type(fd) == \"closed file\" then\n"
"      return nil\n"
"   end   \n"
"   local s = fd:read(1)\n"
"   if not s then\n"
"      fd:close()\n"
"      return nil\n"
"   end\n"
"   return s:byte()\n"
"end\n"
"\n"
"local LITTLE = 1\n"
"-- local BIG = 2\n"
"\n"
"local function bytes2number(s, endian)\n"
"   local r = 0\n"
"   if endian == LITTLE then\n"
"      for i = #s, 1, -1 do\n"
"         r = r*256 + s:byte(i,i)\n"
"      end\n"
"   else\n"
"      for i = 1, #s do\n"
"         r = r*256 + s:byte(i,i)\n"
"      end\n"
"   end\n"
"   return r\n"
"end\n"
"\n"
"local function read(fd, bytes, endian)\n"
"   if io.type(fd) == \"closed file\" then\n"
"      return nil\n"
"   end   \n"
"   local s = fd:read(bytes)\n"
"   if not s\n"
"      then fd:close()\n"
"      return nil\n"
"   end\n"
"   return bytes2number(s, endian)\n"
"end\n"
"\n"
"local function read_int32le(fd)\n"
"   return read(fd, 4, LITTLE)\n"
"end\n"
"\n"
"--------------------------------------------------------------------------------\n"
"-- @section ELF\n"
"--------------------------------------------------------------------------------\n"
"\n"
"local e_osabi = {\n"
"   [0x00] = \"sysv\",\n"
"   [0x01] = \"hpux\",\n"
"   [0x02] = \"netbsd\",\n"
"   [0x03] = \"linux\",\n"
"   [0x04] = \"hurd\",\n"
"   [0x06] = \"solaris\",\n"
"   [0x07] = \"aix\",\n"
"   [0x08] = \"irix\",\n"
"   [0x09] = \"freebsd\",\n"
"   [0x0c] = \"openbsd\",\n"
"}\n"
"\n"
"local e_machines = {\n"
"   [0x02] = \"sparc\",\n"
"   [0x03] = \"x86\",\n"
"   [0x08] = \"mips\",\n"
"   [0x0f] = \"hppa\",\n"
"   [0x12] = \"sparcv8\",\n"
"   [0x14] = \"ppc\",\n"
"   [0x15] = \"ppc64\",\n"
"   [0x16] = \"s390\",\n"
"   [0x28] = \"arm\",\n"
"   [0x2a] = \"superh\",\n"
"   [0x2b] = \"sparcv9\",\n"
"   [0x32] = \"ia_64\",\n"
"   [0x3E] = \"x86_64\",\n"
"   [0xB6] = \"alpha\",\n"
"   [0xB7] = \"aarch64\",\n"
"   [0xF3] = \"riscv64\",\n"
"   [0x9026] = \"alpha\",\n"
"}\n"
"\n"
"local SHT_NOTE = 7\n"
"\n"
"local function read_elf_section_headers(fd, hdr)\n"
"   local endian = hdr.endian\n"
"   local word = hdr.word\n"
"\n"
"   local strtab_offset\n"
"   local sections = {}\n"
"   for i = 0, hdr.e_shnum - 1 do\n"
"      fd:seek(\"set\", hdr.e_shoff + (i * hdr.e_shentsize))\n"
"      local section = {}\n"
"      section.sh_name_off = read(fd, 4, endian)\n"
"      section.sh_type = read(fd, 4, endian)\n"
"      section.sh_flags = read(fd, word, endian)\n"
"      section.sh_addr = read(fd, word, endian)\n"
"      section.sh_offset = read(fd, word, endian)\n"
"      section.sh_size = read(fd, word, endian)\n"
"      section.sh_link = read(fd, 4, endian)\n"
"      section.sh_info = read(fd, 4, endian)\n"
"      if section.sh_type == SHT_NOTE then\n"
"         fd:seek(\"set\", section.sh_offset)\n"
"         section.namesz = read(fd, 4, endian)\n"
"         section.descsz = read(fd, 4, endian)\n"
"         section.type = read(fd, 4, endian)\n"
"         section.namedata = fd:read(section.namesz):gsub(\"%z.*\", \"\")\n"
"         section.descdata = fd:read(section.descsz)\n"
"      elseif i == hdr.e_shstrndx then\n"
"         strtab_offset = section.sh_offset\n"
"      end\n"
"      table.insert(sections, section)\n"
"   end\n"
"   if strtab_offset then\n"
"      for _, section in ipairs(sections) do\n"
"         fd:seek(\"set\", strtab_offset + section.sh_name_off)\n"
"         section.name = fd:read(32):gsub(\"%z.*\", \"\")\n"
"         sections[section.name] = section\n"
"      end\n"
"   end\n"
"   return sections\n"
"end\n"
"\n"
"local function detect_elf_system(fd, hdr, sections)\n"
"   local system = e_osabi[hdr.osabi]\n"
"   local endian = hdr.endian\n"
"   \n"
"   if system == \"sysv\" then\n"
"      local abitag = sections[\".note.ABI-tag\"]\n"
"      if abitag then\n"
"         if abitag.namedata == \"GNU\" and abitag.type == 1\n"
"           and abitag.descdata:sub(0, 4) == \"\\0\\0\\0\\0\" then\n"
"            return \"linux\"\n"
"         end\n"
"      elseif sections[\".SUNW_version\"]\n"
"        or sections[\".SUNW_signature\"] then\n"
"         return \"solaris\"\n"
"      elseif sections[\".note.netbsd.ident\"] then\n"
"         return \"netbsd\"\n"
"      elseif sections[\".note.openbsd.ident\"] then\n"
"         return \"openbsd\"\n"
"      end\n"
"      \n"
"      local gnu_version_r = sections[\".gnu.version_r\"]\n"
"      if gnu_version_r then\n"
"\n"
"         local dynstr = sections[\".dynstr\"].sh_offset\n"
"\n"
"         local idx = 0\n"
"         for _ = 0, gnu_version_r.sh_info - 1 do\n"
"            fd:seek(\"set\", gnu_version_r.sh_offset + idx)\n"
"            assert(read(fd, 2, endian)) -- vn_version\n"
"            local vn_cnt = read(fd, 2, endian)\n"
"            local vn_file = read(fd, 4, endian)\n"
"            local vn_next = read(fd, 2, endian)\n"
"\n"
"            fd:seek(\"set\", dynstr + vn_file)\n"
"            local libname = fd:read(64):gsub(\"%z.*\", \"\")\n"
"            \n"
"            if hdr.e_type == 0x03 and libname == \"libroot.so\" then\n"
"               return \"haiku\"\n"
"            elseif libname:match(\"linux\") then\n"
"               return \"linux\"\n"
"            end\n"
"\n"
"            idx = idx + (vn_next * (vn_cnt + 1))\n"
"         end\n"
"      end\n"
"\n"
"      local procfile = io.open(\"/proc/sys/kernel/ostype\")\n"
"      if procfile then\n"
"         local version = procfile:read(6)\n"
"         procfile:close()\n"
"         if version == \"Linux\\n\" then\n"
"            return \"linux\"\n"
"         end\n"
"      end\n"
"   end\n"
"   \n"
"   return system\n"
"end\n"
"\n"
"local function read_elf_header(fd)\n"
"   local hdr = {}\n"
"\n"
"   hdr.bits = read_int8(fd)\n"
"   hdr.endian = read_int8(fd)\n"
"   hdr.elf_version = read_int8(fd)\n"
"   if hdr.elf_version ~= 1 then\n"
"      return nil\n"
"   end\n"
"   hdr.osabi = read_int8(fd)\n"
"   if not hdr.osabi then\n"
"      return nil\n"
"   end\n"
"   \n"
"   local endian = hdr.endian\n"
"   fd:seek(\"set\", 0x10)\n"
"   hdr.e_type = read(fd, 2, endian)\n"
"   local machine = read(fd, 2, endian)\n"
"   local processor = e_machines[machine] or \"unknown\"\n"
"   if endian == 1 and processor == \"ppc64\" then\n"
"      processor = \"ppc64le\"\n"
"   end\n"
"\n"
"   local elfversion = read(fd, 4, endian)\n"
"   if elfversion ~= 1 then\n"
"      return nil\n"
"   end\n"
"   \n"
"   local word = (hdr.bits == 1) and 4 or 8\n"
"   hdr.word = word\n"
"   \n"
"   hdr.e_entry = read(fd, word, endian)\n"
"   hdr.e_phoff = read(fd, word, endian)\n"
"   hdr.e_shoff = read(fd, word, endian)\n"
"   hdr.e_flags = read(fd, 4, endian)\n"
"   hdr.e_ehsize = read(fd, 2, endian)\n"
"   hdr.e_phentsize = read(fd, 2, endian)\n"
"   hdr.e_phnum = read(fd, 2, endian)\n"
"   hdr.e_shentsize = read(fd, 2, endian)\n"
"   hdr.e_shnum = read(fd, 2, endian)\n"
"   hdr.e_shstrndx = read(fd, 2, endian)\n"
"   \n"
"   return hdr, processor\n"
"end\n"
"\n"
"local function detect_elf(fd)\n"
"   local hdr, processor = read_elf_header(fd)\n"
"   if not hdr then\n"
"      return nil\n"
"   end\n"
"   local sections = read_elf_section_headers(fd, hdr)\n"
"   local system = detect_elf_system(fd, hdr, sections)\n"
"   return system, processor\n"
"end\n"
"\n"
"--------------------------------------------------------------------------------\n"
"-- @section Mach Objects (Apple)\n"
"--------------------------------------------------------------------------------\n"
"\n"
"local mach_l64 = {\n"
"   [7] = \"x86_64\",\n"
"   [12] = \"aarch64\",\n"
"}\n"
"\n"
"local mach_b64 = {\n"
"   [0] = \"ppc64\",\n"
"}\n"
"\n"
"local mach_l32 = {\n"
"   [7] = \"x86\",\n"
"   [12] = \"arm\",\n"
"}\n"
"\n"
"local mach_b32 = {\n"
"   [0] = \"ppc\",\n"
"}\n"
"\n"
"local function detect_mach(magic, fd)\n"
"   if not magic then\n"
"      return nil\n"
"   end\n"
"\n"
"   if magic == hex(\"$CA$FE$BA$BE\") then\n"
"      -- fat binary, go for the first one\n"
"      fd:seek(\"set\", 0x12)\n"
"      local offs = read_int8(fd)\n"
"      if not offs then\n"
"         return nil\n"
"      end\n"
"      fd:seek(\"set\", offs * 256)\n"
"      magic = fd:read(4)\n"
"      return detect_mach(magic, fd)\n"
"   end\n"
"\n"
"   local cputype = read_int8(fd)\n"
"\n"
"   if magic == hex(\"$CF$FA$ED$FE\") then\n"
"      return \"macosx\", mach_l64[cputype] or \"unknown\"\n"
"   elseif magic == hex(\"$FE$ED$CF$FA\") then\n"
"      return \"macosx\", mach_b64[cputype] or \"unknown\"\n"
"   elseif magic == hex(\"$CE$FA$ED$FE\") then\n"
"      return \"macosx\", mach_l32[cputype] or \"unknown\"\n"
"   elseif magic == hex(\"$FE$ED$FA$CE\") then\n"
"      return \"macosx\", mach_b32[cputype] or \"unknown\"\n"
"   end\n"
"end\n"
"\n"
"--------------------------------------------------------------------------------\n"
"-- @section PE (Windows)\n"
"--------------------------------------------------------------------------------\n"
"\n"
"local pe_machine = {\n"
"    [0x8664] = \"x86_64\",\n"
"    [0x01c0] = \"arm\",\n"
"    [0x01c4] = \"armv7l\",\n"
"    [0xaa64] = \"arm64\",\n"
"    [0x014c] = \"x86\",\n"
"}\n"
"\n"
"local function detect_pe(fd)\n"
"   fd:seek(\"set\", 60)                 -- position of PE header position\n"
"   local peoffset = read_int32le(fd)  -- read position of PE header\n"
"   if not peoffset then\n"
"      return nil\n"
"   end\n"
"   local system = \"windows\"\n"
"   fd:seek(\"set\", peoffset + 4)       -- move to position of Machine section\n"
"   local machine = read(fd, 2, LITTLE)\n"
"   local processor = pe_machine[machine]\n"
"  \n"
"   local rdata_pos = fd:read(736):match(\".rdata%z%z............(....)\")\n"
"   if rdata_pos then\n"
"      rdata_pos = bytes2number(rdata_pos, LITTLE)\n"
"      fd:seek(\"set\", rdata_pos)\n"
"      local data = fd:read(512)\n"
"      if data:match(\"cyggcc\") then\n"
"         system = \"cygwin\"\n"
"      end\n"
"   end\n"
"   \n"
"   return system, processor or \"unknown\"\n"
"end\n"
"\n"
"--------------------------------------------------------------------------------\n"
"-- @section API\n"
"--------------------------------------------------------------------------------\n"
"\n"
"function sysdetect.detect_file(file)\n"
"   assert(type(file) == \"string\")\n"
"   local fd = io.open(file, \"rb\")\n"
"   if not fd then\n"
"      return nil\n"
"   end\n"
"   local magic = fd:read(4)\n"
"   if magic == hex(\"$7FELF\") then\n"
"      return detect_elf(fd)\n"
"   end\n"
"   if magic == hex(\"MZ$90$00\") then\n"
"      return detect_pe(fd)\n"
"   end\n"
"   return detect_mach(magic, fd)\n"
"end\n"
"\n"
"local cache_system\n"
"local cache_processor\n"
"\n"
"function sysdetect.detect(input_file)\n"
"   local dirsep = package.config:sub(1,1)\n"
"   local files\n"
"   \n"
"   local arg = rawget(_G, 'arg') or {}\n"
"   if input_file then\n"
"      files = { input_file }\n"
"   else\n"
"      if cache_system then\n"
"         return cache_system, cache_processor\n"
"      end\n"
"\n"
"      local PATHsep\n"
"      local interp = arg and arg[-1]\n"
"      if dirsep == \"/\" then\n"
"         -- Unix\n"
"         files = {\n"
"            \"/bin/sh\", -- Unix: well-known POSIX path\n"
"            \"/proc/self/exe\", -- Linux: this should always have a working binary\n"
"         }\n"
"         PATHsep = \":\"\n"
"      else\n"
"         -- Windows\n"
"         local systemroot = os.getenv(\"SystemRoot\")\n"
"         files = {\n"
"            systemroot .. \"\\\\system32\\\\notepad.exe\", -- well-known Windows path\n"
"            systemroot .. \"\\\\explorer.exe\", -- well-known Windows path\n"
"         }\n"
"         if interp and not interp:lower():match(\"exe$\") then\n"
"            interp = interp .. \".exe\"\n"
"         end\n"
"         PATHsep = \";\"\n"
"      end\n"
"      if interp then\n"
"         if interp:match(dirsep) then\n"
"            -- interpreter path is absolute\n"
"            table.insert(files, interp)\n"
"         else\n"
"            for d in os.getenv(\"PATH\"):gmatch(\"[^\"..PATHsep..\"]+\") do\n"
"               table.insert(files, d .. dirsep .. interp)\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"   for _, f in ipairs(files) do\n"
"      local system, processor = sysdetect.detect_file(f)\n"
"      if system then\n"
"         cache_system = system\n"
"         cache_processor = processor\n"
"         return system, processor\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"return sysdetect\n"
""
;
