const char luarocks_core_vers_lua[] =
"\n"
"local vers = {}\n"
"\n"
"local util = require(\"luarocks.core.util\")\n"
"local require = nil\n"
"--------------------------------------------------------------------------------\n"
"\n"
"local deltas = {\n"
"   dev =    120000000,\n"
"   scm =    110000000,\n"
"   cvs =    100000000,\n"
"   rc =    -1000,\n"
"   pre =   -10000,\n"
"   beta =  -100000,\n"
"   alpha = -1000000\n"
"}\n"
"\n"
"local version_mt = {\n"
"   --- Equality comparison for versions.\n"
"   -- All version numbers must be equal.\n"
"   -- If both versions have revision numbers, they must be equal;\n"
"   -- otherwise the revision number is ignored.\n"
"   -- @param v1 table: version table to compare.\n"
"   -- @param v2 table: version table to compare.\n"
"   -- @return boolean: true if they are considered equivalent.\n"
"   __eq = function(v1, v2)\n"
"      if #v1 ~= #v2 then\n"
"         return false\n"
"      end\n"
"      for i = 1, #v1 do\n"
"         if v1[i] ~= v2[i] then\n"
"            return false\n"
"         end\n"
"      end\n"
"      if v1.revision and v2.revision then\n"
"         return (v1.revision == v2.revision)\n"
"      end\n"
"      return true\n"
"   end,\n"
"   --- Size comparison for versions.\n"
"   -- All version numbers are compared.\n"
"   -- If both versions have revision numbers, they are compared;\n"
"   -- otherwise the revision number is ignored.\n"
"   -- @param v1 table: version table to compare.\n"
"   -- @param v2 table: version table to compare.\n"
"   -- @return boolean: true if v1 is considered lower than v2.\n"
"   __lt = function(v1, v2)\n"
"      for i = 1, math.max(#v1, #v2) do\n"
"         local v1i, v2i = v1[i] or 0, v2[i] or 0\n"
"         if v1i ~= v2i then\n"
"            return (v1i < v2i)\n"
"         end\n"
"      end\n"
"      if v1.revision and v2.revision then\n"
"         return (v1.revision < v2.revision)\n"
"      end\n"
"      return false\n"
"   end,\n"
"   --- Return version as a string.\n"
"   -- @param v The version table. \n"
"   -- @return The string representation.\n"
"   __tostring = function(v)\n"
"      return v.string\n"
"   end,  \n"
"}\n"
"\n"
"local version_cache = {}\n"
"setmetatable(version_cache, {\n"
"   __mode = \"kv\"\n"
"})\n"
"\n"
"--- Parse a version string, converting to table format.\n"
"-- A version table contains all components of the version string\n"
"-- converted to numeric format, stored in the array part of the table.\n"
"-- If the version contains a revision, it is stored numerically\n"
"-- in the 'revision' field. The original string representation of\n"
"-- the string is preserved in the 'string' field.\n"
"-- Returned version tables use a metatable\n"
"-- allowing later comparison through relational operators.\n"
"-- @param vstring string: A version number in string format.\n"
"-- @return table or nil: A version table or nil\n"
"-- if the input string contains invalid characters.\n"
"function vers.parse_version(vstring)\n"
"   if not vstring then return nil end\n"
"   assert(type(vstring) == \"string\")\n"
"\n"
"   local cached = version_cache[vstring]\n"
"   if cached then\n"
"      return cached\n"
"   end\n"
"\n"
"   local version = {}\n"
"   local i = 1\n"
"\n"
"   local function add_token(number)\n"
"      version[i] = version[i] and version[i] + number/100000 or number\n"
"      i = i + 1\n"
"   end\n"
"   \n"
"   -- trim leading and trailing spaces\n"
"   local v = vstring:match(\"^%s*(.*)%s*$\")\n"
"   version.string = v\n"
"   -- store revision separately if any\n"
"   local main, revision = v:match(\"(.*)%-(%d+)$\")\n"
"   if revision then\n"
"      v = main\n"
"      version.revision = tonumber(revision)\n"
"   end\n"
"   while #v > 0 do\n"
"      -- extract a number\n"
"      local token, rest = v:match(\"^(%d+)[%.%-%_]*(.*)\")\n"
"      if token then\n"
"         add_token(tonumber(token))\n"
"      else\n"
"         -- extract a word\n"
"         token, rest = v:match(\"^(%a+)[%.%-%_]*(.*)\")\n"
"         if not token then\n"
"            util.warning(\"version number '\"..v..\"' could not be parsed.\")\n"
"            version[i] = 0\n"
"            break\n"
"         end\n"
"         version[i] = deltas[token] or (token:byte() / 1000)\n"
"      end\n"
"      v = rest\n"
"   end\n"
"   setmetatable(version, version_mt)\n"
"   version_cache[vstring] = version\n"
"   return version\n"
"end\n"
"\n"
"--- Utility function to compare version numbers given as strings.\n"
"-- @param a string: one version.\n"
"-- @param b string: another version.\n"
"-- @return boolean: True if a > b.\n"
"function vers.compare_versions(a, b)\n"
"   if a == b then\n"
"      return false\n"
"   end\n"
"   return vers.parse_version(a) > vers.parse_version(b)\n"
"end\n"
"\n"
"--- A more lenient check for equivalence between versions.\n"
"-- This returns true if the requested components of a version\n"
"-- match and ignore the ones that were not given. For example,\n"
"-- when requesting \"2\", then \"2\", \"2.1\", \"2.3.5-9\"... all match.\n"
"-- When requesting \"2.1\", then \"2.1\", \"2.1.3\" match, but \"2.2\"\n"
"-- doesn't.\n"
"-- @param version string or table: Version to be tested; may be\n"
"-- in string format or already parsed into a table.\n"
"-- @param requested string or table: Version requested; may be\n"
"-- in string format or already parsed into a table.\n"
"-- @return boolean: True if the tested version matches the requested\n"
"-- version, false otherwise.\n"
"local function partial_match(version, requested)\n"
"   assert(type(version) == \"string\" or type(version) == \"table\")\n"
"   assert(type(requested) == \"string\" or type(version) == \"table\")\n"
"\n"
"   if type(version) ~= \"table\" then version = vers.parse_version(version) end\n"
"   if type(requested) ~= \"table\" then requested = vers.parse_version(requested) end\n"
"   if not version or not requested then return false end\n"
"   \n"
"   for i, ri in ipairs(requested) do\n"
"      local vi = version[i] or 0\n"
"      if ri ~= vi then return false end\n"
"   end\n"
"   if requested.revision then\n"
"      return requested.revision == version.revision\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"--- Check if a version satisfies a set of constraints.\n"
"-- @param version table: A version in table format\n"
"-- @param constraints table: An array of constraints in table format.\n"
"-- @return boolean: True if version satisfies all constraints,\n"
"-- false otherwise.\n"
"function vers.match_constraints(version, constraints)\n"
"   assert(type(version) == \"table\")\n"
"   assert(type(constraints) == \"table\")\n"
"   local ok = true\n"
"   setmetatable(version, version_mt)\n"
"   for _, constr in pairs(constraints) do\n"
"      if type(constr.version) == \"string\" then\n"
"         constr.version = vers.parse_version(constr.version)\n"
"      end\n"
"      local constr_version, constr_op = constr.version, constr.op\n"
"      setmetatable(constr_version, version_mt)\n"
"      if     constr_op == \"==\" then ok = version == constr_version\n"
"      elseif constr_op == \"~=\" then ok = version ~= constr_version\n"
"      elseif constr_op == \">\"  then ok = version >  constr_version\n"
"      elseif constr_op == \"<\"  then ok = version <  constr_version\n"
"      elseif constr_op == \">=\" then ok = version >= constr_version\n"
"      elseif constr_op == \"<=\" then ok = version <= constr_version\n"
"      elseif constr_op == \"~>\" then ok = partial_match(version, constr_version)\n"
"      end\n"
"      if not ok then break end\n"
"   end\n"
"   return ok\n"
"end\n"
"\n"
"return vers\n"
""
;
