const char luarocks_dir_lua[] =
"\n"
"--- Generic utilities for handling pathnames.\n"
"local dir = {}\n"
"\n"
"local core = require(\"luarocks.core.dir\")\n"
"\n"
"dir.path = core.path\n"
"dir.split_url = core.split_url\n"
"\n"
"--- Strip the path off a path+filename.\n"
"-- @param pathname string: A path+name, such as \"/a/b/c\"\n"
"-- or \"\\a\\b\\c\".\n"
"-- @return string: The filename without its path, such as \"c\".\n"
"function dir.base_name(pathname)\n"
"   assert(type(pathname) == \"string\")\n"
"\n"
"   local base = pathname:gsub(\"[/\\\\]*$\", \"\"):match(\".*[/\\\\]([^/\\\\]*)\")\n"
"   return base or pathname\n"
"end\n"
"\n"
"--- Strip the name off a path+filename.\n"
"-- @param pathname string: A path+name, such as \"/a/b/c\".\n"
"-- @return string: The filename without its path, such as \"/a/b\".\n"
"-- For entries such as \"/a/b/\", \"/a\" is returned. If there are\n"
"-- no directory separators in input, \"\" is returned.\n"
"function dir.dir_name(pathname)\n"
"   assert(type(pathname) == \"string\")\n"
"   return (pathname:gsub(\"/*$\", \"\"):match(\"(.*)[/]+[^/]*\")) or \"\"\n"
"end\n"
"\n"
"--- Normalize a url or local path.\n"
"-- URLs should be in the \"protocol://path\" format. System independent\n"
"-- forward slashes are used, removing trailing and double slashes\n"
"-- @param url string: an URL or a local pathname.\n"
"-- @return string: Normalized result.\n"
"function dir.normalize(name)\n"
"   local protocol, pathname = dir.split_url(name)\n"
"   pathname = pathname:gsub(\"\\\\\", \"/\"):gsub(\"(.)/*$\", \"%1\"):gsub(\"//\", \"/\")\n"
"   if protocol ~= \"file\" then pathname = protocol ..\"://\"..pathname end\n"
"   return pathname\n"
"end\n"
"\n"
"--- Returns true if protocol does not require additional tools.\n"
"-- @param protocol The protocol name\n"
"function dir.is_basic_protocol(protocol)\n"
"   return protocol == \"http\" or protocol == \"https\" or protocol == \"ftp\" or protocol == \"file\"\n"
"end\n"
"\n"
"function dir.deduce_base_dir(url)\n"
"   -- for extensions like foo.tar.gz, \"gz\" is stripped first\n"
"   local known_exts = {}\n"
"   for _, ext in ipairs{\"zip\", \"git\", \"tgz\", \"tar\", \"gz\", \"bz2\"} do\n"
"      known_exts[ext] = \"\"\n"
"   end\n"
"   local base = dir.base_name(url)\n"
"   return (base:gsub(\"%.([^.]*)$\", known_exts):gsub(\"%.tar\", \"\"))\n"
"end\n"
"\n"
"return dir\n"
""
;
