const char luarocks_fetch_cvs_lua[] =
"\n"
"--- Fetch back-end for retrieving sources from CVS.\n"
"local cvs = {}\n"
"\n"
"local unpack = unpack or table.unpack\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local util = require(\"luarocks.util\")\n"
"\n"
"--- Download sources for building a rock, using CVS.\n"
"-- @param rockspec table: The rockspec table\n"
"-- @param extract boolean: Unused in this module (required for API purposes.)\n"
"-- @param dest_dir string or nil: If set, will extract to the given directory.\n"
"-- @return (string, string) or (nil, string): The absolute pathname of\n"
"-- the fetched source tarball and the temporary directory created to\n"
"-- store it; or nil and an error message.\n"
"function cvs.get_sources(rockspec, extract, dest_dir)\n"
"   assert(rockspec:type() == \"rockspec\")\n"
"   assert(type(dest_dir) == \"string\" or not dest_dir)\n"
"\n"
"   local cvs_cmd = rockspec.variables.CVS\n"
"   local ok, err_msg = fs.is_tool_available(cvs_cmd, \"CVS\")\n"
"   if not ok then\n"
"      return nil, err_msg\n"
"   end\n"
"\n"
"   local name_version = rockspec.name .. \"-\" .. rockspec.version\n"
"   local module = rockspec.source.module or dir.base_name(rockspec.source.url)\n"
"   local command = {cvs_cmd, \"-d\"..rockspec.source.pathname, \"export\", module}\n"
"   if rockspec.source.tag then\n"
"      table.insert(command, 4, \"-r\")\n"
"      table.insert(command, 5, rockspec.source.tag)\n"
"   end\n"
"   local store_dir\n"
"   if not dest_dir then\n"
"      store_dir = fs.make_temp_dir(name_version)\n"
"      if not store_dir then\n"
"         return nil, \"Failed creating temporary directory.\"\n"
"      end\n"
"      util.schedule_function(fs.delete, store_dir)\n"
"   else\n"
"      store_dir = dest_dir\n"
"   end\n"
"   local ok, err = fs.change_dir(store_dir)\n"
"   if not ok then return nil, err end\n"
"   if not fs.execute(unpack(command)) then\n"
"      return nil, \"Failed fetching files from CVS.\"\n"
"   end\n"
"   fs.pop_dir()\n"
"   return module, store_dir\n"
"end\n"
"\n"
"\n"
"return cvs\n"
""
;
