const char luarocks_fetch_git_ssh_lua[] =
"--- Fetch back-end for retrieving sources from Git repositories\n"
"-- that use ssh:// transport. For example, for fetching a repository\n"
"-- that requires the following command line:\n"
"-- `git clone ssh://git@example.com/path/foo.git\n"
"-- you can use this in the rockspec:\n"
"-- source = { url = \"git+ssh://git@example.com/path/foo.git\" }\n"
"-- It also handles scp-style ssh urls: git@example.com:path/foo.git,\n"
"-- but you have to prepend the \"git+ssh://\" and why not use the \"newer\"\n"
"-- style anyway\?\n"
"local git_ssh = {}\n"
"\n"
"local git = require(\"luarocks.fetch.git\")\n"
"\n"
"--- Fetch sources for building a rock from a local Git repository.\n"
"-- @param rockspec table: The rockspec table\n"
"-- @param extract boolean: Unused in this module (required for API purposes.)\n"
"-- @param dest_dir string or nil: If set, will extract to the given directory.\n"
"-- @return (string, string) or (nil, string): The absolute pathname of\n"
"-- the fetched source tarball and the temporary directory created to\n"
"-- store it; or nil and an error message.\n"
"function git_ssh.get_sources(rockspec, extract, dest_dir)\n"
"   rockspec.source.url = rockspec.source.url:gsub(\"^git.\", \"\")\n"
"\n"
"   -- Handle old-style scp-like git ssh urls\n"
"   if rockspec.source.url:match(\"^ssh://[^/]+:[^%d]\") then\n"
"      rockspec.source.url = rockspec.source.url:gsub(\"^ssh://\", \"\")\n"
"   end\n"
"\n"
"   return git.get_sources(rockspec, extract, dest_dir, \"--\")\n"
"end\n"
"\n"
"return git_ssh\n"
""
;
