const char luarocks_fetch_hg_lua[] =
"\n"
"--- Fetch back-end for retrieving sources from HG.\n"
"local hg = {}\n"
"\n"
"local unpack = unpack or table.unpack\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local util = require(\"luarocks.util\")\n"
"\n"
"--- Download sources for building a rock, using hg.\n"
"-- @param rockspec table: The rockspec table\n"
"-- @param extract boolean: Unused in this module (required for API purposes.)\n"
"-- @param dest_dir string or nil: If set, will extract to the given directory.\n"
"-- @return (string, string) or (nil, string): The absolute pathname of\n"
"-- the fetched source tarball and the temporary directory created to\n"
"-- store it; or nil and an error message.\n"
"function hg.get_sources(rockspec, extract, dest_dir)\n"
"   assert(rockspec:type() == \"rockspec\")\n"
"   assert(type(dest_dir) == \"string\" or not dest_dir)\n"
"\n"
"   local hg_cmd = rockspec.variables.HG\n"
"   local ok, err_msg = fs.is_tool_available(hg_cmd, \"Mercurial\")\n"
"   if not ok then\n"
"      return nil, err_msg\n"
"   end\n"
"\n"
"   local name_version = rockspec.name .. \"-\" .. rockspec.version\n"
"   -- Strip off special hg:// protocol type\n"
"   local url = rockspec.source.url:gsub(\"^hg://\", \"\")\n"
"\n"
"   local module = dir.base_name(url)\n"
"\n"
"   local command = {hg_cmd, \"clone\", url, module}\n"
"   local tag_or_branch = rockspec.source.tag or rockspec.source.branch\n"
"   if tag_or_branch then\n"
"      command = {hg_cmd, \"clone\", \"--rev\", tag_or_branch, url, module}\n"
"   end\n"
"   local store_dir\n"
"   if not dest_dir then\n"
"      store_dir = fs.make_temp_dir(name_version)\n"
"      if not store_dir then\n"
"         return nil, \"Failed creating temporary directory.\"\n"
"      end\n"
"      util.schedule_function(fs.delete, store_dir)\n"
"   else\n"
"      store_dir = dest_dir\n"
"   end\n"
"   local ok, err = fs.change_dir(store_dir)\n"
"   if not ok then return nil, err end\n"
"   if not fs.execute(unpack(command)) then\n"
"      return nil, \"Failed cloning hg repository.\"\n"
"   end\n"
"   ok, err = fs.change_dir(module)\n"
"   if not ok then return nil, err end\n"
"\n"
"   fs.delete(dir.path(store_dir, module, \".hg\"))\n"
"   fs.delete(dir.path(store_dir, module, \".hgignore\"))\n"
"   fs.pop_dir()\n"
"   fs.pop_dir()\n"
"   return module, store_dir\n"
"end\n"
"\n"
"\n"
"return hg\n"
""
;
