const char luarocks_fetch_sscm_lua[] =
"\n"
"--- Fetch back-end for retrieving sources from Surround SCM Server\n"
"local sscm = {}\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"local dir = require(\"luarocks.dir\")\n"
"\n"
"--- Download sources via Surround SCM Server for building a rock.\n"
"-- @param rockspec table: The rockspec table\n"
"-- @param extract boolean: Unused in this module (required for API purposes.)\n"
"-- @param dest_dir string or nil: If set, will extract to the given directory.\n"
"-- @return (string, string) or (nil, string): The absolute pathname of\n"
"-- the fetched source tarball and the temporary directory created to\n"
"-- store it; or nil and an error message.\n"
"function sscm.get_sources(rockspec, extract, dest_dir)\n"
"   assert(rockspec:type() == \"rockspec\")\n"
"   assert(type(dest_dir) == \"string\" or not dest_dir)\n"
"\n"
"   local sscm_cmd = rockspec.variables.SSCM\n"
"   local module = rockspec.source.module or dir.base_name(rockspec.source.url)\n"
"   local branch, repository = string.match(rockspec.source.pathname, \"^([^/]*)/(.*)\")\n"
"   if not branch or not repository then\n"
"      return nil, \"Error retrieving branch and repository from rockspec.\"\n"
"   end\n"
"   -- Search for working directory.\n"
"   local working_dir\n"
"   local tmp = io.popen(string.format(sscm_cmd..[[ property \"/\" -d -b%s -p%s]], branch, repository))\n"
"   for line in tmp:lines() do\n"
"      --%c because a chr(13) comes in the end.\n"
"      working_dir = string.match(line, \"Working directory:[%s]*(.*)%c$\")\n"
"      if working_dir then break end\n"
"   end\n"
"   tmp:close()\n"
"   if not working_dir then\n"
"      return nil, \"Error retrieving working directory from SSCM.\"\n"
"   end\n"
"   if not fs.execute(sscm_cmd, \"get\", \"*\", \"-e\" , \"-r\", \"-b\"..branch, \"-p\"..repository, \"-tmodify\", \"-wreplace\") then\n"
"      return nil, \"Failed fetching files from SSCM.\"\n"
"   end\n"
"   -- FIXME: This function does not honor the dest_dir parameter.\n"
"   return module, working_dir\n"
"end\n"
"\n"
"return sscm\n"
""
;
