const char luarocks_fs_lua[] =
"\n"
"--- Proxy module for filesystem and platform abstractions.\n"
"-- All code using \"fs\" code should require \"luarocks.fs\",\n"
"-- and not the various platform-specific implementations.\n"
"-- However, see the documentation of the implementation\n"
"-- for the API reference.\n"
"\n"
"local pairs = pairs\n"
"\n"
"local fs = {}\n"
"-- To avoid a loop when loading the other fs modules.\n"
"package.loaded[\"luarocks.fs\"] = fs\n"
"\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"\n"
"local pack = table.pack or function(...) return { n = select(\"#\", ...), ... } end\n"
"local unpack = table.unpack or unpack\n"
"\n"
"math.randomseed(os.time())\n"
"\n"
"do\n"
"   local old_popen, old_execute\n"
"\n"
"   -- patch io.popen and os.execute to display commands in verbose mode\n"
"   function fs.verbose()\n"
"      if old_popen or old_execute then return end\n"
"      old_popen = io.popen\n"
"      io.popen = function(one, two)\n"
"         if two == nil then\n"
"            print(\"\\nio.popen: \", one)\n"
"         else\n"
"            print(\"\\nio.popen: \", one, \"Mode:\", two)\n"
"         end\n"
"         return old_popen(one, two)\n"
"      end\n"
"\n"
"      old_execute = os.execute\n"
"      os.execute = function(cmd)\n"
"         -- redact api keys if present\n"
"         print(\"\\nos.execute: \", (cmd:gsub(\"(/api/[^/]+/)([^/]+)/\", function(cap, key) return cap..\"<redacted>/\" end)) )\n"
"         local code = pack(old_execute(cmd))\n"
"         print(\"Results: \"..tostring(code.n))\n"
"         for i = 1,code.n do\n"
"            print(\"  \"..tostring(i)..\" (\"..type(code[i])..\"): \"..tostring(code[i]))\n"
"         end\n"
"         return unpack(code, 1, code.n)\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"do\n"
"   local function load_fns(fs_table)\n"
"      for name, fn in pairs(fs_table) do\n"
"         if not fs[name] then\n"
"            fs[name] = fn\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   function fs.init()\n"
"      if fs.current_dir then\n"
"         -- already initialized\n"
"         return\n"
"      end\n"
"\n"
"      if not cfg.each_platform then\n"
"         error(\"cfg is not initialized, please run cfg.init() first\")\n"
"      end\n"
"\n"
"      -- Load platform-specific functions\n"
"      local loaded_platform = nil\n"
"      for platform in cfg.each_platform() do\n"
"         local ok, fs_plat = pcall(require, \"luarocks.fs.\"..platform)\n"
"         if ok and fs_plat then\n"
"            loaded_platform = platform\n"
"            load_fns(fs_plat)\n"
"            break\n"
"         end\n"
"      end\n"
"\n"
"      -- Load platform-independent pure-Lua functionality\n"
"      local fs_lua = require(\"luarocks.fs.lua\")\n"
"      load_fns(fs_lua)\n"
"\n"
"      -- Load platform-specific fallbacks for missing Lua modules\n"
"      local ok, fs_plat_tools = pcall(require, \"luarocks.fs.\"..loaded_platform..\".tools\")\n"
"      if ok and fs_plat_tools then\n"
"         load_fns(fs_plat_tools)\n"
"         load_fns(require(\"luarocks.fs.tools\"))\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"return fs\n"
""
;
