const char luarocks_fs_tools_lua[] =
"\n"
"--- Common fs operations implemented with third-party tools.\n"
"local tools = {}\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"\n"
"local vars = setmetatable({}, { __index = function(_,k) return cfg.variables[k] end })\n"
"\n"
"local dir_stack = {}\n"
"\n"
"do\n"
"   local tool_cache = {}\n"
"   \n"
"   local tool_options = {\n"
"      downloader = {\n"
"         { var = \"CURL\", name = \"curl\" },\n"
"         { var = \"WGET\", name = \"wget\" },\n"
"      },\n"
"      md5checker = {\n"
"         { var = \"MD5SUM\", name = \"md5sum\" },\n"
"         { var = \"OPENSSL\", name = \"openssl\", cmdarg = \"md5\", checkarg = \"version\" },\n"
"         { var = \"MD5\", name = \"md5\", checkarg = \"-shello\" },\n"
"      },\n"
"   }\n"
"\n"
"   function tools.which_tool(tooltype)\n"
"      local tool = tool_cache[tooltype]\n"
"      if not tool then\n"
"         for _, opt in ipairs(tool_options[tooltype]) do\n"
"            if fs.is_tool_available(vars[opt.var], opt.name, opt.checkarg) then\n"
"               tool = opt\n"
"               tool_cache[tooltype] = opt\n"
"               break\n"
"            end\n"
"         end\n"
"      end\n"
"      if not tool then\n"
"         return nil\n"
"      end\n"
"      return tool.name, vars[tool.var] .. (tool.cmdarg and \" \"..tool.cmdarg or \"\")\n"
"   end\n"
"end\n"
"\n"
"do\n"
"   local cache_pwd\n"
"   --- Obtain current directory.\n"
"   -- Uses the module's internal directory stack.\n"
"   -- @return string: the absolute pathname of the current directory.\n"
"   function tools.current_dir()\n"
"      local current = cache_pwd\n"
"      if not current then\n"
"         local pipe = io.popen(fs.quiet_stderr(vars.PWD))\n"
"         current = pipe:read(\"*l\")\n"
"         pipe:close()\n"
"         cache_pwd = current\n"
"      end\n"
"      for _, directory in ipairs(dir_stack) do\n"
"         current = fs.absolute_name(directory, current)\n"
"      end\n"
"      return current\n"
"   end\n"
"end\n"
"\n"
"--- Change the current directory.\n"
"-- Uses the module's internal directory stack. This does not have exact\n"
"-- semantics of chdir, as it does not handle errors the same way,\n"
"-- but works well for our purposes for now.\n"
"-- @param directory string: The directory to switch to.\n"
"-- @return boolean or (nil, string): true if successful, (nil, error message) if failed.\n"
"function tools.change_dir(directory)\n"
"   assert(type(directory) == \"string\")\n"
"   if fs.is_dir(directory) then\n"
"      table.insert(dir_stack, directory)\n"
"      return true\n"
"   end\n"
"   return nil, \"directory not found: \"..directory\n"
"end\n"
"\n"
"--- Change directory to root.\n"
"-- Allows leaving a directory (e.g. for deleting it) in\n"
"-- a crossplatform way.\n"
"function tools.change_dir_to_root()\n"
"   local curr_dir = fs.current_dir()\n"
"   if not curr_dir or not fs.is_dir(curr_dir) then\n"
"      return false\n"
"   end\n"
"   table.insert(dir_stack, \"/\")\n"
"   return true\n"
"end\n"
"\n"
"--- Change working directory to the previous in the directory stack.\n"
"function tools.pop_dir()\n"
"   local directory = table.remove(dir_stack)\n"
"   return directory ~= nil\n"
"end\n"
"\n"
"--- Run the given command.\n"
"-- The command is executed in the current directory in the directory stack.\n"
"-- @param cmd string: No quoting/escaping is applied to the command.\n"
"-- @return boolean: true if command succeeds (status code 0), false\n"
"-- otherwise.\n"
"function tools.execute_string(cmd)\n"
"   local current = fs.current_dir()\n"
"   if not current then return false end\n"
"   cmd = fs.command_at(current, cmd)\n"
"   local code = os.execute(cmd)\n"
"   if code == 0 or code == true then\n"
"      return true\n"
"   else\n"
"      return false\n"
"   end\n"
"end\n"
"\n"
"--- Internal implementation function for fs.dir.\n"
"-- Yields a filename on each iteration.\n"
"-- @param at string: directory to list\n"
"-- @return nil\n"
"function tools.dir_iterator(at)\n"
"   local pipe = io.popen(fs.command_at(at, vars.LS, true))\n"
"   for file in pipe:lines() do\n"
"      if file ~= \".\" and file ~= \"..\" then\n"
"         coroutine.yield(file)\n"
"      end\n"
"   end\n"
"   pipe:close()\n"
"end\n"
"\n"
"--- Download a remote file.\n"
"-- @param url string: URL to be fetched.\n"
"-- @param filename string or nil: this function attempts to detect the\n"
"-- resulting local filename of the remote file as the basename of the URL;\n"
"-- if that is not correct (due to a redirection, for example), the local\n"
"-- filename can be given explicitly as this second argument.\n"
"-- @param cache boolean: compare remote timestamps via HTTP HEAD prior to\n"
"-- re-downloading the file.\n"
"-- @return (boolean, string): true and the filename on success,\n"
"-- false and the error message on failure.\n"
"function tools.use_downloader(url, filename, cache)\n"
"   assert(type(url) == \"string\")\n"
"   assert(type(filename) == \"string\" or not filename)\n"
"\n"
"   filename = fs.absolute_name(filename or dir.base_name(url))\n"
"   \n"
"   local downloader = fs.which_tool(\"downloader\")\n"
"\n"
"   local ok\n"
"   if downloader == \"wget\" then\n"
"      local wget_cmd = vars.WGET..\" \"..vars.WGETNOCERTFLAG..\" --no-cache --user-agent=\\\"\"..cfg.user_agent..\" via wget\\\" --quiet \"\n"
"      if cfg.connection_timeout and cfg.connection_timeout > 0 then\n"
"        wget_cmd = wget_cmd .. \"--timeout=\"..tostring(cfg.connection_timeout)..\" --tries=1 \"\n"
"      end\n"
"      if cache then\n"
"         -- --timestamping is incompatible with --output-document,\n"
"         -- but that's not a problem for our use cases.\n"
"         fs.delete(filename .. \".unixtime\")\n"
"         fs.change_dir(dir.dir_name(filename))\n"
"         ok = fs.execute_quiet(wget_cmd..\" --timestamping \", url)\n"
"         fs.pop_dir()\n"
"      elseif filename then\n"
"         ok = fs.execute_quiet(wget_cmd..\" --output-document \", filename, url)\n"
"      else\n"
"         ok = fs.execute_quiet(wget_cmd, url)\n"
"      end\n"
"   elseif downloader == \"curl\" then\n"
"      local curl_cmd = vars.CURL..\" \"..vars.CURLNOCERTFLAG..\" -f -L --user-agent \\\"\"..cfg.user_agent..\" via curl\\\" \"\n"
"      if cfg.connection_timeout and cfg.connection_timeout > 0 then\n"
"        curl_cmd = curl_cmd .. \"--connect-timeout \"..tostring(cfg.connection_timeout)..\" \"\n"
"      end\n"
"      ok = fs.execute_string(fs.quiet_stderr(curl_cmd..fs.Q(url)..\" > \"..fs.Q(filename)))\n"
"   else\n"
"      return false, \"No downloader tool available -- please install 'wget' or 'curl' in your system\"\n"
"   end\n"
"   if ok then\n"
"      return true, filename\n"
"   else\n"
"      os.remove(filename)\n"
"      return false, \"Failed downloading \" .. url\n"
"   end\n"
"end\n"
"\n"
"--- Get the MD5 checksum for a file.\n"
"-- @param file string: The file to be computed.\n"
"-- @return string: The MD5 checksum or nil + message\n"
"function tools.get_md5(file)\n"
"   local ok, md5checker = fs.which_tool(\"md5checker\")\n"
"   if ok then\n"
"      local pipe = io.popen(md5checker..\" \"..fs.Q(fs.absolute_name(file)))\n"
"      local computed = pipe:read(\"*l\")\n"
"      pipe:close()\n"
"      if computed then\n"
"         computed = computed:match(\"(\"..(\"%x\"):rep(32)..\")\")\n"
"      end\n"
"      if computed then\n"
"         return computed\n"
"      else\n"
"         return nil, \"Failed to compute MD5 hash for file \"..tostring(fs.absolute_name(file))\n"
"      end\n"
"   else\n"
"      return false, \"No MD5 checking tool available -- please install 'md5', 'md5sum' or 'openssl' in your system\"\n"
"   end\n"
"end\n"
"\n"
"return tools\n"
""
;
