const char luarocks_fun_lua[] =
"\n"
"--- A set of basic functional utilities\n"
"local fun = {}\n"
"\n"
"local unpack = table.unpack or unpack\n"
"\n"
"function fun.concat(xs, ys)\n"
"   local rs = {}\n"
"   local n = #xs\n"
"   for i = 1, n do\n"
"      rs[i] = xs[i]\n"
"   end\n"
"   for i = 1, #ys do\n"
"      rs[i + n] = ys[i]\n"
"   end\n"
"   return rs\n"
"end\n"
"\n"
"function fun.contains(xs, v)\n"
"   for _, x in ipairs(xs) do\n"
"      if v == x then\n"
"         return true\n"
"      end\n"
"   end\n"
"   return false\n"
"end\n"
"\n"
"function fun.map(xs, f)\n"
"   local rs = {}\n"
"   for i = 1, #xs do\n"
"      rs[i] = f(xs[i])\n"
"   end\n"
"   return rs\n"
"end\n"
"\n"
"function fun.filter(xs, f)\n"
"   local rs = {}\n"
"   for i = 1, #xs do\n"
"      local v = xs[i]\n"
"      if f(v) then\n"
"         rs[#rs+1] = v\n"
"      end\n"
"   end\n"
"   return rs\n"
"end\n"
"\n"
"function fun.traverse(t, f)\n"
"   return fun.map(t, function(x)\n"
"      return type(x) == \"table\" and fun.traverse(x, f) or f(x)\n"
"   end)\n"
"end\n"
"\n"
"function fun.reverse_in(t)\n"
"   for i = 1, math.floor(#t/2) do\n"
"      local m, n = i, #t - i + 1\n"
"      local a, b = t[m], t[n]\n"
"      t[m] = b\n"
"      t[n] = a\n"
"   end\n"
"   return t\n"
"end\n"
"\n"
"function fun.sort_in(t, f)\n"
"   table.sort(t, f)\n"
"   return t\n"
"end\n"
"\n"
"function fun.flip(f)\n"
"   return function(a, b)\n"
"      return f(b, a)\n"
"   end\n"
"end\n"
"\n"
"function fun.find(xs, f)\n"
"   if type(xs) == \"function\" then\n"
"      for v in xs do\n"
"         local x = f(v)\n"
"         if x then\n"
"            return x\n"
"         end\n"
"      end\n"
"   elseif type(xs) == \"table\" then\n"
"      for _, v in ipairs(xs) do\n"
"         local x = f(v)\n"
"         if x then\n"
"            return x\n"
"         end\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"function fun.partial(f, ...)\n"
"   local n = select(\"#\", ...)\n"
"   if n == 1 then\n"
"      local a = ...\n"
"      return function(...)\n"
"         return f(a, ...)\n"
"      end\n"
"   elseif n == 2 then\n"
"      local a, b = ...\n"
"      return function(...)\n"
"         return f(a, b, ...)\n"
"      end\n"
"   else\n"
"      local pargs = { n = n, ... }\n"
"      return function(...)\n"
"         local m = select(\"#\", ...)\n"
"         local fargs = { ... }\n"
"         local args = {}\n"
"         for i = 1, n do\n"
"            args[i] = pargs[i]\n"
"         end\n"
"         for i = 1, m do\n"
"            args[i+n] = fargs[i]\n"
"         end\n"
"         local unpack = unpack or table.unpack\n"
"         return f(unpack(args, 1, n+m))\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"function fun.memoize(fn)\n"
"   local memory = setmetatable({}, { __mode = \"k\" })\n"
"   local errors = setmetatable({}, { __mode = \"k\" })\n"
"   local NIL = {}\n"
"   return function(arg)\n"
"      if memory[arg] then\n"
"         if memory[arg] == NIL then\n"
"            return nil, errors[arg]\n"
"         end\n"
"         return memory[arg]\n"
"      end\n"
"      local ret1, ret2 = fn(arg)\n"
"      if ret1 ~= nil then\n"
"         memory[arg] = ret1\n"
"      else\n"
"         memory[arg] = NIL\n"
"         errors[arg] = ret2\n"
"      end\n"
"      return ret1, ret2\n"
"   end\n"
"end\n"
"\n"
"return fun\n"
""
;
