const char luarocks_manif_writer_lua[] =
"\n"
"local writer = {}\n"
"\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local search = require(\"luarocks.search\")\n"
"local repos = require(\"luarocks.repos\")\n"
"local deps = require(\"luarocks.deps\")\n"
"local vers = require(\"luarocks.core.vers\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local util = require(\"luarocks.util\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local fetch = require(\"luarocks.fetch\")\n"
"local path = require(\"luarocks.path\")\n"
"local persist = require(\"luarocks.persist\")\n"
"local manif = require(\"luarocks.manif\")\n"
"local queries = require(\"luarocks.queries\")\n"
"\n"
"--- Update storage table to account for items provided by a package.\n"
"-- @param storage table: a table storing items in the following format:\n"
"-- keys are item names and values are arrays of packages providing each item,\n"
"-- where a package is specified as string `name/version`.\n"
"-- @param items table: a table mapping item names to paths.\n"
"-- @param name string: package name.\n"
"-- @param version string: package version.\n"
"local function store_package_items(storage, name, version, items)\n"
"   assert(type(storage) == \"table\")\n"
"   assert(type(items) == \"table\")\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"\n"
"   local package_identifier = name..\"/\"..version\n"
"\n"
"   for item_name, path in pairs(items) do\n"
"      if not storage[item_name] then\n"
"         storage[item_name] = {}\n"
"      end\n"
"\n"
"      table.insert(storage[item_name], package_identifier)\n"
"   end\n"
"end\n"
"\n"
"--- Update storage table removing items provided by a package.\n"
"-- @param storage table: a table storing items in the following format:\n"
"-- keys are item names and values are arrays of packages providing each item,\n"
"-- where a package is specified as string `name/version`.\n"
"-- @param items table: a table mapping item names to paths.\n"
"-- @param name string: package name.\n"
"-- @param version string: package version.\n"
"local function remove_package_items(storage, name, version, items)\n"
"   assert(type(storage) == \"table\")\n"
"   assert(type(items) == \"table\")\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"\n"
"   local package_identifier = name..\"/\"..version\n"
"\n"
"   for item_name, path in pairs(items) do\n"
"      local all_identifiers = storage[item_name]\n"
"\n"
"      for i, identifier in ipairs(all_identifiers) do\n"
"         if identifier == package_identifier then\n"
"            table.remove(all_identifiers, i)\n"
"            break\n"
"         end\n"
"      end\n"
"\n"
"      if #all_identifiers == 0 then\n"
"         storage[item_name] = nil\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"--- Process the dependencies of a manifest table to determine its dependency\n"
"-- chains for loading modules. The manifest dependencies information is filled\n"
"-- and any dependency inconsistencies or missing dependencies are reported to\n"
"-- standard error.\n"
"-- @param manifest table: a manifest table.\n"
"-- @param deps_mode string: Dependency mode: \"one\" for the current default tree,\n"
"-- \"all\" for all trees, \"order\" for all trees with priority >= the current default,\n"
"-- \"none\" for no trees.\n"
"local function update_dependencies(manifest, deps_mode)\n"
"   assert(type(manifest) == \"table\")\n"
"   assert(type(deps_mode) == \"string\")\n"
"   \n"
"   for pkg, versions in pairs(manifest.repository) do\n"
"      for version, repositories in pairs(versions) do\n"
"         for _, repo in ipairs(repositories) do\n"
"            if repo.arch == \"installed\" then\n"
"               repo.dependencies = {}\n"
"               deps.scan_deps(repo.dependencies, manifest, pkg, version, deps_mode)\n"
"               repo.dependencies[pkg] = nil\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"\n"
"\n"
"--- Sort function for ordering rock identifiers in a manifest's\n"
"-- modules table. Rocks are ordered alphabetically by name, and then\n"
"-- by version which greater first.\n"
"-- @param a string: Version to compare.\n"
"-- @param b string: Version to compare.\n"
"-- @return boolean: The comparison result, according to the\n"
"-- rule outlined above.\n"
"local function sort_pkgs(a, b)\n"
"   assert(type(a) == \"string\")\n"
"   assert(type(b) == \"string\")\n"
"\n"
"   local na, va = a:match(\"(.*)/(.*)$\")\n"
"   local nb, vb = b:match(\"(.*)/(.*)$\")\n"
"\n"
"   return (na == nb) and vers.compare_versions(va, vb) or na < nb\n"
"end\n"
"\n"
"--- Sort items of a package matching table by version number (higher versions first).\n"
"-- @param tbl table: the package matching table: keys should be strings\n"
"-- and values arrays of strings with packages names in \"name/version\" format.\n"
"local function sort_package_matching_table(tbl)\n"
"   assert(type(tbl) == \"table\")\n"
"\n"
"   if next(tbl) then\n"
"      for item, pkgs in pairs(tbl) do\n"
"         if #pkgs > 1 then\n"
"            table.sort(pkgs, sort_pkgs)\n"
"            -- Remove duplicates from the sorted array.\n"
"            local prev = nil\n"
"            local i = 1\n"
"            while pkgs[i] do\n"
"               local curr = pkgs[i]\n"
"               if curr == prev then\n"
"                  table.remove(pkgs, i)\n"
"               else\n"
"                  prev = curr\n"
"                  i = i + 1\n"
"               end\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"--- Filter manifest table by Lua version, removing rockspecs whose Lua version\n"
"-- does not match.\n"
"-- @param manifest table: a manifest table.\n"
"-- @param lua_version string or nil: filter by Lua version\n"
"-- @param repodir string: directory of repository being scanned\n"
"-- @param cache table: temporary rockspec cache table\n"
"local function filter_by_lua_version(manifest, lua_version, repodir, cache)\n"
"   assert(type(manifest) == \"table\")\n"
"   assert(type(repodir) == \"string\")\n"
"   assert((not cache) or type(cache) == \"table\")\n"
"   \n"
"   cache = cache or {}\n"
"   lua_version = vers.parse_version(lua_version)\n"
"   for pkg, versions in pairs(manifest.repository) do\n"
"      local to_remove = {}\n"
"      for version, repositories in pairs(versions) do\n"
"         for _, repo in ipairs(repositories) do\n"
"            if repo.arch == \"rockspec\" then\n"
"               local pathname = dir.path(repodir, pkg..\"-\"..version..\".rockspec\")\n"
"               local rockspec, err = cache[pathname]\n"
"               if not rockspec then\n"
"                  rockspec, err = fetch.load_local_rockspec(pathname, true)\n"
"               end\n"
"               if rockspec then\n"
"                  cache[pathname] = rockspec\n"
"                  for _, dep in ipairs(rockspec.dependencies) do\n"
"                     if dep.name == \"lua\" then \n"
"                        if not vers.match_constraints(lua_version, dep.constraints) then\n"
"                           table.insert(to_remove, version)\n"
"                        end\n"
"                        break\n"
"                     end\n"
"                  end\n"
"               else\n"
"                  util.printerr(\"Error loading rockspec for \"..pkg..\" \"..version..\": \"..err)\n"
"               end\n"
"            end\n"
"         end\n"
"      end\n"
"      if next(to_remove) then\n"
"         for _, incompat in ipairs(to_remove) do\n"
"            versions[incompat] = nil\n"
"         end\n"
"         if not next(versions) then\n"
"            manifest.repository[pkg] = nil\n"
"         end\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"--- Store search results in a manifest table.\n"
"-- @param results table: The search results as returned by search.disk_search.\n"
"-- @param manifest table: A manifest table (must contain repository, modules, commands tables).\n"
"-- It will be altered to include the search results.\n"
"-- @return boolean or (nil, string): true in case of success, or nil followed by an error message.\n"
"local function store_results(results, manifest)\n"
"   assert(type(results) == \"table\")\n"
"   assert(type(manifest) == \"table\")\n"
"\n"
"   for name, versions in pairs(results) do\n"
"      local pkgtable = manifest.repository[name] or {}\n"
"      for version, entries in pairs(versions) do\n"
"         local versiontable = {}\n"
"         for _, entry in ipairs(entries) do\n"
"            local entrytable = {}\n"
"            entrytable.arch = entry.arch\n"
"            if entry.arch == \"installed\" then\n"
"               local rock_manifest, err = manif.load_rock_manifest(name, version)\n"
"               if not rock_manifest then return nil, err end\n"
"\n"
"               entrytable.modules = repos.package_modules(name, version)\n"
"               store_package_items(manifest.modules, name, version, entrytable.modules)\n"
"               entrytable.commands = repos.package_commands(name, version)\n"
"               store_package_items(manifest.commands, name, version, entrytable.commands)\n"
"            end\n"
"            table.insert(versiontable, entrytable)\n"
"         end\n"
"         pkgtable[version] = versiontable\n"
"      end\n"
"      manifest.repository[name] = pkgtable\n"
"   end\n"
"   sort_package_matching_table(manifest.modules)\n"
"   sort_package_matching_table(manifest.commands)\n"
"   return true\n"
"end\n"
"\n"
"--- Commit a table to disk in given local path.\n"
"-- @param where string: The directory where the table should be saved.\n"
"-- @param name string: The filename.\n"
"-- @param tbl table: The table to be saved.\n"
"-- @return boolean or (nil, string): true if successful, or nil and a\n"
"-- message in case of errors.\n"
"local function save_table(where, name, tbl)\n"
"   assert(type(where) == \"string\")\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(tbl) == \"table\")\n"
"\n"
"   local filename = dir.path(where, name)\n"
"   local ok, err = persist.save_from_table(filename..\".tmp\", tbl)\n"
"   if ok then\n"
"      ok, err = fs.replace_file(filename, filename..\".tmp\")\n"
"   end\n"
"   return ok, err\n"
"end\n"
"\n"
"function writer.make_rock_manifest(name, version)\n"
"   local install_dir = path.install_dir(name, version)\n"
"   local tree = {}\n"
"   for _, file in ipairs(fs.find(install_dir)) do\n"
"      local full_path = dir.path(install_dir, file)\n"
"      local walk = tree\n"
"      local last\n"
"      local last_name\n"
"      for filename in file:gmatch(\"[^/]+\") do\n"
"         local next = walk[filename]\n"
"         if not next then\n"
"            next = {}\n"
"            walk[filename] = next\n"
"         end\n"
"         last = walk\n"
"         last_name = filename\n"
"         walk = next\n"
"      end\n"
"      if fs.is_file(full_path) then\n"
"         local sum, err = fs.get_md5(full_path)\n"
"         if not sum then\n"
"            return nil, \"Failed producing checksum: \"..tostring(err)\n"
"         end\n"
"         last[last_name] = sum\n"
"      end\n"
"   end\n"
"   local rock_manifest = { rock_manifest=tree }\n"
"   manif.rock_manifest_cache[name..\"/\"..version] = rock_manifest\n"
"   save_table(install_dir, \"rock_manifest\", rock_manifest )\n"
"   return true\n"
"end\n"
"\n"
"-- Writes a 'rock_namespace' file in a locally installed rock directory.\n"
"-- @param name string: the rock name (may be in user/rock format)\n"
"-- @param version string: the rock version\n"
"-- @param namespace string\?: the namespace\n"
"-- @return true if successful (or unnecessary, if there is no namespace),\n"
"-- or nil and an error message.\n"
"function writer.make_namespace_file(name, version, namespace)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   assert(type(namespace) == \"string\" or not namespace)\n"
"   name = util.adjust_name_and_namespace(name, { namespace = namespace })\n"
"   name, namespace = util.split_namespace(name)\n"
"   if not namespace then\n"
"      return true\n"
"   end\n"
"   local fd, err = io.open(path.rock_namespace_file(name, version), \"w\")\n"
"   if not fd then\n"
"      return nil, err\n"
"   end\n"
"   local ok, err = fd:write(namespace)\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"   fd:close()\n"
"   return true\n"
"end\n"
"\n"
"--- Scan a LuaRocks repository and output a manifest file.\n"
"-- A file called 'manifest' will be written in the root of the given\n"
"-- repository directory.\n"
"-- @param repo A local repository directory.\n"
"-- @param deps_mode string: Dependency mode: \"one\" for the current default tree,\n"
"-- \"all\" for all trees, \"order\" for all trees with priority >= the current default,\n"
"-- \"none\" for the default dependency mode from the configuration.\n"
"-- @param remote boolean: 'true' if making a manifest for a rocks server.\n"
"-- @return boolean or (nil, string): True if manifest was generated,\n"
"-- or nil and an error message.\n"
"function writer.make_manifest(repo, deps_mode, remote)\n"
"   assert(type(repo) == \"string\")\n"
"   assert(type(deps_mode) == \"string\")\n"
"\n"
"   if deps_mode == \"none\" then deps_mode = cfg.deps_mode end\n"
"\n"
"   if not fs.is_dir(repo) then\n"
"      return nil, \"Cannot access repository at \"..repo\n"
"   end\n"
"\n"
"   local query = queries.all(\"any\")\n"
"   local results = search.disk_search(repo, query)\n"
"   local manifest = { repository = {}, modules = {}, commands = {} }\n"
"\n"
"   manif.cache_manifest(repo, nil, manifest)\n"
"\n"
"   local ok, err = store_results(results, manifest)\n"
"   if not ok then return nil, err end\n"
"\n"
"   if remote then\n"
"      local cache = {}\n"
"      for luaver in util.lua_versions() do\n"
"         local vmanifest = { repository = {}, modules = {}, commands = {} }\n"
"         local ok, err = store_results(results, vmanifest)\n"
"         filter_by_lua_version(vmanifest, luaver, repo, cache)\n"
"         save_table(repo, \"manifest-\"..luaver, vmanifest)\n"
"      end\n"
"   else\n"
"      update_dependencies(manifest, deps_mode)\n"
"   end\n"
"\n"
"   return save_table(repo, \"manifest\", manifest)\n"
"end\n"
"\n"
"--- Update manifest file for a local repository\n"
"-- adding information about a version of a package installed in that repository.\n"
"-- @param name string: Name of a package from the repository.\n"
"-- @param version string: Version of a package from the repository.\n"
"-- @param repo string or nil: Pathname of a local repository. If not given,\n"
"-- the default local repository is used.\n"
"-- @param deps_mode string: Dependency mode: \"one\" for the current default tree,\n"
"-- \"all\" for all trees, \"order\" for all trees with priority >= the current default,\n"
"-- \"none\" for using the default dependency mode from the configuration.\n"
"-- @return boolean or (nil, string): True if manifest was updated successfully,\n"
"-- or nil and an error message.\n"
"function writer.add_to_manifest(name, version, repo, deps_mode)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   local rocks_dir = path.rocks_dir(repo or cfg.root_dir)\n"
"   assert(type(deps_mode) == \"string\")\n"
"\n"
"   if deps_mode == \"none\" then deps_mode = cfg.deps_mode end\n"
"\n"
"   local manifest, err = manif.load_manifest(rocks_dir)\n"
"   if not manifest then\n"
"      util.printerr(\"No existing manifest. Attempting to rebuild...\")\n"
"      -- Manifest built by `writer.make_manifest` should already\n"
"      -- include information about given name and version,\n"
"      -- no need to update it.\n"
"      return writer.make_manifest(rocks_dir, deps_mode)\n"
"   end\n"
"\n"
"   local results = {[name] = {[version] = {{arch = \"installed\", repo = rocks_dir}}}}\n"
"\n"
"   local ok, err = store_results(results, manifest)\n"
"   if not ok then return nil, err end\n"
"\n"
"   update_dependencies(manifest, deps_mode)\n"
"   return save_table(rocks_dir, \"manifest\", manifest)\n"
"end\n"
"\n"
"--- Update manifest file for a local repository\n"
"-- removing information about a version of a package.\n"
"-- @param name string: Name of a package removed from the repository.\n"
"-- @param version string: Version of a package removed from the repository.\n"
"-- @param repo string or nil: Pathname of a local repository. If not given,\n"
"-- the default local repository is used.\n"
"-- @param deps_mode string: Dependency mode: \"one\" for the current default tree,\n"
"-- \"all\" for all trees, \"order\" for all trees with priority >= the current default,\n"
"-- \"none\" for using the default dependency mode from the configuration.\n"
"-- @return boolean or (nil, string): True if manifest was updated successfully,\n"
"-- or nil and an error message.\n"
"function writer.remove_from_manifest(name, version, repo, deps_mode)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   local rocks_dir = path.rocks_dir(repo or cfg.root_dir)\n"
"   assert(type(deps_mode) == \"string\")\n"
"\n"
"   if deps_mode == \"none\" then deps_mode = cfg.deps_mode end\n"
"\n"
"   local manifest, err = manif.load_manifest(rocks_dir)\n"
"   if not manifest then\n"
"      util.printerr(\"No existing manifest. Attempting to rebuild...\")\n"
"      -- Manifest built by `writer.make_manifest` should already\n"
"      -- include up-to-date information, no need to update it.\n"
"      return writer.make_manifest(rocks_dir, deps_mode)\n"
"   end\n"
"\n"
"   local package_entry = manifest.repository[name]\n"
"\n"
"   local version_entry = package_entry[version][1]\n"
"   remove_package_items(manifest.modules, name, version, version_entry.modules)\n"
"   remove_package_items(manifest.commands, name, version, version_entry.commands)\n"
"\n"
"   package_entry[version] = nil\n"
"   manifest.dependencies[name][version] = nil\n"
"\n"
"   if not next(package_entry) then\n"
"      -- No more versions of this package.\n"
"      manifest.repository[name] = nil\n"
"      manifest.dependencies[name] = nil\n"
"   end\n"
"\n"
"   update_dependencies(manifest, deps_mode)\n"
"   return save_table(rocks_dir, \"manifest\", manifest)\n"
"end\n"
"\n"
"--- Report missing dependencies for all rocks installed in a repository.\n"
"-- @param repo string or nil: Pathname of a local repository. If not given,\n"
"-- the default local repository is used.\n"
"-- @param deps_mode string: Dependency mode: \"one\" for the current default tree,\n"
"-- \"all\" for all trees, \"order\" for all trees with priority >= the current default,\n"
"-- \"none\" for using the default dependency mode from the configuration.\n"
"function writer.check_dependencies(repo, deps_mode)\n"
"   local rocks_dir = path.rocks_dir(repo or cfg.root_dir)\n"
"   assert(type(deps_mode) == \"string\")\n"
"   if deps_mode == \"none\" then deps_mode = cfg.deps_mode end\n"
"\n"
"   local manifest = manif.load_manifest(rocks_dir)\n"
"   if not manifest then\n"
"      return\n"
"   end\n"
"\n"
"   for name, versions in util.sortedpairs(manifest.repository) do\n"
"      for version, version_entries in util.sortedpairs(versions, vers.compare_versions) do\n"
"         for _, entry in ipairs(version_entries) do\n"
"            if entry.arch == \"installed\" then\n"
"               if manifest.dependencies[name] and manifest.dependencies[name][version] then\n"
"                  deps.report_missing_dependencies(name, version, manifest.dependencies[name][version], deps_mode, cfg.rocks_provided_3_0)\n"
"               end\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"return writer\n"
""
;
