const char luarocks_path_lua[] =
"\n"
"--- LuaRocks-specific path handling functions.\n"
"-- All paths are configured in this module, making it a single\n"
"-- point where the layout of the local installation is defined in LuaRocks.\n"
"local path = {}\n"
"\n"
"local core = require(\"luarocks.core.path\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local util = require(\"luarocks.util\")\n"
"\n"
"path.rocks_dir = core.rocks_dir\n"
"path.versioned_name = core.versioned_name\n"
"path.path_to_module = core.path_to_module\n"
"path.deploy_lua_dir = core.deploy_lua_dir\n"
"path.deploy_lib_dir = core.deploy_lib_dir\n"
"\n"
"--- Infer rockspec filename from a rock filename.\n"
"-- @param rock_name string: Pathname of a rock file.\n"
"-- @return string: Filename of the rockspec, without path.\n"
"function path.rockspec_name_from_rock(rock_name)\n"
"   assert(type(rock_name) == \"string\")\n"
"   local base_name = dir.base_name(rock_name)\n"
"   return base_name:match(\"(.*)%.[^.]*.rock\") .. \".rockspec\"\n"
"end\n"
"\n"
"function path.root_from_rocks_dir(rocks_dir)\n"
"   assert(type(rocks_dir) == \"string\")\n"
"   return rocks_dir:match(\"(.*)\" .. util.matchquote(cfg.rocks_subdir) .. \".*$\")\n"
"end\n"
"\n"
"function path.root_dir(tree)\n"
"   if type(tree) == \"string\" then\n"
"      return tree\n"
"   else\n"
"      assert(type(tree) == \"table\")\n"
"      return tree.root\n"
"   end\n"
"end\n"
"\n"
"function path.deploy_bin_dir(tree)\n"
"   return dir.path(path.root_dir(tree), \"bin\")\n"
"end\n"
"\n"
"function path.manifest_file(tree)\n"
"   return dir.path(path.rocks_dir(tree), \"manifest\")\n"
"end\n"
"\n"
"--- Get the directory for all versions of a package in a tree.\n"
"-- @param name string: The package name. \n"
"-- @return string: The resulting path -- does not guarantee that\n"
"-- @param tree string or nil: If given, specifies the local tree to use.\n"
"-- the package (and by extension, the path) exists.\n"
"function path.versions_dir(name, tree)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   return dir.path(path.rocks_dir(tree), name)\n"
"end\n"
"\n"
"--- Get the local installation directory (prefix) for a package.\n"
"-- @param name string: The package name. \n"
"-- @param version string: The package version.\n"
"-- @param tree string or nil: If given, specifies the local tree to use.\n"
"-- @return string: The resulting path -- does not guarantee that\n"
"-- the package (and by extension, the path) exists.\n"
"function path.install_dir(name, version, tree)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   return dir.path(path.rocks_dir(tree), name, version)\n"
"end\n"
"\n"
"--- Get the local filename of the rockspec of an installed rock.\n"
"-- @param name string: The package name. \n"
"-- @param version string: The package version.\n"
"-- @param tree string or nil: If given, specifies the local tree to use.\n"
"-- @return string: The resulting path -- does not guarantee that\n"
"-- the package (and by extension, the file) exists.\n"
"function path.rockspec_file(name, version, tree)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   return dir.path(path.rocks_dir(tree), name, version, name..\"-\"..version..\".rockspec\")\n"
"end\n"
"\n"
"--- Get the local filename of the rock_manifest file of an installed rock.\n"
"-- @param name string: The package name. \n"
"-- @param version string: The package version.\n"
"-- @param tree string or nil: If given, specifies the local tree to use.\n"
"-- @return string: The resulting path -- does not guarantee that\n"
"-- the package (and by extension, the file) exists.\n"
"function path.rock_manifest_file(name, version, tree)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   return dir.path(path.rocks_dir(tree), name, version, \"rock_manifest\")\n"
"end\n"
"\n"
"--- Get the local filename of the rock_namespace file of an installed rock.\n"
"-- @param name string: The package name (without a namespace).\n"
"-- @param version string: The package version.\n"
"-- @param tree string or nil: If given, specifies the local tree to use.\n"
"-- @return string: The resulting path -- does not guarantee that\n"
"-- the package (and by extension, the file) exists.\n"
"function path.rock_namespace_file(name, version, tree)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   return dir.path(path.rocks_dir(tree), name, version, \"rock_namespace\")\n"
"end\n"
"\n"
"--- Get the local installation directory for C libraries of a package.\n"
"-- @param name string: The package name. \n"
"-- @param version string: The package version.\n"
"-- @param tree string or nil: If given, specifies the local tree to use.\n"
"-- @return string: The resulting path -- does not guarantee that\n"
"-- the package (and by extension, the path) exists.\n"
"function path.lib_dir(name, version, tree)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   return dir.path(path.rocks_dir(tree), name, version, \"lib\")\n"
"end\n"
"\n"
"--- Get the local installation directory for Lua modules of a package.\n"
"-- @param name string: The package name. \n"
"-- @param version string: The package version.\n"
"-- @param tree string or nil: If given, specifies the local tree to use.\n"
"-- @return string: The resulting path -- does not guarantee that\n"
"-- the package (and by extension, the path) exists.\n"
"function path.lua_dir(name, version, tree)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   return dir.path(path.rocks_dir(tree), name, version, \"lua\")\n"
"end\n"
"\n"
"--- Get the local installation directory for documentation of a package.\n"
"-- @param name string: The package name. \n"
"-- @param version string: The package version.\n"
"-- @param tree string or nil: If given, specifies the local tree to use.\n"
"-- @return string: The resulting path -- does not guarantee that\n"
"-- the package (and by extension, the path) exists.\n"
"function path.doc_dir(name, version, tree)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   return dir.path(path.rocks_dir(tree), name, version, \"doc\")\n"
"end\n"
"\n"
"--- Get the local installation directory for configuration files of a package.\n"
"-- @param name string: The package name. \n"
"-- @param version string: The package version.\n"
"-- @param tree string or nil: If given, specifies the local tree to use.\n"
"-- @return string: The resulting path -- does not guarantee that\n"
"-- the package (and by extension, the path) exists.\n"
"function path.conf_dir(name, version, tree)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   return dir.path(path.rocks_dir(tree), name, version, \"conf\")\n"
"end\n"
"\n"
"--- Get the local installation directory for command-line scripts\n"
"-- of a package.\n"
"-- @param name string: The package name. \n"
"-- @param version string: The package version.\n"
"-- @param tree string or nil: If given, specifies the local tree to use.\n"
"-- @return string: The resulting path -- does not guarantee that\n"
"-- the package (and by extension, the path) exists.\n"
"function path.bin_dir(name, version, tree)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   return dir.path(path.rocks_dir(tree), name, version, \"bin\")\n"
"end\n"
"\n"
"--- Extract name, version and arch of a rock filename,\n"
"-- or name, version and \"rockspec\" from a rockspec name.\n"
"-- @param file_name string: pathname of a rock or rockspec\n"
"-- @return (string, string, string) or nil: name, version and arch\n"
"-- or nil if name could not be parsed\n"
"function path.parse_name(file_name)\n"
"   assert(type(file_name) == \"string\")\n"
"   if file_name:match(\"%.rock$\") then\n"
"      return dir.base_name(file_name):match(\"(.*)-([^-]+-%d+)%.([^.]+)%.rock$\")\n"
"   else\n"
"      return dir.base_name(file_name):match(\"(.*)-([^-]+-%d+)%.(rockspec)\")\n"
"   end\n"
"end\n"
"\n"
"--- Make a rockspec or rock URL.\n"
"-- @param pathname string: Base URL or pathname.\n"
"-- @param name string: Package name.\n"
"-- @param version string: Package version.\n"
"-- @param arch string: Architecture identifier, or \"rockspec\" or \"installed\".\n"
"-- @return string: A URL or pathname following LuaRocks naming conventions.\n"
"function path.make_url(pathname, name, version, arch)\n"
"   assert(type(pathname) == \"string\")\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   assert(type(arch) == \"string\")\n"
"\n"
"   local filename = name..\"-\"..version\n"
"   if arch == \"installed\" then\n"
"      filename = dir.path(name, version, filename..\".rockspec\")\n"
"   elseif arch == \"rockspec\" then\n"
"      filename = filename..\".rockspec\"\n"
"   else\n"
"      filename = filename..\".\"..arch..\".rock\"\n"
"   end\n"
"   return dir.path(pathname, filename)\n"
"end\n"
"\n"
"--- Obtain the directory name where a module should be stored.\n"
"-- For example, on Unix, \"foo.bar.baz\" will return \"foo/bar\".\n"
"-- @param mod string: A module name in Lua dot-separated format.\n"
"-- @return string: A directory name using the platform's separator.\n"
"function path.module_to_path(mod)\n"
"   assert(type(mod) == \"string\")\n"
"   return (mod:gsub(\"[^.]*$\", \"\"):gsub(\"%.\", \"/\"))\n"
"end\n"
"\n"
"function path.use_tree(tree)\n"
"   cfg.root_dir = tree\n"
"   cfg.rocks_dir = path.rocks_dir(tree)\n"
"   cfg.deploy_bin_dir = path.deploy_bin_dir(tree)\n"
"   cfg.deploy_lua_dir = path.deploy_lua_dir(tree)\n"
"   cfg.deploy_lib_dir = path.deploy_lib_dir(tree)\n"
"\n"
"   -- TODO Do we want LuaRocks itself to use whatever tree is in use\?\n"
"   -- package.path = dir.path(path.deploy_lua_dir(tree), \"\?.lua\") .. \";\"\n"
"   --             .. dir.path(path.deploy_lua_dir(tree), \"\?/init.lua\") .. \";\"\n"
"   --             .. package.path\n"
"   -- package.cpath = dir.path(path.deploy_lib_dir(tree), \"\?.\" .. cfg.lib_extension) .. \";\"\n"
"   --              .. package.cpath\n"
"end\n"
"\n"
"function path.rocks_tree_to_string(tree)\n"
"   if type(tree) == \"string\" then\n"
"      return tree\n"
"   else\n"
"      assert(type(tree) == \"table\")\n"
"      return tree.root\n"
"   end\n"
"end\n"
"\n"
"--- Apply a given function to the active rocks trees based on chosen dependency mode.\n"
"-- @param deps_mode string: Dependency mode: \"one\" for the current default tree,\n"
"-- \"all\" for all trees, \"order\" for all trees with priority >= the current default,\n"
"-- \"none\" for no trees (this function becomes a nop).\n"
"-- @param fn function: function to be applied, with the tree dir (string) as the first\n"
"-- argument and the remaining varargs of map_trees as the following arguments.\n"
"-- @return a table with all results of invocations of fn collected.\n"
"function path.map_trees(deps_mode, fn, ...)\n"
"   local result = {}\n"
"   if deps_mode == \"one\" then\n"
"      table.insert(result, (fn(cfg.root_dir, ...)) or 0)\n"
"   elseif deps_mode == \"all\" or deps_mode == \"order\" then\n"
"      local use = false\n"
"      if deps_mode == \"all\" then\n"
"         use = true\n"
"      end\n"
"      for _, tree in ipairs(cfg.rocks_trees) do\n"
"         if dir.normalize(path.rocks_tree_to_string(tree)) == dir.normalize(path.rocks_tree_to_string(cfg.root_dir)) then\n"
"            use = true\n"
"         end\n"
"         if use then\n"
"            table.insert(result, (fn(tree, ...)) or 0)\n"
"         end\n"
"      end\n"
"   end\n"
"   return result\n"
"end\n"
"\n"
"--- Get the namespace of a locally-installed rock, if any.\n"
"-- @param name string: The rock name, without a namespace.\n"
"-- @param version string: The rock version.\n"
"-- @param tree string: The local tree to use.\n"
"-- @return string\?: The namespace if it exists, or nil.\n"
"function path.read_namespace(name, version, tree)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   assert(type(tree) == \"string\")\n"
"\n"
"   local namespace\n"
"   local fd = io.open(path.rock_namespace_file(name, version, tree), \"r\")\n"
"   if fd then\n"
"      namespace = fd:read(\"*a\")\n"
"      fd:close()\n"
"   end\n"
"   return namespace\n"
"end\n"
"\n"
"function path.package_paths(deps_mode)\n"
"   local lpaths = {}\n"
"   local lcpaths = {}\n"
"   path.map_trees(deps_mode, function(tree)\n"
"      local root = path.root_dir(tree)\n"
"      table.insert(lpaths, dir.path(root, cfg.lua_modules_path, \"\?.lua\"))\n"
"      table.insert(lpaths, dir.path(root, cfg.lua_modules_path, \"\?/init.lua\"))\n"
"      table.insert(lcpaths, dir.path(root, cfg.lib_modules_path, \"\?.\" .. cfg.lib_extension))\n"
"   end)\n"
"   return table.concat(lpaths, \";\"), table.concat(lcpaths, \";\")\n"
"end\n"
"\n"
"return path\n"
""
;
