const char luarocks_queries_lua[] =
"\n"
"local queries = {}\n"
"\n"
"local vers = require(\"luarocks.core.vers\")\n"
"local util = require(\"luarocks.util\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"\n"
"local query_mt = {}\n"
"\n"
"query_mt.__index = query_mt\n"
"\n"
"function query_mt.type()\n"
"   return \"query\"\n"
"end\n"
"\n"
"-- Fallback default value for the `arch` field, if not explicitly set.\n"
"query_mt.arch = {\n"
"   src = true,\n"
"   all = true,\n"
"   rockspec = true,\n"
"   installed = true,\n"
"   -- [cfg.arch] = true, -- this is set later\n"
"}\n"
"\n"
"-- Fallback default value for the `substring` field, if not explicitly set.\n"
"query_mt.substring = false\n"
"\n"
"--- Convert the arch field of a query table to table format.\n"
"-- @param input string, table or nil\n"
"local function arch_to_table(input)\n"
"   if type(input) == \"table\" then\n"
"      return input\n"
"   elseif type(input) == \"string\" then\n"
"      local arch = {}\n"
"      for a in input:gmatch(\"[%w_-]+\") do\n"
"         arch[a] = true\n"
"      end\n"
"      return arch\n"
"   end\n"
"end\n"
"\n"
"--- Prepare a query in dependency table format.\n"
"-- @param ns_name string: the package name, may contain a namespace.\n"
"-- @param version string\?: the package version.\n"
"-- @param substring boolean\?: match substrings of the name\n"
"-- (default is false, match full name)\n"
"-- @param arch string\?: a string with pipe-separated accepted arch values\n"
"-- @param operator string\?: operator for version matching (default is \"==\")\n"
"-- @return table: A query in table format\n"
"function queries.new(ns_name, version, substring, arch, operator)\n"
"   assert(type(ns_name) == \"string\")\n"
"   assert(type(version) == \"string\" or not version)\n"
"   assert(type(substring) == \"boolean\" or not substring)\n"
"   assert(type(arch) == \"string\" or not arch)\n"
"   assert(type(operator) == \"string\" or not operator)\n"
"   \n"
"   operator = operator or \"==\"\n"
"\n"
"   local name, namespace = util.split_namespace(ns_name)\n"
"   \n"
"   local self = {\n"
"      name = name,\n"
"      namespace = namespace,\n"
"      constraints = {},\n"
"      substring = substring,\n"
"      arch = arch_to_table(arch),\n"
"   }\n"
"   if version then\n"
"      table.insert(self.constraints, { op = operator, version = vers.parse_version(version)})\n"
"   end\n"
"\n"
"   query_mt.arch[cfg.arch] = true\n"
"   return setmetatable(self, query_mt)\n"
"end\n"
"\n"
"-- Query for all packages\n"
"-- @param arch string (optional)\n"
"function queries.all(arch)\n"
"   assert(type(arch) == \"string\" or not arch)\n"
"\n"
"   return queries.new(\"\", nil, true, arch)\n"
"end\n"
"\n"
"do\n"
"   local parse_constraints\n"
"   do\n"
"      local parse_constraint\n"
"      do\n"
"         local operators = {\n"
"            [\"==\"] = \"==\",\n"
"            [\"~=\"] = \"~=\",\n"
"            [\">\"] = \">\",\n"
"            [\"<\"] = \"<\",\n"
"            [\">=\"] = \">=\",\n"
"            [\"<=\"] = \"<=\",\n"
"            [\"~>\"] = \"~>\",\n"
"            -- plus some convenience translations\n"
"            [\"\"] = \"==\",\n"
"            [\"=\"] = \"==\",\n"
"            [\"!=\"] = \"~=\"\n"
"         }\n"
"         \n"
"         --- Consumes a constraint from a string, converting it to table format.\n"
"         -- For example, a string \">= 1.0, > 2.0\" is converted to a table in the\n"
"         -- format {op = \">=\", version={1,0}} and the rest, \"> 2.0\", is returned\n"
"         -- back to the caller.\n"
"         -- @param input string: A list of constraints in string format.\n"
"         -- @return (table, string) or nil: A table representing the same\n"
"         -- constraints and the string with the unused input, or nil if the\n"
"         -- input string is invalid.\n"
"         parse_constraint = function(input)\n"
"            assert(type(input) == \"string\")\n"
"         \n"
"            local no_upgrade, op, version, rest = input:match(\"^(@\?)([<>=~!]*)%s*([%w%.%_%-]+)[%s,]*(.*)\")\n"
"            local _op = operators[op]\n"
"            version = vers.parse_version(version)\n"
"            if not _op then\n"
"               return nil, \"Encountered bad constraint operator: '\"..tostring(op)..\"' in '\"..input..\"'\"\n"
"            end\n"
"            if not version then \n"
"               return nil, \"Could not parse version from constraint: '\"..input..\"'\"\n"
"            end\n"
"            return { op = _op, version = version, no_upgrade = no_upgrade==\"@\" and true or nil }, rest\n"
"         end\n"
"      end\n"
"      \n"
"      --- Convert a list of constraints from string to table format.\n"
"      -- For example, a string \">= 1.0, < 2.0\" is converted to a table in the format\n"
"      -- {{op = \">=\", version={1,0}}, {op = \"<\", version={2,0}}}.\n"
"      -- Version tables use a metatable allowing later comparison through\n"
"      -- relational operators.\n"
"      -- @param input string: A list of constraints in string format.\n"
"      -- @return table or nil: A table representing the same constraints,\n"
"      -- or nil if the input string is invalid.\n"
"      parse_constraints = function(input)\n"
"         assert(type(input) == \"string\")\n"
"      \n"
"         local constraints, oinput, constraint = {}, input\n"
"         while #input > 0 do\n"
"            constraint, input = parse_constraint(input)\n"
"            if constraint then\n"
"               table.insert(constraints, constraint)\n"
"            else\n"
"               return nil, \"Failed to parse constraint '\"..tostring(oinput)..\"' with error: \".. input\n"
"            end\n"
"         end\n"
"         return constraints\n"
"      end\n"
"   end\n"
"   \n"
"   --- Prepare a query in dependency table format.\n"
"   -- @param depstr string: A dependency in string format\n"
"   -- as entered in rockspec files.\n"
"   -- @return table: A query in table format, or nil and an error message in case of errors.\n"
"   function queries.from_dep_string(depstr)\n"
"      assert(type(depstr) == \"string\")\n"
"   \n"
"      local ns_name, rest = depstr:match(\"^%s*([a-zA-Z0-9%.%-%_]*/\?[a-zA-Z0-9][a-zA-Z0-9%.%-%_]*)%s*([^/]*)\")\n"
"      if not ns_name then\n"
"         return nil, \"failed to extract dependency name from '\"..depstr..\"'\"\n"
"      end\n"
"   \n"
"      local constraints, err = parse_constraints(rest)\n"
"      if not constraints then\n"
"         return nil, err\n"
"      end\n"
"   \n"
"      local name, namespace = util.split_namespace(ns_name)\n"
"   \n"
"      local self = {\n"
"         name = name,\n"
"         namespace = namespace,\n"
"         constraints = constraints,\n"
"      }\n"
"\n"
"      query_mt.arch[cfg.arch] = true\n"
"      return setmetatable(self, query_mt)\n"
"   end\n"
"end\n"
"\n"
"function queries.from_persisted_table(tbl)\n"
"   query_mt.arch[cfg.arch] = true\n"
"   return setmetatable(tbl, query_mt)\n"
"end\n"
"\n"
"--- Build a string representation of a query package name.\n"
"-- Includes namespace, name and version, but not arch or constraints.\n"
"-- @param query table: a query table\n"
"-- @return string: a result such as `my_user/my_rock 1.0` or `my_rock`.\n"
"function query_mt:__tostring()\n"
"   local out = {}\n"
"   if self.namespace then\n"
"      table.insert(out, self.namespace)\n"
"      table.insert(out, \"/\")\n"
"   end\n"
"   table.insert(out, self.name)\n"
"\n"
"   if #self.constraints > 0 then\n"
"      local pretty = {}\n"
"      for _, c in ipairs(self.constraints) do\n"
"         local v = c.version.string\n"
"         if c.op == \"==\" then\n"
"            table.insert(pretty, v)\n"
"         else\n"
"            table.insert(pretty, c.op .. \" \" .. v)\n"
"         end\n"
"      end\n"
"      table.insert(out, \" \")\n"
"      table.insert(out, table.concat(pretty, \", \"))\n"
"   end\n"
"\n"
"   return table.concat(out)\n"
"end\n"
"\n"
"return queries\n"
""
;
