const char luarocks_remove_lua[] =
"local remove = {}\n"
"\n"
"local search = require(\"luarocks.search\")\n"
"local deps = require(\"luarocks.deps\")\n"
"local fetch = require(\"luarocks.fetch\")\n"
"local repos = require(\"luarocks.repos\")\n"
"local path = require(\"luarocks.path\")\n"
"local util = require(\"luarocks.util\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local queries = require(\"luarocks.queries\")\n"
"\n"
"--- Obtain a list of packages that depend on the given set of packages\n"
"-- (where all packages of the set are versions of one program).\n"
"-- @param name string: the name of a program\n"
"-- @param versions array of string: the versions to be deleted.\n"
"-- @return array of string: an empty table if no packages depend on any\n"
"-- of the given list, or an array of strings in \"name/version\" format.\n"
"local function check_dependents(name, versions, deps_mode)\n"
"   local dependents = {}\n"
"   local blacklist = {}\n"
"   blacklist[name] = {}\n"
"   for version, _ in pairs(versions) do\n"
"      blacklist[name][version] = true\n"
"   end\n"
"   local local_rocks = {}\n"
"   local query_all = queries.all()\n"
"   search.local_manifest_search(local_rocks, cfg.rocks_dir, query_all)\n"
"   local_rocks[name] = nil\n"
"   for rock_name, rock_versions in pairs(local_rocks) do\n"
"      for rock_version, _ in pairs(rock_versions) do\n"
"         local rockspec, err = fetch.load_rockspec(path.rockspec_file(rock_name, rock_version))\n"
"         if rockspec then\n"
"            local _, missing = deps.match_deps(rockspec.dependencies, rockspec.rocks_provided, blacklist, deps_mode)\n"
"            if missing[name] then\n"
"               table.insert(dependents, { name = rock_name, version = rock_version })\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"   return dependents\n"
"end\n"
"\n"
"--- Delete given versions of a program.\n"
"-- @param name string: the name of a program\n"
"-- @param versions array of string: the versions to be deleted.\n"
"-- @param deps_mode: string: Which trees to check dependencies for:\n"
"-- \"one\" for the current default tree, \"all\" for all trees,\n"
"-- \"order\" for all trees with priority >= the current default, \"none\" for no trees.\n"
"-- @return boolean or (nil, string): true on success or nil and an error message.\n"
"local function delete_versions(name, versions, deps_mode) \n"
"\n"
"   for version, _ in pairs(versions) do\n"
"      util.printout(\"Removing \"..name..\" \"..version..\"...\")\n"
"      local ok, err = repos.delete_version(name, version, deps_mode)\n"
"      if not ok then return nil, err end\n"
"   end\n"
"   \n"
"   return true\n"
"end\n"
"\n"
"function remove.remove_search_results(results, name, deps_mode, force, fast)\n"
"   local versions = results[name]\n"
"\n"
"   local version = next(versions)\n"
"   local second = next(versions, version)\n"
"   \n"
"   local dependents = {}\n"
"   if not fast then\n"
"      util.printout(\"Checking stability of dependencies in the absence of\")\n"
"      util.printout(name..\" \"..table.concat(util.keys(versions), \", \")..\"...\")\n"
"      util.printout()\n"
"      dependents = check_dependents(name, versions, deps_mode)\n"
"   end\n"
"   \n"
"   if #dependents > 0 then\n"
"      if force or fast then\n"
"         util.printerr(\"The following packages may be broken by this forced removal:\")\n"
"         for _, dependent in ipairs(dependents) do\n"
"            util.printerr(dependent.name..\" \"..dependent.version)\n"
"         end\n"
"         util.printerr()\n"
"      else\n"
"         if not second then\n"
"            util.printerr(\"Will not remove \"..name..\" \"..version..\".\")\n"
"            util.printerr(\"Removing it would break dependencies for: \")\n"
"         else\n"
"            util.printerr(\"Will not remove installed versions of \"..name..\".\")\n"
"            util.printerr(\"Removing them would break dependencies for: \")\n"
"         end\n"
"         for _, dependent in ipairs(dependents) do\n"
"            util.printerr(dependent.name..\" \"..dependent.version)\n"
"         end\n"
"         util.printerr()\n"
"         util.printerr(\"Use --force to force removal (warning: this may break modules).\")\n"
"         return nil, \"Failed removing.\"\n"
"      end\n"
"   end\n"
"   \n"
"   local ok, err = delete_versions(name, versions, deps_mode)\n"
"   if not ok then return nil, err end\n"
"\n"
"   util.printout(\"Removal successful.\")\n"
"   return true\n"
"end\n"
"\n"
"function remove.remove_other_versions(name, version, force, fast)\n"
"   local results = {}\n"
"   local query = queries.new(name, version, false, nil, \"~=\")\n"
"   search.local_manifest_search(results, cfg.rocks_dir, query)\n"
"   if results[name] then\n"
"      return remove.remove_search_results(results, name, cfg.deps_mode, force, fast)\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"return remove\n"
""
;
