const char luarocks_rockspecs_lua[] =
"local rockspecs = {}\n"
"   \n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local path = require(\"luarocks.path\")\n"
"local queries = require(\"luarocks.queries\")\n"
"local type_rockspec = require(\"luarocks.type.rockspec\")\n"
"local util = require(\"luarocks.core.util\")\n"
"local vers = require(\"luarocks.core.vers\")\n"
"\n"
"local rockspec_mt = {}\n"
"\n"
"rockspec_mt.__index = rockspec_mt\n"
"\n"
"function rockspec_mt.type()\n"
"   return \"rockspec\"\n"
"end\n"
"\n"
"--- Perform platform-specific overrides on a table.\n"
"-- Overrides values of table with the contents of the appropriate\n"
"-- subset of its \"platforms\" field. The \"platforms\" field should\n"
"-- be a table containing subtables keyed with strings representing\n"
"-- platform names. Names that match the contents of the global\n"
"-- detected platforms setting are used. For example, if\n"
"-- platform \"unix\" is detected, then the fields of\n"
"-- tbl.platforms.unix will overwrite those of tbl with the same\n"
"-- names. For table values, the operation is performed recursively\n"
"-- (tbl.platforms.foo.x.y.z overrides tbl.x.y.z; other contents of\n"
"-- tbl.x are preserved).\n"
"-- @param tbl table or nil: Table which may contain a \"platforms\" field;\n"
"-- if it doesn't (or if nil is passed), this function does nothing.\n"
"local function platform_overrides(tbl)\n"
"   assert(type(tbl) == \"table\" or not tbl)\n"
"   \n"
"   if not tbl then return end\n"
"   \n"
"   if tbl.platforms then\n"
"      for platform in cfg.each_platform() do\n"
"         local platform_tbl = tbl.platforms[platform]\n"
"         if platform_tbl then\n"
"            util.deep_merge(tbl, platform_tbl)\n"
"         end\n"
"      end\n"
"   end\n"
"   tbl.platforms = nil\n"
"end\n"
"\n"
"local function convert_dependencies(rockspec, key)\n"
"   if rockspec[key] then\n"
"      for i = 1, #rockspec[key] do\n"
"         local parsed, err = queries.from_dep_string(rockspec[key][i])\n"
"         if not parsed then\n"
"            return nil, \"Parse error processing dependency '\"..rockspec[key][i]..\"': \"..tostring(err)\n"
"         end\n"
"         rockspec[key][i] = parsed\n"
"      end\n"
"   else\n"
"      rockspec[key] = {}\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"--- Set up path-related variables for a given rock.\n"
"-- Create a \"variables\" table in the rockspec table, containing\n"
"-- adjusted variables according to the configuration file.\n"
"-- @param rockspec table: The rockspec table.\n"
"local function configure_paths(rockspec)\n"
"   local vars = {}\n"
"   for k,v in pairs(cfg.variables) do\n"
"      vars[k] = v\n"
"   end\n"
"   local name, version = rockspec.name, rockspec.version\n"
"   vars.PREFIX = path.install_dir(name, version)\n"
"   vars.LUADIR = path.lua_dir(name, version)\n"
"   vars.LIBDIR = path.lib_dir(name, version)\n"
"   vars.CONFDIR = path.conf_dir(name, version)\n"
"   vars.BINDIR = path.bin_dir(name, version)\n"
"   vars.DOCDIR = path.doc_dir(name, version)\n"
"   rockspec.variables = vars\n"
"end\n"
"\n"
"function rockspecs.from_persisted_table(filename, rockspec, globals, quick)\n"
"   assert(type(rockspec) == \"table\")\n"
"   assert(type(globals) == \"table\" or globals == nil)\n"
"   assert(type(filename) == \"string\")\n"
"   assert(type(quick) == \"boolean\" or quick == nil)\n"
"\n"
"   if rockspec.rockspec_format then\n"
"      if vers.compare_versions(rockspec.rockspec_format, type_rockspec.rockspec_format) then\n"
"         return nil, \"Rockspec format \"..rockspec.rockspec_format..\" is not supported, please upgrade LuaRocks.\"\n"
"      end\n"
"   end\n"
"\n"
"   if not quick then\n"
"      local ok, err = type_rockspec.check(rockspec, globals or {})\n"
"      if not ok then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"\n"
"   --- Check if rockspec format version satisfies version requirement.\n"
"   -- @param rockspec table: The rockspec table.\n"
"   -- @param version string: required version.\n"
"   -- @return boolean: true if rockspec format matches version or is newer, false otherwise.\n"
"   do\n"
"      local parsed_format = vers.parse_version(rockspec.rockspec_format or \"1.0\")\n"
"      rockspec.format_is_at_least = function(self, version)\n"
"         return parsed_format >= vers.parse_version(version)\n"
"      end\n"
"   end\n"
"\n"
"   platform_overrides(rockspec.build)\n"
"   platform_overrides(rockspec.dependencies)\n"
"   platform_overrides(rockspec.build_dependencies)\n"
"   platform_overrides(rockspec.test_dependencies)\n"
"   platform_overrides(rockspec.external_dependencies)\n"
"   platform_overrides(rockspec.source)\n"
"   platform_overrides(rockspec.hooks)\n"
"   platform_overrides(rockspec.test)\n"
"\n"
"   rockspec.name = rockspec.package:lower()\n"
"\n"
"   local protocol, pathname = dir.split_url(rockspec.source.url)\n"
"   if dir.is_basic_protocol(protocol) then\n"
"      rockspec.source.file = rockspec.source.file or dir.base_name(rockspec.source.url)\n"
"   end\n"
"   rockspec.source.protocol, rockspec.source.pathname = protocol, pathname\n"
"\n"
"   -- Temporary compatibility\n"
"   if rockspec.source.cvs_module then rockspec.source.module = rockspec.source.cvs_module end\n"
"   if rockspec.source.cvs_tag then rockspec.source.tag = rockspec.source.cvs_tag end\n"
"\n"
"   rockspec.local_abs_filename = filename\n"
"   local filebase = rockspec.source.file or rockspec.source.url\n"
"   local base = dir.deduce_base_dir(filebase)\n"
"   rockspec.source.dir_set = rockspec.source.dir ~= nil\n"
"   rockspec.source.dir = rockspec.source.dir\n"
"                      or rockspec.source.module\n"
"                      or ( (filebase:match(\"%.lua$\") or filebase:match(\"%.c$\"))\n"
"                           and (rockspec:format_is_at_least(\"3.0\")\n"
"                                and (dir.is_basic_protocol(protocol) and \".\" or base)\n"
"                                or  \".\") )\n"
"                      or base\n"
"\n"
"   rockspec.rocks_provided = (rockspec:format_is_at_least(\"3.0\")\n"
"                              and cfg.rocks_provided_3_0\n"
"                              or  cfg.rocks_provided)\n"
"\n"
"   for _, key in ipairs({\"dependencies\", \"build_dependencies\", \"test_dependencies\"}) do\n"
"      local ok, err = convert_dependencies(rockspec, key)\n"
"      if not ok then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"\n"
"   if not quick then\n"
"      configure_paths(rockspec)\n"
"   end\n"
"\n"
"   return setmetatable(rockspec, rockspec_mt)\n"
"end\n"
"\n"
"return rockspecs"
;
