const char luarocks_search_lua[] =
"local search = {}\n"
"\n"
"local dir = require(\"luarocks.dir\")\n"
"local path = require(\"luarocks.path\")\n"
"local manif = require(\"luarocks.manif\")\n"
"local vers = require(\"luarocks.core.vers\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local util = require(\"luarocks.util\")\n"
"local queries = require(\"luarocks.queries\")\n"
"local results = require(\"luarocks.results\")\n"
"\n"
"--- Store a search result (a rock or rockspec) in the result tree.\n"
"-- @param result_tree table: The result tree, where keys are package names and\n"
"-- values are tables matching version strings to arrays of\n"
"-- tables with fields \"arch\" and \"repo\".\n"
"-- @param result table: A result.\n"
"function search.store_result(result_tree, result)\n"
"   assert(type(result_tree) == \"table\")\n"
"   assert(result:type() == \"result\")\n"
"\n"
"   local name = result.name\n"
"   local version = result.version\n"
"\n"
"   if not result_tree[name] then result_tree[name] = {} end\n"
"   if not result_tree[name][version] then result_tree[name][version] = {} end\n"
"   table.insert(result_tree[name][version], {\n"
"      arch = result.arch,\n"
"      repo = result.repo,\n"
"      namespace = result.namespace,\n"
"   })\n"
"end\n"
"\n"
"--- Store a match in a result tree if version matches query.\n"
"-- Name, version, arch and repository path are stored in a given\n"
"-- table, optionally checking if version and arch (if given) match\n"
"-- a query.\n"
"-- @param result_tree table: The result tree, where keys are package names and\n"
"-- values are tables matching version strings to arrays of\n"
"-- tables with fields \"arch\" and \"repo\".\n"
"-- @param result table: a result object.\n"
"-- @param query table: a query object.\n"
"local function store_if_match(result_tree, result, query)\n"
"   assert(result:type() == \"result\")\n"
"   assert(query:type() == \"query\")\n"
"\n"
"   if result:satisfies(query) then\n"
"      search.store_result(result_tree, result)\n"
"   end\n"
"end\n"
"\n"
"--- Perform search on a local repository.\n"
"-- @param repo string: The pathname of the local repository.\n"
"-- @param query table: a query object.\n"
"-- @param result_tree table or nil: If given, this table will store the\n"
"-- result tree; if not given, a new table will be created.\n"
"-- @return table: The result tree, where keys are package names and\n"
"-- values are tables matching version strings to arrays of\n"
"-- tables with fields \"arch\" and \"repo\".\n"
"-- If a table was given in the \"result_tree\" parameter, that is the result value.\n"
"function search.disk_search(repo, query, result_tree)\n"
"   assert(type(repo) == \"string\")\n"
"   assert(query:type() == \"query\")\n"
"   assert(type(result_tree) == \"table\" or not result_tree)\n"
"   \n"
"   local fs = require(\"luarocks.fs\")\n"
"     \n"
"   if not result_tree then\n"
"      result_tree = {}\n"
"   end\n"
"   \n"
"   for name in fs.dir(repo) do\n"
"      local pathname = dir.path(repo, name)\n"
"      local rname, rversion, rarch = path.parse_name(name)\n"
"\n"
"      if rname and (pathname:match(\".rockspec$\") or pathname:match(\".rock$\")) then\n"
"         local result = results.new(rname, rversion, repo, rarch)\n"
"         store_if_match(result_tree, result, query)\n"
"      elseif fs.is_dir(pathname) then\n"
"         for version in fs.dir(pathname) do\n"
"            if version:match(\"-%d+$\") then\n"
"               local namespace = path.read_namespace(name, version, repo)\n"
"               local result = results.new(name, version, repo, \"installed\", namespace)\n"
"               store_if_match(result_tree, result, query)\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"   return result_tree\n"
"end\n"
"\n"
"--- Perform search on a rocks server or tree.\n"
"-- @param result_tree table: The result tree, where keys are package names and\n"
"-- values are tables matching version strings to arrays of\n"
"-- tables with fields \"arch\" and \"repo\".\n"
"-- @param repo string: The URL of a rocks server or\n"
"-- the pathname of a rocks tree (as returned by path.rocks_dir()).\n"
"-- @param query table: a query object.\n"
"-- @param lua_version string: Lua version in \"5.x\" format, defaults to installed version.\n"
"-- @param is_local boolean\n"
"-- @return true or, in case of errors, nil, an error message and an optional error code.\n"
"local function manifest_search(result_tree, repo, query, lua_version, is_local)\n"
"   assert(type(result_tree) == \"table\")\n"
"   assert(type(repo) == \"string\")\n"
"   assert(query:type() == \"query\")\n"
"\n"
"   -- FIXME do not add this in local repos\n"
"   if (not is_local) and query.namespace then\n"
"      repo = repo .. \"/manifests/\" .. query.namespace\n"
"   end\n"
"\n"
"   local manifest, err, errcode = manif.load_manifest(repo, lua_version)\n"
"   if not manifest then\n"
"      return nil, err, errcode\n"
"   end\n"
"   for name, versions in pairs(manifest.repository) do\n"
"      for version, items in pairs(versions) do\n"
"         local namespace = is_local and path.read_namespace(name, version, repo) or query.namespace\n"
"         for _, item in ipairs(items) do\n"
"            local result = results.new(name, version, repo, item.arch, namespace)\n"
"            store_if_match(result_tree, result, query)\n"
"         end\n"
"      end\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"local function remote_manifest_search(result_tree, repo, query, lua_version)\n"
"   return manifest_search(result_tree, repo, query, lua_version, false)\n"
"end\n"
"\n"
"function search.local_manifest_search(result_tree, repo, query, lua_version)\n"
"   return manifest_search(result_tree, repo, query, lua_version, true)\n"
"end\n"
"\n"
"--- Search on all configured rocks servers.\n"
"-- @param query table: a query object.\n"
"-- @param lua_version string: Lua version in \"5.x\" format, defaults to installed version.\n"
"-- @return table: A table where keys are package names\n"
"-- and values are tables matching version strings to arrays of\n"
"-- tables with fields \"arch\" and \"repo\".\n"
"function search.search_repos(query, lua_version)\n"
"   assert(query:type() == \"query\")\n"
"   local query_version = nil\n"
"   for _, constraint in pairs(query.constraints) do\n"
"      if constraint.op == '==' then\n"
"       query_version = constraint.version.string\n"
"       break\n"
"      end\n"
"   end\n"
"\n"
"   local result_tree = {}\n"
"   for _, repo in ipairs(cfg.rocks_servers) do\n"
"      if not cfg.disabled_servers[repo] then\n"
"         if type(repo) == \"string\" then\n"
"            repo = { repo }\n"
"         end\n"
"         for _, mirror in ipairs(repo) do\n"
"            local protocol, pathname = dir.split_url(mirror)\n"
"            if protocol == \"file\" then\n"
"               mirror = pathname\n"
"            end\n"
"            local ok, err, errcode = remote_manifest_search(result_tree, mirror, query, lua_version)\n"
"            if errcode == \"network\" then\n"
"               cfg.disabled_servers[repo] = true\n"
"            end\n"
"            if ok then\n"
"               break\n"
"            else\n"
"               util.warning(\"Failed searching manifest: \"..err)\n"
"            end\n"
"         end\n"
"      end\n"
"\n"
"      -- stop searching repos if exact match was found\n"
"      local result = result_tree[query.name]\n"
"      if result and result[query_version] then\n"
"         break\n"
"      end\n"
"   end\n"
"   -- search through rocks in cfg.rocks_provided\n"
"   local provided_repo = \"provided by VM or rocks_provided\"\n"
"   for name, version in pairs(cfg.rocks_provided) do\n"
"      local result = results.new(name, version, provided_repo, \"installed\")\n"
"      store_if_match(result_tree, result, query)\n"
"   end\n"
"   return result_tree\n"
"end\n"
"\n"
"--- Get the URL for the latest in a set of versions.\n"
"-- @param name string: The package name to be used in the URL.\n"
"-- @param versions table: An array of version informations, as stored\n"
"-- in search result trees.\n"
"-- @return string or nil: the URL for the latest version if one could\n"
"-- be picked, or nil.\n"
"local function pick_latest_version(name, versions)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(versions) == \"table\")\n"
"\n"
"   local vtables = {}\n"
"   for v, _ in pairs(versions) do\n"
"      table.insert(vtables, vers.parse_version(v))\n"
"   end\n"
"   table.sort(vtables)\n"
"   local version = vtables[#vtables].string\n"
"   local items = versions[version]\n"
"   if items then\n"
"      local pick = 1\n"
"      for i, item in ipairs(items) do\n"
"         if (item.arch == 'src' and items[pick].arch == 'rockspec')\n"
"         or (item.arch ~= 'src' and item.arch ~= 'rockspec') then\n"
"            pick = i\n"
"         end\n"
"      end\n"
"      return path.make_url(items[pick].repo, name, version, items[pick].arch)\n"
"   end\n"
"   return nil\n"
"end\n"
"\n"
"-- Find out which other Lua versions provide rock versions matching a query,\n"
"-- @param query table: a query object.\n"
"-- @param cli boolean: print status messages as it works\n"
"-- @return table: array of Lua versions supported, in \"5.x\" format.\n"
"local function supported_lua_versions(query, cli)\n"
"   assert(query:type() == \"query\")\n"
"   local result_tree = {}\n"
"\n"
"   for lua_version in util.lua_versions() do\n"
"      if lua_version ~= cfg.lua_version then\n"
"         if cli then\n"
"            util.printout(\"Checking for Lua \" .. lua_version .. \"...\")\n"
"         end\n"
"         if search.search_repos(query, lua_version)[query.name] then\n"
"            table.insert(result_tree, lua_version)\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   return result_tree\n"
"end\n"
"\n"
"function search.find_src_or_rockspec(ns_name, version, cli)\n"
"   local query = queries.new(ns_name, version, false, \"src|rockspec\")\n"
"   local url, err = search.find_suitable_rock(query, cli)\n"
"   if not url then\n"
"      return nil, \"Could not find a result named \"..tostring(query)..\": \"..err\n"
"   end\n"
"   return url\n"
"end\n"
"\n"
"--- Attempt to get a single URL for a given search for a rock.\n"
"-- @param query table: a query object.\n"
"-- @param cli boolean: print status messages as it works\n"
"-- @return string or (nil, string): URL for latest matching version\n"
"-- of the rock if it was found, or nil followed by an error message.\n"
"function search.find_suitable_rock(query, cli)\n"
"   assert(query:type() == \"query\")\n"
"\n"
"   if cfg.rocks_provided[query.name] ~= nil then\n"
"      -- Do not install versions listed in cfg.rocks_provided.\n"
"      return nil, \"Rock \"..query.name..\" \"..cfg.rocks_provided[query.name]..\n"
"         \" is already provided by VM or via 'rocks_provided' in the config file.\"\n"
"   end\n"
"   \n"
"   local result_tree = search.search_repos(query)\n"
"   local first_rock = next(result_tree)\n"
"   if not first_rock then\n"
"      if cfg.version_check_on_fail then\n"
"         if cli then\n"
"            util.printout(query.name .. \" not found for Lua \" .. cfg.lua_version .. \".\")\n"
"            util.printout(\"Checking if available for other Lua versions...\")\n"
"         end\n"
"      \n"
"         -- Check if constraints are satisfiable with other Lua versions.\n"
"         local lua_versions = supported_lua_versions(query, cli)\n"
"   \n"
"         if #lua_versions ~= 0 then\n"
"            -- Build a nice message in \"only Lua 5.x and 5.y but not 5.z.\" format\n"
"            for i, lua_version in ipairs(lua_versions) do\n"
"               lua_versions[i] = \"Lua \"..lua_version\n"
"            end\n"
"   \n"
"            local versions_message = \"only \"..table.concat(lua_versions, \" and \")..\n"
"               \" but not Lua \"..cfg.lua_version..\".\\n\"..\n"
"               \"(To suppress these checks run '\"..\n"
"               \"luarocks --lua-version=\"..cfg.lua_version..\" config version_check_on_fail false')\"\n"
"   \n"
"            if #query.constraints == 0 then\n"
"               return nil, query.name..\" supports \"..versions_message\n"
"            elseif #query.constraints == 1 and query.constraints[1].op == \"==\" then\n"
"               return nil, query.name..\" \"..query.constraints[1].version.string..\" supports \"..versions_message\n"
"            else\n"
"               return nil, \"Matching \"..query.name..\" versions support \"..versions_message\n"
"            end\n"
"         end\n"
"      end\n"
"\n"
"      return nil, \"No results matching query were found for Lua \" .. cfg.lua_version .. \".\"\n"
"   elseif next(result_tree, first_rock) then\n"
"      -- Shouldn't happen as query must match only one package.\n"
"      return nil, \"Several rocks matched query.\"\n"
"   else\n"
"      return pick_latest_version(query.name, result_tree[first_rock])\n"
"   end\n"
"end\n"
"\n"
"--- Print a list of rocks/rockspecs on standard output.\n"
"-- @param result_tree table: A result tree.\n"
"-- @param porcelain boolean or nil: A flag to force machine-friendly output.\n"
"function search.print_result_tree(result_tree, porcelain)\n"
"   assert(type(result_tree) == \"table\")\n"
"   assert(type(porcelain) == \"boolean\" or not porcelain)\n"
"   \n"
"   if porcelain then\n"
"      for package, versions in util.sortedpairs(result_tree) do\n"
"         for version, repos in util.sortedpairs(versions, vers.compare_versions) do\n"
"            for _, repo in ipairs(repos) do\n"
"               local nrepo = dir.normalize(repo.repo)\n"
"               util.printout(package, version, repo.arch, nrepo, repo.namespace)\n"
"            end\n"
"         end\n"
"      end\n"
"      return\n"
"   end\n"
"   \n"
"   for package, versions in util.sortedpairs(result_tree) do\n"
"      local namespaces = {}\n"
"      for version, repos in util.sortedpairs(versions, vers.compare_versions) do\n"
"         for _, repo in ipairs(repos) do\n"
"            local key = repo.namespace or \"\"\n"
"            local list = namespaces[key] or {}\n"
"            namespaces[key] = list\n"
"\n"
"            repo.repo = dir.normalize(repo.repo)\n"
"            table.insert(list, \"   \"..version..\" (\"..repo.arch..\") - \"..path.root_dir(repo.repo))\n"
"         end\n"
"      end\n"
"      for key, list in util.sortedpairs(namespaces) do\n"
"         util.printout(key == \"\" and package or key .. \"/\" .. package)\n"
"         for _, line in ipairs(list) do\n"
"            util.printout(line)\n"
"         end\n"
"         util.printout()\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"function search.pick_installed_rock(query, given_tree)\n"
"   assert(query:type() == \"query\")\n"
"\n"
"   local result_tree = {}\n"
"   local tree_map = {}\n"
"   local trees = cfg.rocks_trees\n"
"   if given_tree then\n"
"      trees = { given_tree }\n"
"   end\n"
"   for _, tree in ipairs(trees) do\n"
"      local rocks_dir = path.rocks_dir(tree)\n"
"      tree_map[rocks_dir] = tree\n"
"      search.local_manifest_search(result_tree, rocks_dir, query)\n"
"   end\n"
"   if not next(result_tree) then\n"
"      return nil, \"cannot find package \"..tostring(query)..\"\\nUse 'list' to find installed rocks.\"\n"
"   end\n"
"\n"
"   local version = nil\n"
"   local repo_url\n"
"   local _, versions = util.sortedpairs(result_tree)()\n"
"   --question: what do we do about multiple versions\? This should\n"
"   --give us the latest version on the last repo (which is usually the global one)\n"
"   for vs, repositories in util.sortedpairs(versions, vers.compare_versions) do\n"
"      if not version then version = vs end\n"
"      for _, rp in ipairs(repositories) do repo_url = rp.repo end\n"
"   end\n"
"\n"
"   local repo = tree_map[repo_url]\n"
"   return query.name, version, repo, repo_url\n"
"end\n"
"\n"
"return search\n"
""
;
