const char luarocks_tools_patch_lua[] =
"--- Patch utility to apply unified diffs.\n"
"--\n"
"-- http://lua-users.org/wiki/LuaPatch\n"
"--\n"
"-- (c) 2008 David Manura, Licensed under the same terms as Lua (MIT license).\n"
"-- Code is heavily based on the Python-based patch.py version 8.06-1\n"
"--   Copyright (c) 2008 rainforce.org, MIT License\n"
"--   Project home: http://code.google.com/p/python-patch/ .\n"
"--   Version 0.1\n"
"\n"
"local patch = {}\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"local fun = require(\"luarocks.fun\")\n"
"\n"
"local io = io\n"
"local os = os\n"
"local string = string\n"
"local table = table\n"
"local format = string.format\n"
"\n"
"-- logging\n"
"local debugmode = false\n"
"local function debug(_) end\n"
"local function info(_) end\n"
"local function warning(s) io.stderr:write(s .. '\\n') end\n"
"\n"
"-- Returns boolean whether string s2 starts with string s.\n"
"local function startswith(s, s2)\n"
"  return s:sub(1, #s2) == s2\n"
"end\n"
"\n"
"-- Returns boolean whether string s2 ends with string s.\n"
"local function endswith(s, s2)\n"
"  return #s >= #s2 and s:sub(#s-#s2+1) == s2\n"
"end\n"
"\n"
"-- Returns string s after filtering out any new-line characters from end.\n"
"local function endlstrip(s)\n"
"  return s:gsub('[\\r\\n]+$', '')\n"
"end\n"
"\n"
"-- Returns shallow copy of table t.\n"
"local function table_copy(t)\n"
"  local t2 = {}\n"
"  for k,v in pairs(t) do t2[k] = v end\n"
"  return t2\n"
"end\n"
"\n"
"local function exists(filename)\n"
"  local fh = io.open(filename)\n"
"  local result = fh ~= nil\n"
"  if fh then fh:close() end\n"
"  return result\n"
"end\n"
"local function isfile() return true end --FIX\?\n"
"\n"
"local function string_as_file(s)\n"
"   return {\n"
"      at = 0,\n"
"      str = s,\n"
"      len = #s,\n"
"      eof = false,\n"
"      read = function(self, n)\n"
"         if self.eof then return nil end\n"
"         local chunk = self.str:sub(self.at, self.at + n - 1)\n"
"         self.at = self.at + n\n"
"         if self.at > self.len then\n"
"            self.eof = true\n"
"         end\n"
"         return chunk\n"
"      end,\n"
"      close = function(self)\n"
"         self.eof = true\n"
"      end,\n"
"   }\n"
"end\n"
"\n"
"--\n"
"-- file_lines(f) is similar to f:lines() for file f.\n"
"-- The main difference is that read_lines includes\n"
"-- new-line character sequences (\"\\n\", \"\\r\\n\", \"\\r\"),\n"
"-- if any, at the end of each line.  Embedded \"\\0\" are also handled.\n"
"-- Caution: The newline behavior can depend on whether f is opened\n"
"-- in binary or ASCII mode.\n"
"-- (file_lines - version 20080913)\n"
"--\n"
"local function file_lines(f)\n"
"  local CHUNK_SIZE = 1024\n"
"  local buffer = \"\"\n"
"  local pos_beg = 1\n"
"  return function()\n"
"    local pos, chars\n"
"    while 1 do\n"
"      pos, chars = buffer:match('()([\\r\\n].)', pos_beg)\n"
"      if pos or not f then\n"
"        break\n"
"      elseif f then\n"
"        local chunk = f:read(CHUNK_SIZE)\n"
"        if chunk then\n"
"          buffer = buffer:sub(pos_beg) .. chunk\n"
"          pos_beg = 1\n"
"        else\n"
"          f = nil\n"
"        end\n"
"      end\n"
"    end\n"
"    if not pos then\n"
"      pos = #buffer\n"
"    elseif chars == '\\r\\n' then\n"
"      pos = pos + 1\n"
"    end\n"
"    local line = buffer:sub(pos_beg, pos)\n"
"    pos_beg = pos + 1\n"
"    if #line > 0 then\n"
"      return line\n"
"    end    \n"
"  end\n"
"end\n"
"\n"
"local function match_linerange(line)\n"
"  local m1, m2, m3, m4 =      line:match(\"^@@ %-(%d+),(%d+) %+(%d+),(%d+)\")\n"
"  if not m1 then m1, m3, m4 = line:match(\"^@@ %-(%d+) %+(%d+),(%d+)\") end\n"
"  if not m1 then m1, m2, m3 = line:match(\"^@@ %-(%d+),(%d+) %+(%d+)\") end\n"
"  if not m1 then m1, m3     = line:match(\"^@@ %-(%d+) %+(%d+)\") end\n"
"  return m1, m2, m3, m4\n"
"end\n"
"\n"
"local function match_epoch(str)\n"
"  return str:match(\"[^0-9]1969[^0-9]\") or str:match(\"[^0-9]1970[^0-9]\")\n"
"end\n"
"\n"
"function patch.read_patch(filename, data)\n"
"  -- define possible file regions that will direct the parser flow\n"
"  local state = 'header'\n"
"    -- 'header'    - comments before the patch body\n"
"    -- 'filenames' - lines starting with --- and +++\n"
"    -- 'hunkhead'  - @@ -R +R @@ sequence\n"
"    -- 'hunkbody'\n"
"    -- 'hunkskip'  - skipping invalid hunk mode\n"
"\n"
"  local all_ok = true\n"
"  local lineends = {lf=0, crlf=0, cr=0}\n"
"  local files = {source={}, target={}, epoch={}, hunks={}, fileends={}, hunkends={}}\n"
"  local nextfileno = 0\n"
"  local nexthunkno = 0    --: even if index starts with 0 user messages\n"
"                          --  number hunks from 1\n"
"\n"
"  -- hunkinfo holds parsed values, hunkactual - calculated\n"
"  local hunkinfo = {\n"
"    startsrc=nil, linessrc=nil, starttgt=nil, linestgt=nil,\n"
"    invalid=false, text={}\n"
"  }\n"
"  local hunkactual = {linessrc=nil, linestgt=nil}\n"
"\n"
"  info(format(\"reading patch %s\", filename))\n"
"\n"
"  local fp\n"
"  if data then\n"
"    fp = string_as_file(data)\n"
"  else\n"
"    fp = filename == '-' and io.stdin or assert(io.open(filename, \"rb\"))\n"
"  end\n"
"  local lineno = 0\n"
"\n"
"  for line in file_lines(fp) do\n"
"    lineno = lineno + 1\n"
"    if state == 'header' then\n"
"      if startswith(line, \"--- \") then\n"
"        state = 'filenames'\n"
"      end\n"
"      -- state is 'header' or 'filenames'\n"
"    end\n"
"    if state == 'hunkbody' then\n"
"      -- skip hunkskip and hunkbody code until definition of hunkhead read\n"
"\n"
"      if line:match\"^[\\r\\n]*$\" then\n"
"          -- prepend space to empty lines to interpret them as context properly\n"
"          line = \" \" .. line\n"
"      end\n"
"\n"
"      -- process line first\n"
"      if line:match\"^[- +\\\\]\" then\n"
"          -- gather stats about line endings\n"
"          local he = files.hunkends[nextfileno]\n"
"          if endswith(line, \"\\r\\n\") then\n"
"            he.crlf = he.crlf + 1\n"
"          elseif endswith(line, \"\\n\") then\n"
"            he.lf = he.lf + 1\n"
"          elseif endswith(line, \"\\r\") then\n"
"            he.cr = he.cr + 1\n"
"          end\n"
"          if startswith(line, \"-\") then\n"
"            hunkactual.linessrc = hunkactual.linessrc + 1\n"
"          elseif startswith(line, \"+\") then\n"
"            hunkactual.linestgt = hunkactual.linestgt + 1\n"
"          elseif startswith(line, \"\\\\\") then\n"
"            -- nothing\n"
"          else\n"
"            hunkactual.linessrc = hunkactual.linessrc + 1\n"
"            hunkactual.linestgt = hunkactual.linestgt + 1\n"
"          end\n"
"          table.insert(hunkinfo.text, line)\n"
"          -- todo: handle \\ No newline cases\n"
"      else\n"
"          warning(format(\"invalid hunk no.%d at %d for target file %s\",\n"
"                         nexthunkno, lineno, files.target[nextfileno]))\n"
"          -- add hunk status node\n"
"          table.insert(files.hunks[nextfileno], table_copy(hunkinfo))\n"
"          files.hunks[nextfileno][nexthunkno].invalid = true\n"
"          all_ok = false\n"
"          state = 'hunkskip'\n"
"      end\n"
"\n"
"      -- check exit conditions\n"
"      if hunkactual.linessrc > hunkinfo.linessrc or\n"
"         hunkactual.linestgt > hunkinfo.linestgt\n"
"      then\n"
"          warning(format(\"extra hunk no.%d lines at %d for target %s\",\n"
"                         nexthunkno, lineno, files.target[nextfileno]))\n"
"          -- add hunk status node\n"
"          table.insert(files.hunks[nextfileno], table_copy(hunkinfo))\n"
"          files.hunks[nextfileno][nexthunkno].invalid = true\n"
"          state = 'hunkskip'\n"
"      elseif hunkinfo.linessrc == hunkactual.linessrc and\n"
"             hunkinfo.linestgt == hunkactual.linestgt\n"
"      then\n"
"          table.insert(files.hunks[nextfileno], table_copy(hunkinfo))\n"
"          state = 'hunkskip'\n"
"\n"
"          -- detect mixed window/unix line ends\n"
"          local ends = files.hunkends[nextfileno]\n"
"          if (ends.cr~=0 and 1 or 0) + (ends.crlf~=0 and 1 or 0) +\n"
"             (ends.lf~=0 and 1 or 0) > 1\n"
"          then\n"
"            warning(format(\"inconsistent line ends in patch hunks for %s\",\n"
"                    files.source[nextfileno]))\n"
"          end\n"
"      end\n"
"      -- state is 'hunkbody' or 'hunkskip'\n"
"    end\n"
"\n"
"    if state == 'hunkskip' then\n"
"      if match_linerange(line) then\n"
"        state = 'hunkhead'\n"
"      elseif startswith(line, \"--- \") then\n"
"        state = 'filenames'\n"
"        if debugmode and #files.source > 0 then\n"
"            debug(format(\"- %2d hunks for %s\", #files.hunks[nextfileno],\n"
"                         files.source[nextfileno]))\n"
"        end\n"
"      end\n"
"      -- state is 'hunkskip', 'hunkhead', or 'filenames'\n"
"    end\n"
"    local advance\n"
"    if state == 'filenames' then\n"
"      if startswith(line, \"--- \") then\n"
"        if fun.contains(files.source, nextfileno) then\n"
"          all_ok = false\n"
"          warning(format(\"skipping invalid patch for %s\",\n"
"                         files.source[nextfileno+1]))\n"
"          table.remove(files.source, nextfileno+1)\n"
"          -- double source filename line is encountered\n"
"          -- attempt to restart from this second line\n"
"        end\n"
"        -- Accept a space as a terminator, like GNU patch does.\n"
"        -- Breaks patches containing filenames with spaces...\n"
"        -- FIXME Figure out what does GNU patch do in those cases.\n"
"        local match, rest = line:match(\"^%-%-%- ([^ \\t\\r\\n]+)(.*)\")\n"
"        if not match then\n"
"          all_ok = false\n"
"          warning(format(\"skipping invalid filename at line %d\", lineno+1))\n"
"          state = 'header'\n"
"        else\n"
"          if match_epoch(rest) then\n"
"            files.epoch[nextfileno + 1] = true\n"
"          end\n"
"          table.insert(files.source, match)\n"
"        end\n"
"      elseif not startswith(line, \"+++ \") then\n"
"        if fun.contains(files.source, nextfileno) then\n"
"          all_ok = false\n"
"          warning(format(\"skipping invalid patch with no target for %s\",\n"
"                         files.source[nextfileno+1]))\n"
"          table.remove(files.source, nextfileno+1)\n"
"        else\n"
"          -- this should be unreachable\n"
"          warning(\"skipping invalid target patch\")\n"
"        end\n"
"        state = 'header'\n"
"      else\n"
"        if fun.contains(files.target, nextfileno) then\n"
"          all_ok = false\n"
"          warning(format(\"skipping invalid patch - double target at line %d\",\n"
"                         lineno+1))\n"
"          table.remove(files.source, nextfileno+1)\n"
"          table.remove(files.target, nextfileno+1)\n"
"          nextfileno = nextfileno - 1\n"
"          -- double target filename line is encountered\n"
"          -- switch back to header state\n"
"          state = 'header'\n"
"        else\n"
"          -- Accept a space as a terminator, like GNU patch does.\n"
"          -- Breaks patches containing filenames with spaces...\n"
"          -- FIXME Figure out what does GNU patch do in those cases.\n"
"          local re_filename = \"^%+%+%+ ([^ \\t\\r\\n]+)(.*)$\"\n"
"          local match, rest = line:match(re_filename)\n"
"          if not match then\n"
"            all_ok = false\n"
"            warning(format(\n"
"              \"skipping invalid patch - no target filename at line %d\",\n"
"              lineno+1))\n"
"            state = 'header'\n"
"          else\n"
"            table.insert(files.target, match)\n"
"            nextfileno = nextfileno + 1\n"
"            if match_epoch(rest) then\n"
"              files.epoch[nextfileno] = true\n"
"            end\n"
"            nexthunkno = 0\n"
"            table.insert(files.hunks, {})\n"
"            table.insert(files.hunkends, table_copy(lineends))\n"
"            table.insert(files.fileends, table_copy(lineends))\n"
"            state = 'hunkhead'\n"
"            advance = true\n"
"          end\n"
"        end\n"
"      end\n"
"      -- state is 'filenames', 'header', or ('hunkhead' with advance)\n"
"    end\n"
"    if not advance and state == 'hunkhead' then\n"
"      local m1, m2, m3, m4 = match_linerange(line)\n"
"      if not m1 then\n"
"        if not fun.contains(files.hunks, nextfileno-1) then\n"
"          all_ok = false\n"
"          warning(format(\"skipping invalid patch with no hunks for file %s\",\n"
"                         files.target[nextfileno]))\n"
"        end\n"
"        state = 'header'\n"
"      else\n"
"        hunkinfo.startsrc = tonumber(m1)\n"
"        hunkinfo.linessrc = tonumber(m2 or 1)\n"
"        hunkinfo.starttgt = tonumber(m3)\n"
"        hunkinfo.linestgt = tonumber(m4 or 1)\n"
"        hunkinfo.invalid = false\n"
"        hunkinfo.text = {}\n"
"\n"
"        hunkactual.linessrc = 0\n"
"        hunkactual.linestgt = 0\n"
"\n"
"        state = 'hunkbody'\n"
"        nexthunkno = nexthunkno + 1\n"
"      end\n"
"      -- state is 'header' or 'hunkbody'\n"
"    end\n"
"  end\n"
"  if state ~= 'hunkskip' then\n"
"    warning(format(\"patch file incomplete - %s\", filename))\n"
"    all_ok = false\n"
"    -- os.exit(\?)\n"
"  else\n"
"    -- duplicated message when an eof is reached\n"
"    if debugmode and #files.source > 0 then\n"
"      debug(format(\"- %2d hunks for %s\", #files.hunks[nextfileno],\n"
"                   files.source[nextfileno]))\n"
"    end\n"
"  end\n"
"\n"
"  local sum = 0; for _,hset in ipairs(files.hunks) do sum = sum + #hset end\n"
"  info(format(\"total files: %d  total hunks: %d\", #files.source, sum))\n"
"  fp:close()\n"
"  return files, all_ok\n"
"end\n"
"\n"
"local function find_hunk(file, h, hno)\n"
"  for fuzz=0,2 do\n"
"    local lineno = h.startsrc\n"
"    for i=0,#file do\n"
"      local found = true\n"
"      local location = lineno\n"
"      for l, hline in ipairs(h.text) do\n"
"        if l > fuzz then\n"
"          -- todo: \\ No newline at the end of file\n"
"          if startswith(hline, \" \") or startswith(hline, \"-\") then\n"
"            local line = file[lineno]\n"
"            lineno = lineno + 1\n"
"            if not line or #line == 0 then\n"
"              found = false\n"
"              break\n"
"            end\n"
"            if endlstrip(line) ~= endlstrip(hline:sub(2)) then\n"
"              found = false\n"
"              break\n"
"            end\n"
"          end\n"
"        end\n"
"      end\n"
"      if found then\n"
"        local offset = location - h.startsrc - fuzz\n"
"        if offset ~= 0 then\n"
"          warning(format(\"Hunk %d found at offset %d%s...\", hno, offset, fuzz == 0 and \"\" or format(\" (fuzz %d)\", fuzz)))\n"
"        end\n"
"        h.startsrc = location\n"
"        h.starttgt = h.starttgt + offset\n"
"        for _=1,fuzz do\n"
"           table.remove(h.text, 1)\n"
"           table.remove(h.text, #h.text)\n"
"        end\n"
"        return true\n"
"      end\n"
"      lineno = i\n"
"    end\n"
"  end\n"
"  return false\n"
"end\n"
"\n"
"local function load_file(filename)\n"
"  local fp = assert(io.open(filename))\n"
"  local file = {}\n"
"  local readline = file_lines(fp)\n"
"  while true do\n"
"    local line = readline()\n"
"    if not line then break end\n"
"    table.insert(file, line)\n"
"  end\n"
"  fp:close()\n"
"  return file\n"
"end\n"
"\n"
"local function find_hunks(file, hunks)\n"
"  for hno, h in ipairs(hunks) do\n"
"    find_hunk(file, h, hno)\n"
"  end\n"
"end\n"
"\n"
"local function check_patched(file, hunks)\n"
"  local lineno = 1\n"
"  local ok, err = pcall(function()\n"
"    if #file == 0 then\n"
"      error('nomatch', 0)\n"
"    end\n"
"    for hno, h in ipairs(hunks) do\n"
"      -- skip to line just before hunk starts\n"
"      if #file < h.starttgt then\n"
"        error('nomatch', 0)\n"
"      end\n"
"      lineno = h.starttgt\n"
"      for _, hline in ipairs(h.text) do\n"
"        -- todo: \\ No newline at the end of file\n"
"        if not startswith(hline, \"-\") and not startswith(hline, \"\\\\\") then\n"
"          local line = file[lineno]\n"
"          lineno = lineno + 1\n"
"          if #line == 0 then\n"
"            error('nomatch', 0)\n"
"          end\n"
"          if endlstrip(line) ~= endlstrip(hline:sub(2)) then\n"
"            warning(format(\"file is not patched - failed hunk: %d\", hno))\n"
"            error('nomatch', 0)\n"
"          end\n"
"        end\n"
"      end\n"
"    end\n"
"  end)\n"
"  -- todo: display failed hunk, i.e. expected/found\n"
"  return err ~= 'nomatch'\n"
"end\n"
"\n"
"local function patch_hunks(srcname, tgtname, hunks)\n"
"  local src = assert(io.open(srcname, \"rb\"))\n"
"  local tgt = assert(io.open(tgtname, \"wb\"))\n"
"\n"
"  local src_readline = file_lines(src)\n"
"\n"
"  -- todo: detect linefeeds early - in apply_files routine\n"
"  --       to handle cases when patch starts right from the first\n"
"  --       line and no lines are processed. At the moment substituted\n"
"  --       lineends may not be the same at the start and at the end\n"
"  --       of patching. Also issue a warning about mixed lineends\n"
"\n"
"  local srclineno = 1\n"
"  local lineends = {['\\n']=0, ['\\r\\n']=0, ['\\r']=0}\n"
"  for hno, h in ipairs(hunks) do\n"
"    debug(format(\"processing hunk %d for file %s\", hno, tgtname))\n"
"    -- skip to line just before hunk starts\n"
"    while srclineno < h.startsrc do\n"
"      local line = src_readline()\n"
"      -- Python 'U' mode works only with text files\n"
"      if endswith(line, \"\\r\\n\") then\n"
"        lineends[\"\\r\\n\"] = lineends[\"\\r\\n\"] + 1\n"
"      elseif endswith(line, \"\\n\") then\n"
"        lineends[\"\\n\"] = lineends[\"\\n\"] + 1\n"
"      elseif endswith(line, \"\\r\") then\n"
"        lineends[\"\\r\"] = lineends[\"\\r\"] + 1\n"
"      end\n"
"      tgt:write(line)\n"
"      srclineno = srclineno + 1\n"
"    end\n"
"\n"
"    for _,hline in ipairs(h.text) do\n"
"      -- todo: check \\ No newline at the end of file\n"
"      if startswith(hline, \"-\") or startswith(hline, \"\\\\\") then\n"
"        src_readline()\n"
"        srclineno = srclineno + 1\n"
"      else\n"
"        if not startswith(hline, \"+\") then\n"
"          src_readline()\n"
"          srclineno = srclineno + 1\n"
"        end\n"
"        local line2write = hline:sub(2)\n"
"        -- detect if line ends are consistent in source file\n"
"        local sum = 0\n"
"        for _,v in pairs(lineends) do if v > 0 then sum=sum+1 end end\n"
"        if sum == 1 then\n"
"          local newline\n"
"          for k,v in pairs(lineends) do if v ~= 0 then newline = k end end\n"
"          tgt:write(endlstrip(line2write) .. newline)\n"
"        else -- newlines are mixed or unknown\n"
"          tgt:write(line2write)\n"
"        end\n"
"      end\n"
"    end\n"
"  end\n"
"  for line in src_readline do\n"
"    tgt:write(line)\n"
"  end\n"
"  tgt:close()\n"
"  src:close()\n"
"  return true\n"
"end \n"
"\n"
"local function strip_dirs(filename, strip)\n"
"  if strip == nil then return filename end\n"
"  for _=1,strip do\n"
"    filename=filename:gsub(\"^[^/]*/\", \"\")\n"
"  end\n"
"  return filename\n"
"end\n"
"\n"
"local function write_new_file(filename, hunk)\n"
"  local fh = io.open(filename, \"wb\")\n"
"  if not fh then return false end\n"
"  for _, hline in ipairs(hunk.text) do\n"
"    if not hline:sub(1,1) == \"+\" then\n"
"      return false, \"malformed patch\"\n"
"    end\n"
"    fh:write(hline:sub(2))\n"
"  end\n"
"  fh:close()\n"
"  return true\n"
"end\n"
"\n"
"local function patch_file(source, target, epoch, hunks, strip, create_delete)\n"
"  local create_file = false\n"
"  if create_delete then\n"
"    local is_src_epoch = epoch and #hunks == 1 and hunks[1].startsrc == 0 and hunks[1].linessrc == 0\n"
"    if is_src_epoch or source == \"/dev/null\" then\n"
"      info(format(\"will create %s\", target))\n"
"      create_file = true\n"
"    end\n"
"  end\n"
"  if create_file then\n"
"    return write_new_file(fs.absolute_name(strip_dirs(target, strip)), hunks[1])\n"
"  end\n"
"  source = strip_dirs(source, strip)\n"
"  local f2patch = source\n"
"  if not exists(f2patch) then\n"
"    f2patch = strip_dirs(target, strip)\n"
"    f2patch = fs.absolute_name(f2patch)\n"
"    if not exists(f2patch) then  --FIX:if f2patch nil\n"
"      warning(format(\"source/target file does not exist\\n--- %s\\n+++ %s\",\n"
"              source, f2patch))\n"
"      return false\n"
"    end\n"
"  end\n"
"  if not isfile(f2patch) then\n"
"    warning(format(\"not a file - %s\", f2patch))\n"
"    return false\n"
"  end\n"
"\n"
"  source = f2patch\n"
"\n"
"  -- validate before patching\n"
"  local file = load_file(source)\n"
"  local hunkno = 1\n"
"  local hunk = hunks[hunkno]\n"
"  local hunkfind = {}\n"
"  local validhunks = 0\n"
"  local canpatch = false\n"
"  local hunklineno\n"
"  if not file then\n"
"    return nil, \"failed reading file \" .. source\n"
"  end\n"
"\n"
"  if create_delete then\n"
"    if epoch and #hunks == 1 and hunks[1].starttgt == 0 and hunks[1].linestgt == 0 then\n"
"      local ok = os.remove(source)\n"
"      if not ok then\n"
"        return false\n"
"      end\n"
"      info(format(\"successfully removed %s\", source))\n"
"      return true\n"
"    end\n"
"  end\n"
"\n"
"  find_hunks(file, hunks)\n"
"\n"
"  local function process_line(line, lineno)\n"
"    if not hunk or lineno < hunk.startsrc then\n"
"      return false\n"
"    end\n"
"    if lineno == hunk.startsrc then\n"
"      hunkfind = {}\n"
"      for _,x in ipairs(hunk.text) do\n"
"        if x:sub(1,1) == ' ' or x:sub(1,1) == '-' then\n"
"          hunkfind[#hunkfind+1] = endlstrip(x:sub(2))\n"
"        end\n"
"      end\n"
"      hunklineno = 1\n"
"  \n"
"      -- todo \\ No newline at end of file\n"
"    end\n"
"    -- check hunks in source file\n"
"    if lineno < hunk.startsrc + #hunkfind - 1 then\n"
"      if endlstrip(line) == hunkfind[hunklineno] then\n"
"        hunklineno = hunklineno + 1\n"
"      else\n"
"        debug(format(\"hunk no.%d doesn't match source file %s\",\n"
"                     hunkno, source))\n"
"        -- file may be already patched, but check other hunks anyway\n"
"        hunkno = hunkno + 1\n"
"        if hunkno <= #hunks then\n"
"          hunk = hunks[hunkno]\n"
"          return false\n"
"        else\n"
"          return true\n"
"        end\n"
"      end\n"
"    end\n"
"    -- check if processed line is the last line\n"
"    if lineno == hunk.startsrc + #hunkfind - 1 then\n"
"      debug(format(\"file %s hunk no.%d -- is ready to be patched\",\n"
"                   source, hunkno))\n"
"      hunkno = hunkno + 1\n"
"      validhunks = validhunks + 1\n"
"      if hunkno <= #hunks then\n"
"        hunk = hunks[hunkno]\n"
"      else\n"
"        if validhunks == #hunks then\n"
"          -- patch file\n"
"          canpatch = true\n"
"          return true\n"
"        end\n"
"      end\n"
"    end\n"
"    return false\n"
"  end\n"
"\n"
"  local done = false\n"
"  for lineno, line in ipairs(file) do\n"
"    done = process_line(line, lineno)\n"
"    if done then\n"
"      break\n"
"    end\n"
"  end\n"
"  if not done then\n"
"    if hunkno <= #hunks and not create_file then\n"
"      warning(format(\"premature end of source file %s at hunk %d\",\n"
"                     source, hunkno))\n"
"      return false\n"
"    end\n"
"  end\n"
"  if validhunks < #hunks then\n"
"    if check_patched(file, hunks) then\n"
"      warning(format(\"already patched  %s\", source))\n"
"    elseif not create_file then\n"
"      warning(format(\"source file is different - %s\", source))\n"
"      return false\n"
"    end\n"
"  end\n"
"  if not canpatch then\n"
"    return true\n"
"  end\n"
"  local backupname = source .. \".orig\"\n"
"  if exists(backupname) then\n"
"    warning(format(\"can't backup original file to %s - aborting\",\n"
"                   backupname))\n"
"    return false\n"
"  end\n"
"  local ok = os.rename(source, backupname)\n"
"  if not ok then\n"
"    warning(format(\"failed backing up %s when patching\", source))\n"
"    return false\n"
"  end\n"
"  patch_hunks(backupname, source, hunks)\n"
"  info(format(\"successfully patched %s\", source))\n"
"  os.remove(backupname)\n"
"  return true\n"
"end\n"
"\n"
"function patch.apply_patch(the_patch, strip, create_delete)\n"
"  local all_ok = true\n"
"  local total = #the_patch.source\n"
"  for fileno, source in ipairs(the_patch.source) do\n"
"    local target = the_patch.target[fileno]\n"
"    local hunks = the_patch.hunks[fileno]\n"
"    local epoch = the_patch.epoch[fileno]\n"
"    info(format(\"processing %d/%d:\\t %s\", fileno, total, source))\n"
"    local ok = patch_file(source, target, epoch, hunks, strip, create_delete)\n"
"    all_ok = all_ok and ok\n"
"  end\n"
"  -- todo: check for premature eof\n"
"  return all_ok\n"
"end\n"
"\n"
"return patch\n"
""
;
