const char luarocks_tools_tar_lua[] =
"\n"
"--- A pure-Lua implementation of untar (unpacking .tar archives)\n"
"local tar = {}\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local fun = require(\"luarocks.fun\")\n"
"\n"
"local blocksize = 512\n"
"\n"
"local function get_typeflag(flag)\n"
"   if flag == \"0\" or flag == \"\\0\" then return \"file\"\n"
"   elseif flag == \"1\" then return \"link\"\n"
"   elseif flag == \"2\" then return \"symlink\" -- \"reserved\" in POSIX, \"symlink\" in GNU\n"
"   elseif flag == \"3\" then return \"character\"\n"
"   elseif flag == \"4\" then return \"block\"\n"
"   elseif flag == \"5\" then return \"directory\"\n"
"   elseif flag == \"6\" then return \"fifo\"\n"
"   elseif flag == \"7\" then return \"contiguous\" -- \"reserved\" in POSIX, \"contiguous\" in GNU\n"
"   elseif flag == \"x\" then return \"next file\"\n"
"   elseif flag == \"g\" then return \"global extended header\"\n"
"   elseif flag == \"L\" then return \"long name\"\n"
"   elseif flag == \"K\" then return \"long link name\"\n"
"   end\n"
"   return \"unknown\"\n"
"end\n"
"\n"
"local function octal_to_number(octal)\n"
"   local exp = 0\n"
"   local number = 0\n"
"   octal = octal:gsub(\"%s\", \"\")\n"
"   for i = #octal,1,-1 do\n"
"      local digit = tonumber(octal:sub(i,i)) \n"
"      if not digit then\n"
"         break\n"
"      end\n"
"      number = number + (digit * 8^exp)\n"
"      exp = exp + 1\n"
"   end\n"
"   return number\n"
"end\n"
"\n"
"local function checksum_header(block)\n"
"   local sum = 256\n"
"   for i = 1,148 do\n"
"      local b = block:byte(i) or 0\n"
"      sum = sum + b\n"
"   end\n"
"   for i = 157,500 do\n"
"      local b = block:byte(i) or 0\n"
"      sum = sum + b\n"
"   end\n"
"   return sum\n"
"end\n"
"\n"
"local function nullterm(s)\n"
"   return s:match(\"^[^%z]*\")\n"
"end\n"
"\n"
"local function read_header_block(block)\n"
"   local header = {}\n"
"   header.name = nullterm(block:sub(1,100))\n"
"   header.mode = nullterm(block:sub(101,108)):gsub(\" \", \"\")\n"
"   header.uid = octal_to_number(nullterm(block:sub(109,116)))\n"
"   header.gid = octal_to_number(nullterm(block:sub(117,124)))\n"
"   header.size = octal_to_number(nullterm(block:sub(125,136)))\n"
"   header.mtime = octal_to_number(nullterm(block:sub(137,148)))\n"
"   header.chksum = octal_to_number(nullterm(block:sub(149,156)))\n"
"   header.typeflag = get_typeflag(block:sub(157,157))\n"
"   header.linkname = nullterm(block:sub(158,257))\n"
"   header.magic = block:sub(258,263)\n"
"   header.version = block:sub(264,265)\n"
"   header.uname = nullterm(block:sub(266,297))\n"
"   header.gname = nullterm(block:sub(298,329))\n"
"   header.devmajor = octal_to_number(nullterm(block:sub(330,337)))\n"
"   header.devminor = octal_to_number(nullterm(block:sub(338,345)))\n"
"   header.prefix = block:sub(346,500)\n"
"   -- if header.magic ~= \"ustar \" and header.magic ~= \"ustar\\0\" then\n"
"   --    return false, (\"Invalid header magic %6x\"):format(bestring_to_number(header.magic))\n"
"   -- end\n"
"   -- if header.version ~= \"00\" and header.version ~= \" \\0\" then\n"
"   --    return false, \"Unknown version \"..header.version\n"
"   -- end\n"
"   if not checksum_header(block) == header.chksum then\n"
"      return false, \"Failed header checksum\"\n"
"   end\n"
"   return header\n"
"end\n"
"\n"
"function tar.untar(filename, destdir)\n"
"   assert(type(filename) == \"string\")\n"
"   assert(type(destdir) == \"string\")\n"
"\n"
"   local tar_handle = io.open(filename, \"rb\")\n"
"   if not tar_handle then return nil, \"Error opening file \"..filename end\n"
"   \n"
"   local long_name, long_link_name\n"
"   local ok, err\n"
"   local make_dir = fun.memoize(fs.make_dir)\n"
"   while true do\n"
"      local block\n"
"      repeat\n"
"         block = tar_handle:read(blocksize)\n"
"      until (not block) or checksum_header(block) > 256\n"
"      if not block then break end\n"
"      if #block < blocksize then\n"
"         ok, err = nil, \"Invalid block size -- corrupted file\?\"\n"
"         break\n"
"      end\n"
"      local header\n"
"      header, err = read_header_block(block)\n"
"      if not header then\n"
"         ok = false\n"
"         break\n"
"      end\n"
"\n"
"      local file_data = tar_handle:read(math.ceil(header.size / blocksize) * blocksize):sub(1,header.size)\n"
"\n"
"      if header.typeflag == \"long name\" then\n"
"         long_name = nullterm(file_data)\n"
"      elseif header.typeflag == \"long link name\" then\n"
"         long_link_name = nullterm(file_data)\n"
"      else\n"
"         if long_name then\n"
"            header.name = long_name\n"
"            long_name = nil\n"
"         end\n"
"         if long_link_name then\n"
"            header.name = long_link_name\n"
"            long_link_name = nil\n"
"         end\n"
"      end\n"
"      local pathname = dir.path(destdir, header.name)\n"
"      pathname = fs.absolute_name(pathname)\n"
"      if header.typeflag == \"directory\" then\n"
"         ok, err = make_dir(pathname)\n"
"         if not ok then\n"
"            break\n"
"         end\n"
"      elseif header.typeflag == \"file\" then\n"
"         local dirname = dir.dir_name(pathname)\n"
"         if dirname ~= \"\" then\n"
"            ok, err = make_dir(dirname)\n"
"            if not ok then\n"
"               break\n"
"            end\n"
"         end\n"
"         local file_handle\n"
"         file_handle, err = io.open(pathname, \"wb\")\n"
"         if not file_handle then\n"
"            ok = nil\n"
"            break\n"
"         end\n"
"         file_handle:write(file_data)\n"
"         file_handle:close()\n"
"         fs.set_time(pathname, header.mtime)\n"
"         if header.mode:match(\"[75]\") then\n"
"            fs.set_permissions(pathname, \"exec\", \"all\")\n"
"         else\n"
"            fs.set_permissions(pathname, \"read\", \"all\")\n"
"         end\n"
"      end\n"
"      --[[\n"
"      for k,v in pairs(header) do\n"
"         util.printout(\"[\\\"\"..tostring(k)..\"\\\"] = \"..(type(v)==\"number\" and v or \"\\\"\"..v:gsub(\"%z\", \"\\\\0\")..\"\\\"\"))\n"
"      end\n"
"      util.printout()\n"
"      --]]\n"
"   end\n"
"   tar_handle:close()\n"
"   return ok, err\n"
"end\n"
"\n"
"return tar\n"
""
;
