const char luarocks_tools_zip_lua[] =
"\n"
"--- A Lua implementation of .zip and .gz file compression and decompression,\n"
"-- using only lzlib or lua-lzib.\n"
"local zip = {}\n"
"\n"
"local zlib = require(\"zlib\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local fun = require(\"luarocks.fun\")\n"
"local dir = require(\"luarocks.dir\")\n"
"\n"
"local stat_ok, stat = pcall(require, \"posix.sys.stat\")\n"
"\n"
"local function shr(n, m)\n"
"   return math.floor(n / 2^m)\n"
"end\n"
"\n"
"local function shl(n, m)\n"
"   return n * 2^m\n"
"end\n"
"local function lowbits(n, m)\n"
"   return n % 2^m\n"
"end\n"
"\n"
"local function mode_to_windowbits(mode)\n"
"   if mode == \"gzip\" then\n"
"      return 31\n"
"   elseif mode == \"zlib\" then\n"
"      return 0\n"
"   elseif mode == \"raw\" then\n"
"      return -15\n"
"   end\n"
"end\n"
"\n"
"-- zlib module can be provided by both lzlib and lua-lzib packages.\n"
"-- Create a compatibility layer.\n"
"local zlib_compress, zlib_uncompress, zlib_crc32\n"
"if zlib._VERSION:match \"^lua%-zlib\" then\n"
"   function zlib_compress(data, mode)\n"
"      return (zlib.deflate(6, mode_to_windowbits(mode))(data, \"finish\"))\n"
"   end\n"
"\n"
"   function zlib_uncompress(data, mode)\n"
"      return (zlib.inflate(mode_to_windowbits(mode))(data))\n"
"   end\n"
"\n"
"   function zlib_crc32(data)\n"
"      return zlib.crc32()(data)\n"
"   end\n"
"elseif zlib._VERSION:match \"^lzlib\" then\n"
"   function zlib_compress(data, mode)\n"
"      return zlib.compress(data, -1, nil, mode_to_windowbits(mode))\n"
"   end\n"
"\n"
"   function zlib_uncompress(data, mode)\n"
"      return zlib.decompress(data, mode_to_windowbits(mode))\n"
"   end\n"
"\n"
"   function zlib_crc32(data)\n"
"      return zlib.crc32(zlib.crc32(), data)\n"
"   end\n"
"else\n"
"   error(\"unknown zlib library\", 0)\n"
"end\n"
"\n"
"local function number_to_lestring(number, nbytes)\n"
"   local out = {}\n"
"   for _ = 1, nbytes do\n"
"      local byte = number % 256\n"
"      table.insert(out, string.char(byte))\n"
"      number = (number - byte) / 256\n"
"   end\n"
"   return table.concat(out)\n"
"end\n"
"\n"
"local function lestring_to_number(str)\n"
"   local n = 0\n"
"   local bytes = { string.byte(str, 1, #str) }\n"
"   for b = 1, #str do\n"
"      n = n + shl(bytes[b], (b-1)*8)\n"
"   end\n"
"   return math.floor(n)\n"
"end\n"
"\n"
"local LOCAL_FILE_HEADER_SIGNATURE  = number_to_lestring(0x04034b50, 4)\n"
"local DATA_DESCRIPTOR_SIGNATURE    = number_to_lestring(0x08074b50, 4)\n"
"local CENTRAL_DIRECTORY_SIGNATURE  = number_to_lestring(0x02014b50, 4)\n"
"local END_OF_CENTRAL_DIR_SIGNATURE = number_to_lestring(0x06054b50, 4)\n"
"\n"
"--- Begin a new file to be stored inside the zipfile.\n"
"-- @param self handle of the zipfile being written.\n"
"-- @param filename filenome of the file to be added to the zipfile.\n"
"-- @return true if succeeded, nil in case of failure.\n"
"local function zipwriter_open_new_file_in_zip(self, filename)\n"
"   if self.in_open_file then\n"
"      self:close_file_in_zip()\n"
"      return nil\n"
"   end\n"
"   local lfh = {}\n"
"   self.local_file_header = lfh\n"
"   lfh.last_mod_file_time = 0 -- TODO\n"
"   lfh.last_mod_file_date = 0 -- TODO\n"
"   lfh.file_name_length = #filename\n"
"   lfh.extra_field_length = 0\n"
"   lfh.file_name = filename:gsub(\"\\\\\", \"/\")\n"
"   lfh.external_attr = shl(493, 16) -- TODO proper permissions\n"
"   self.in_open_file = true\n"
"   return true\n"
"end\n"
"\n"
"--- Write data to the file currently being stored in the zipfile.\n"
"-- @param self handle of the zipfile being written.\n"
"-- @param data string containing full contents of the file.\n"
"-- @return true if succeeded, nil in case of failure.\n"
"local function zipwriter_write_file_in_zip(self, data)\n"
"   if not self.in_open_file then\n"
"      return nil\n"
"   end\n"
"   local lfh = self.local_file_header\n"
"   local compressed = zlib_compress(data, \"raw\")\n"
"   lfh.crc32 = zlib_crc32(data)\n"
"   lfh.compressed_size = #compressed\n"
"   lfh.uncompressed_size = #data\n"
"   self.data = compressed\n"
"   return true\n"
"end\n"
"\n"
"--- Complete the writing of a file stored in the zipfile.\n"
"-- @param self handle of the zipfile being written.\n"
"-- @return true if succeeded, nil in case of failure.\n"
"local function zipwriter_close_file_in_zip(self)\n"
"   local zh = self.ziphandle\n"
"   \n"
"   if not self.in_open_file then\n"
"      return nil\n"
"   end\n"
"\n"
"   -- Local file header\n"
"   local lfh = self.local_file_header\n"
"   lfh.offset = zh:seek()\n"
"   zh:write(LOCAL_FILE_HEADER_SIGNATURE)\n"
"   zh:write(number_to_lestring(20, 2)) -- version needed to extract: 2.0\n"
"   zh:write(number_to_lestring(4, 2)) -- general purpose bit flag\n"
"   zh:write(number_to_lestring(8, 2)) -- compression method: deflate\n"
"   zh:write(number_to_lestring(lfh.last_mod_file_time, 2))\n"
"   zh:write(number_to_lestring(lfh.last_mod_file_date, 2))\n"
"   zh:write(number_to_lestring(lfh.crc32, 4))\n"
"   zh:write(number_to_lestring(lfh.compressed_size, 4))\n"
"   zh:write(number_to_lestring(lfh.uncompressed_size, 4))\n"
"   zh:write(number_to_lestring(lfh.file_name_length, 2))\n"
"   zh:write(number_to_lestring(lfh.extra_field_length, 2))\n"
"   zh:write(lfh.file_name)\n"
"\n"
"   -- File data\n"
"   zh:write(self.data)\n"
"   \n"
"   -- Data descriptor\n"
"   zh:write(DATA_DESCRIPTOR_SIGNATURE)\n"
"   zh:write(number_to_lestring(lfh.crc32, 4))\n"
"   zh:write(number_to_lestring(lfh.compressed_size, 4))\n"
"   zh:write(number_to_lestring(lfh.uncompressed_size, 4))\n"
"   \n"
"   table.insert(self.files, lfh)\n"
"   self.in_open_file = false\n"
"   \n"
"   return true\n"
"end\n"
"\n"
"-- @return boolean or (boolean, string): true on success,\n"
"-- false and an error message on failure.\n"
"local function zipwriter_add(self, file)\n"
"   local fin\n"
"   local ok, err = self:open_new_file_in_zip(file)\n"
"   if not ok then\n"
"      err = \"error in opening \"..file..\" in zipfile\"\n"
"   else\n"
"      fin = io.open(fs.absolute_name(file), \"rb\")\n"
"      if not fin then\n"
"         ok = false\n"
"         err = \"error opening \"..file..\" for reading\"\n"
"      end\n"
"   end\n"
"   if ok then\n"
"      local data = fin:read(\"*a\")\n"
"      if not data then\n"
"         err = \"error reading \"..file\n"
"         ok = false\n"
"      else\n"
"         ok = self:write_file_in_zip(data)\n"
"         if not ok then\n"
"            err = \"error in writing \"..file..\" in the zipfile\"\n"
"         end\n"
"      end\n"
"   end\n"
"   if fin then\n"
"      fin:close()\n"
"   end\n"
"   if ok then\n"
"      ok = self:close_file_in_zip()\n"
"      if not ok then\n"
"         err = \"error in writing \"..file..\" in the zipfile\"\n"
"      end\n"
"   end\n"
"   return ok == true, err\n"
"end\n"
"\n"
"--- Complete the writing of the zipfile.\n"
"-- @param self handle of the zipfile being written.\n"
"-- @return true if succeeded, nil in case of failure.\n"
"local function zipwriter_close(self)\n"
"   local zh = self.ziphandle\n"
"   \n"
"   local central_directory_offset = zh:seek()\n"
"   \n"
"   local size_of_central_directory = 0\n"
"   -- Central directory structure\n"
"   for _, lfh in ipairs(self.files) do\n"
"      zh:write(CENTRAL_DIRECTORY_SIGNATURE) -- signature\n"
"      zh:write(number_to_lestring(3, 2)) -- version made by: UNIX\n"
"      zh:write(number_to_lestring(20, 2)) -- version needed to extract: 2.0\n"
"      zh:write(number_to_lestring(0, 2)) -- general purpose bit flag\n"
"      zh:write(number_to_lestring(8, 2)) -- compression method: deflate\n"
"      zh:write(number_to_lestring(lfh.last_mod_file_time, 2))\n"
"      zh:write(number_to_lestring(lfh.last_mod_file_date, 2))\n"
"      zh:write(number_to_lestring(lfh.crc32, 4))\n"
"      zh:write(number_to_lestring(lfh.compressed_size, 4))\n"
"      zh:write(number_to_lestring(lfh.uncompressed_size, 4))\n"
"      zh:write(number_to_lestring(lfh.file_name_length, 2))\n"
"      zh:write(number_to_lestring(lfh.extra_field_length, 2))\n"
"      zh:write(number_to_lestring(0, 2)) -- file comment length\n"
"      zh:write(number_to_lestring(0, 2)) -- disk number start\n"
"      zh:write(number_to_lestring(0, 2)) -- internal file attributes\n"
"      zh:write(number_to_lestring(lfh.external_attr, 4)) -- external file attributes\n"
"      zh:write(number_to_lestring(lfh.offset, 4)) -- relative offset of local header\n"
"      zh:write(lfh.file_name)\n"
"      size_of_central_directory = size_of_central_directory + 46 + lfh.file_name_length\n"
"   end\n"
"   \n"
"   -- End of central directory record\n"
"   zh:write(END_OF_CENTRAL_DIR_SIGNATURE) -- signature\n"
"   zh:write(number_to_lestring(0, 2)) -- number of this disk\n"
"   zh:write(number_to_lestring(0, 2)) -- number of disk with start of central directory\n"
"   zh:write(number_to_lestring(#self.files, 2)) -- total number of entries in the central dir on this disk\n"
"   zh:write(number_to_lestring(#self.files, 2)) -- total number of entries in the central dir\n"
"   zh:write(number_to_lestring(size_of_central_directory, 4))\n"
"   zh:write(number_to_lestring(central_directory_offset, 4))\n"
"   zh:write(number_to_lestring(0, 2)) -- zip file comment length\n"
"   zh:close()\n"
"\n"
"   return true\n"
"end\n"
"\n"
"--- Return a zip handle open for writing.\n"
"-- @param name filename of the zipfile to be created.\n"
"-- @return a zip handle, or nil in case of error.\n"
"function zip.new_zipwriter(name)\n"
"   \n"
"   local zw = {}\n"
"  \n"
"   zw.ziphandle = io.open(fs.absolute_name(name), \"wb\")\n"
"   if not zw.ziphandle then\n"
"      return nil\n"
"   end\n"
"   zw.files = {}\n"
"   zw.in_open_file = false\n"
"   \n"
"   zw.add = zipwriter_add\n"
"   zw.close = zipwriter_close\n"
"   zw.open_new_file_in_zip = zipwriter_open_new_file_in_zip\n"
"   zw.write_file_in_zip = zipwriter_write_file_in_zip\n"
"   zw.close_file_in_zip = zipwriter_close_file_in_zip\n"
"\n"
"   return zw\n"
"end\n"
"\n"
"--- Compress files in a .zip archive.\n"
"-- @param zipfile string: pathname of .zip archive to be created.\n"
"-- @param ... Filenames to be stored in the archive are given as\n"
"-- additional arguments.\n"
"-- @return boolean or (boolean, string): true on success,\n"
"-- false and an error message on failure.\n"
"function zip.zip(zipfile, ...)\n"
"   local zw = zip.new_zipwriter(zipfile)\n"
"   if not zw then\n"
"      return nil, \"error opening \"..zipfile\n"
"   end\n"
"\n"
"   local ok, err\n"
"   for _, file in pairs({...}) do\n"
"      if fs.is_dir(file) then\n"
"         for _, entry in pairs(fs.find(file)) do\n"
"            local fullname = dir.path(file, entry)\n"
"            if fs.is_file(fullname) then\n"
"               ok, err = zw:add(fullname)\n"
"               if not ok then break end\n"
"            end\n"
"         end\n"
"      else\n"
"         ok, err = zw:add(file)\n"
"         if not ok then break end\n"
"      end\n"
"   end\n"
"\n"
"   zw:close()\n"
"   return ok, err\n"
"end\n"
"\n"
"\n"
"local function ziptime_to_luatime(ztime, zdate)\n"
"   local date = {\n"
"      year = shr(zdate, 9) + 1980,\n"
"      month = shr(lowbits(zdate, 9), 5),\n"
"      day = lowbits(zdate, 5),\n"
"      hour = shr(ztime, 11),\n"
"      min = shr(lowbits(ztime, 11), 5),\n"
"      sec = lowbits(ztime, 5) * 2,\n"
"   }\n"
"\n"
"   if date.month == 0 then date.month = 1 end\n"
"   if date.day == 0 then date.day = 1 end\n"
"\n"
"   return date\n"
"end\n"
"\n"
"local function read_file_in_zip(zh, cdr)\n"
"   local sig = zh:read(4)\n"
"   if sig ~= LOCAL_FILE_HEADER_SIGNATURE then\n"
"      return nil, \"failed reading Local File Header signature\"\n"
"   end\n"
"\n"
"   local lfh = {}\n"
"   lfh.version_needed = lestring_to_number(zh:read(2))\n"
"   lfh.bitflag = lestring_to_number(zh:read(2))\n"
"   lfh.compression_method = lestring_to_number(zh:read(2))\n"
"   lfh.last_mod_file_time = lestring_to_number(zh:read(2))\n"
"   lfh.last_mod_file_date = lestring_to_number(zh:read(2))\n"
"   lfh.crc32 = lestring_to_number(zh:read(4))\n"
"   lfh.compressed_size = lestring_to_number(zh:read(4))\n"
"   lfh.uncompressed_size = lestring_to_number(zh:read(4))\n"
"   lfh.file_name_length = lestring_to_number(zh:read(2))\n"
"   lfh.extra_field_length = lestring_to_number(zh:read(2))\n"
"   lfh.file_name = zh:read(lfh.file_name_length)\n"
"   lfh.extra_field = zh:read(lfh.extra_field_length)\n"
"\n"
"   local data = zh:read(cdr.compressed_size)\n"
"\n"
"   local uncompressed\n"
"   if cdr.compression_method == 8 then\n"
"      uncompressed = zlib_uncompress(data, \"raw\")\n"
"   elseif cdr.compression_method == 0 then\n"
"      uncompressed = data\n"
"   else\n"
"      return nil, \"unknown compression method \" .. cdr.compression_method\n"
"   end\n"
"\n"
"   if #uncompressed ~= cdr.uncompressed_size then\n"
"      return nil, \"uncompressed size doesn't match\"\n"
"   end\n"
"   if cdr.crc32 ~= zlib_crc32(uncompressed) then\n"
"      return nil, \"crc32 failed (expected \" .. cdr.crc32 .. \") - data: \" .. uncompressed\n"
"   end\n"
"\n"
"   return uncompressed\n"
"end\n"
"\n"
"local function process_end_of_central_dir(zh)\n"
"   local at, err = zh:seek(\"end\", -22)\n"
"   if not at then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   while true do\n"
"      local sig = zh:read(4)\n"
"      if sig == END_OF_CENTRAL_DIR_SIGNATURE then\n"
"         break\n"
"      end\n"
"      at = at - 1\n"
"      local at1, err = zh:seek(\"set\", at)\n"
"      if at1 ~= at then\n"
"         return nil, \"Could not find End of Central Directory signature\"\n"
"      end\n"
"   end\n"
"   \n"
"   -- number of this disk (2 bytes)\n"
"   -- number of the disk with the start of the central directory (2 bytes)\n"
"   -- total number of entries in the central directory on this disk (2 bytes)\n"
"   -- total number of entries in the central directory (2 bytes)\n"
"   zh:seek(\"cur\", 6)\n"
"\n"
"   local central_directory_entries = lestring_to_number(zh:read(2))\n"
"\n"
"   -- central directory size (4 bytes)\n"
"   zh:seek(\"cur\", 4)\n"
"\n"
"   local central_directory_offset = lestring_to_number(zh:read(4))\n"
"\n"
"   return central_directory_entries, central_directory_offset\n"
"end\n"
"\n"
"local function process_central_dir(zh, cd_entries)\n"
"\n"
"   local files = {}\n"
"\n"
"   for i = 1, cd_entries do\n"
"      local sig = zh:read(4)\n"
"      if sig ~= CENTRAL_DIRECTORY_SIGNATURE then\n"
"         return nil, \"failed reading Central Directory signature\"\n"
"      end\n"
"\n"
"      local cdr = {}\n"
"      files[i] = cdr\n"
"\n"
"      cdr.version_made_by = lestring_to_number(zh:read(2))\n"
"      cdr.version_needed = lestring_to_number(zh:read(2))\n"
"      cdr.bitflag = lestring_to_number(zh:read(2))\n"
"      cdr.compression_method = lestring_to_number(zh:read(2))\n"
"      cdr.last_mod_file_time = lestring_to_number(zh:read(2))\n"
"      cdr.last_mod_file_date = lestring_to_number(zh:read(2))\n"
"      cdr.last_mod_luatime = ziptime_to_luatime(cdr.last_mod_file_time, cdr.last_mod_file_date)\n"
"      cdr.crc32 = lestring_to_number(zh:read(4))\n"
"      cdr.compressed_size = lestring_to_number(zh:read(4))\n"
"      cdr.uncompressed_size = lestring_to_number(zh:read(4))\n"
"      cdr.file_name_length = lestring_to_number(zh:read(2))\n"
"      cdr.extra_field_length = lestring_to_number(zh:read(2))\n"
"      cdr.file_comment_length = lestring_to_number(zh:read(2))\n"
"      cdr.disk_number_start = lestring_to_number(zh:read(2))\n"
"      cdr.internal_attr = lestring_to_number(zh:read(2))\n"
"      cdr.external_attr = lestring_to_number(zh:read(4))\n"
"      cdr.offset = lestring_to_number(zh:read(4))\n"
"      cdr.file_name = zh:read(cdr.file_name_length)\n"
"      cdr.extra_field = zh:read(cdr.extra_field_length)\n"
"      cdr.file_comment = zh:read(cdr.file_comment_length)\n"
"   end\n"
"   return files\n"
"end\n"
"\n"
"--- Uncompress files from a .zip archive.\n"
"-- @param zipfile string: pathname of .zip archive to be created.\n"
"-- @return boolean or (boolean, string): true on success,\n"
"-- false and an error message on failure.\n"
"function zip.unzip(zipfile)\n"
"   zipfile = fs.absolute_name(zipfile)\n"
"   local zh, err = io.open(zipfile, \"rb\")\n"
"   if not zh then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   local cd_entries, cd_offset = process_end_of_central_dir(zh)\n"
"   if not cd_entries then\n"
"      return nil, cd_offset\n"
"   end\n"
"   \n"
"   local ok, err = zh:seek(\"set\", cd_offset)\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   local files, err = process_central_dir(zh, cd_entries)\n"
"   if not files then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   for _, cdr in ipairs(files) do\n"
"      local file = cdr.file_name\n"
"      if file:sub(#file) == \"/\" then\n"
"         local ok, err = fs.make_dir(dir.path(fs.current_dir(), file))\n"
"         if not ok then\n"
"            return nil, err\n"
"         end\n"
"      else\n"
"         local base = dir.dir_name(file)\n"
"         if base ~= \"\" then\n"
"            base = dir.path(fs.current_dir(), base)\n"
"            if not fs.is_dir(base) then\n"
"               local ok, err = fs.make_dir(base)\n"
"               if not ok then\n"
"                  return nil, err\n"
"               end\n"
"            end\n"
"         end\n"
"\n"
"         local ok, err = zh:seek(\"set\", cdr.offset)\n"
"         if not ok then\n"
"            return nil, err\n"
"         end\n"
"\n"
"         local contents, err = read_file_in_zip(zh, cdr)\n"
"         if not contents then\n"
"            return nil, err\n"
"         end\n"
"         local pathname = dir.path(fs.current_dir(), file)\n"
"         local wf, err = io.open(pathname, \"wb\")\n"
"         if not wf then\n"
"            zh:close()\n"
"            return nil, err\n"
"         end\n"
"         wf:write(contents)\n"
"         wf:close()\n"
"\n"
"         if cdr.external_attr > 0 then\n"
"            fs.set_permissions(pathname, \"exec\", \"all\")\n"
"         else\n"
"            fs.set_permissions(pathname, \"read\", \"all\")\n"
"         end\n"
"         fs.set_time(pathname, cdr.last_mod_luatime)\n"
"      end\n"
"   end\n"
"   zh:close()\n"
"   return true\n"
"end\n"
"\n"
"function zip.gzip(input_filename, output_filename)\n"
"   assert(type(input_filename) == \"string\")\n"
"   assert(output_filename == nil or type(output_filename) == \"string\")\n"
"   \n"
"   if not output_filename then\n"
"      output_filename = input_filename .. \".gz\"\n"
"   end\n"
"\n"
"   local fn = fun.partial(fun.flip(zlib_compress), \"gzip\")\n"
"   return fs.filter_file(fn, input_filename, output_filename)\n"
"end\n"
"\n"
"function zip.gunzip(input_filename, output_filename)\n"
"   assert(type(input_filename) == \"string\")\n"
"   assert(output_filename == nil or type(output_filename) == \"string\")\n"
"   \n"
"   if not output_filename then\n"
"      output_filename = input_filename:gsub(\"%.gz$\", \"\")\n"
"   end\n"
"\n"
"   local fn = fun.partial(fun.flip(zlib_uncompress), \"gzip\")\n"
"   return fs.filter_file(fn, input_filename, output_filename)\n"
"end\n"
"\n"
"return zip\n"
""
;
