const char luarocks_type_rockspec_lua[] =
"local type_rockspec = {}\n"
"\n"
"local type_check = require(\"luarocks.type_check\")\n"
"\n"
"type_rockspec.rockspec_format = \"3.0\"\n"
"\n"
"-- Syntax for type-checking tables:\n"
"--\n"
"-- A type-checking table describes typing data for a value.\n"
"-- Any key starting with an underscore has a special meaning:\n"
"-- _type (string) is the Lua type of the value. Default is \"table\".\n"
"-- _mandatory (boolean) indicates if the value is a mandatory key in its container table. Default is false.\n"
"-- For \"string\" types only:\n"
"--    _pattern (string) is the string-matching pattern, valid for string types only. Default is \".*\".\n"
"-- For \"table\" types only:\n"
"--    _any (table) is the type-checking table for unspecified keys, recursively checked.\n"
"--    _more (boolean) indicates that the table accepts unspecified keys and does not type-check them.\n"
"--    Any other string keys that don't start with an underscore represent known keys and are type-checking tables, recursively checked.\n"
"\n"
"local rockspec_formats, versions = type_check.declare_schemas({\n"
"   [\"1.0\"] = {\n"
"      rockspec_format = { _type = \"string\" },\n"
"      package = { _type = \"string\", _mandatory = true },\n"
"      version = { _type = \"string\", _pattern = \"[%w.]+-[%d]+\", _mandatory = true },\n"
"      description = {\n"
"         summary = { _type = \"string\" },\n"
"         detailed = { _type = \"string\" },\n"
"         homepage = { _type = \"string\" },\n"
"         license = { _type = \"string\" },\n"
"         maintainer = { _type = \"string\" },\n"
"      },\n"
"      dependencies = {\n"
"         platforms = type_check.MAGIC_PLATFORMS,\n"
"         _any = {\n"
"            _type = \"string\",\n"
"            _name = \"a valid dependency string\",\n"
"            _pattern = \"%s*([a-zA-Z0-9][a-zA-Z0-9%.%-%_]*)%s*([^/]*)\",\n"
"         },\n"
"      },\n"
"      supported_platforms = {\n"
"         _any = { _type = \"string\" },\n"
"      },\n"
"      external_dependencies = {\n"
"         platforms = type_check.MAGIC_PLATFORMS,\n"
"         _any = {\n"
"            program = { _type = \"string\" },\n"
"            header = { _type = \"string\" },\n"
"            library = { _type = \"string\" },\n"
"         }\n"
"      },\n"
"      source = {\n"
"         _mandatory = true,\n"
"         platforms = type_check.MAGIC_PLATFORMS,\n"
"         url = { _type = \"string\", _mandatory = true },\n"
"         md5 = { _type = \"string\" },\n"
"         file = { _type = \"string\" },\n"
"         dir = { _type = \"string\" },\n"
"         tag = { _type = \"string\" },\n"
"         branch = { _type = \"string\" },\n"
"         module = { _type = \"string\" },\n"
"         cvs_tag = { _type = \"string\" },\n"
"         cvs_module = { _type = \"string\" },\n"
"      },\n"
"      build = {\n"
"         platforms = type_check.MAGIC_PLATFORMS,\n"
"         type = { _type = \"string\" },\n"
"         install = {\n"
"            lua = {\n"
"               _more = true\n"
"            },\n"
"            lib = {\n"
"               _more = true\n"
"            },\n"
"            conf = {\n"
"               _more = true\n"
"            },\n"
"            bin = {\n"
"               _more = true\n"
"            }\n"
"         },\n"
"         copy_directories = {\n"
"            _any = { _type = \"string\" },\n"
"         },\n"
"         _more = true,\n"
"         _mandatory = true\n"
"      },\n"
"      hooks = {\n"
"         platforms = type_check.MAGIC_PLATFORMS,\n"
"         post_install = { _type = \"string\" },\n"
"      },\n"
"   },\n"
"   \n"
"   [\"1.1\"] = {\n"
"      deploy = {\n"
"         wrap_bin_scripts = { _type = \"boolean\" },\n"
"      }\n"
"   },\n"
"   \n"
"   [\"3.0\"] = {\n"
"      description = {\n"
"         labels = {\n"
"            _any = { _type = \"string\" }\n"
"         },\n"
"         issues_url = { _type = \"string\" },\n"
"      },\n"
"      dependencies = {\n"
"         _any = {\n"
"            _pattern = \"%s*([a-zA-Z0-9%.%-%_]*/\?[a-zA-Z0-9][a-zA-Z0-9%.%-%_]*)%s*([^/]*)\",\n"
"         },\n"
"      },\n"
"      build_dependencies = {\n"
"         platforms = type_check.MAGIC_PLATFORMS,\n"
"         _any = {\n"
"            _type = \"string\",\n"
"            _name = \"a valid dependency string\",\n"
"            _pattern = \"%s*([a-zA-Z0-9%.%-%_]*/\?[a-zA-Z0-9][a-zA-Z0-9%.%-%_]*)%s*([^/]*)\",\n"
"         },\n"
"      },\n"
"      test_dependencies = {\n"
"         platforms = type_check.MAGIC_PLATFORMS,\n"
"         _any = {\n"
"            _type = \"string\",\n"
"            _name = \"a valid dependency string\",\n"
"            _pattern = \"%s*([a-zA-Z0-9%.%-%_]*/\?[a-zA-Z0-9][a-zA-Z0-9%.%-%_]*)%s*([^/]*)\",\n"
"         },\n"
"      },\n"
"      build = {\n"
"         _mandatory = false,\n"
"      },\n"
"      test = {\n"
"         platforms = type_check.MAGIC_PLATFORMS,\n"
"         type = { _type = \"string\" },\n"
"         _more = true,\n"
"      },\n"
"   }\n"
"})\n"
"\n"
"type_rockspec.order = {\"rockspec_format\", \"package\", \"version\", \n"
"   { \"source\", { \"url\", \"tag\", \"branch\", \"md5\" } },\n"
"   { \"description\", {\"summary\", \"detailed\", \"homepage\", \"license\" } },\n"
"   \"supported_platforms\", \"dependencies\", \"build_dependencies\", \"external_dependencies\",\n"
"   { \"build\", {\"type\", \"modules\", \"copy_directories\", \"platforms\"} },\n"
"   \"test_dependencies\", { \"test\", {\"type\"} },\n"
"   \"hooks\"}\n"
"\n"
"local function check_rockspec_using_version(rockspec, globals, version)\n"
"   local schema = rockspec_formats[version]\n"
"   if not schema then\n"
"      return nil, \"unknown rockspec format \" .. version\n"
"   end\n"
"   local ok, err = type_check.check_undeclared_globals(globals, schema)\n"
"   if ok then\n"
"      ok, err = type_check.type_check_table(version, rockspec, schema, \"\")\n"
"   end\n"
"   if ok then\n"
"      return true\n"
"   else\n"
"      return nil, err\n"
"   end\n"
"end\n"
"\n"
"--- Type check a rockspec table.\n"
"-- Verify the correctness of elements from a \n"
"-- rockspec table, reporting on unknown fields and type\n"
"-- mismatches.\n"
"-- @return boolean or (nil, string): true if type checking\n"
"-- succeeded, or nil and an error message if it failed.\n"
"function type_rockspec.check(rockspec, globals)\n"
"   assert(type(rockspec) == \"table\")\n"
"\n"
"   local version = rockspec.rockspec_format or \"1.0\"\n"
"   local ok, err = check_rockspec_using_version(rockspec, globals, version)\n"
"   if ok then\n"
"      return true\n"
"   end\n"
"\n"
"   -- Rockspec parsing failed.\n"
"   -- Let's see if it would pass using a later version.\n"
"\n"
"   local found = false\n"
"   for _, v in ipairs(versions) do\n"
"      if not found then\n"
"         if v == version then\n"
"            found = true\n"
"         end\n"
"      else\n"
"         local v_ok, v_err = check_rockspec_using_version(rockspec, globals, v)\n"
"         if v_ok then\n"
"            return nil, err .. \" (using rockspec format \" .. version .. \" -- \" ..\n"
"               [[adding 'rockspec_format = \"]] .. v .. [[\"' to the rockspec ]] ..\n"
"               [[will fix this)]]\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   return nil, err .. \" (using rockspec format \" .. version .. \")\"\n"
"end\n"
"\n"
"return type_rockspec\n"
""
;
