const char luarocks_type_check_lua[] =
"\n"
"local type_check = {}\n"
"\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local fun = require(\"luarocks.fun\")\n"
"local util = require(\"luarocks.util\")\n"
"local vers = require(\"luarocks.core.vers\")\n"
"--------------------------------------------------------------------------------\n"
"\n"
"-- A magic constant that is not used anywhere in a schema definition\n"
"-- and retains equality when the table is deep-copied.\n"
"type_check.MAGIC_PLATFORMS = 0xEBABEFAC\n"
"\n"
"do\n"
"   local function fill_in_version(tbl, version)\n"
"      for _, v in pairs(tbl) do\n"
"         if type(v) == \"table\" then\n"
"            if v._version == nil then\n"
"               v._version = version\n"
"            end\n"
"            fill_in_version(v)\n"
"         end\n"
"      end\n"
"   end\n"
"   \n"
"   local function expand_magic_platforms(tbl)\n"
"      for k,v in pairs(tbl) do\n"
"         if v == type_check.MAGIC_PLATFORMS then\n"
"            tbl[k] = {\n"
"               _any = util.deep_copy(tbl)\n"
"            }\n"
"            tbl[k]._any[k] = nil\n"
"         elseif type(v) == \"table\" then\n"
"            expand_magic_platforms(v)\n"
"         end\n"
"      end\n"
"   end\n"
"   \n"
"   -- Build a table of schemas.\n"
"   -- @param versions a table where each key is a version number as a string,\n"
"   -- and the value is a schema specification. Schema versions are considered\n"
"   -- incremental: version \"2.0\" only needs to specify what's new/changed from\n"
"   -- version \"1.0\".\n"
"   function type_check.declare_schemas(inputs)\n"
"      local schemas = {}\n"
"      local parent_version\n"
"   \n"
"      local versions = fun.reverse_in(fun.sort_in(util.keys(inputs), vers.compare_versions))\n"
"\n"
"      for _, version in ipairs(versions) do\n"
"         local schema = inputs[version]\n"
"         if parent_version ~= nil then\n"
"            local copy = util.deep_copy(schemas[parent_version])\n"
"            util.deep_merge(copy, schema)\n"
"            schema = copy\n"
"         end\n"
"         fill_in_version(schema, version)\n"
"         expand_magic_platforms(schema)\n"
"         parent_version = version\n"
"         schemas[version] = schema\n"
"      end\n"
"\n"
"      return schemas, versions\n"
"   end\n"
"end\n"
"\n"
"--------------------------------------------------------------------------------\n"
"\n"
"local function check_version(version, typetbl, context)\n"
"   local typetbl_version = typetbl._version or \"1.0\"\n"
"   if vers.compare_versions(typetbl_version, version) then\n"
"      if context == \"\" then\n"
"         return nil, \"Invalid rockspec_format version number in rockspec\? Please fix rockspec accordingly.\"\n"
"      else\n"
"         return nil, context..\" is not supported in rockspec format \"..version..\" (requires version \"..typetbl_version..\"), please fix the rockspec_format field accordingly.\"\n"
"      end\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"--- Type check an object.\n"
"-- The object is compared against an archetypical value\n"
"-- matching the expected type -- the actual values don't matter,\n"
"-- only their types. Tables are type checked recursively.\n"
"-- @param version string: The version of the item.\n"
"-- @param item any: The object being checked.\n"
"-- @param typetbl any: The type-checking table for the object.\n"
"-- @param context string: A string indicating the \"context\" where the\n"
"-- error occurred (the full table path), for error messages.\n"
"-- @return boolean or (nil, string): true if type checking\n"
"-- succeeded, or nil and an error message if it failed.\n"
"-- @see type_check_table\n"
"local function type_check_item(version, item, typetbl, context)\n"
"   assert(type(version) == \"string\")\n"
"\n"
"   if typetbl._version and typetbl._version ~= \"1.0\" then\n"
"      local ok, err = check_version(version, typetbl, context)\n"
"      if not ok then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"   \n"
"   local item_type = type(item) or \"nil\"\n"
"   local expected_type = typetbl._type or \"table\"\n"
"   \n"
"   if expected_type == \"number\" then\n"
"      if not tonumber(item) then\n"
"         return nil, \"Type mismatch on field \"..context..\": expected a number\"\n"
"      end\n"
"   elseif expected_type == \"string\" then\n"
"      if item_type ~= \"string\" then\n"
"         return nil, \"Type mismatch on field \"..context..\": expected a string, got \"..item_type\n"
"      end\n"
"      local pattern = typetbl._pattern\n"
"      if pattern then\n"
"         if not item:match(\"^\"..pattern..\"$\") then\n"
"            local what = typetbl._name or (\"'\"..pattern..\"'\")\n"
"            return nil, \"Type mismatch on field \"..context..\": invalid value '\"..item..\"' does not match \" .. what\n"
"         end\n"
"      end\n"
"   elseif expected_type == \"table\" then\n"
"      if item_type ~= expected_type then\n"
"         return nil, \"Type mismatch on field \"..context..\": expected a table\"\n"
"      else\n"
"         return type_check.type_check_table(version, item, typetbl, context)\n"
"      end\n"
"   elseif item_type ~= expected_type then\n"
"      return nil, \"Type mismatch on field \"..context..\": expected \"..expected_type\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"local function mkfield(context, field)\n"
"   if context == \"\" then\n"
"      return tostring(field)\n"
"   elseif type(field) == \"string\" then\n"
"      return context..\".\"..field\n"
"   else\n"
"      return context..\"[\"..tostring(field)..\"]\"\n"
"   end\n"
"end\n"
"\n"
"--- Type check the contents of a table.\n"
"-- The table's contents are compared against a reference table,\n"
"-- which contains the recognized fields, with archetypical values\n"
"-- matching the expected types -- the actual values of items in the\n"
"-- reference table don't matter, only their types (ie, for field x\n"
"-- in tbl that is correctly typed, type(tbl.x) == type(types.x)).\n"
"-- If the reference table contains a field called MORE, then\n"
"-- unknown fields in the checked table are accepted.\n"
"-- If it contains a field called ANY, then its type will be \n"
"-- used to check any unknown fields. If a field is prefixed\n"
"-- with MUST_, it is mandatory; its absence from the table is\n"
"-- a type error.\n"
"-- Tables are type checked recursively.\n"
"-- @param version string: The version of tbl.\n"
"-- @param tbl table: The table to be type checked.\n"
"-- @param typetbl table: The type-checking table, containing\n"
"-- values for recognized fields in the checked table.\n"
"-- @param context string: A string indicating the \"context\" where the\n"
"-- error occurred (such as the name of the table the item is a part of),\n"
"-- to be used by error messages.\n"
"-- @return boolean or (nil, string): true if type checking\n"
"-- succeeded, or nil and an error message if it failed.\n"
"function type_check.type_check_table(version, tbl, typetbl, context)\n"
"   assert(type(version) == \"string\")\n"
"   assert(type(tbl) == \"table\")\n"
"   assert(type(typetbl) == \"table\")\n"
"\n"
"   local ok, err = check_version(version, typetbl, context)\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   for k, v in pairs(tbl) do\n"
"      local t = typetbl[k] or typetbl._any\n"
"      if t then \n"
"         local ok, err = type_check_item(version, v, t, mkfield(context, k))\n"
"         if not ok then return nil, err end\n"
"      elseif typetbl._more then\n"
"         -- Accept unknown field\n"
"      else\n"
"         if not cfg.accept_unknown_fields then\n"
"            return nil, \"Unknown field \"..k\n"
"         end\n"
"      end\n"
"   end\n"
"   for k, v in pairs(typetbl) do\n"
"      if k:sub(1,1) ~= \"_\" and v._mandatory then\n"
"         if not tbl[k] then\n"
"            return nil, \"Mandatory field \"..mkfield(context, k)..\" is missing.\"\n"
"         end\n"
"      end\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"function type_check.check_undeclared_globals(globals, typetbl)\n"
"   local undeclared = {}\n"
"   for glob, _ in pairs(globals) do\n"
"      if not (typetbl[glob] or typetbl[\"MUST_\"..glob]) then\n"
"         table.insert(undeclared, glob)\n"
"      end\n"
"   end\n"
"   if #undeclared == 1 then\n"
"      return nil, \"Unknown variable: \"..undeclared[1]\n"
"   elseif #undeclared > 1 then\n"
"      return nil, \"Unknown variables: \"..table.concat(undeclared, \", \")\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"return type_check\n"
""
;
