const char luarocks_upload_api_lua[] =
"\n"
"local api = {}\n"
"\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local util = require(\"luarocks.util\")\n"
"local persist = require(\"luarocks.persist\")\n"
"local multipart = require(\"luarocks.upload.multipart\")\n"
"\n"
"local Api = {}\n"
"\n"
"local function upload_config_file()\n"
"   if not cfg.config_files.user.file then\n"
"      return nil\n"
"   end\n"
"   return (cfg.config_files.user.file:gsub(\"/[^/]+$\", \"/upload_config.lua\"))\n"
"end\n"
"\n"
"function Api:load_config()\n"
"   local upload_conf = upload_config_file()\n"
"   if not upload_conf then return nil end\n"
"   local config, err = persist.load_into_table(upload_conf)\n"
"   return config\n"
"end\n"
"\n"
"function Api:save_config()\n"
"   -- Test configuration before saving it.\n"
"   local res, err = self:raw_method(\"status\")\n"
"   if not res then\n"
"      return nil, err\n"
"   end\n"
"   if res.errors then\n"
"      util.printerr(\"Server says: \" .. tostring(res.errors[1]))\n"
"      return\n"
"   end\n"
"   local upload_conf = upload_config_file()\n"
"   if not upload_conf then return nil end\n"
"   local ok, err = fs.make_dir(dir.dir_name(upload_conf))\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"   persist.save_from_table(upload_conf, self.config)\n"
"   fs.set_permissions(upload_conf, \"read\", \"user\")\n"
"end\n"
"\n"
"function Api:check_version()\n"
"   if not self._server_tool_version then\n"
"      local tool_version = cfg.upload.tool_version\n"
"      local res, err = self:request(tostring(self.config.server) .. \"/api/tool_version\", {\n"
"        current = tool_version\n"
"      })\n"
"      if not res then\n"
"         return nil, err\n"
"      end\n"
"      if not res.version then\n"
"         return nil, \"failed to fetch tool version\"\n"
"      end\n"
"      self._server_tool_version = res.version\n"
"      if res.force_update then\n"
"         return nil, \"Your upload client is too out of date to continue, please upgrade LuaRocks.\"\n"
"      end\n"
"      if res.version ~= tool_version then\n"
"         util.warning(\"your LuaRocks is out of date, consider upgrading.\")\n"
"      end\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"function Api:method(...)\n"
"   local res, err = self:raw_method(...)\n"
"   if not res then\n"
"      return nil, err\n"
"   end\n"
"   if res.errors then\n"
"      if res.errors[1] == \"Invalid key\" then\n"
"         return nil, res.errors[1] .. \" (use the --api-key flag to change)\"\n"
"      end\n"
"      local msg = table.concat(res.errors, \", \")\n"
"      return nil, \"API Failed: \" .. msg\n"
"   end\n"
"   return res\n"
"end\n"
"\n"
"function Api:raw_method(path, ...)\n"
"   self:check_version()\n"
"   local url = tostring(self.config.server) .. \"/api/\" .. tostring(cfg.upload.api_version) .. \"/\" .. tostring(self.config.key) .. \"/\" .. tostring(path)\n"
"   return self:request(url, ...)\n"
"end\n"
"\n"
"local function encode_query_string(t, sep)\n"
"   if sep == nil then\n"
"      sep = \"&\"\n"
"   end\n"
"   local i = 0\n"
"   local buf = { }\n"
"   for k, v in pairs(t) do\n"
"      if type(k) == \"number\" and type(v) == \"table\" then\n"
"         k, v = v[1], v[2]\n"
"      end\n"
"      buf[i + 1] = multipart.url_escape(k)\n"
"      buf[i + 2] = \"=\"\n"
"      buf[i + 3] = multipart.url_escape(v)\n"
"      buf[i + 4] = sep\n"
"      i = i + 4\n"
"   end\n"
"   buf[i] = nil\n"
"   return table.concat(buf)\n"
"end\n"
"\n"
"local function redact_api_url(url)\n"
"   url = tostring(url)\n"
"   return (url:gsub(\".*/api/[^/]+/[^/]+\", \"\")) or \"\"\n"
"end\n"
"\n"
"local ltn12_ok, ltn12 = pcall(require, \"ltn12\")\n"
"if not ltn12_ok then -- If not using LuaSocket and/or LuaSec...\n"
"\n"
"function Api:request(url, params, post_params)\n"
"   local vars = cfg.variables\n"
"   local json_ok, json = util.require_json()\n"
"   if not json_ok then return nil, \"A JSON library is required for this command. \"..json end\n"
"   \n"
"   if fs.which_tool(\"downloader\") == \"wget\" then\n"
"      local curl_ok, err = fs.is_tool_available(vars.CURL, \"curl\")\n"
"      if not curl_ok then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"\n"
"   if not self.config.key then\n"
"      return nil, \"Must have API key before performing any actions.\"\n"
"   end\n"
"   if params and next(params) then\n"
"      url = url .. (\"\?\" .. encode_query_string(params))\n"
"   end\n"
"   local method = \"GET\"\n"
"   local out \n"
"   local tmpfile = fs.tmpname()\n"
"   if post_params then\n"
"      method = \"POST\"\n"
"      local curl_cmd = vars.CURL..\" -f -k -L --silent --user-agent \\\"\"..cfg.user_agent..\" via curl\\\" \"\n"
"      for k,v in pairs(post_params) do\n"
"         local var = v\n"
"         if type(v) == \"table\" then\n"
"            var = \"@\"..v.fname\n"
"         end\n"
"         curl_cmd = curl_cmd .. \"--form \\\"\"..k..\"=\"..var..\"\\\" \"\n"
"      end\n"
"      if cfg.connection_timeout and cfg.connection_timeout > 0 then\n"
"        curl_cmd = curl_cmd .. \"--connect-timeout \"..tonumber(cfg.connection_timeout)..\" \" \n"
"      end\n"
"      local ok = fs.execute_string(curl_cmd..fs.Q(url)..\" -o \"..fs.Q(tmpfile))\n"
"      if not ok then\n"
"         return nil, \"API failure: \" .. redact_api_url(url)\n"
"      end\n"
"   else\n"
"      local ok, err = fs.download(url, tmpfile)\n"
"      if not ok then\n"
"         return nil, \"API failure: \" .. tostring(err) .. \" - \" .. redact_api_url(url)\n"
"      end\n"
"   end\n"
"\n"
"   local tmpfd = io.open(tmpfile)\n"
"   if not tmpfd then\n"
"      os.remove(tmpfile)\n"
"      return nil, \"API failure reading temporary file - \" .. redact_api_url(url)\n"
"   end\n"
"   out = tmpfd:read(\"*a\")\n"
"   tmpfd:close()\n"
"   os.remove(tmpfile)\n"
"\n"
"   if self.debug then\n"
"      util.printout(\"[\" .. tostring(method) .. \" via curl] \" .. redact_api_url(url) .. \" ... \")\n"
"   end\n"
"\n"
"   return json.decode(out)\n"
"end\n"
"\n"
"else -- use LuaSocket and LuaSec\n"
"\n"
"local warned_luasec = false\n"
"\n"
"function Api:request(url, params, post_params)\n"
"   local json_ok, json = util.require_json()\n"
"   if not json_ok then return nil, \"A JSON library is required for this command. \"..json end\n"
"   local server = tostring(self.config.server)\n"
"   local http_ok, http\n"
"   local via = \"luasocket\"\n"
"   if server:match(\"^https://\") then\n"
"      http_ok, http = pcall(require, \"ssl.https\")\n"
"      if http_ok then\n"
"         via = \"luasec\"\n"
"      else\n"
"         if not warned_luasec then\n"
"            util.printerr(\"LuaSec is not available; using plain HTTP. Install 'luasec' to enable HTTPS.\")\n"
"            warned_luasec = true\n"
"         end\n"
"         http_ok, http = pcall(require, \"socket.http\")\n"
"         url = url:gsub(\"^https\", \"http\")\n"
"         via = \"luasocket\"\n"
"      end\n"
"   else\n"
"      http_ok, http = pcall(require, \"socket.http\")\n"
"   end\n"
"   if not http_ok then\n"
"      return nil, \"Failed loading socket library!\"\n"
"   end\n"
"   \n"
"   if not self.config.key then\n"
"      return nil, \"Must have API key before performing any actions.\"\n"
"   end\n"
"   local body\n"
"   local headers = {}\n"
"   if params and next(params) then\n"
"      url = url .. (\"\?\" .. encode_query_string(params))\n"
"   end\n"
"   if post_params then\n"
"      local boundary\n"
"      body, boundary = multipart.encode(post_params)\n"
"      headers[\"Content-length\"] = #body\n"
"      headers[\"Content-type\"] = \"multipart/form-data; boundary=\" .. tostring(boundary)\n"
"   end\n"
"   local method = post_params and \"POST\" or \"GET\"\n"
"   if self.debug then\n"
"      util.printout(\"[\" .. tostring(method) .. \" via \"..via..\"] \" .. redact_api_url(url) .. \" ... \")\n"
"   end\n"
"   local out = {}\n"
"   local _, status = http.request({\n"
"      url = url,\n"
"      headers = headers,\n"
"      method = method,\n"
"      sink = ltn12.sink.table(out),\n"
"      source = body and ltn12.source.string(body)\n"
"   })\n"
"   if self.debug then\n"
"      util.printout(tostring(status))\n"
"   end\n"
"   local pok, ret, err = pcall(json.decode, table.concat(out))\n"
"   if pok and ret then\n"
"      return ret\n"
"   end\n"
"   return nil, \"API returned \" .. tostring(status) .. \" - \" .. redact_api_url(url)\n"
"end\n"
"\n"
"end\n"
"\n"
"function api.new(flags)\n"
"   local self = {}\n"
"   setmetatable(self, { __index = Api })\n"
"   self.config = self:load_config() or {}\n"
"   self.config.server = flags[\"server\"] or self.config.server or cfg.upload.server\n"
"   self.config.version = self.config.version or cfg.upload.version\n"
"   self.config.key = flags[\"temp-key\"] or flags[\"api-key\"] or self.config.key\n"
"   self.debug = flags[\"debug\"]\n"
"   if not self.config.key then\n"
"      return nil, \"You need an API key to upload rocks.\\n\" ..\n"
"                  \"Navigate to \"..self.config.server..\"/settings to get a key\\n\" ..\n"
"                  \"and then pass it through the --api-key=<key> flag.\"\n"
"   end\n"
"   if flags[\"api-key\"] then\n"
"      self:save_config()\n"
"   end\n"
"   return self\n"
"end\n"
"\n"
"return api\n"
""
;
